/**
 * 
 * @author Octavian Patrascoiu
 *
 */

// Usercode Section
package rwd.tableaux.syntax.parser;

import java_cup.runtime.*;
import uk.ac.kent.cs.kmf.util.*;
      
%%  
// Options Sections
%unicode
%cup
%line
%column
   
// Declarations Section
%{
	// Debug flag
	public static boolean lexDebug = false;
	protected void debug(int type) {
    	if (lexDebug) {
    		log.reportMessage(yyline+":"+yycolumn+" Token "+type+" '"+yytext()+"'");		
    	}
	}

	// Output log
	protected ILog log;
	public void setLog(ILog log) {
		this.log = log;
	}
	
    // Create a new java_cup.runtime.Symbol with information about the current token
    protected Symbol symbol(int type) {
    	debug(type);
        return new Symbol(type, yyline, yycolumn, new String(yytext()));
    }
    protected Symbol symbol(int type, Object value) {
		debug(type);
        return new Symbol(type, yyline, yycolumn, value);
    }
%}
   
%eofval{
	return symbol(sym.EOF);
%eofval}


// Macro declarations
lineTerminator 		= \r|\n|\r\n|\n\r
whiteSpace     		= [ \t\f\n\r]
comment				= {paragraphComment} | {lineComment}
paragraphComment	= "/*" ~"*/"
lineComment			= "--" ~{lineTerminator}

lowerCase			= [a-z]
upperCase			= [A-Z]
digit				= [0-9]
letter				= {lowerCase} | {upperCase} | [_]
alpha				= {letter} | {digit}
simpleName			= {letter}{alpha}*

%%
// Lexical Rules Section 
   
<YYINITIAL> {  
	{whiteSpace}		{ /* just skip what was found, do nothing */ }   
	{comment}			{ /* just skip what was found, do nothing */ }   

	"not"				{ return symbol(sym.NOT); }
	"and"				{ return symbol(sym.AND); }
	"or"				{ return symbol(sym.OR); }
	"=>"				{ return symbol(sym.IMPLIES); }
	"<=>"				{ return symbol(sym.EQUIVALENT); }
	
	"true"				{ return symbol(sym.TRUE); }
	"false"				{ return symbol(sym.FALSE); }

	"("					{ return symbol(sym.LEFT_PAR); }
	")"					{ return symbol(sym.RIGHT_PAR); }
	{simpleName}		{ return symbol(sym.NAME); }
}

[^]						{ return symbol(sym.BAD);}
