package rwd.abstractsyntax.constraintdiagram;

import java.io.Serializable;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.spiderdiagram.Spider;
//import rwd.abstractsyntax.eulerdiagram.Contour;

/**
 * An abstract arrow, consisting of a source object,
 * target object and label.
 * 
 * @author Octavian Patrascoiu, Richard Bosworth
 * @version 24.8.05   19:59
 *
 */
public class Arrow 
	implements Serializable
{
	//
	// Constructors
	//
	
	/**
	 * Post: a new arrow object has been created as specified
	 * @param s source object of this arrow (may be null)
	 * @param t target object of this arrow (may be null)
	 * @param l label of this arrow (may be zero-length)
	 */
	/*
	 * test: once thru
	 */
	public Arrow(Object s, Object t, String l) {
		this.source = s;
		this.target = t;
		this.label = l;
	}
	
	
	/**
	 * Post: a new arrow object has been created with the same source,
	 * target and label as arrow a.
	 * @param a arrow to be cloned
	 */
	/*
	 * test: once thru
	 */
	public Arrow(Arrow a) {
		this.source = a.source;
		this.target = a.target;
		this.label = a.label;
	}
	
	//
	// Object's methods
	//
	
	/*
	 * returns true iff both o1 and o2 are null, or both o1 and o2 are
	 *   of the same type (spider or contour) and have the same label.
	 */
	/*
	 * test:
	 *   o1 null
	 *    o2 non-null
	 *    o2 null
	 *   o1 contour
	 *    o2 not contour
	 *    o2 contour with same label
	 *    o2 contour with different label
	 *   o1 spider
	 *    o2 not spider
	 *    o2 spider with same label
	 *    o2 spider with different label
	 */
	private boolean isEqual(Object o1, Object o2){
		if (o1==null)
			return o2==null;
		else if (o1 instanceof Spider) {
			Spider s1 = (Spider) o1;
			if (!(o2 instanceof Spider)) return false;
			Spider s2 = (Spider) o2;
			return (s1.getLabel().equals(s2.getLabel()));
		}
		else if (o1 instanceof Contour) {
			Contour c1 = (Contour) o1;
			if (!(o2 instanceof Contour)) return false;
			Contour c2 = (Contour) o2;
			return (c1.getLabel().equals(c2.getLabel()));
		}
		else return false;
	}
	
	
	/**
	 * @param o object to be compared with this arrow
	 * @return true, iff this arrow has the same label, source and target as o,
	 *         false otherwise
	 *         (Note: source and/or target may be null, label may be zero-length)
	 */
	/*
	 * test:
	 *   o is not an arrow
	 *   o is an arrow
	 *     o label not equal to this label
	 *     o label equal to this label
	 *       o source not equal to this source
	 *       o source equal to this source
	 *         o target not equal to this target
	 *         o target equal to this target
	 */
	public boolean equals(Object o) {
		if (!(o instanceof Arrow)) return false;
		Arrow a = (Arrow) o;
		return this.label.equals(a.getLabel()) &&
		       isEqual(source,a.getSource()) && 
			   isEqual(target,a.getTarget());
	}
	
	
	/**
	 * @return a hash code value for the Arrow.
	 * Equal arrows return the same hashcode.
	 */
	/*
	 * test:
	 *   arrow has null source
	 *     arrow has null target
	 *     arrow has non-null target
	 *   arrow has non-null source
	 *     arrow has null target
	 *     arrow has non-null target
	 */
	public int hashCode(){
		int sourceHash;
		int targetHash;
		
		if (source==null)
			sourceHash = 0;
		else
			sourceHash = source.hashCode();
		
		if (target==null)
			targetHash = 0;
		else
			targetHash = target.hashCode();
		
		return label.hashCode() + sourceHash + targetHash;
	}
	
	/**
	 * @return a clone of this arrow
	 */
	/*
	 * test: once thru
	 */
	public Object clone() {
		return new Arrow(this);
	}
	
	/**
	 * @return a string representation of this arrow
	 */
	/*
	 * test: once thru
	 */
	public String toString() {
		return label + ":"+ source+"->"+target;

	}
	
	
	//
	// Getters and setters
	//
	
	/**
	 * @return the target of this arrow (may be null)
	 */
	/*
	 * test: once thru
	 * 
	 */
	public Object getTarget() {
		return target;
	}
	
	/**
	 * Post: target of this arrow has been set to t
	 * @param t required target of this arrow
	 */
	/*
	 * test: once thru
	 */
	public void setTarget(Object t) {
		target = t;
	}
	
	/**
	 * @return source object of this arrow (may be null)
	 */
	/*
	 * test: once thru
	 */
	public Object getSource() {
		return source;
	}
	
	/**
	 * Post: source of this arrow has been set to s
	 * @param s required source object of this arrow
	 */
	public void setSource(Object s) {
		source = s;
	}
	
	/**
	 * @return label of this arrow (may be zero-length)
	 */
	/*
	 * test: once thru
	 */
	public String getLabel() {
		return label;
	}
	
	/**
	 * Post: label of this arrow has been set to l
	 * @param l required label of this arrow
	 */
	/*
	 * test: once thru with non-null label
	 */
	public void setLabel(String l) {
		label = l;
	}
	
	
	//
	// Local properties
	//
	protected Object source;
	protected Object target;
	protected String label; 

	
	/*
	// test program
    // ------------

    private static void pr(String s) {
      System.out.println(s);
    }

    private static void hd(String s) {
      System.out.println("\n" + s + "\n");
    }

    private static void pg() {
      try {
        int i;
        System.out.print("next page? (CR)");
        do i = System.in.read(); while (i != '\n');
        System.out.println();
      }
      catch (Exception e) {System.out.println("raised exception : " + e);}
    }

    public static void main(String args[]) throws Exception {
    	
    	Arrow ar1 = new Arrow(null,null,"ar");
    	pr("ar1="+ar1);
    	pg();
    	
    	hd("Tests of isEqual");
    	Contour a = Contour.getContour("a");
    	Contour b = Contour.getContour("b");
    	Spider s = new Spider("s",null);
    	Spider t = new Spider("t",null);
    	pr("ar1.isEqual(null,a)="+ar1.isEqual(null,a));
    	pr("ar1.isEqual(null,null)="+ar1.isEqual(null,null));
    	pr("ar1.isEqual(a,s)="+ar1.isEqual(a,s));
    	pr("ar1.isEqual(a,a)="+ar1.isEqual(a,a));
    	pr("ar1.isEqual(a,b)="+ar1.isEqual(a,b));
    	pr("ar1.isEqual(s,a)="+ar1.isEqual(s,a));
    	pr("ar1.isEqual(s,s)="+ar1.isEqual(s,s));
    	pr("ar1.isEqual(s,t)="+ar1.isEqual(s,t));
    	pg();

    	hd("tests of equals and hashCode");    	
    	pr("ar1="+ar1);
    	pr("ar1.hashCode()="+ar1.hashCode());
    	pr("ar1.equals(a)="+ar1.equals(a));
    	pg();
    	
    	Arrow ar2 = new Arrow(a,null,"ar");
    	pr("ar2="+ar2);
    	pr("ar2.hashCode()="+ar2.hashCode());
    	pr("ar1.equals(ar2)="+ar1.equals(ar2));
    	pg();
    	
    	Arrow ar3 = new Arrow(null,a,"ar");
    	pr("ar3="+ar3);
    	pr("ar3.hashCode()="+ar3.hashCode());
    	pr("ar1.equals(ar3)="+ar1.equals(ar3));
    	pg();
    	
    	Arrow ar4 = new Arrow(null,null,"ar4");
    	pr("ar4="+ar4);
    	pr("ar4.hashCode()="+ar4.hashCode());
    	pr("ar1.equals(ar4)="+ar1.equals(ar4));
    	pg();
    	
    	Arrow ar5 = new Arrow(null,null,"ar");
    	pr("ar5="+ar5);
    	pr("ar5.hashCode()="+ar5.hashCode());
    	pr("ar1.equals(ar5)="+ar1.equals(ar5));
    	pg();
    	
    	Arrow ar6 = new Arrow(a,a,"ar6");
    	pr("ar6="+ar6);
    	pr("ar6.hashCode()="+ar6.hashCode());
    	pr("ar1.equals(ar6)="+ar1.equals(ar6));
    	pg();
    	
    	Arrow ar6clone = (Arrow) ar6.clone();
    	pr("ar6clone="+ar6clone);
    	pr("ar6clone.hashCode()="+ar6clone.hashCode());
    	pr("ar6.equals(ar6clone)="+ar6.equals(ar6clone));
    	pg();
    	
    	Arrow ar7 = new Arrow(ar6);
    	pr("ar7="+ar7);
    	pr("ar7.hashCode()="+ar7.hashCode());
    	pr("ar7.equals(ar6)="+ar7.equals(ar6));
    	pg();
    	
    	hd("Tests of setters");
    	pr("ar1="+ar1);
    	pg();
    	
    	pr("now set to ar11,a,a");
    	ar1.setLabel("ar11");
    	ar1.setSource(a);
    	ar1.setTarget(a);
    	pr("ar1="+ar1);
    	pg();
    	
    	
    }
    */

}
