package rwd.abstractsyntax.spiderdiagram;
import java.awt.Component;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Iterator;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.gui.StaticObjects;
public class DiagramFileUser {
    static File directory = new File(System.getProperty("user.dir"));

    static private String nameIntroduction = "";
    static private String contoursIntroduction = "";
    static private String zonesIntroduction = "";
    static private String shadedZonesIntro = "";
    static private String spidersIntroduction = "";

    static private Hashtable readers = new Hashtable();
    static private Hashtable writers = new Hashtable();

    public static Diagram getDiagramFromWherever(String fileName) {
        try {
            BufferedReader bf;
            //look for filename in readers
            bf = (BufferedReader) readers.get(fileName);
            // if it's not there, build one and add it to tha hashtable
            if (bf == null) {
                bf = new BufferedReader(new FileReader(new File(fileName)));
                readers.put(fileName, bf);
            }
            // use it to get a diagram and return it
            return getDiagram(bf);
        } catch (IOException e) {
            System.out.println("IO exception: " + e);
            e.printStackTrace();
            System.exit(0);
            return null;
        }
    }

    public static Diagram getDiagramFromStart(String filename) {
        try {
            BufferedReader bf;
            //look for filename in readers
            bf = (BufferedReader) readers.get(filename);
            // if it's not there, build one and add it to tha hashtable
            if (bf != null) {
                readers.remove(bf);
                bf.close();
            }
            // create a fresh writer
            bf = new BufferedReader(new FileReader(new File(filename)));
            readers.put(filename, bf);

            // use it to get a diagram and return it
            return getDiagram(bf);
        } catch (IOException e) {
            System.out.println("IO exception: " + e);
            e.printStackTrace();
            System.exit(0);
            return null;
        }
    }
    public static void addDiagramAppend(Diagram d, String filename) {
        try {
            // look for filename in writers
            BufferedWriter bf = (BufferedWriter) writers.get(filename);
            // if it's not there, build one and add it to the hashtable
            if (bf == null) {
                bf =
                    new BufferedWriter(
                        new FileWriter(new File(filename), true));
                writers.put(filename, bf);
            }
            writeToFile(d, bf);
        } catch (IOException e) {
            System.out.println("IO exception: " + e);
            e.printStackTrace();
            System.exit(0);
        }
    }

    public static void addDiagramFreshFile(Diagram d, String filename) {
        try {
            // look for filename in writers
            BufferedWriter bf = (BufferedWriter) writers.get(filename);
            // if it's not there, build one and add it to tha hashtable
            if (bf != null) {
                writers.remove(bf);
                bf.flush();
                bf.close();
            }
            // create a fresh writer
            bf = new BufferedWriter(new FileWriter(new File(filename)));
            writers.put(filename, bf);
            writeToFile(d, bf);
        } catch (IOException e) {
            System.out.println("IO exception: " + e);
            e.printStackTrace();
            System.exit(0);
        }
    }

    public static void clearFile(String filename) {
        try {
            // look for filename in writers
            BufferedWriter bf = (BufferedWriter) writers.get(filename);
            // if it's not there, build one and add it to tha hashtable
            if (bf != null) {
                writers.remove(bf);
                bf.flush();
                bf.close();
            } else {
                throw new IllegalArgumentException();
            }
            // create a fresh writer
            bf = new BufferedWriter(new FileWriter(new File(filename)));
            writers.put(filename, bf);
        } catch (IOException e) {
            System.out.println("IO exception: " + e);
            e.printStackTrace();
            System.exit(0);
        }
    }

    public static void closeReader(String filename) {
        try {
            // look for filename in writers
            BufferedWriter bf = (BufferedWriter) writers.get(filename);
            // if it's not there, build one and add it to tha hashtable
            if (bf != null) {
                writers.remove(bf);
                bf.flush();
                bf.close();
            } else {
                throw new IllegalArgumentException();
            }
        } catch (IOException e) {
            System.out.println("IO exception: " + e);
            e.printStackTrace();
            System.exit(0);
        }
    }

    public static void saveToFile(Diagram d) {
        JFileChooser chooser = new JFileChooser(directory);
        chooser.setSelectedFile(new File(d.getName() + ".diagram"));
        //System.out.println("built FileChooser");
        int returnVal = chooser.showSaveDialog(StaticObjects.mdf);
        if (returnVal != JFileChooser.APPROVE_OPTION) {
            return;
        }
        File f = chooser.getSelectedFile();
        directory = chooser.getCurrentDirectory();
        //System.out.println("got file "+f+" from FileChooser");
        saveToFile(d, f);
    }

    static void saveToFile(Diagram d, File f) {
        boolean fileIsThereAlready = true;
        try {
            new FileInputStream(f);
        } catch (FileNotFoundException e) {
            fileIsThereAlready = false;
        }
        if (fileIsThereAlready
            && JOptionPane.showConfirmDialog(StaticObjects.mdf, "overwrite:" + f)
                != JOptionPane.OK_OPTION) {
            return;
        }
        //System.out.println("save to file "+f);
        try {
            FileWriter fw = new FileWriter(f);
            BufferedWriter b = new BufferedWriter(fw);
            writeToFile(d, b);
            b.flush();
            b.close();
        } catch (IOException e) {
            System.out.println(
                "An IO exception occured when executing "
                    + "saveToFile(..): error is "
                    + e
                    + "\n");
        }
    }
    private static void writeToFile(Diagram d, BufferedWriter b) {
        try {
            // save the outcome of toString()
            String s = toSaveString(d);
            //System.out.println("saving "+s);
            int newLinePlace = s.indexOf('\n');
            while (newLinePlace != -1) {
                b.write(s.substring(0, newLinePlace));
                b.newLine();
                s = s.substring(newLinePlace + 1, s.length());
                newLinePlace = s.indexOf('\n');
            };
            b.write(s);
            b.newLine();
            b.flush();
        } catch (IOException e) {
            System.out.println(
                "An IO exception occured when executing "
                    + "writeToFile(..): error is "
                    + e
                    + "\n");
        }
    }

    public static Vector loadFromFile(Component c) {
        JFileChooser chooser = new JFileChooser(directory);
        chooser.setMultiSelectionEnabled(true);
        if (chooser.showOpenDialog(c) != JFileChooser.APPROVE_OPTION) {
            //System.out.println("other than approval from file opener");
            return null;
        }
        File[] fs = chooser.getSelectedFiles();
        directory = chooser.getCurrentDirectory();
        Vector result = new Vector();
        for (int i = 0; i < fs.length; i++) {
            File f = fs[i];
            InputStream inStream;
            try {
                inStream = new FileInputStream(f);
                //BufferedReader b = new BufferedReader(new FileReader(file));
                InputStreamReader isReader = new InputStreamReader(inStream);
                BufferedReader b = new BufferedReader(isReader);
                Diagram diag = getDiagram(b);
                //System.out.println("about to return:"+result);
                result.add(diag);
            } catch (Exception e) {
                System.out.println("exception thrown " + e);
            }
        }
        return result;
    }

    public static Diagram getDiagram(BufferedReader b) {
        try {
            String line = b.readLine();
            //System.out.println("In DFU: line is " + line);
            line = line.substring(0, 2);
            if (line.equals("UD")) {
                //System.out.println("about to get a unitary diagram");
                Diagram d = getUnitaryDiagram(b);
                //System.out.println("got a unitary diagram " + d.getName());
                return d;
            } else if (line.equals("CD")) {
                //System.out.println("about to get a compound diagram");
                Diagram d = getCompoundDiagram(b);
                //System.out.println("got a compound diagram:"+d.getName());
                return d;
            } else if (line.equals("FD")) { //
                //System.out.println("about to get a compound diagram");
                Diagram d = getFalseDiagram(b);
                //System.out.println("got a false diagram:"+d.getName());
                return d;
            }
        } catch (Exception e) { /**/
            //System.out.println("exception in getDiagram : " + e);
        }
        return null;
    }

    private static Diagram getUnitaryDiagram(BufferedReader b) { //
        //System.out.println("START GET UNITARY DIAGRAM");
        try {
            String diagramName = "not loaded properly";
            Vector contours = new Vector();
            Vector zones = new Vector();
            UnitaryDiagram ud = null;
            //////////////////////////////////////////////////////////////////
            String line = b.readLine(); // line is "name:..."
            //System.out.println("line is "+line);
            diagramName =
                line.substring(nameIntroduction.length(), line.length());
            //System.out.println("diagram name is "+diagramName);

            //////////////////////////////////////////////////////////////////
            line = b.readLine(); // line is "contours:..."
            //System.out.println("line for contours is '"+line+"'");
            line =
                line.substring(
                    contoursIntroduction.length() + 1,
                    line.length() - 1);
            if (line.length() > 0) {
                line = " " + line;
            }
            //System.out.println("line for contours reduces to '"+line+"'");
            int spacePlace = line.indexOf(" ");
            while (spacePlace != -1) {
                String contourName =
                    line.substring(spacePlace + 1, spacePlace + 2);
                //System.out.println("line is '"+line+"'");
                //System.out.println("contour name is "+contourName);
                contours.add(Contour.getContour(contourName));
                line = line.substring(spacePlace + 2, line.length());
                spacePlace = line.indexOf(" ");
            }
            if (line.length() > 2) {
                String contourName = line.substring(1, 2);
                //System.out.println("contour name is "+contourName);
                contours.add(Contour.getContour(contourName));
            }
            //System.out.println("contours are "+contours);
            //////////////////////////////////////////////////////////////////
            line = b.readLine(); // line is "zones:..."
            //System.out.println("zones:"+line);
            //System.out.println(line);
            line = line.substring(line.indexOf("[") + 1, line.length() - 1);
            while (line.length() > 0) {
                String zoneLine;
                try {
                    zoneLine = line.substring(0, line.indexOf(","));
                    line = line.substring(line.indexOf(",") + 2, line.length());
                } catch (StringIndexOutOfBoundsException x) {
                    zoneLine = line;
                    line = "";
                }
                //System.out.println("zoneLine:"+zoneLine);
                //System.out.println("line:"+line);
                Vector containingContours = new Vector();
                while (zoneLine.length() > 0) {
                    if (!zoneLine.equals("-")) {
                        String contourName = zoneLine.substring(0, 1);
                        //System.out.println("contourName is "+contourName);
                        containingContours.add(Contour.getContour(contourName));
                        try {
                            zoneLine = zoneLine.substring(1, zoneLine.length());
                        } catch (StringIndexOutOfBoundsException e) {
                            zoneLine = "";
                        }
                        //System.out.println("zone string is reduced to '"+zoneLine+"'");
                    } else {
                        zoneLine = "";
                    }
                }
                //System.out.println("containing contours are "+containingContours);
                Vector excludingContours = (Vector) (contours.clone());
                //System.out.println("all contours are "+excludingContours);
                excludingContours.removeAll(containingContours);
                //System.out.println("so outside contours must be "+excludingContours);
                zones.add(Zone.getZone(containingContours, excludingContours));
                //System.out.println("zones so far are "+zones);
            }
            ud = new UnitaryDiagram(zones, diagramName);
            //for(int i=0; i<zones.size(); i++)
            //{Zone z = (Zone)zones.get(i);
            //	System.out.println("z"+i+":["+z.getContainingContours()+","+z.getExcludingContours()+"}");
            //}
            //System.out.println("zones are "+zones);
            //////////////////////////////////////////////////////////////////
            line = b.readLine(); // line is "shadedzones:..."
            //System.out.println("shaded zones line "+line);
            line = line.substring(line.indexOf("[") + 1, line.length() - 1);
            while (line.length() > 0) {
                String zoneLine;
                try {
                    zoneLine = line.substring(0, line.indexOf(","));
                    line = line.substring(line.indexOf(",") + 2, line.length());
                } catch (StringIndexOutOfBoundsException x) {
                    zoneLine = line;
                    line = "";
                }
                //System.out.println("zoneLine is "+zoneLine);
                //System.out.println("line is "+line);

                Vector containingContours = new Vector();
                while (zoneLine.length() > 0) {
                    if (!zoneLine.equals("-")) {
                        String contourName = zoneLine.substring(0, 1);
                        //System.out.println("contourName is "+contourName);
                        containingContours.add(Contour.getContour(contourName));
                        try {
                            zoneLine = zoneLine.substring(1, zoneLine.length());
                        } catch (StringIndexOutOfBoundsException e) {
                            zoneLine = "";
                        }
                        //System.out.println("zone string is reduced to '"+zoneLine+"'");
                    } else {
                        zoneLine = "";
                    }
                }
                Vector excludingContours =
                    (Vector) (ud.getCloneOfContourSet());
                excludingContours.removeAll(containingContours);
                ud.shadeZone(
                    Zone.getZone(containingContours, excludingContours));
            }
            //////////////////////////////////////////////////////////////////
            line = b.readLine();
            // line is "spiders: [{-,}, {a,-,}, {a,}, {a,}]"
            line = line.substring(line.indexOf("[") + 1, line.length() - 1);
            // line is "{-,}, {a,-,}, {a,}, {a,}"
            //System.out.println("line is '" + line + "'");
            while (line.length() > 0 && line.charAt(0) == '{') {
                // there are more spiders
                String zonesLine = line.substring(1, line.indexOf("}"));
                line = line.substring(1, line.length());
                // take zonesLine and build zones for that
                //System.out.println("zonesline is '" + zonesLine + "'");
                Vector habitat = new Vector();
                contours = ud.getCloneOfContourSet();
                while (zonesLine != null) {
                    // remember zonesLine might be "-" indicating just outside Zone!!
                    //System.out.println("zonesLine is "+zonesLine);
                    Vector insideContours = new Vector();
                    buildUpZone : while (
                        zonesLine.length() > 0 && zonesLine.charAt(0) != ',') {
                        //System.out.println("extract letter from '"+zonesLine+"'");
                        String letter = String.valueOf(zonesLine.charAt(0));
                        //System.out.println("letter is " + letter);
                        zonesLine = zonesLine.substring(1, zonesLine.length());
                        if (letter.equals("-")) {
                            break buildUpZone;
                        } else {
                            insideContours.add(Contour.getContour(letter));
                        }

                    }
                    //System.out.println("leaving 'buildUpZone' loop, insideContours is "+insideContours);
                    Vector contoursCopy = (Vector) (contours.clone());
                    contoursCopy.removeAll(insideContours);
                    //System.out.println("add "+insideContours+" to habitat");
                    habitat.add(Zone.getZone(insideContours, contoursCopy));

                    // either got just "" or got ",qwbgdkueqgfdk" or "khjh, ljknl"
                    if (zonesLine.length() > 0
                        && zonesLine.charAt(0) == ',') // remove ","
                        {
                        zonesLine = zonesLine.substring(1, zonesLine.length());
                    }
                    if (zonesLine.length() == 0) {
                        zonesLine = null;
                    } // breaks out of loop
                }
                //System.out.println("add spider in "+habitat);
                ud.addSpider(habitat);
                //System.out.println("line before cropping is '" + line + "'");
                if (line.indexOf("{") > 0) {
                    line = line.substring(line.indexOf("{"), line.length());
                } else {
                    line = "-";
                }
                //System.out.println("line after cropping is '" + line + "'");
                //System.out.println("end of loop is '" + line + "'");
            }
            //System.out.println("line is "+line);
            return ud;
        } catch (Exception e) {
            /**/
            System.out.println("exception loading unitary diag" + e);
            e.printStackTrace();
            System.exit(0);
            return null;
        }
    }
    private static Diagram getCompoundDiagram(BufferedReader b) { //
        //System.out.println("getting a compound diagram from the buffer");
        try {
            // we need (i) the name
            // and to recursively call getDiagram(b)
            String line = b.readLine();
            //System.out.println("'name: zzz' is "+line);
            // should be "name: blah"
            String name =
                line.substring(nameIntroduction.length(), line.length());
            Vector diagrams = new Vector();
            //System.out.println("line is "+line);
            while (line.charAt(0) != ')') {
                Diagram d = getDiagram(b);
                diagrams.add(d);
                line = b.readLine(); // should be "And," or "Or" or "Not"
            }
            //System.out.println("line is "+line);
            if (line.charAt(1) == 'A') {
                return new AndCompoundDiagram(diagrams, name);
            } else if (line.charAt(1) == 'O') {
                return new OrCompoundDiagram(diagrams, name);
            } else {
                return new NotCompoundDiagram(diagrams, name);
            }
        } catch (Exception e) {
            System.out.println("exception thrown in compound reader: " + e);
        }
        return null;
    }
    private static Diagram getFalseDiagram(BufferedReader b) {
        try {
            // we need (i) the name
            // and to recursively call getDiagram(b)
            String line = b.readLine();
            //System.out.println("'name: zzz' is "+line);
            // should be "name: blah"
            String name =
                line.substring(nameIntroduction.length(), line.length());
            return new FalseDiagram(name);
        } catch (Exception e) {
            System.out.println(
                "exception thrown in false-diagram reader: " + e);
        }
        return null;
    }

    // recreate diagrams: if they were serialized, they don't respect the
    // flyweight pattern
    private static UnitaryDiagram makeClone(UnitaryDiagram ud) {
        Vector oldZoneSet = ud.getCloneOfZoneSet();
        Vector newZoneSet = new Vector();
        for (int i = 0; i < oldZoneSet.size(); i++) {
            Zone z = (Zone) oldZoneSet.get(i);
            newZoneSet.add(
                Zone.getZone(
                    z.getContainingContours(),
                    z.getExcludingContours()));
        }
        Vector newShading = ud.getCloneOfShadedZones();
        UnitaryDiagram result = new UnitaryDiagram(newZoneSet, ud.getName());
        result.setShadedZones(newShading);
        Vector oldSpiders = ud.getSpiders();
        for (int i = 0; i < oldSpiders.size(); i++) {
            Spider s = (Spider) oldSpiders.get(i);
            Vector oldH = s.getHabitat();
            Vector newH = new Vector();
            for (int j = 0; j < oldH.size(); j++) {
                Zone z = (Zone) oldH.get(j);
                newH.add(
                    Zone.getZone(
                        z.getContainingContours(),
                        z.getExcludingContours()));
            }
            result.addSpider(newH);
        }
        return result;
    }

    public static String toSaveString(Vector region) {
        Vector a = new Vector();
        for (int i = 0; i < region.size(); i++) {
            a.add(toSaveString((Zone) (region.get(i))));
        }
        return "" + a;
    }

    public static Vector getRegionFrom(String s, Vector allContours) {
        //
        // expect - for empty region
        // or a,ab,c
        s = s.substring(1, s.length() - 1);
        //System.out.println("s is reduced to " + s);
        Vector region = new Vector();
        String line = s;

        while (line.length() > 0) {
            String zoneLine;
            try {
                zoneLine = line.substring(0, line.indexOf(","));
                line = line.substring(line.indexOf(",") + 2, line.length());
            } catch (StringIndexOutOfBoundsException x) {
                zoneLine = line;
                line = "";
            }
            //System.out.println("zoneLine is "+zoneLine);
            //System.out.println("line is "+line);
            region.add(getZoneFrom(zoneLine, allContours));
        }
        return region;
    }

    public static Zone getZoneFrom(String zoneLine, Vector allContours) {
        Vector containingContours = new Vector();
        while (zoneLine.length() > 0) {
            if (!zoneLine.equals("-")) {
                String contourName = zoneLine.substring(0, 1);
                //System.out.println("contourName is "+contourName);
                containingContours.add(Contour.getContour(contourName));
                try {
                    zoneLine = zoneLine.substring(1, zoneLine.length());
                } catch (StringIndexOutOfBoundsException e) {
                    zoneLine = "";
                }
                //System.out.println("zone string is reduced to '"+zoneLine+"'");
            } else {
                zoneLine = "";
            }
        }
        Vector excludingContours = (Vector) (allContours.clone());
        excludingContours.removeAll(containingContours);
        //System.out.println("containingContours numbers "+containingContours.size());
        //System.out.println("excludingContours numbers "+excludingContours.size());
        return Zone.getZone(containingContours, excludingContours);
    }

    /*	public static void main(String args[]) {
    		Contour a = Contour.getContour("a");
    		Contour b = Contour.getContour("b");
    		Vector allContours = new Vector();
    		allContours.add(a);
    		allContours.add(b);
    		Vector inside = new Vector();
    		Vector outside = new Vector();
    		inside.add(a);
    		Zone z = Zone.getZone(inside, outside);
    		Vector region = new Vector();
    		region.add(z);
    		inside = new Vector();
    		outside = new Vector();
    		outside.add(a);
    		z = Zone.getZone(inside, outside);
    		region.add(z);
    		System.out.println("region is " + toSaveString(region));
    		System.out.println(""+getRegionFrom(toSaveString(region),allContours));
    	}
    	*/
    public static String toSaveString(Diagram d) {

        try {
            FalseDiagram fd = (FalseDiagram) d;
            return "FD:\n" + nameIntroduction + fd.getName();
        } catch (ClassCastException x) {
        }
        try {
            UnitaryDiagram ud = (UnitaryDiagram) d;
            String result = "UD: \n" + nameIntroduction + ud.getName() + "\n";
            result = result + contoursIntroduction;
            Vector a = new Vector();
            for (int i = 0; i < ud.getCloneOfContourSet().size(); i++) {
                a.add(toSaveString((Contour) (ud.getCloneOfContourSet().get(i))));
            }
            result = result + a + "\n" + zonesIntroduction;
            a = new Vector();
            Iterator zi = ud.getZoneIterator();
            while(zi.hasNext()){
                Zone z = (Zone)zi.next();                
                a.add(toSaveString(z));
            }
            result = result + a + "\n" + shadedZonesIntro;
            a = new Vector();
            Iterator it=ud.getSZIterator();
            while(it.hasNext()){
                Zone sz = (Zone)it.next();
                a.add(toSaveString(sz));
            }
            result = result + a + "\n" + spidersIntroduction;
            a = new Vector();
            for (int i = 0; i < ud.getSpiders().size(); i++) {
                a.add(toSaveString((Spider) (ud.getSpiders().get(i))));
            }
            result = result + a;
            return result;
        } catch (ClassCastException x) {
        }
        try {
            AndCompoundDiagram acd = (AndCompoundDiagram) d;
            String result = "CD:And(";
            result = result + "\n" + nameIntroduction + acd.getName();
            for (int i = 0; i < acd.diagrams.size() - 1; i++) {
                result =
                    result
                        + "\n"
                        + toSaveString((Diagram) (acd.diagrams.get(i)))
                        + "\nand";
            }
            result =
                result
                    + "\n"
                    + toSaveString(
                        (Diagram) (acd.diagrams.get(acd.diagrams.size() - 1)))
                    + "\n)And";
            return result;
        } catch (ClassCastException x) {
        }
        try {
            OrCompoundDiagram ocd = (OrCompoundDiagram) d;
            String result = "CD:Or(";
            result = result + "\n" + nameIntroduction + ocd.getName();
            for (int i = 0; i < ocd.diagrams.size() - 1; i++) {
                result =
                    result
                        + "\n"
                        + toSaveString((Diagram) (ocd.diagrams.get(i)))
                        + "\nor";
            }
            result =
                result
                    + "\n"
                    + toSaveString(
                        (Diagram) (ocd.diagrams.get(ocd.diagrams.size() - 1)))
                    + "\n)Or";
            return result;
        } catch (ClassCastException x) {
        }
        try {
            NotCompoundDiagram ncd = (NotCompoundDiagram) d;
            String result = "CD:Not(";
            result = result + "\n" + nameIntroduction + ncd.getName();
            result =
                result
                    + "\n"
                    + toSaveString(
                        (Diagram) (ncd.diagrams.get(0)))
                    + "\n)Not";
            return result;
        } catch (ClassCastException x) {
        }
        System.out.println(
            "shouldn't happen: passed a Diagram "
                + "that's not Unitary, And, Or, Not or False:"
                + d.getClass());
        System.exit(0);
        return "BUG";
    }

/*    public static String toSaveStringWithNot(Diagram d) {
        //System.out.println("in toSaveStringWithNot");
        if (d instanceof NotCompoundDiagram) {
            //System.out.println("stripped off a Not");
            return "CD:Not:\n("
                + toSaveStringWithNot(
                    (Diagram) ((NotCompoundDiagram) d).diagrams.get(0))
                + "\n)Not";
        }

        try {
            FalseDiagram fd = (FalseDiagram) d;
            return "FD:\n" + nameIntroduction + fd.getName();
        } catch (ClassCastException x) {
        }
        try {
            UnitaryDiagram ud = (UnitaryDiagram) d;
            String result = "UD: \n" + nameIntroduction + ud.getName() + "\n";
            result = result + contoursIntroduction;
            Vector a = new Vector();
            for (int i = 0; i < ud.getContours().size(); i++) {
                a.add(toSaveString((Contour) (ud.getContours().get(i))));
            }
            result = result + a + "\n" + zonesIntroduction;
            a = new Vector();
            for (int i = 0; i < ud.getZones().size(); i++) {
                a.add(toSaveString((Zone) (ud.getZones().get(i))));
            }
            result = result + a + "\n" + shadedZonesIntro;
            a = new Vector();
            Iterator it=ud.getSZIterator();
            while(it.hasNext()){
                Zone sz = (Zone)it.next();
                a.add(toSaveString(sz));
            }
            result = result + a + "\n" + spidersIntroduction;
            a = new Vector();
            for (int i = 0; i < ud.getSpiders().size(); i++) {
                a.add(toSaveString((Spider) (ud.getSpiders().get(i))));
            }
            result = result + a;
            return result;
        } catch (ClassCastException x) {
        }
        try {
            AndCompoundDiagram acd = (AndCompoundDiagram) d;
            String result = "CD:And(";
            result = result + "\n" + nameIntroduction + acd.getName();
            for (int i = 0; i < acd.diagrams.size() - 1; i++) {
                result =
                    result
                        + "\n"
                        + toSaveStringWithNot((Diagram) (acd.diagrams.get(i)))
                        + "\nand";
            }
            result =
                result
                    + "\n"
                    + toSaveStringWithNot(
                        (Diagram) (acd.diagrams.get(acd.diagrams.size() - 1)))
                    + "\n)And";
            return result;
        } catch (ClassCastException x) {
        }
        try {
            OrCompoundDiagram ocd = (OrCompoundDiagram) d;
            String result = "CD:Or(";
            result = result + "\n" + nameIntroduction + ocd.getName();
            for (int i = 0; i < ocd.diagrams.size() - 1; i++) {
                result =
                    result
                        + "\n"
                        + toSaveStringWithNot((Diagram) (ocd.diagrams.get(i)))
                        + "\nor";
            }
            result =
                result
                    + "\n"
                    + toSaveStringWithNot(
                        (Diagram) (ocd.diagrams.get(ocd.diagrams.size() - 1)))
                    + "\n)Or";
            return result;
        } catch (ClassCastException x) {
        }
        try {
            NotCompoundDiagram ncd = (NotCompoundDiagram) d;
            String result = "CD:Not(";
            result = result + "\n" + nameIntroduction + ncd.getName();
            result =
                result
                    + "\n"
                    + toSaveStringWithNot(
                        (Diagram) (ncd.diagrams.get(0)))
                    + "\n)Not";
            return result;
        } catch (ClassCastException x) {
        }
        System.out.println(
            "shouldn't happen: passed a Diagram "
                + "that's not Unitary, And, Or, Not or False:"
                + d.getClass());
        System.exit(0);
        return "BUG";
    }*/

    public static String toSaveString(Spider s) {
        String result = "{";
        Vector habitat = s.getHabitat();
        for (int i = 0; i < habitat.size(); i++) {
            result = result + toSaveString((Zone) habitat.get(i)) + ",";
        }
        return result + "}";
    }
    public static String toSaveString(Zone z) {
        String result = "";
        for (int i = 0; i < z.getContainingContours().size(); i++) {
            result += ("" + z.getContainingContours().get(i));
        }
        if (result == "") {
            return "-";
        } else {
            return result;
        }

    }
    public static String toSaveString(Contour c) {
        return c.toString();
    }

    public static void main(String args[]) {
        String testFile = "test.txt";
        Diagram d = new UnitaryDiagram(new Vector(), "test");
        DiagramFileUser.addDiagramFreshFile(d, testFile);
        // check it's there
        Diagram d2 = DiagramFileUser.getDiagramFromStart(testFile);
        if (d2 == null) {
            System.out.println("bug1");
        }
        d2 = DiagramFileUser.getDiagramFromStart(testFile);
        if (d2 == null) {
            System.out.println("bug2");
        }
        d2 = DiagramFileUser.getDiagramFromWherever(testFile);
        if (d2 != null) {
            System.out.println("bug3");
        }
        DiagramFileUser.addDiagramAppend(d, testFile);
        d2 = DiagramFileUser.getDiagramFromWherever(testFile);
        if (d2 == null) {
            System.out.println("bug4");
        }
        d2 = DiagramFileUser.getDiagramFromStart(testFile);
        if (d2 == null) {
            System.out.println("bug5");
        }
        d2 = DiagramFileUser.getDiagramFromWherever(testFile);
        if (d2 == null) {
            System.out.println("bug6");
        }
        d2 = DiagramFileUser.getDiagramFromWherever(testFile);
        if (d2 != null) {
            System.out.println("bug7");
        }

        System.out.println("done");

    }
}
