package rwd.concretesyntax.constraintdiagram;

import java.awt.BasicStroke;
import java.awt.Graphics2D;
//import java.awt.Polygon;
import java.awt.Shape;
import java.awt.geom.GeneralPath;
//import java.awt.geom.Point2D;

import rwd.abstractsyntax.constraintdiagram.Arrow;
//import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
//import rwd.concretesyntax.eulerdiagram.DecorationBox;
import rwd.concretesyntax.spiderdiagram.ConcreteSpider;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.gui.editor.viewers.DiagramViewerInterface;

/**
 * A concrete arrow.
 * 
 * @author Octavian Patrascoiu, Richard Bosworth
 * @version 18.8.05   12:56
 *
 */
public class ConcreteArc 
	extends DiagramItem 
	implements java.io.Serializable 
{
	public ConcreteArc() {
		ConcreteArcEnd start = new ConcreteArcEnd(0, 0);
		ConcreteArcEnd end = new ConcreteArcEnd(arrowSize, 0); 
		init(start, end, LINE);
	}
	
	public ConcreteArc(int type, int x, int y) {
		ConcreteArcEnd start = new ConcreteArcEnd(x, y);
		ConcreteArcEnd end = new ConcreteArcEnd(x+arrowSize, y); 
		init(start, end, type);
	}

	public ConcreteArc(int type, int x1, int y1, int x2, int y2) {
		this(new ConcreteArcEnd(x1, y1), new ConcreteArcEnd(x2, y2), type);
	}

	public ConcreteArc(int x, int y) {
		this(LINE, x, y);
	}

	public ConcreteArc(int type) {
		ConcreteArcEnd start = new ConcreteArcEnd(0, 0);
		ConcreteArcEnd end = new ConcreteArcEnd(arrowSize, 0); 
		init(start, end, type);
	}

	public ConcreteArc(ConcreteArcEnd start, ConcreteArcEnd end) {
		this(start, end, LINE);
	}
    
	public ConcreteArc(ConcreteArcEnd start, ConcreteArcEnd end, int type) {
		init(start, end, type);
	}
	
	/*
	 * test: 
	 *   type = line
	 *   type = quad
	 */
	protected void init(ConcreteArcEnd start, ConcreteArcEnd end, int type) {
		this.start = start;
		this.end = end;
		if (type == LINE) {
			middle = new ConcreteArcEnd((float)((start.getX()+end.getX())/2.0),
			                            (float)((start.getY()+end.getY())/2.0));
		}
		else
			middle = getDefaultMiddle();
		this.type = type;
		// Set label location		
		autoPlaceLabel();
	}

	public ConcreteArc(DiagramItem source, DiagramItem target) {
		this(source, target, LINE);
	}

	public ConcreteArc(DiagramItem source, DiagramItem target, int type) {
		this.start = new ConcreteArcEnd(source);
		this.end = new ConcreteArcEnd(target);
		middle = getDefaultMiddle();
		this.type = type;		
	}
    
	public int getType() {
		return type;
	}
	public void setType(int type) {
		this.type = type;
	} 
	
	public ConcreteArcEnd getStart() {
		return start;
	}
	public void setStart(ConcreteArcEnd end) {
		start = end;
	}
	    
	public ConcreteArcEnd getMiddle() {
		if (type == LINE) {
			middle.setLocation((float)((start.getX()+end.getX())/2.0),
					           (float)((start.getY()+end.getY())/2.0));
		}
		return middle;
	}
	public void setMiddle(ConcreteArcEnd end) {
		middle = end;
		// Set label location
		autoPlaceLabel();
	}

	public ConcreteArcEnd getEnd() {
		return end;
	}
	public void setEnd(ConcreteArcEnd end) {
		this.end = end;
	}

	public DiagramItem getSource() {
		return getStart().getAttachedItem();
	}

	public DiagramItem getTarget() {
		return getEnd().getAttachedItem();
	}
	
	
	/**
	 * Attach closest source and target
	 * 
	 */	
	public void attachEnds(DiagramViewerInterface viewer) {
		DiagramItem item = viewer.getNearestItem(start.getX(), start.getY()); 
		start.attachItem(item);
		item = viewer.getNearestItem(end.getX(), end.getY()); 
		end.attachItem(item);
	}

	/**
	 * Update location if a source/target has moved 
	 */
	public void updateLocation() {
		start.updateLocation();
		end.updateLocation();
	}

	public void translateTo(float dx, float dy) {
		start.translateTo(dx, dy);
		middle.translateTo(dx, dy);
		end.translateTo(dx, dy);
		super.translateTo(dx,dy);
	}

	public Shape getShape() {
		return getShape(false);
	}

	public Shape getShape(boolean smooth) {
		Shape shape = null;
		// Get points coordinates
		int x1 = start.getX();
		int y1 = start.getY();
		int x2 = end.getX();
		int y2 = end.getY();
		int x3 = middle.getX();
		int y3 = middle.getY();
		// Draw arrow
		GeneralPath arrow = new GeneralPath();
		if (type == LINE) {
			// Compute coordinates for arrow head
			computeArrow(x1, y1, x2, y2);
			// Draw arrow's body
			arrow.moveTo(x1, y1);
			arrow.lineTo(x2, y2);
		} else {
			// Compute coordinates for arrow head
			computeArrow(x3, y3, x2, y2);
			// Draw arrow's body
			arrow.moveTo(x1, y1);
			arrow.quadTo(x3, y3, x2, y2);
		}
		// Draw arrow's head
		arrow.lineTo((float) x4, (float) y4);
		arrow.lineTo((float) x4, (float) y4);
		arrow.quadTo(x2, y2, (float)x5, (float)y5);
		arrow.lineTo((float) x2, (float) y2);
		shape = arrow;
		return shape;
	}

	public void draw(Graphics2D g, boolean smoothed) {
		// Draw shape
		Shape shape = getShape(smoothed);
		g.setColor(getForeColor());
		BasicStroke stroke = new BasicStroke(
				getThickness(),
				BasicStroke.CAP_BUTT,
				BasicStroke.JOIN_ROUND);
		g.setStroke(stroke);
		g.draw(shape);
		g.fill(getArrow());
		// Draw label
		drawLabel(g);
	}
	
	private static final int LABEL_Y_OFFSET = -80;

	public void autoPlaceLabel() {
		_labelX = getMiddle().getX();
		_labelY = getMiddle().getY() + LABEL_Y_OFFSET;
	}

	public double getLength() {
		int x1 = start.getX();
		int y1 = start.getY();
		int x2 = end.getX();
		int y2 = end.getY();
		double len = (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1);
		return len;
	}
	
	public java.util.Set getHandles() {
		// Set handles
		java.util.Set _handles = new java.util.LinkedHashSet();
		_handles.addAll(start.getHandles());
		if (type == ConcreteArc.QUAD)
			_handles.addAll(middle.getHandles());
		_handles.addAll(end.getHandles());
		return _handles;
	}

	public Shape getArrow() {
		// Get points coordinates
		int x1 = start.getX();
		int y1 = start.getY();
		int x2 = end.getX();
		int y2 = end.getY();
		int x3 = middle.getX();
		int y3 = middle.getY();
		if (type == LINE) {
			// Compute coordinates for arrow head
			computeArrow(x1, y1, x2, y2);
		} else {
			// Compute coordinates for arrow head
			computeArrow(x3, y3, x2, y2);
		}
		// Draw arrow's head
		GeneralPath arrow = new GeneralPath();
		arrow.moveTo(x2, y2);
		arrow.lineTo((float) x4, (float) y4);
//		arrow.quadTo(x2, y2, (float)x5, (float)y5);
		arrow.lineTo((float) x5, (float) y5);
		arrow.lineTo(x2, y2);
		arrow.closePath();
		return arrow;
	}

	// Arrow's head coordinates
	protected double x4, y4, x5, y5;
	// Compute arrow's head coordinates
	protected void computeArrow(double x1, double y1, double x2, double y2) {
		double dx = x2-x1;
		double dy = y2-y1;
		double d = Math.sqrt(dx*dx+dy*dy);
		// Horizontal
		if (y1 == y2) {
			double dy1 = headSize*Math.tan(arrowAngle); 
			if (x1 <= x2) {
				x4 = x5 = x2-headSize;
				y4 = y1+dy1;
				y5 = y1-dy1;				
			} else {
				x4 = x5 = x2+headSize;
				y4 = y1+dy1;
				y5 = y1-dy1;				
			}
		// Vertical
		} else if (x1 == x2) {
			if (y1 <= y2) {
				double dx1 = headSize*Math.tan(arrowAngle); 
				y4 = y5 = y2-headSize;
				x4 = x1+dx1;
				x5 = x1-dx1;
			} else {
				double dx1 = headSize*Math.tan(arrowAngle); 
				y4 = y5 = y2+headSize;
				x4 = x1+dx1;
				x5 = x1-dx1;					
			}
		// Else
		} else {
			// Compute for the first quadrant
			double adx = Math.abs(x2-x1);
			double ady = Math.abs(y2-y1);
			double angle = Math.atan(1.0*Math.abs(ady)/adx);
			double dx4, dy4, dx5, dy5;
				 
			if (d >= headSize) {
				double x = headSize*Math.tan(arrowAngle);
				double dangle = Math.atan(x/(d-headSize));
				double r = Math.sqrt((d-headSize)*(d-headSize)+x*x);
				dx4 = r*Math.cos(angle+dangle);
				dy4 = r*Math.sin(angle+dangle);
				dx5 = r*Math.cos(angle-dangle);
				dy5 = r*Math.sin(angle-dangle);
			} else {
				double dangle = Math.toRadians(45);
				dx4 = (d-headSize)*Math.cos(angle+dangle);
				dy4 = (d-headSize)*Math.sin(angle+dangle);
				dx5 = (d-headSize)*Math.cos(angle-dangle);
				dy5 = (d-headSize)*Math.sin(angle-dangle);					
			}
			if (x1 <= x2) {
				x4 = x1+dx4;
				x5 = x1+dx5;
			} else {
				x4 = x1-dx4;
				x5 = x1-dx5;
			}
			if (y1 <= y2) {
				y4 = y1+dy4;
				y5 = y1+dy5;
			} else {
				y4 = y1-dy4;
				y5 = y1-dy5;
			}
		}		    
	}
        
	protected ConcreteArcEnd getDefaultMiddle() {
		int x1 = start.getX();
		int y1 = start.getY();
		int x3 = end.getX();
		int y3 = end.getY();
		int x2 = (x1+x3)/2;
		int y2 = (y1+y3)/2;
		if (Math.abs(x3-x1) >= Math.abs(y3-y1)) {
			y2 += -300;
		} else {
			x2 += -300;	
		}
		ConcreteArcEnd middle = new ConcreteArcEnd(x2, y2);
		middle.setIsMiddle(true); 
		return middle;
	}
	
	/**
	 * 
	 * @param cud concrete unitary diagram containing this concrete arrow
	 * @return abstract arrow corresponding to this concrete arrow
	 */
	/*
	 * test: once thru
	 */
	public Arrow getAbstractArrow(ConcreteUnitaryDiagram cud) {
		Object s = getAbstractItem(getSource(), cud);
		Object t = getAbstractItem(getTarget(), cud);
		String l = this.getLabel();
		return new Arrow(s,t,l);
	}
	
	protected Object getAbstractItem(DiagramItem i, ConcreteUnitaryDiagram cud) {
		if (i instanceof ConcreteSpiderFoot) {
			i = cud.getConcreteSpider((ConcreteSpiderFoot)i);
		}
		if (i instanceof ConcreteSpider) {
			return ((ConcreteSpider)i).getAbstractSpider(cud);
		} else if (i instanceof ConcreteContour) {
			return ((ConcreteContour)i).getContour();
		}
		return null;
	}

	public DiagramItem getClone() {
		// Create clone
		ConcreteArc res = new ConcreteArc();
		res.start = (start == null ? null : (ConcreteArcEnd)this.start.getClone());
		res.middle = (middle == null ? null : (ConcreteArcEnd)this.middle.getClone());
		res.end = (end == null ? null : (ConcreteArcEnd)this.end.getClone());
		res.type = type;
		// Copy attributes
		res.setAttributes(this);
		return res;
	}
	
	/**
	 * 
	 * @return a user-friendly description of this item
	 */
	/*
	 * test:
	 *   arrow with no start or end or label
	 *   arrow with start and end but no attached items
	 *   arrow with label and with attached items at each end
	 */
	public String id() {
		String s = "arrow ";
		if (getLabel().length() > 0) s += getLabel() + " ";
		s += "from ";
		if (start != null) {
			DiagramItem di = start.getAttachedItem(); 
			if (di != null)
				s += di.id();
			else
				s += "nowhere";
		}
		s += " to ";
		if (end != null) {
			DiagramItem di = end.getAttachedItem(); 
			if (di != null)
				s += di.id();
			else
				s += "nowhere";
		}
		return s;
	}
	
	public String toString() {
		return "Arc '"+_label+"'"; 
	}
    
    // Local properties
	protected ConcreteArcEnd start;
	protected ConcreteArcEnd middle;
	protected ConcreteArcEnd end;
	protected int type;

	public static final int LINE = 1;
	public static final int QUAD = 2; 
	protected double arrowAngle = Math.toRadians(20); 
	public double headSize = 110;
	protected int arrowSize = 1000;
	
	/*
	// test program
    // ------------

    private static void pr(String s) {
      System.out.println(s);
    }

    private static void hd(String s) {
      System.out.println("\n" + s + "\n");
    }

    private static void pg() {
      try {
        int i;
        System.out.print("next page? (CR)");
        do i = System.in.read(); while (i != '\n');
        System.out.println();
      }
      catch (Exception e) {System.out.println("raised exception : " + e);}
    }

    public static void main(String args[]) throws Exception {
    	hd("Create new null arrow");
    	ConcreteArc carc = new ConcreteArc();
    	pr("carc="+carc);
    	pr("carc.id()="+carc.id());
    	pg();
    	/*
    	{
	    	pr("Create arrow with null ends");
	    	ConcreteArcEnd cs = new ConcreteArcEnd(null);
	    	ConcreteArcEnd ct = new ConcreteArcEnd(null);
	    	ConcreteArc carc2 = new ConcreteArc();
	    	pr("carc2="+carc2);
	    	pr("carc2.id()="+carc2.id());
	    	pg();
	    	
    	}
    	
    	{
	    	hd("Create new concrete contours");
	    	Polygon p = new Polygon();
	    	p.addPoint(1,2);
	    	ConcreteContour ca = 
	    		new ConcreteContour(Contour.getContour("a"),p);
	    	pr("ca="+ca);
	    	ConcreteContour cb = 
	    		new ConcreteContour(Contour.getContour("b"),p);
	    	pr("cb="+cb);
	    	pg();
	    	
	    	pr("create new concrete unitary diagram with a and b");
	    	DecorationBox db = new DecorationBox("db","",p);
	    	ConcreteUnitaryDiagram cud = new ConcreteUnitaryDiagram(db);
	    	cud.addConcreteContour(ca);
	    	cud.addConcreteContour(cb);
	    	pr("add arrow from a to b");
	    	ConcreteArcEnd cs = new ConcreteArcEnd(ca);
	    	ConcreteArcEnd ct = new ConcreteArcEnd(cb);
	    	ConcreteArc carrow = new ConcreteArc(cs,ct);
	    	carrow.setLabel("arrow1");
	    	pr("carrow="+carrow);
	    	pr("carrow.id()="+carrow.id());
	    	pg();
	    	
	    	cud.addConcreteArc(carrow);
	    	pr("cud="+cud);
	    	pg();
	    	
	    	pr("carrow.getAbstractArrow(cud)="+carrow.getAbstractArrow(cud));
	    	pg();
	    	
	    	pr("making clone");
	    	pr("carrow="+carrow);
	    	pr("carrow.id()="+carrow.id());
	    	ConcreteArc carrowc = (ConcreteArc) carrow.getClone();
	    	pr("carrowc="+carrowc);
	    	pr("carrowc.id()="+carrowc.id());
	    	pr("carrow.equals(carrowc)="+carrow.equals(carrowc));
	    	pg();
	    	
    	}
    	*
    	{
    		hd("Test of getMiddle");
    		ConcreteArc ca1 = new ConcreteArc(LINE,0,0,10,10);
    		pr("ca1="+ca1);
    		pr("ca1.id()="+ca1.id());
    		pr("ca1.start.id()="+ca1.start.id());
    		pr("ca1.middle.id()="+ca1.middle.id());
    		pr("ca1.end.id()="+ca1.end.id());
    		pg();
    		
    		pr("ca1.getMiddle()="+ca1.getMiddle());
    		pr("ca1.start.id()="+ca1.start.id());
    		pr("ca1.middle.id()="+ca1.middle.id());
    		pr("ca1.end.id()="+ca1.end.id());
    		pg();
    		
    		ConcreteArc ca2 = new ConcreteArc(QUAD,0,0,10,10);
    		pr("ca2="+ca2);
    		pr("ca2.id()="+ca2.id());
    		pr("ca2.start.id()="+ca2.start.id());
    		pr("ca2.middle.id()="+ca2.middle.id());
    		pr("ca2.end.id()="+ca2.end.id());
    		pg();
    		
    		pr("ca2.getMiddle()="+ca2.getMiddle());
    		pr("ca2.start.id()="+ca2.start.id());
    		pr("ca2.middle.id()="+ca2.middle.id());
    		pr("ca2.end.id()="+ca2.end.id());
    		pg();
    	}
    	
    }
    */
}
