package rwd.concretesyntax.eulerdiagram;

import java.awt.Polygon;
import java.awt.Rectangle;
import java.util.Vector;
import java.util.Iterator;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.utilities.AreaRWD;
import rwd.views.graph.BlockGraph;
import rwd.views.graph.BlockNode;

public class AEDVComponent {
	private BlockGraph bg;

	private AEDVZone parent;

	private Vector children = new Vector(); // of AEDVZone objects

	private AEDView aedv; // the actual image!!

	public AEDVComponent(AEDVZone parent, BlockGraph bg) {
		this.bg = bg;
		this.parent = parent;
		Vector children = bg.children();
		for (int i = 0; i < children.size(); i++) {
			BlockNode bn = (BlockNode) children.get(i);
			if (bn != null) {
				this.children.add(new AEDVZone(bn));
			}
		}
	}

	public AEDView aedview() {
		//System.out.println("making aedv");
		for (int i = 0; i < children.size(); i++) {
			AEDVZone child = (AEDVZone) children.get(i);
			//System.out.println("about to make rectangles");
			makeRectangles(
				aedv,
				child.getZone(),
				child.aedvComponents().size());
			//System.out.println("made rectangles");
			for (int j = 0; j < child.aedvComponents().size(); j++) {
				AEDVComponent aedvc =
					(AEDVComponent) (child.aedvComponents().get(j));
//					System.out.println(
//						"about to add a subview in "
//						+ aedv.hashCode()
//						+ " in zone "
//						+ child.getZone());
				Zone childZone = child.getZone();
				Vector childContours =
					(Vector) (childZone.getContainingContours().clone());
				childContours.removeAll(
					parent.getZone().getContainingContours());
				Vector a = new Vector();
				Iterator k = childContours.iterator();
				while (k.hasNext()) {
					Object o = k.next();
//						System.out.println("getting contour " + o.getClass());
					a.add(Contour.getContour("" + o));
				}
				//Zone zoneInContext = Zone.getZone(a, new Vector());
				// highly dubious allocation 
				//aedv.add(aedvc.aedview(),zoneInContext);
				aedv.add(aedvc.aedview(), childZone);
			}
		}
		return aedv;
	}

	public boolean add(AEDView aedv, BlockGraph bg) {
		if (this.bg == bg) {
			this.aedv = aedv;
			return true;
		} else // try to add to children
			{
			for (int i = 0; i < children.size(); i++) {
				AEDVZone child = (AEDVZone) children.get(i);
				if (child.add(aedv, bg)) {
					return true;
				}
			}
		}
		return false;
	}

	int countAEDViews() {
		int result = 1;
		for (int i = 0; i < children.size(); i++) {
			result = result + ((AEDVZone) children.get(i)).countAEDViews();
		}
		return result;
	}

	public String toString() {
		String result = "component view ";
		result = result + "aedv=null?" + (aedv == null);
		if (aedv != null) {
			result =
				result
					+ aedv.hashCode()
					+ ":"
					+ aedv.getContours()
					+ "\nwith zone children:";
			for (int i = 0; i < children.size(); i++) {
				AEDVZone z = (AEDVZone) children.get(i);
				result = result + "\n" + z;
			}
		}
		return result;
	}

	private void makeRectangles(AEDView view, Zone zp, int numRequired) {
		// first find a rectangle in the right zone
		Vector contours = (Vector) (zp.getContainingContours().clone());
		//System.out.println("making rectangles");
		//System.out.println("contours are "+contours);
		//System.out.println("parent.getZone().contours() are "+parent.getZone().contours());
		contours.removeAll(parent.getZone().getContainingContours());
		//System.out.println("contours.removeAll(parent.getZone().contours()); are "+contours);
		//System.out.println("removed some contours");
		Vector al = new Vector();
		//System.out.println("build a zone");
		Iterator i = contours.iterator();
		while (i.hasNext()) {
			Object o = i.next();
			//System.out.println("o is "+o.getClass());
			al.add((Contour) o);
		}
		//Zone z = Zone.getZone(al, new Vector());
		//System.out.println("built a zone");
		//System.out.println("Parent zone contours "+parent.getZone().contours());
		//System.out.println("Remaining zone contours are "+contours);
		//System.out.println("AEDV contours are "+view.getContours());
		//System.out.println("AEDV zones are "+view.zones());
		//System.out.println("~~~~~~~~~~~~~~~~~~~ ");
		Rectangle sub;
		if (contours.size() == 0) { // build a rectangle next to existing view
			// ready to hold numRequired new diagrams
			Rectangle bounds = view.getFittedBoundingBox();
			sub =
				new Rectangle(
					bounds.x + bounds.width,
					bounds.y,
					bounds.width * numRequired,
					bounds.height);
		} else {
			i = contours.iterator();
			Object o = i.next();
			Contour c = (Contour) o;
			//System.out.println("About to build zone by intersection");
			Polygon p = view.getPolyFor(c);
			//System.out.println("got polygon for contour "+c);
			AreaRWD a = new AreaRWD(p);
			while (i.hasNext()) {
				Object o2 = i.next();
				Contour c2 = Contour.getContour(o2.toString());
				//System.out.println("intersect with contour "+c2);
				p = view.getPolyFor(c2);
				a.intersect(new AreaRWD(p));
				//System.out.println("shrunk to "+a.getBounds());
			}
			Vector excludingContours =
				(Vector) (view.getContours().clone());
			Iterator itemp = contours.iterator();
			while (itemp.hasNext()) {
				excludingContours.remove(Contour.getContour("" + itemp.next()));
			}
			i = excludingContours.iterator();
			while (i.hasNext()) {
				Object o2 = i.next();
				Contour c2 = Contour.getContour(o2.toString());
				//System.out.println("exclude contour "+c2);
				p = view.getPolyFor(c2);
				a.subtract(new AreaRWD(p));
				//System.out.println("shrunk to "+a.getBounds());
			}
			//System.out.println("a has bounds "+a.getBounds());
			// now a shows where the rectangle could be
			Rectangle bounds = a.getBounds();
			int boundsx = bounds.x;
			int boundsy = bounds.y;
			int boundsw = bounds.width;
			int boundsh = bounds.height;
			sub = bounds;
			/*int factorBy = 0;  // 1 means 1 out of 3 = middle box from 9
			while(!a.contains(sub))
			{ factorBy++;
			  double xstep = boundsw/(factorBy+2);
			  double ystep = boundsh/(factorBy+2);
			  // the bounding box will be split into factorBy+2*factorBy+2
			  for(int c=1; c<factorBy+1; c++) // e.g. factorBy = 1, c from 1 to 1
			  { // build a new "sub"
			    double subx = boundsx + xstep*c; // e.g. factorBy = 1, subx = x+1/3 width
			    double suby = boundsy + ystep*c; // e.g. factorBy = 1, subx = x+1/3 width
			    sub = new Rectangle((int)subx,(int)suby,(int)xstep,(int)ystep);
			    if(a.contains(sub))
			    { break; }
			  }
			}*/
			int factorBy = 20;
			//System.out.println("starting rectangle loop");
			rectangleSearch : while (true) {
				//System.out.println("in rectangle loop");
				double xstep = boundsw / factorBy;
				double ystep = boundsh / factorBy;
				// fix a size for the subrectangle
				for (int subSize = factorBy - 2; subSize > 0; subSize--) {
					// fix an x/y relative to bounds
					for (int x = 1; x < factorBy - 2; x++) {
						for (int y = 1; y < factorBy - 2; y++) {
							double subx = boundsx + x * xstep;
							double suby = boundsy + y * ystep;
							sub =
								new Rectangle(
									(int) subx,
									(int) suby,
									(int) xstep * subSize,
									(int) ystep * subSize);
							//System.out.println("see whether "+a.getBounds()+" contains "+sub.getBounds());
							if (a.contains(sub)) {
								break rectangleSearch;
							}
						}
					}

				}
				factorBy = factorBy * 2;
			}
			//System.out.println("after rectangle loop");
		}
		// now split a up into numReqd pieces and
		//System.out.println("split up into numReqd pieces");
		int subw = sub.width;
		double subStep = subw / numRequired;
		for (int ii = 0; ii < numRequired; ii++) {
			Rectangle partSub =
				new Rectangle(
					(int) (sub.x + ii * subStep),
					sub.y,
					(int) subStep,
					sub.height);
			//System.out.println(
			//	"adding a rectangle in " + view.hashCode() + " for zone " + zp);
			view.add(partSub, zp);
			//System.out.println(
			//	"added a rectangle in " + view.hashCode() + " for zone " + zp);
		}
		//System.out.println("finished adding rectangles");
	}
}
