package rwd.concretesyntax.eulerdiagram;

import java.awt.Polygon;
import java.awt.Rectangle;
import java.io.Serializable;
import java.util.Vector;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;
import java.util.TreeSet;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;

public class AEDView
	implements Serializable { // 
	// has concrete information about positioning of polygons for
	// contour representation using ContourView objects
	protected Vector polygons = new Vector();
	protected Vector contours = new Vector();
	private Vector rectangles = new Vector();
	// contains arraylists of rectangles
	protected Vector zones = new Vector();
	private String error = null;
	private String warning = "";
	private String description = "";

	private double aspectRatio = 1.0;

	public AEDView(String description, String error) {
		this.description = description;
		this.error = error;
	}

	public AEDView(String description, TreeSet zones) {
		this.zones.addAll(zones);
		this.description = description;
		for (int index = 0; index < zones.size(); index++) {
			rectangles.add(new Vector());
		}
		//System.out.println("just built an AEDView with ");
		//System.out.println("zones are "+zones);
		//System.out.println("rectangles are "+rectangles);
	}

	public void add(Polygon p, Contour c) {
		polygons.add(p);
		contours.add(c);
	}

	public void add(Rectangle r, Zone z) { 
		
		// find the right arraylist for z
		Vector rects = this.getRectanglesFor(z);
		//System.out.println("zones are "+zones+" should include "+z);
		//System.out.println("rectangles are "+rectangles);
		rects.add(r);
	}

	public Vector getContours() {
		return (Vector) (contours.clone());
	}

	public Vector getPolygons() {
		return (Vector) (polygons.clone());
	}

	public Polygon getPolyFor(Contour c) {
		//System.out.println("getting a polygon for contour "+c.getClass());
		Iterator ic = contours.iterator();
		Iterator ip = polygons.iterator();
		while (ic.hasNext() && ip.hasNext()) {
			Contour cc = (Contour) (ic.next());
			Polygon p = (Polygon) (ip.next());
			//System.out.println("c is "+cc+" class "+cc.getClass());


			if(c.toString().equals(cc.toString()))
			{ c = Contour.getContour(c.toString());
				cc = Contour.getContour(c.toString());
			}
			// a hack to fix flyweight/serialization problems upon reloading diagrams


			if (c == cc) {
				return p;
			}
		}
		return null;
	}

	private Vector getRectanglesFor(Zone z) { 
		//System.out.println("looking for zone "+z+" in "+zones);
		//System.out.println("then we will extract rectangle from "+rectangles);
		Iterator irs = rectangles.iterator();
		Iterator iz = zones.iterator();
		while (iz.hasNext() && irs.hasNext()) {
			Zone zz = (Zone) (iz.next());
			Vector rs=(Vector)irs.next();
			if (zz==z) {
				//System.out.println("returning rectangle "+rs+" in zone "+z+" in "+this);
				return rs;
			}
		}
		return null;
	}

	public Rectangle getARectangleFor(Zone z) {
		Vector rectanglesForZ = getRectanglesFor(z);
		//System.out.println("rectangles before extracting one are "+rectangles);
		//System.out.println("rectanglesForZ are "+rectanglesForZ);
		Rectangle r = null;
		if (rectanglesForZ.size() > 0) {
			Object o = rectanglesForZ.iterator().next();
			//System.out.println("object o.getClass() is "+o.getClass());
			r = (Rectangle) (o); ////////////////
			rectanglesForZ.remove(r);
		}
		//System.out.println("rectangles after extracting one are "+rectangles);
		return r;
	}

    public Rectangle getExactBoundingBox() {
        if (polygons.size() == 0) {
            return null;
        }
        Iterator i = polygons.iterator();
        Polygon p = (Polygon) (i.next());
        p.invalidate();
        Rectangle r = p.getBounds();
        int minX = (int) (r.getMinX());
        int maxX = (int) (r.getMaxX());
        int minY = (int) (r.getMinY());
        int maxY = (int) (r.getMaxY());
        //System.out.println("minx,maxx are "+minX+","+maxX);
        while (i.hasNext()) {
            p = (Polygon) (i.next());
            p.invalidate();
            r = p.getBoundingBox();
            minX = Math.min((int) (r.getMinX()), minX);
            maxX = Math.max((int) (r.getMaxX()), maxX);
            minY = Math.min((int) (r.getMinY()), minY);
            maxY = Math.max((int) (r.getMaxY()), maxY);
        }
        int width = maxX - minX;
        int height = maxY - minY;
        return new Rectangle(minX, minY, width, height);
        //return new Rectangle(minX, minY, width, height);
    }

	public Rectangle getFittedBoundingBox() {
        Rectangle r = getExactBoundingBox();
		int minX = (int) (r.getMinX());
		int maxX = (int) (r.getMaxX());
		int minY = (int) (r.getMinY());
		int maxY = (int) (r.getMaxY());
		int width = maxX - minX;
		int height = maxY - minY;
		int newminX = (int) (minX - 0.2 * width);
		int newminY = (int) (minY - 0.2 * height);
		int newwidth = (int) (1.4 * width);
		int newheight = (int) (1.4 * height);
		return new Rectangle(newminX, newminY, newwidth, newheight);
		//return new Rectangle(minX, minY, width, height);
	}

	public void scaleToFit(Rectangle newR) 
	{
		// take polygons, which fit inside r,
		// and rescale integer coords according to width, height
		// ie. map from r.getMinX() -> r.getMaxX()
		// to           0           -> width
		// ie   (old-r.getMinX())/(r.getMaxX()-r.getMinX())
		Rectangle r = this.getExactBoundingBox();
		if (r == null) {
			return;
		}
		int rminx = (int) (r.getMinX());
		int rmaxx = (int) (r.getMaxX());
		int rminy = (int) (r.getMinY());
		int rmaxy = (int) (r.getMaxY());
		//System.out.println("bounding box was "+rminx+"-"+rmaxx+" and "+rminy+"-"+rmaxy);
		int newrminx = 0, newrmaxx = 0, newrminy = 0, newrmaxy = 0;
		newrminx = (int) (newR.getMinX());
		newrmaxx = (int) (newR.getMaxX());
		newrminy = (int) (newR.getMinY());
		newrmaxy = (int) (newR.getMaxY());
		//System.out.println("bounding box should become "+newrminx+"-"+
		//                            newrmaxx+" and "+newrminy+"-"+newrmaxy);
		Iterator i = polygons.iterator();
		while (i.hasNext()) {
			Polygon p = (Polygon) (i.next());
			int[] xps = p.xpoints;
			int[] yps = p.ypoints;
			int numpts = p.npoints;
			//int[] newxs = new int[numpts];
			//int[] newys = new int[numpts];
			for (int n = 0; n < numpts; n++) {
                int oldx = xps[n];
				xps[n] =
					(int) ((oldx - rminx)
						* (newrmaxx - newrminx)
						/ (rmaxx - rminx)
						+ newrminx);
				//System.out.println("x went "+oldx+"->"+xps[n]);
                int oldy = yps[n];
				yps[n] =
					(int) ((oldy - rminy)
						* (newrmaxy - newrminy)
						/ (rmaxy - rminy)
						+ newrminy);
				//System.out.println("y went "+yps[n]+"->"+yps[n]);
			}
			//System.out.println("rescaler replacing "+p+" with "+pol);
		}
		//System.out.println("view is now "+this);
		// also scale all rectangles too
		Iterator rectSetIt = rectangles.iterator();
		while (rectSetIt.hasNext()) {
			Vector rectangleSet = (Vector) (rectSetIt.next());
			Iterator rectIt = rectangleSet.iterator();
			while (rectIt.hasNext()) {
				Rectangle rr = (Rectangle) (rectIt.next());
				int newx =
					((rr.x - rminx) * (newrmaxx - newrminx) / (rmaxx - rminx)
						+ newrminx);
				int newy =
					((rr.y - rminy) * (newrmaxy - newrminy) / (rmaxy - rminy)
						+ newrminy);
				int neww =
					((rr.width) * (newrmaxx - newrminx) / (rmaxx - rminx));
				int newh =
					((rr.height) * (newrmaxy - newrminy) / (rmaxy - rminy));
				//System.out.println("old rectangle "+rr);
				rr.setBounds(newx, newy, neww, newh);
				//System.out.println("new rectangle "+rr);
			}
		}
	}
	/////////////////////////////////////////////////////////////////////////
	public void add(
		AEDView otherView,
		Zone z) { //System.out.println("calling add in AEDVIew");
		//System.out.println("before add we have "+polygons.size()+" polys");

		// we should look for a rectangle for z and insert otherView in there
		Rectangle r = getARectangleFor(z);
		// scale polygons to fit into r
		//System.out.println("put "+otherView+" into rectangle "+r+" of "+this);
		otherView.scaleToFit(r);
		contours.addAll(otherView.contours);
		polygons.addAll(otherView.polygons);
		//System.out.println(""+this+" now has "+contours.size()+" contours");
		//System.out.println("polygons are "+polygons);
		// also add otherView's zones to our zones array
		// (but don't duplicate the outside zone = smallest zone)
		// also scale rectangles in otherView and add them into our rectangles/zones
		// array
		Iterator rectSetIt = otherView.rectangles.iterator();
		Iterator zoneIt = otherView.zones.iterator();
		while (rectSetIt.hasNext() && zoneIt.hasNext()) {
			Vector rectangles = (Vector) (rectSetIt.next());
			Zone zz = (Zone) (zoneIt.next());
			if (zz!=z) {
				Iterator rectIt = rectangles.iterator();
				while (rectIt.hasNext()) {
					Rectangle rr = (Rectangle) (rectIt.next());
					this.add(rr, zz);
				}
			}
		}
		// (but don't need to insert rectangles from outside zone = smallest zone)
		//System.out.println("giving "+this);
		//System.out.println("after add we have "+polygons.size()+" polys");
	}

	public String toString() {
		return "polys "
			+ this.polygons
			+ " for contours "
			+ contours
			+ " rectangles "
			+ rectangles
			+ " for zones "
			+ zones;
	}

	public double getAspectRatio() {
		return aspectRatio;
	}
	public void setAspectRatio(double ratio) {
		aspectRatio = ratio;
	}

	public String getDescription() {
		return description + "\n" + warning;
	}

	public String error() {
		return error;
	}

	public void setWarning(String warning) {
		this.warning = warning;
	}

	public Vector zones() {
		return (Vector) (zones.clone());
	}
    
    public Set getConcreteContours(){
        LinkedHashSet result = new LinkedHashSet();
        //System.out.println("  "+aedv.getContours());
        if (getContours().size() == 0) {
            //System.out.println("no contours");
            return result;
        }
        Vector polys = getPolygons();
        Vector contours = getContours();
        int numPolys = polys.size();
        ConcreteContour cp;
        for (int index = 0; index < numPolys; index++) {
            cp =
                new ConcreteContour(
                    (Contour) (contours.get(index)),
                    (Polygon) (polys.get(index)));
            result.add(cp);
        }
        return result;
    }
}
