package rwd.concretesyntax.eulerdiagram;
import java.awt.Polygon;
import java.util.Vector;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.metrics.AreaPolygonAdapter;

public class ContourPolygonSet implements java.io.Serializable {
	public ContourPolygonSet() {
	}

	private Vector contourPolygons = new Vector(); // the contourPolygons
    //private Vector zoneShapes = null;
    private transient Vector zoneAreas = new Vector();
    // needs to be calculated before it can be "got"
    //	private Vector lines = null;
    // needs to be calculated before it can be "got"

    public ContourPolygonSet(Vector contourPolygons) {
        //System.out.println("in CPS constructor");
        this.contourPolygons = contourPolygons;
        //System.out.println("about to leave CPS constructor");
    }

    // should we get the constructors to call calcZonePolys?

    public ContourPolygonSet(AEDView aedv) { // extract polygons and contour labels from AEDView object
        this(new Vector());
        //System.out.println("  "+aedv.getContours());
        if (aedv.getContours().size() == 0) {
            //System.out.println("no contours");
            return;
        }
        Vector polys = aedv.getPolygons();
        Vector contours = aedv.getContours();
        Vector contourPolys = new Vector();
        int numPolys = polys.size();
        ConcreteContour cp;
        for (int index = 0; index < numPolys; index++) {
            cp =
                new ConcreteContour(
                    (Contour) (contours.get(index)),
                    (Polygon) (polys.get(index)));
            contourPolys.add(cp);
        }
        this.contourPolygons = contourPolys;
    }

    public Vector getZoneAreasForStructureChecking() { // 
        //find all intersections - returns an arraylist size <= 2^(#contours)
        return zoneAreas;
    }

    public Vector getContourPolys() {
        return contourPolygons;
    }

    public void calcZoneAreasForStructureChecking()
    // should this include "outside" zone? I guess no
    {
        if (contourPolygons.size() == 0) { //zoneShapes = new Vector();
            zoneAreas = new Vector();
            return;
        }
        // should be called after a polygon is moved
        // uses intersection, subtraction of polygons
        // assigns Zone objects to each Polygon obtained
        // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
        // iterate through nonempty subsets of the polygons
        int numPolys = contourPolygons.size();
        Vector result = new Vector();
        boolean[] isIn = new boolean[numPolys];
        for (int index = 0; index < numPolys; index++) {
            isIn[index] = false;
        }
        // use this boolean array to indicate which contour are in subset
        boolean hasFinished = false;
        subSetLoop : while (!hasFinished) { // change the boolean array
            // scan along true's, changing them to false's
            // then when we find a false, change it to a true

            boolean foundFalse = false;
            int thisIndex = 0;
            while (!foundFalse && thisIndex < isIn.length) {
                if (isIn[thisIndex] == true) // looking at a "true"
                    {
                    isIn[thisIndex] = false;
                    if (thisIndex == numPolys - 1)
                        // hve finished through subsets
                        {
                        hasFinished = true;
                        break subSetLoop;
                    } else {
                        thisIndex++;
                    }
                } else { //System.out.println("changing a false to a true");
                    isIn[thisIndex] = true;
                    foundFalse = true;
                }
            }
            //for(int index = 0; index<numPolys; index++)
            //{ /**/System.out.print("-"+isIn[index]); }
            //System.out.println("");
            // we have an array of booleans: isIn
            // use this to construct a subset of ContourPolygons from polygons
            Vector containingContourPs = new Vector();
            Vector excludingContourPs = new Vector();
            for (int index = 0; index < numPolys; index++) {
                if (isIn[index]) {
                    ConcreteContour cp =
                        (ConcreteContour) (contourPolygons.get(index));
                    containingContourPs.add(cp);
                } else {
                    ConcreteContour cp =
                        (ConcreteContour) (contourPolygons.get(index));
                    excludingContourPs.add(cp);
                }
            }
            // now we know which polygons our zone should be inside/outside
            //System.out.println("inside "+insideSubset+" and outside "+outsideSubset);
            //System.out.println("working on zone "+z);
            // construct a Polygon as the intersection of the insides
            ConcreteZone cz =
                new ConcreteZone(containingContourPs, excludingContourPs);
            if (!cz.getArea().isEmpty()) {
                if (AreaPolygonAdapter.getAreaOfConnectedArea(cz.getArea())
                    == 0) {
                    zoneAreas = new Vector();
                    return;
                } else {
                    result.add(cz);
                }
            }
        }
        zoneAreas = result;
        //System.out.println("storing zone polys : "+zonePolys);
    }
    public String toString() {
        String result = "";
        for (int polyNum = 0; polyNum < contourPolygons.size(); polyNum++) {
            ConcreteContour cp = (ConcreteContour) (contourPolygons.get(polyNum));
            Contour c = cp.getContour();
            result = result + " " + c;
            Polygon p = cp.getPolygon();
            int[] xs = p.xpoints;
            int[] ys = p.ypoints;
            for (int index = 0; index < p.npoints; index++) {
                result = result + "(" + xs[index] + "," + ys[index] + ")";
            }
        }
        return result;
    }
    
    public Object clone() {
    	return new ContourPolygonSet((Vector)this.contourPolygons.clone());
    }

    // test zoneshape construction
    /*
    	public static void main(String args[]) { // build some Contours, Polygons, ContourPolygons...
    		Vector polys = new Vector();
    		Polygon p;
    		ConcreteContour cp;
    		Contour a = Contour.getContour("a");
    		Contour b = Contour.getContour("b");
    		//Contour c = Contour.getContour("c");
    		int[] x1s = { 0, 2, 2, 0 };
    		int[] y1s = { 1, 1, 3, 3 };
    		p = new Polygon(x1s, y1s, 4);
    		cp = new ConcreteContour(a, p);
    		polys.add(cp);
    		int[] x2s = { 1, 3, 3, 1 };
    		int[] y2s = { 0, 0, 2, 2 };
    		p = new Polygon(x2s, y2s, 4);
    		cp = new ConcreteContour(b, p);
    		polys.add(cp);
    		ContourPolygonSet polySet = new ContourPolygonSet(polys);
    		System.out.println(
    			"built a polygon set with polygons " + polySet.getContourPolys());
    		polySet.calcZonePolys();
    		//System.out.println("3 zone shapes are "+polySet.getZoneShapes());
    		System.out.println("3 zone polys are " + polySet.getZonePolys());
    		System.out.println("second example");
    		polys = new Vector();
    		int[] x3s = { 0, 1, 1, 0 };
    		int[] y3s = { 2, 2, 3, 3 };
    		p = new Polygon(x3s, y3s, 4);
    		cp = new ConcreteContour(a, p);
    		polys.add(cp);
    		int[] x4s = { 2, 3, 3, 2 };
    		int[] y4s = { 0, 0, 1, 1 };
    		p = new Polygon(x4s, y4s, 4);
    		cp = new ConcreteContour(b, p);
    		polys.add(cp);
    		polySet = new ContourPolygonSet(polys);
    		System.out.println(
    			"built a polygon set with polygons " + polySet.getContourPolys());
    		polySet.calcZonePolys();
    		//System.out.println("2 zone shapes are "+polySet.getZoneShapes());
    		System.out.println("2 zone polys are " + polySet.getZonePolys());
    		System.out.println("third example");
    
    		///OldAbstractEulerDiagram aed = new OldAbstractEulerDiagram();
    		//OldZone z = new OldZone();
    		aed.add(z);
    		z = new OldZone();
    		z.add(a);
    		aed.add(z);
    		z = new OldZone();
    		z.add(b);
    		aed.add(z);
    		Result r = new ResultDirector().makeResult(aed);
    
    		AEDView aedv = ((CompositeResult) r).getAEDV();
    		polySet = new ContourPolygonSet(aedv);
    		
    		System.out.println(
    			"built a polygon set with polygons " + polySet.getContourPolys());
    		polySet.calcZonePolys();
    		//System.out.println("2 zone shapes are "+polySet.getZoneShapes());
    		
    		System.out.println("2 zone polys are " + polySet.getZonePolys());
    
    		
    		System.out.println("fourth example");
    
    		aed = new OldAbstractEulerDiagram();
    		z = new OldZone();
    		aed.add(z);
    		z = new OldZone();
    		z.add(a);
    		aed.add(z);
    		z = new OldZone();
    		z.add(b);
    		aed.add(z);
    		z = new OldZone();
    		z.add(a);
    		z.add(b);
    		aed.add(z);
    		aedv =
    			((CompositeResult) new ResultDirector().makeResult(aed)).getAEDV();
    		polySet = new ContourPolygonSet(aedv);
    		System.out.println(
    			"built a polygon set with polygons " + polySet.getContourPolys());
    		polySet.calcZonePolys();
    		//System.out.println("3 zone shapes are "+polySet.getZoneShapes());
    		System.out.println("3 zone polys are " + polySet.getZonePolys());
    		System.out.println("fifth example");
    		polys = new Vector();
    		int[] x5s = { 0, 4, 4, 0 };
    		int[] y5s = { 2, 2, 3, 3 };
    		p = new Polygon(x5s, y5s, 4);
    		cp = new ConcreteContour(a, p);
    		polys.add(cp);
    		int[] x6s = { 177, 313, 262, 100 };
    		int[] y6s = { 251, 299, 322, 400 };
    		p = new Polygon(x6s, y6s, 4);
    		cp = new ConcreteContour(b, p);
    		polys.add(cp);
    		polySet = new ContourPolygonSet(polys);
    		System.out.println(
    			"built a polygon set with polygons " + polySet.getContourPolys());
    		polySet.calcZonePolys();
    		//System.out.println("2 zone shapes are "+polySet.getZoneShapes());
    		System.out.println("2 zone polys are " + polySet.getZonePolys());
    		System.out.println("sixth example");
    		polys = new Vector();
    		int[] x7s = { 210, 227, 221, 205 };
    		int[] y7s = { 148, 146, 162, 250 };
    		p = new Polygon(x7s, y7s, 4);
    		cp = new ConcreteContour(a, p);
    		polys.add(cp);
    		int[] x8s = { 216, 231, 212, 200 };
    		int[] y8s = { 183, 159, 175, 250 };
    		p = new Polygon(x8s, y8s, 4);
    		cp = new ConcreteContour(b, p);
    		polys.add(cp);
    		polySet = new ContourPolygonSet(polys);
    		System.out.println(
    			"built a polygon set with polygons " + polySet.getContourPolys());
    		polySet.calcZonePolys();
    		//System.out.println("2 zone shapes are "+polySet.getZoneShapes());
    		System.out.println("2 zone polys are " + polySet.getZonePolys());
    	}
    	/*
    	      Polygon p = ((ConcreteContour)(insideSubset.get(0))).getPolygon();
    	      Vector polySet = new Vector();
    	      polySet.add(p);
    	      //System.out.println("polyset is empty? "+inContours.isEmpty());
    	      for(int index = 1; index<insideSubset.size(); index++)
    	      { ConcreteContour cp = (ConcreteContour)(insideSubset.get(index));
    	        Polygon other = cp.getPolygon();
    	        // intersect Polygons in ploySet with other
    	        Vector newPolySet = new Vector();
    	        for(int i = 0; i<polySet.size();i++)
    	        { newPolySet.addAll(new AreaPolygonAdapter((Polygon)(polySet.get(i)))
    	                                               .intersect(other));
    	        }
    	        polySet = newPolySet;
    	      }
    	      // exclude area from contours it should be outside
    	      for(int index = 0; index<outsideSubset.size(); index++)
    	      { ConcreteContour cp = (ConcreteContour)(outsideSubset.get(index));
    	        AreaPolygonAdapter other = new AreaPolygonAdapter(cp.getPolygon());
    	        //System.out.println("subtract contour "+cp.contour);
    	        // subtracts Area other from inContours
    	        //polySet.subtract(other);
    	      }
    	      // now polySet is the right Area for the zone
    	      // and Zone is the right zone for the zone:
    	      // check that the Area for the zone is not empty
    	      // build a ZoneShape object and add it to result
    	      //System.out.println("adding a zone shape");
    	      //TO DO handle case where polyset.size()>1..................!!!!!!!!!!!
    	      //ToDo;
    	      if(!polySet.isEmpty())
    	      { result.add(new ZonePoly(z,(Polygon)(polySet.get(0)))); }
    	      //else
    	      //{ //System.out.println("found an absent zone "+z);
    	      //}
    	    }
    	    //zoneShapes = result;
    	    zonePolys = result;
    	*/
    /*
      public void addMorePoints(int n)
      { // iterate through polys and check they have >=8 points
        // otherwise add some at midpoints
        //System.out.println("adding more points");
        Iterator polyIt = contourPolygons.iterator();
        while(polyIt.hasNext())
        { ConcreteContour cp = (ConcreteContour)(polyIt.next());
          Polygon p = cp.getPolygon();
          int npoints;
          while((npoints = p.npoints)<n)
          { int[] newxs = new int[2*npoints];
            int[] newys = new int[2*npoints];
            //System.out.println("npoints is "+npoints);
            int evenIndex = 0;
            PathIterator pi = p.getPathIterator(null);
            double[] sixDoubles = new double[6];
            int oldx, nextx, oldy, nexty;
            pi.currentSegment(sixDoubles);
            nextx = (int)(sixDoubles[0]);
            nexty = (int)(sixDoubles[1]);
            int firstx = nextx;
            int firsty = nexty;
            while(evenIndex<2*npoints - 2)
            { oldx = nextx;
              oldy = nexty;
              // use the "last" coords for old(x,y)
              // now pick up the "next" coords for next(x,y)
              pi.next();
              pi.currentSegment(sixDoubles);
              nextx = (int)(sixDoubles[0]);
              nexty = (int)(sixDoubles[1]);
              //System.out.println("old (x,y) is ("+oldx+","+oldy
              //                  +") and next(x,y) is ("+nextx+","+nexty+")");
              newxs[evenIndex]=oldx;
              newys[evenIndex]=oldy;
              newxs[evenIndex+1]=(oldx+nextx)/2;
              newys[evenIndex+1]=(oldy+nexty)/2;
              evenIndex = evenIndex+2;
            }
            oldx = nextx;
            oldy = nexty;
            nextx = firstx;
            nexty = firsty;
            //System.out.println("old (x,y) is ("+oldx+","+oldy
            //                    +") and next(x,y) is ("+nextx+","+nexty+")");
            newxs[evenIndex]=oldx;
            newys[evenIndex]=oldy;
            newxs[evenIndex+1]=(oldx+nextx)/2;
            newys[evenIndex+1]=(oldy+nexty)/2;
    
            p = new Polygon(newxs,newys,2*npoints);
            cp.setPolygon(p);
          }
        }
      }*/
}
