package rwd.concretesyntax.spiderdiagram;

import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;
import java.awt.geom.Rectangle2D;
import java.util.*;
import rwd.concretesyntax.*;
import rwd.concretesyntax.eulerdiagram.*;
import rwd.abstractsyntax.eulerdiagram.*;
import rwd.abstractsyntax.spiderdiagram.*;
import rwd.utilities.AreaRWD;
import rwd.views.abstracttoconcrete.forcemodel.*;

/**
 * Holds:
 *   concrete spider feet
 *   concrete spider legs
 *   ties between concrete spiders
 *   strands between concrete spiders.
 * As ties, strands and legs are held as varieties of ConcreteSpiderLeg, 
 * this makes sense.
 * However, it is not good modeling.  Better would be to have a superclass
 * ConcreteConnector with subclasses ConcreteSpiderLeg, ConcreteStrand, 
 * ConcreteTie; to make ConcreteSpider have type EXISTENTIAL, UNIVERSAL,
 * or SCHROEDINGER (instead of ConcreteSpiderFoot as now) and give 
 * components ConcreteSpiderLeg and ConcreteSpiderFoot to ConcreteSpider.
 * 
 * @author  op, rnb
 * @version 6.7.05   15:05
 */
public class ConcreteSpiderFeetAndLegsSet 
	implements java.io.Serializable 
{
	//
	// Constructors
	//
	public ConcreteSpiderFeetAndLegsSet() {
	}
	public ConcreteSpiderFeetAndLegsSet(Collection feet, Collection legs) {
		_feet = new LinkedHashSet(feet);
		_legs = new LinkedHashSet(legs);
		_changed = true;
	}
    public ConcreteSpiderFeetAndLegsSet(Map zoneMapping) {
        this.zoneMapping = zoneMapping;
    }

	//
	// Spider methods
	//
	/** Add an abstract spider */
	public void addAbstractSpider(ConcreteDiagram d, Spider s){
		HashMap mapping = 
			ConcreteUnitaryConverter.mapUnitaryDiagramCalled(
				d,
				(String)ConcreteUnitaryConverter.getNamesOfUnitaryBoxes(d).get(0));
		addAbstractSpider(s, mapping, new Vector(d.getConcreteZones()));
	}
	/** Add an abstract spider */
	public void addAbstractSpider(Spider s, HashMap zoneMapping, Vector concreteZones){
		this.zoneMapping = zoneMapping;
		this.addSpider((Vector)zoneMapping.get("czones"),s);
	}

    /**
     * Add an abstract spider to this concrete spider set. The spider
     * is converted to a collection of concrete spider feet and legs.
     */
    private void addSpider(Vector zoneAreas, Spider spider) {
    	// get label
    	String label = spider.getLabel();
    	// Get zones
        Vector zones = spider.getHabitat();
		// Compute feet
        Vector spiderFeet = new Vector();
        for (int i = 0; i < zones.size(); i++) {
            Zone zone = (Zone) zones.get(i);
            ConcreteSpiderFoot foot =
                new ConcreteSpiderFoot(
                    label,
                    ConcreteSpiderFoot.NORMAL);
            // blank label for any foot after the first
            label = "";
            foot.setZone(zone.toString());
            spiderFeet.add(foot);
            _feet.add(foot);
        }
		// Place feet
        placeNodes(zoneAreas, new Vector(_feet), true, null);
        // Now create a spanning tree (this may not be minimum)
        for (int i = 1; i < spiderFeet.size(); i++) {
            ConcreteSpiderFoot source =
                (ConcreteSpiderFoot) spiderFeet.get(i - 1);
            ConcreteSpiderFoot target = (ConcreteSpiderFoot) spiderFeet.get(i);
            _legs.add(
                new ConcreteSpiderLeg(
                    "",
                    source,
                    target,
                    ConcreteSpiderLeg.NORMAL));
        }
		_changed = true;
    }

	/** Place nodes from feet */
    public void placeNodes(
        List zoneAreas,
        List spiderFeet,
        boolean isAbstract,
        HashMap footMapping) {

        // Group the spider feet into zone groups.
        HashMap groups = new HashMap();
        for (int i = 0; i < spiderFeet.size(); i++) {
            ConcreteSpiderFoot foot = (ConcreteSpiderFoot) spiderFeet.get(i);
                        
            for (int j = 0; j < zoneAreas.size(); j++) {
                ConcreteZone za = (ConcreteZone) zoneAreas.get(j);
                System.out.println("maybe it should go in "+za+"?");
                AreaRWD a = za.getArea();

                //System.out.println("za string is "+getZoneString(za)+"");
                if (isAbstract
                    && getZoneString(za).equals(foot.getZone())
                    || !isAbstract
                    && a.contains(foot.getX(), foot.getY())) {

                    if (!groups.containsKey(za)) {
                        groups.put(za, new Vector());
                    }
                    Vector nodes = (Vector) groups.get(za);
                    Node node = new Node();
                    node.setObject(foot);
                    node.setX(foot.getX());
                    node.setY(foot.getY());
                    nodes.add(node);
                    System.out.println("put foot in "+za);
                    continue;
                }
            }
        }

        // Get the set of line segments for each zone area.
        HashMap segmentGroup = new HashMap();
        for (int i = 0; i < zoneAreas.size(); i++) {
            ConcreteZone za = (ConcreteZone) zoneAreas.get(i);
            AreaRWD a = za.getArea(false);
            if(a!=null){
                Vector segments = getLineSegments(a);
                System.out.println("- got "+segments.size()+" segments for "+ za);
                segmentGroup.put(za, segments);
            }
        }

        // For each zone, let's work out where to automatically
        // place the nodes in it.
        Iterator it = groups.keySet().iterator();
        while (it.hasNext()) {
            ConcreteZone za = (ConcreteZone) it.next();
            System.out.println("consider "+za);
            
            AreaRWD a = za.getArea();
            Vector nodes = (Vector) groups.get(za);
            System.out.println("relevant nodes :"+nodes);
            Vector segments = (Vector) segmentGroup.get(za);
            Rectangle2D rect = a.getBounds2D();

            for (int n = 0; n < nodes.size(); n++) {
                Node node = (Node) nodes.get(n);
                System.out.println("consider node :"+node);

                ConcreteSpiderFoot matchingFoot = null;
                if (footMapping != null) {
                    matchingFoot = (ConcreteSpiderFoot) node.getObject();
                    System.out.println("found mapped foot");
                }

                LinkedHashSet xIntersects = new LinkedHashSet();

                double y = (2 * rect.getY() + rect.getHeight()) / 2;
                y += ((rect.getHeight() / 2) * (Math.random() - 0.5));

                // if the corresponding foot from d1 is within the range, use its
                // y coordinate instead of a random one.
                if (matchingFoot != null
                    && matchingFoot.getY() > rect.getY()
                    && matchingFoot.getY() < rect.getY() + rect.getHeight()) {
                    y = matchingFoot.getY();
                }

                Line2D horizontal =
                    new Line2D.Double(
                        rect.getX(),
                        y,
                        rect.getX() + rect.getWidth(),
                        y);
                for (int i = 0; i < segments.size(); i++) {
                    Line2D line = (Line2D) segments.get(i);
                    if (line.intersectsLine(horizontal)) {
                        // this is where the horizontal line enters or leaves the polygon
                        // ...let's work out exactly where it intersects
                        double dx = line.getX2() - line.getX1();
                        double dy = line.getY2() - line.getY1();
                        double xIntersect =
                            line.getX1() + dx * (y - line.getY1()) / dy;
                        xIntersects.add(new Double(xIntersect));
                    }
                }
                Vector xIntersectsList = new Vector(xIntersects);

                Collections.sort(xIntersectsList);

                if (xIntersectsList.size() < 2) {
                    continue;
                }

                int segmentNumber =
                    RAND.nextInt(xIntersectsList.size() / 2) * 2;

                // We can pick an x-coordinate anywhere between these two values.
                double x1 =
                    ((Double) xIntersectsList.get(segmentNumber)).doubleValue();
                double x2 =
                    ((Double) xIntersectsList.get(segmentNumber + 1))
                        .doubleValue();
                        
                double x = x1 + (x2 - x1) * (Math.random() / 2 + 0.25);

                // however, if the corresponding spider foot is in the zone, then
                // let's just use that coordinate instead!
                if (matchingFoot != null
                    && a.contains(matchingFoot.getX(), y)) {
                    x = matchingFoot.getX();
                }

                node.setX(x);
                node.setY(y);
                ConcreteSpiderFoot foot = (ConcreteSpiderFoot) node.getObject();
                foot.setLocation((int) node.getX(), (int) node.getY());
            }
        }
		_changed = true;
    }
	private String getZoneString(ConcreteZone cz) {
		return "" + zoneMapping.get(cz);
	}

    private static Vector getLineSegments(AreaRWD area) {
        Vector segments = new Vector();
        int type = 0;
        float oldX = 0;
        float oldY = 0;
        float[] coords = new float[6];
        PathIterator it = area.getPathIterator(null);
        while (!it.isDone()) {
            type = it.currentSegment(coords);
            float x1 = coords[0];
            float y1 = coords[1];

            if (type == PathIterator.SEG_MOVETO) {
                oldX = x1;
                oldY = y1;
            } else if (
                type == PathIterator.SEG_LINETO
                    || type == PathIterator.SEG_CUBICTO
                    || type == PathIterator.SEG_QUADTO) {
                segments.add(new Line2D.Double(oldX, oldY, x1, y1));
                oldX = x1;
                oldY = y1;
            }
            it.next();
        }
        return segments;
    }

    public List getFeet() {
        return new Vector(_feet);
    }
    public void addFoot(ConcreteSpiderFoot foot) {
    	_feet.add(foot);
		_changed = true;
    }
	public void removeFoot(ConcreteSpiderFoot foot) {
		_feet.remove(foot);
		_changed = true;
	}

    public Vector getLegs() {
        return new Vector(_legs);
    }
	public void addLegs(Collection legs) {
		Iterator i = legs.iterator();
		while (i.hasNext()) {
			ConcreteSpiderLeg leg = (ConcreteSpiderLeg)i.next();
			addLeg(leg);
		}
	}
	public void removeLegs(Collection legs) {
		Iterator i = legs.iterator();
		while (i.hasNext()) {
			ConcreteSpiderLeg leg = (ConcreteSpiderLeg)i.next();
			removeLeg(leg);
		}
	}
	public void addLeg(ConcreteSpiderLeg leg) {
		_legs.add(leg);
		_feet.add(leg.getSource());
		_feet.add(leg.getTarget());
		_changed = true;
	}
	public void removeLeg(ConcreteSpiderLeg leg) {
		_legs.remove(leg);
//		_feet.remove(leg.getSource());
//		_feet.remove(leg.getTarget());
		_changed = true;
	}

    // Use edge shape intersections to calculate the number of edge crossings
    public int getEdgeCrossings() {
        int crossings = 0;
        for (int a = 0; a < getLegs().size(); a++) {
            for (int b = a + 1; b < getLegs().size(); b++) {
                ConcreteSpiderLeg legA = (ConcreteSpiderLeg) getLegs().get(a);
                ConcreteSpiderLeg legB = (ConcreteSpiderLeg) getLegs().get(b);
                if (Line2D
                    .linesIntersect(
                        legA.getSource().getX(),
                        legA.getSource().getY(),
                        legA.getTarget().getX(),
                        legA.getTarget().getY(),
                        legB.getSource().getX(),
                        legB.getSource().getY(),
                        legB.getTarget().getX(),
                        legB.getTarget().getY())) {
                    crossings++;
                }
            }
        }
        return crossings;
    }

    public double totalEdgeLengths() {
        double length = 0;
        for (int i = 0; i < getLegs().size(); i++) {
            ConcreteSpiderLeg leg = (ConcreteSpiderLeg) getLegs().get(i);
            double dx = leg.getTarget().getX() - leg.getSource().getX();
            double dy = leg.getTarget().getY() - leg.getSource().getY();
            length += Math.sqrt(dx * dx + dy * dy);
        }
        return length;
    }

    public double totalSquareEdgeLengths() {
        double lengthSq = 0;
        for (int i = 0; i < getLegs().size(); i++) {
            ConcreteSpiderLeg leg = (ConcreteSpiderLeg) getLegs().get(i);
            double dx = leg.getTarget().getX() - leg.getSource().getX();
            double dy = leg.getTarget().getY() - leg.getSource().getY();
            lengthSq += dx * dx + dy * dy;
        }
        return lengthSq;
    }

    public double getShortestEdgeLength() {
        double shortest = Double.POSITIVE_INFINITY;
        for (int i = 0; i < getLegs().size(); i++) {
            ConcreteSpiderLeg leg = (ConcreteSpiderLeg) getLegs().get(i);
            double dx = leg.getTarget().getX() - leg.getSource().getX();
            double dy = leg.getTarget().getY() - leg.getSource().getY();
            double length = Math.sqrt(dx * dx + dy * dy);
            if (length < shortest) {
                shortest = length;
            }
        }
        return shortest;
    }

	public ConcreteSpiderFoot getNearestFoot(int x, int y) {
		int bestDistSq = Integer.MAX_VALUE;
		ConcreteSpiderFoot nearest = null;
		for (int i = 0; i < getFeet().size(); i++) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot) getFeet().get(i);
			int dx = foot.getX() - x;
			int dy = foot.getY() - y;
			int distSq = dx * dx + dy * dy;
			if (distSq < bestDistSq) {
				nearest = foot;
				bestDistSq = distSq;
			}
		}
		return nearest;
	}

//    public ConcreteSpiderFoot getNearestNormalFoot(int x, int y) {
//        int bestDistSq = Integer.MAX_VALUE;
//        ConcreteSpiderFoot nearest = null;
//        for (int i = 0; i < getFeet().size(); i++) {
//            ConcreteSpiderFoot foot = (ConcreteSpiderFoot) getFeet().get(i);
//            if (foot.getType() == ConcreteSpiderFoot.NORMAL) {
//                int dx = foot.getX() - x;
//                int dy = foot.getY() - y;
//                int distSq = dx * dx + dy * dy;
//                if (distSq < bestDistSq) {
//                    nearest = foot;
//                    bestDistSq = distSq;
//                }
//            }
//        }
//        return nearest;
//    }

	
    private Map foot2spider = new HashMap(); // 30.6.05 changed this to map
                                             // feet to spiders, rather than
                                             // foot-labels to spiders
    private List _spiders = new Vector();
    
    public List getSpiders() {
    	if (!_changed)
    		return _spiders;
    	_changed = false;
        // Initialize the set of spiders
        Set spiders = new LinkedHashSet();
        Iterator i = getFeet().iterator();
        while (i.hasNext()) {
            ConcreteSpiderFoot foot = (ConcreteSpiderFoot) i.next();
            ConcreteSpider spider = new ConcreteSpider();
            spider.addFoot(foot);
            spiders.add(spider);
            foot2spider.put(foot, spider);
        }

        // Make a temporary copy of the legs
        List legs = (List) getLegs().clone();
        // Refine the set of spiders by merging
        boolean ok = true;
        do {
            ok = true;
            // Search for legs that connect 2 spiders
            i = legs.iterator();
            while (i.hasNext()) {
                // Compute the feet of the current leg
                ConcreteSpiderLeg leg = (ConcreteSpiderLeg) i.next();
                ConcreteSpiderFoot f1 = leg.getSource();
                ConcreteSpiderFoot f2 = leg.getTarget();
                // Compute the spider that contains f1
                ConcreteSpider s1 = (ConcreteSpider) foot2spider.get(f1);
                ConcreteSpider s2 = (ConcreteSpider) foot2spider.get(f2);
                // If different 
                if (!(s1.equals(s2))) {
                    // Merge the spiders by adding legs and feet
                    s1.getFeet().addAll(s2.getFeet());
                    s1.getLegs().addAll(s2.getLegs());
                    s1.getLegs().add(leg);
                    // Update the mapping foot2spider every feet
                    Iterator j = s2.getFeet().iterator();
                    while (j.hasNext()) {
                        ConcreteSpiderFoot f = (ConcreteSpiderFoot) j.next();
                        foot2spider.put(f, s1);
                    }
                    // Remove the extra spider
                    spiders.remove(s2);
                    
                    // Set flag
                    ok = false;
                }
            }
        }
        while (!ok);
        _spiders = new Vector(spiders); 
        return _spiders;
    }
//	public List getSpiders() {
//		// Initialize the set of spiders
//		List spiders = new Vector();
//		Iterator i = getFeet().iterator();
//		while (i.hasNext()) {
//			ConcreteSpiderFoot foot = (ConcreteSpiderFoot) i.next();
//			ConcreteSpider spider = new ConcreteSpider();
//			spider.addFoot(foot);
//			spiders.add(spider);
//			foot2spider.put(foot.toString(), spider);
//		}
//		// Make a temporary copy of the legs
//		List legs = (List) getLegs().clone();
//		// Refine the set of spiders by merging
//		boolean ok = true;
//		do {
//			ok = true;
//			// For each pair of distinct spiders
//			for(int j=0; j<spiders.size(); j++) {
//				ConcreteSpider cs1 = (ConcreteSpider)spiders.get(j);
//				for(int k=j+1; k<spiders.size(); k++) {
//					ConcreteSpider cs2 = (ConcreteSpider)spiders.get(k);
//					// If there is a leg connecting feet from different spiders
//					Iterator fIt = getLegs().iterator();
//					while (fIt.hasNext()) {
//						ConcreteSpiderLeg leg = (ConcreteSpiderLeg)fIt.next();
//						ConcreteSpiderFoot foot1 = (ConcreteSpiderFoot)leg.getSource(); 
//						ConcreteSpiderFoot foot2 = (ConcreteSpiderFoot)leg.getTarget();
//						// Chech if they belong to different spiders 
//					}
//				}
//			}
//			// Search for legs that connect 2 spiders
//			// and store them
//			Set mLegs = new LinkedHashSet();
//			i = legs.iterator();
//			while (i.hasNext()) {
//				// Compute the feet of the current leg
//				ConcreteSpiderLeg leg = (ConcreteSpiderLeg) i.next();
//				ConcreteSpiderFoot f1 = leg.getSource();
//				ConcreteSpiderFoot f2 = leg.getTarget();
//				// Compute the spider that contains f1
//				ConcreteSpider s1 = (ConcreteSpider) foot2spider.get(f1.toString());
//				ConcreteSpider s2 = (ConcreteSpider) foot2spider.get(f2.toString());
//				// If different 
//				if (s1 != s2) {
//					// Merge the spiders by adding legs and feet
//					s1.getFeet().addAll(s2.getFeet());
//					s1.getLegs().addAll(s2.getLegs());
//					s1.getLegs().add(leg);
//					// Uodate the mapping foot2spider every feet
//					Iterator j = s2.getFeet().iterator();
//					while (j.hasNext()) {
//						ConcreteSpiderFoot f = (ConcreteSpiderFoot) j.next();
//						foot2spider.put(f, s1);
//					}
//					// Remove the extra spider
//					spiders.remove(s2);
//					// Set flag
//					ok = false;
//					// Store the leg
//					mLegs.add(leg);
//				}
//			}
//			// Remove the legs that were used to merge spiders
//			legs.removeAll(mLegs);
//		}
//		while (!ok);
//		return new Vector(spiders);
//	}

    public ConcreteSpiderFeetAndLegsSet getClone() {
    	Set newLegs = getClone(_legs);
    	Set newFeet = new LinkedHashSet();
    	if (newLegs.isEmpty()) {
    		newFeet = getClone(_feet);
    	} else {
    		Iterator i = newLegs.iterator();
    		while (i.hasNext()) {
    			ConcreteSpiderLeg newLeg = (ConcreteSpiderLeg)i.next();
    			newFeet.add(newLeg.getSource());
    			newFeet.add(newLeg.getTarget());
    		}
    	}
        return new ConcreteSpiderFeetAndLegsSet(
            newFeet,
            newLegs);
    }
    protected Set getClone(Set list) {
		Set res = new LinkedHashSet();
		Iterator i = list.iterator();
		while (i.hasNext()) {
			DiagramItem item = (DiagramItem)i.next();
			if (item != null)
				res.add(item.getClone());
		}
    	return res;
    }

    
	// 
	// toString
	//
	/**
	 * @return string representation of ConcreteSpiderFeetAndLegsSet
	 */
    /*
     * test: once thru
     */
	public String toString(){
		return "(feet="+ _feet + "," +
		       "legs="+_legs + ")";
	}
	
    //
    // Local properties
    //
    protected Set _feet = new LinkedHashSet();
	protected Set _legs = new LinkedHashSet();
	protected static final Random RAND = new Random();
	static final long serialVersionUID = -1205790233765361457L;
	protected Map zoneMapping;
	protected boolean _changed = true;
}
