package rwd.deltas;

import java.awt.Point;

import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.constraintdiagram.ConcreteArcEnd;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import uk.ac.kent.cs.kmf.util.ILog;
import uk.ac.kent.cs.kmf.util.OutputStreamLog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class AngularResolutionDelta 
	extends Delta 
{
	// Constructor
	public AngularResolutionDelta(DiagramItem item1, DiagramItem item2, DiagramItem item3, ILog log) {
		this.log = log;
		args.add(item1);
		args.add(item2);	
		args.add(item3);
		item1.addObserver(this);
		item2.addObserver(this);
		item3.addObserver(this);
		computeValue();
	}
	public AngularResolutionDelta(DiagramItem item1, DiagramItem item2, DiagramItem item3) {
		this(item1, item2, item3, new OutputStreamLog(System.out));
	}
	
	protected void computeValue() {
		// Get arguments
		DiagramItem item1 = (DiagramItem)args.get(0);
		DiagramItem item2 = (DiagramItem)args.get(1);
		DiagramItem item3 = (DiagramItem)args.get(2);
		// Compute points
		Point p1 = point(item1);
		Point p2 = point(item2);
		Point p3 = point(item3);
		double a = length(p2, p3);
		double b = length(p1, p2);
		double c = length(p1, p3);
		double cosA = ((b*b+c*c)-a*a)/(2*b*c);
		double angle = Math.acos(cosA);
		value = (int)(angle*180/3.14);
		value = angle*angle;
		if (value != 0)
			value = 1/value;
	}
	
	protected Point point(DiagramItem item) {
		Point res[] = new Point[2];
		if (item instanceof ConcreteSpiderFoot) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot)item;
			return new Point(foot.getX(), foot.getY());
		} else if (item instanceof ConcreteArcEnd) {
			ConcreteArcEnd end = (ConcreteArcEnd)item;
			return new Point(end.getX(), end.getY());
		} else {
			return null;
		}
	}
	protected double length(Point p1, Point p2) {
		if (p1 == null || p2 == null) return 0;
		double x1 = p1.getX();
		double y1 = p1.getY();
		double x2 = p2.getX();
		double y2 = p2.getY();
		double value = (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1);
		return Math.sqrt(value);
	}
	public String toString() {
		String res = this.getClass().getName().substring(11);
		res += "("+args.get(0).toString()+", "+args.get(1).toString()+", "+args.get(2).toString()+")";
		return res;
	}
}
