package rwd.deltas;

import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.constraintdiagram.ConcreteArc;
import rwd.concretesyntax.constraintdiagram.ConcreteArcEnd;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderLeg;
import uk.ac.kent.cs.kmf.util.ILog;
import uk.ac.kent.cs.kmf.util.OutputStreamLog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class EdgeLengthDelta 
	extends Delta
{
	// Constructor - item either leg or arc
	public EdgeLengthDelta(DiagramItem item, ILog log) {
		this.log = log;
		if (item instanceof ConcreteSpiderLeg) {
			ConcreteSpiderLeg leg = (ConcreteSpiderLeg)item;
			ConcreteSpiderFoot foot1 = leg.getSource();
			ConcreteSpiderFoot foot2 = leg.getTarget();
			args.add(foot1);
			args.add(foot2);
			foot1.addObserver(this);
			foot2.addObserver(this);
		} else if (item instanceof ConcreteArc) {
			ConcreteArc arc = (ConcreteArc)item;
			ConcreteArcEnd start = arc.getStart();
			ConcreteArcEnd end = arc.getEnd();
			args.add(start);
			args.add(end);
			start.addObserver(this);
			end.addObserver(this);
		}
		computeValue();
	}
	public EdgeLengthDelta(DiagramItem item) {
		this(item, new OutputStreamLog(System.out));
	}
	protected void computeValue() {
		DiagramItem arg1 = (DiagramItem)args.get(0);
		DiagramItem arg2 = (DiagramItem)args.get(1);
		if (arg1 instanceof ConcreteSpiderFoot) {
			ConcreteSpiderFoot foot1 = (ConcreteSpiderFoot)arg1;
			ConcreteSpiderFoot foot2 = (ConcreteSpiderFoot)arg2;
			double x1 = foot1.getX();
			double y1 = foot1.getY();
			double x2 = foot2.getX();
			double y2 = foot2.getY();
			value = (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1);				
		} else if (arg1 instanceof ConcreteArcEnd) {
			ConcreteArcEnd start = (ConcreteArcEnd)arg1;;
			ConcreteArcEnd end = (ConcreteArcEnd)arg2;
			double x1 = start.getX();
			double y1 = start.getY();
			double x2 = end.getX();
			double y2 = end.getY();
			value = (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1);				
		}
	}
	public String toString() {
		String res = this.getClass().getName().substring(11);
		res += "("+args.get(0).toString()+")";
		return res;
	}
}
