package rwd.deltas;

import java.awt.Polygon;
import java.util.List;

import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.Event;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.ConcreteZone;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class NodeZoneSeparationDelta 
	extends Delta
{
	// Constructor
	public NodeZoneSeparationDelta(ConcreteSpiderFoot foot, ConcreteZone cz) {
		args.add(foot);
		args.add(cz);	
		computeValue();
	}
	public NodeZoneSeparationDelta(ConcreteSpiderFoot foot, ConcreteZone cz, ILog log) {
		this.log = log;
		args.add(foot);
		args.add(cz);	
		computeValue();
	}
	
	protected void computeValue() {
		ConcreteSpiderFoot foot = (ConcreteSpiderFoot)args.get(0);
		ConcreteZone cz = (ConcreteZone)args.get(1);
		value = distance(foot, cz);
		if (value != 0)
			value = 1/value;		
	}

	public void update(DiagramItem item, Event e) {
		if (args.contains(item)) {
//			log.reportMessage("Update delta "+toString());
			computeValue();
//			log.reportMessage("Old value:"+value);
//			log.reportMessage("New value:"+value);
		}
	}
	
	protected double distance(ConcreteSpiderFoot sf, ConcreteZone cz) {
		double res = Double.POSITIVE_INFINITY;
		if (cz == null || sf == null) return res;
		List cp = cz.getContainingPolygonals();
		double x1 = sf.getX();
		double y1 = sf.getY();
		for(int i=0; i<cp.size(); i++) {
			Object obj = cp.get(i);
			if (obj instanceof ConcreteContour) {
				ConcreteContour c = (ConcreteContour)obj;
				Polygon poly = c.getPolygon();
				for(int j=0; j<poly.npoints; j++) {
					double x2 = poly.xpoints[j];
					double y2 = poly.ypoints[j];
					double d = (x2-x1)*(x2-x1)+(y2-y1)*(y2-y1);
					if (d < res)
						res = d;
				}
			}
		}
		return res;
	}	 
	public String toString() {
		String res = this.getClass().getName().substring(11);
		res += "("+args.get(0).toString()+", "+args.get(1).toString()+")";
		return res;
	}
}
