package rwd.gui;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;

import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;

import java.util.Enumeration;

import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.FalseDiagram;
import rwd.abstractsyntax.spiderdiagram.NotCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.CompoundDiagram;
import java.util.Vector;

import rwd.gui.editor.viewers.DiagramViewerComponent;

public class DiagramShowerTree {
    private DefaultTreeModel dtm;
    //private DefaultMutableTreeNode selectedNode;
    private TreePath selectedPath;
    private JTree tree;
    private final FontPanel treePanel;
    private final FontPanel allPanel;
    private final boolean diagramByDefault;
    private final boolean canSelect;
    private Diagram partDiagram;
    private Diagram diagram;
    private int diagramSize;
    private int jspSize;

    DiagramShowerTree(
        final FontPanel p,
        final Diagram d,
        boolean canSelect,
        boolean pictureByDefault,
        int diagramSize) {

        //System.out.println(
        //    "Building a new DiagramShowerTree : with pictures? "
        //        + pictureByDefault);

        this.diagram = d;
        this.diagramSize = diagramSize;
        this.jspSize = (int) (diagramSize * 1.1);
        this.partDiagram = d;
        this.diagramByDefault = pictureByDefault;
        this.canSelect = canSelect;
        allPanel = p;
        treePanel = new FontPanel();
        allPanel.removeAll();
        allPanel.setLayout(new BorderLayout());
        allPanel.add(treePanel, BorderLayout.CENTER);
        FontPanel tmp = new FontPanel();
        tmp.setLayout(new FlowLayout());
        allPanel.add(tmp, BorderLayout.SOUTH);
        makeTree(d);
        allPanel.validate();
        allPanel.repaint();
    }

    private void makeTree(Diagram d) {
        dtm = makeTreeModel(d, diagramByDefault);
        tree = new JTree(dtm);
        selectedPath = tree.getPathForLocation(0,0);
        DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer() {

            public Component getTreeCellRendererComponent(
                JTree t,
                Object value,
                boolean selected,
                boolean expanded,
                boolean leaf,
                int row,
                boolean hasFocus) {

                if (value instanceof DefaultMutableTreeNode) {
                    Object userObject =
                        ((DefaultMutableTreeNode) value).getUserObject();
                    if (userObject instanceof DiagramViewerComponent) {
                        DiagramViewerComponent dvc =
                            (DiagramViewerComponent) userObject;
                        dvc.setPreferredSize(new Dimension(jspSize, jspSize));
                        return dvc;
                    }
                }
                return super.getTreeCellRendererComponent(
                    t,
                    value,
                    selected,
                    expanded,
                    leaf,
                    row,
                    hasFocus);
            }
        };
        tree.setCellRenderer(renderer);

        final boolean canSelectF = canSelect;
        tree.addTreeSelectionListener(new TreeSelectionListener() {
            public void valueChanged(TreeSelectionEvent e) {
                DefaultMutableTreeNode node =
                    (DefaultMutableTreeNode) tree
                        .getLastSelectedPathComponent();
                Object userObject = node.getUserObject();
                if (userObject instanceof String) {
                    String name = (String) (node.getUserObject());
                    //System.out.println("selected 'name' is '"+name+"'");
                    if (name.length()>7 && name.substring(0, 7).equals("name: '") && canSelectF) {
                        name = name.substring(7, name.length());
                        name = name.substring(0, name.indexOf("'"));
                        //System.out.println("stripped down name is '"+name+"'");
                        StaticObjects.mdf.selectPart(name);
                        selectedPath = tree.getSelectionPath();
                        return;
                    } 
                }
                tree.setSelectionPath(selectedPath);
                // either clicked on other text or on diagram
            }
        });
        refreshDiagram(false);
    }

    private void refreshDiagram(boolean isdrawn) {
        Runnable r = new Runnable() {
            public void run() {
                FontScrollPane treeView = new FontScrollPane(tree);
                treeView.setBounds(
                    0,
                    0,
                    treePanel.getWidth(),
                    treePanel.getHeight());
                tree.setBounds(
                    0,
                    0,
                    treePanel.getWidth(),
                    treePanel.getHeight());
                treePanel.setLayout(new BorderLayout());
                treePanel.removeAll();
                treePanel.add(treeView);
                allPanel.removeAll();
                allPanel.setLayout(new BorderLayout());
                allPanel.add(treePanel, BorderLayout.CENTER);
                FontPanel tmp = new FontPanel();
                tmp.setLayout(new FlowLayout());
                allPanel.add(tmp, BorderLayout.SOUTH);
                allPanel.repaint();
            }
        };
        //System.out.println("refreshing");
        if (isdrawn && SwingUtilities.isEventDispatchThread()) {
            //System.out.println(
            //    "refreshing " + this.hashCode() + "tree within event thread");
            //r.run();
            SwingUtilities.invokeLater(r);
        } else if (isdrawn && !SwingUtilities.isEventDispatchThread()) {
            //System.out.println(
            //    "refreshing " + this.hashCode() + "tree outside event thread");
            // desperate attempt to get repainting working - switched lines here 11/11/02
            // don't know which of these should be used...
            // r.run();
            SwingUtilities.invokeLater(r);
        } else if (!isdrawn && SwingUtilities.isEventDispatchThread()) {
            //System.out.println(
            //    "painting " + this.hashCode() + "tree within event thread");
            r.run();
            //SwingUtilities.invokeLater(r);
        } else if (!isdrawn && !SwingUtilities.isEventDispatchThread()) {
            //System.out.println(
            //    "painting " + this.hashCode() + "tree outside event thread");
            SwingUtilities.invokeLater(r);
        }
        //repaintThings();
        allPanel.repaint();
    }

    public void refreshDiagramAfterEdit(Diagram partDiagram) { // 
        // check user objects of leaf nodes against existing Diagram
        this.partDiagram = partDiagram;
        String s = "name: '" + partDiagram.getName() + "'";
        // System.out.println("refreshing "+s);
        DefaultMutableTreeNode selectedNode =
            (DefaultMutableTreeNode) selectedPath.getLastPathComponent();
        selectedNode.setUserObject(s);
        if (partDiagram instanceof UnitaryDiagram && diagramByDefault) {
            try {
                //System.out.println("building a new dvc from the diagram");
                DiagramViewerComponent dvc =
                    new DiagramViewerComponent(640, 480, (UnitaryDiagram) partDiagram, true, false);
                dvc.fitToWidth(diagramSize);
                DefaultMutableTreeNode node = new DefaultMutableTreeNode(dvc);
                //System.out.println("removing children from tree node");
                selectedNode.removeAllChildren();
                //System.out.println("adding new tree node");
                selectedNode.add(node);
                //System.out.println("refreshing diagram");
                //refreshDiagram(true);
            } catch (IllegalArgumentException x) {
                UnitaryDiagram ud = (UnitaryDiagram) partDiagram;
                DefaultMutableTreeNode node;
                Enumeration e = selectedNode.children();
                node = (DefaultMutableTreeNode) e.nextElement();
                node.setUserObject(fillOut("contours: " + ud.getCloneOfContourSet()));
                node = (DefaultMutableTreeNode) e.nextElement();
                node.setUserObject(fillOut("zones: " + ud.getCloneOfZoneSet()));
                node = (DefaultMutableTreeNode) e.nextElement();
                node.setUserObject(fillOut("shaded zones: " + ud.getCloneOfShadedZones()));
                node = (DefaultMutableTreeNode) e.nextElement();
                node.setUserObject(fillOut("spiders: " + ud.getSpiders()));
            }
        } else if (partDiagram instanceof UnitaryDiagram) {
            UnitaryDiagram ud = (UnitaryDiagram) partDiagram;
            // each child node of selectedNode needs checking against
            // the attributes of partDiagram
            DefaultMutableTreeNode node;
            Enumeration e = selectedNode.children();
            node = (DefaultMutableTreeNode) e.nextElement();
            node.setUserObject(fillOut("contours: " + ud.getCloneOfContourSet()));
            node = (DefaultMutableTreeNode) e.nextElement();
            node.setUserObject(fillOut("zones: " + ud.getCloneOfZoneSet()));
            node = (DefaultMutableTreeNode) e.nextElement();
            node.setUserObject(fillOut("shaded zones: " + ud.getCloneOfShadedZones()));
            node = (DefaultMutableTreeNode) e.nextElement();
            node.setUserObject(fillOut("spiders: " + ud.getSpiders()));
        }
        refreshDiagram(true);
    }
    private String fillOut(String p) {
        while (p.length() < 80) {
            p = p + " ";
        }
        return p;
    }

    private DefaultTreeModel makeTreeModel(
        Diagram d,
        boolean try4Pictures) { //
        //System.out.println("building a new TreeModel");
        DefaultMutableTreeNode root =
            new DefaultMutableTreeNode(fillOut("name: '" + d.getName() + "'"));
        DefaultTreeModel result = new DefaultTreeModel(root);
        if (d instanceof CompoundDiagram) {
            CompoundDiagram cd = (CompoundDiagram) d;
            DefaultMutableTreeNode composition = null;
            if (cd instanceof AndCompoundDiagram) {
                composition = new DefaultMutableTreeNode("and");
            } else if (cd instanceof OrCompoundDiagram) {
                composition = new DefaultMutableTreeNode("or");
            } else if (cd instanceof NotCompoundDiagram) {
                composition = new DefaultMutableTreeNode("not");
            }
            root.add(composition);
            Vector children = cd.getChildren();
            for (int i = 0; i < children.size(); i++) {
                Diagram child = (Diagram) children.get(i);
                DefaultTreeModel model1 = makeTreeModel(child, try4Pictures);
                composition.insert(
                    (DefaultMutableTreeNode) (model1.getRoot()),
                    i);
            }
        } else if (d instanceof FalseDiagram) {
            DefaultMutableTreeNode composition =
                new DefaultMutableTreeNode("false");
            root.add(composition);
            //System.out.println("built a false diagram node");
        } else if (d instanceof UnitaryDiagram) {
            UnitaryDiagram ud = (UnitaryDiagram) d;
            if (try4Pictures) {
                try {
                    DiagramViewerComponent dvc =
                        new DiagramViewerComponent(jspSize, jspSize, ud, true, false);

                    dvc.fitToWidth(diagramSize);
                    root.add(new DefaultMutableTreeNode(dvc));
                    return result;
                } catch (IllegalArgumentException x) {
                    root.add(
                        new DefaultMutableTreeNode(
                            fillOut("contours: " + ud.getCloneOfContourSet())));
                    root.add(
                        new DefaultMutableTreeNode(
                            fillOut("zones: " + ud.getCloneOfZoneSet())));
                    root.add(
                        new DefaultMutableTreeNode(
                            fillOut("shaded zones: " + ud.getCloneOfShadedZones())));
                    root.add(
                        new DefaultMutableTreeNode(
                            fillOut("spiders: " + ud.getSpiders())));
                }
            } else {
                root.add(
                    new DefaultMutableTreeNode(
                        fillOut("contours: " + ud.getCloneOfContourSet())));
                root.add(
                    new DefaultMutableTreeNode(
                        fillOut("zones: " + ud.getCloneOfZoneSet())));
                root.add(
                    new DefaultMutableTreeNode(
                        fillOut("shaded zones: " + ud.getCloneOfShadedZones())));
                root.add(
                    new DefaultMutableTreeNode(
                        fillOut("spiders: " + ud.getSpiders())));
            }
        }
        return result;
    }
}
