package rwd.gui;

import java.awt.event.WindowEvent;
import javax.swing.JButton;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;

import rwd.rules.Proof;
import rwd.rules.ProofFileUser;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.abstractsyntax.spiderdiagram.Diagram;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.Dimension;
import java.util.Vector;
import java.util.Observer;
import java.util.Observable;
import javax.swing.JLabel;
import java.awt.Rectangle;

public class ProofViewer extends FontFrame implements Observer {
    Proof p; // "current"
    private FontPanel bigPanel = new FontPanel();
    private FontScrollPane jsp;
    private JLabel statusLabel = new JLabel();
    private FontFrame thisFrame = this;
    
    private static final int bigjspwidth = 780;
    private static final int bigjspheight = 500;
    private static final int smalljspwidth = 250;
    private static final int smalljspheight = 480;
    

    private boolean isUserExtensible = false;
    private int lastProofStepShown = 0;
    private JButton lastProofStepButton = null;
    private ActionListener extendProofListener = new ActionListener() {
            public void actionPerformed(ActionEvent e) {
            	//
            	//System.out.println("extension listener triggered");
            	
		    extendVisibleProof();
        }
    };
    Vector thingsToRepaint = new Vector();
    
    private boolean tryToDrawPictures;

    public ProofViewer(Proof p, boolean autoUpdate, boolean tryToDrawPictures) { //

        //System.out.println("proof viewer about to show proof "+p);
        super("proof with " + p.numSteps() + " steps");
        this.tryToDrawPictures = tryToDrawPictures;
        initGUI();
        isUserExtensible = !autoUpdate;
        this.p = p;
        if (autoUpdate) {
            p.addObserver(this);
            update(null, null);
        } else {
            startWithSome();
        }
        p.goToStart();
        pack();
        Dimension screenSize =
            java.awt.Toolkit.getDefaultToolkit().getScreenSize();
        setBounds(
            new Rectangle(
                (int) (screenSize.getWidth() - getWidth()),
                (int) (screenSize.getHeight() - getHeight()),
                getWidth(),
                getHeight()));
        JMenuBar jmb = new JMenuBar();
        this.setJMenuBar(jmb);
        JMenu fileMenu = new JMenu("File");
        jmb.add(fileMenu);
        JMenuItem load = new JMenuItem("load");
        fileMenu.add(load);
        load.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                loadProof();
            }
        });
        JMenuItem save = new JMenuItem("save");
        fileMenu.add(save);
        save.addActionListener(new ActionListener(){
            public void actionPerformed(ActionEvent e){
                saveProof();
            }
        });
        show();
    }
    private void loadProof(){
        p = (Proof)(ProofFileUser.loadFromFile(this).get(0));
        update(null,null);
    }
    private void saveProof(){
        ProofFileUser.saveToFile(p);
    }

    // for autoUpdate
    public void update(Observable o, Object args) {
        FontPanel jp = new FontPanel();
        thisFrame.getContentPane().remove(jsp);
        jsp = new FontScrollPane(jp);
        jsp.setBounds(0, 0, bigjspwidth, bigjspheight);
        jsp.setMaximumSize(new Dimension(bigjspwidth, bigjspheight));
        jsp.setPreferredSize(new Dimension(bigjspwidth, bigjspheight));
        jsp.setMinimumSize(new Dimension(bigjspwidth, bigjspheight));
        jp.setLayout(new FlowLayout());
        ////
        thisFrame.getContentPane().add(jsp, BorderLayout.CENTER);
        bigPanel = jp;
        //System.out.println("set bigPanel to be new jp");
        //bigPanel.setBackground(Color.red);
        thingsToRepaint = new Vector();
        thingsToRepaint.add(addDiagramPanel(p.getFirstDiagram()));
        //System.out.println("added a diagram");
        p.goToStart();
        int stepNumber = 0;
        while (p.goToNextStep()) //&& stepNumber<20)
            {
            stepNumber++;
            Rule r = p.getPreviousRule();
            RuleApplicationSpec ras = p.getPreviousRuleApplicationSpec();
            addStepPanel(r, ras);
            thingsToRepaint.add(addDiagramPanel(p.getCurrentDiagram()));
            //System.out.print(","+stepNumber);
        }
        setTitle("proof with " + p.numSteps() + " steps");
        //repaintThings();
    }

    // for user-extensible diagrams
    public void startWithSome() { //System.out.println("start with some");
        FontPanel jp = new FontPanel();
        thisFrame.getContentPane().remove(jsp);
        jsp = new FontScrollPane(jp);
        //jsp.setBounds(0, 0, 700, 330);
        //jsp.setMaximumSize(new Dimension(700, 330));
        //jsp.setPreferredSize(new Dimension(700, 330));
        jsp.setMinimumSize(new Dimension(bigjspwidth, bigjspheight));
        jp.setLayout(new FlowLayout());
        ////
        thisFrame.getContentPane().add(jsp, BorderLayout.CENTER);
        bigPanel = jp;
        //System.out.println("set bigPanel to be new jp");
        //bigPanel.setBackground(Color.red);
        //System.out.println("adding one panel");
        thingsToRepaint = new Vector();
        thingsToRepaint.add(addDiagramPanel(p.getFirstDiagram()));
        //System.out.println("added a diagram");
        p.goToStart();
        if (p.goToNextStep()) //&& stepNumber<20)
            {
            Rule r = p.getPreviousRule();
            RuleApplicationSpec ras = p.getPreviousRuleApplicationSpec();
            addStepPanel(r, ras);
            //System.out.println("adding a 2nd panel");
            thingsToRepaint.add(addDiagramPanel(p.getCurrentDiagram()));
            lastProofStepShown = 1;
        }
        if (p.goToNextStep()) //&& stepNumber<20)
            {
            Rule r = p.getPreviousRule();
            RuleApplicationSpec ras = p.getPreviousRuleApplicationSpec();
            addStepPanel(r, ras);
        }
        //repaintThings();
    }

    private void extendVisibleProof() {
        p.goToStart();
        int n = 0;
        while (n < lastProofStepShown) {
            p.goToNextStep();
            n++;
        }
        // add diagram
		thingsToRepaint.add(addDiagramPanel(p.getCurrentDiagram()));
		// if possible, add next step
		if (p.goToNextStep()) {
			Rule r = p.getPreviousRule();
			RuleApplicationSpec ras = p.getPreviousRuleApplicationSpec();
			addStepPanel(r, ras);
		}
		//repaintThings();
    }

    private DiagramShowerTree addDiagramPanel(Diagram d) {
        if (lastProofStepButton != null) {
            lastProofStepButton.removeActionListener(extendProofListener);
        }
        //System.out.println("adding diagram panel for "+d);
        FontPanel proofPanel = new FontPanel();
        DiagramShowerTree dst =
            new DiagramShowerTree(proofPanel, d, false, tryToDrawPictures, smalljspwidth-80);
        proofPanel.setPreferredSize(new Dimension(smalljspwidth, smalljspheight));
        bigPanel.add(proofPanel);
        jsp.validate();
        //System.out.println("scrolling to show panel at "+bigPanel.getWidth());
        bigPanel.scrollRectToVisible(
            new Rectangle(bigPanel.getWidth(), 0, 0, 0));
        //repaintThings();
        return dst;
    }

    private void addStepPanel(final Rule r, final RuleApplicationSpec ras) {
        JButton step = new JButton(">");
        step.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) { //System.out.println("description writing listener triggered");
                String description = "rwd.rules: " + r.name + "   ";
                description =
                    description
                        + "\napplied to (sub)diagram: "
                        + ras.d.getName()
                        + "   ";
                description = description + "\napplied in case: " + ras + "   ";
                statusLabel.setText(description);
            }
        });
        lastProofStepShown++;
        lastProofStepButton = step;
        lastProofStepButton.addActionListener(extendProofListener);
        step.setPreferredSize(new Dimension(30, 50));
        bigPanel.add(step);
        jsp.validate();
        //System.out.println("scrolling to show button at "+bigPanel.getWidth());
        bigPanel.scrollRectToVisible(
            new Rectangle(bigPanel.getWidth(), 0, 0, 0));
        //repaintThings();
    }

    /** This method is called from within the constructor to initialize the form. */
    private void initGUI() {
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });
        bigPanel.setLayout(new FlowLayout());
        jsp = new FontScrollPane(bigPanel);
        jsp.setBounds(0, 0, bigjspwidth, bigjspheight);
        jsp.setMaximumSize(new Dimension(bigjspwidth, bigjspheight));
        jsp.setMinimumSize(new Dimension(bigjspwidth, bigjspheight));
        jsp.setPreferredSize(new Dimension(bigjspwidth, bigjspheight));
        getContentPane().add(jsp, BorderLayout.CENTER);
        statusLabel.setPreferredSize(new Dimension(bigjspwidth, 50));
        getContentPane().add(statusLabel, BorderLayout.SOUTH);
    }

    /** Exit the Application */
    private void exitForm(WindowEvent evt) {
        //System.exit(0);
        ProofManager.close(p);
    }

    /*public void repaintThings() { // who should call this, when?
        System.out.println("repainting things in proofviewer");
        for (int i = 0; i < thingsToRepaint.size(); i++) {
            ((DiagramShowerTree) thingsToRepaint.get(i)).repaintThings();
        }
    }*/

    //public void show() {
    //    super.show();
        //repaintThings();
    //}
}
