package rwd.gui.editor;
import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;

import rwd.concretesyntax.*;
import rwd.concretesyntax.constraintdiagram.*;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderLeg;
import rwd.gui.editor.listeners.DrawNewArcListener;
import rwd.gui.editor.listeners.AutoSpiderListener;
import rwd.gui.editor.listeners.FormatBackColorListener;
import rwd.gui.editor.listeners.DrawNewBoxListener;
import rwd.gui.editor.listeners.DrawNewContourListener;
import rwd.gui.editor.listeners.EditDeleteListener;
import rwd.gui.editor.listeners.DrawNewFootListener;
import rwd.gui.editor.listeners.FormatBezierListener;
import rwd.gui.editor.listeners.FormatEllipseListener;
import rwd.gui.editor.listeners.FormatForeColorListener;
import rwd.gui.editor.listeners.FormatSetLabelListener;
import rwd.gui.editor.listeners.DrawNewLegListener;
import rwd.gui.editor.listeners.LayoutMotionListener;
import rwd.gui.editor.listeners.PlaceFeetListener;
import rwd.gui.editor.listeners.EditSelectListener;
import rwd.gui.editor.listeners.SmoothListener;
import rwd.gui.editor.listeners.DrawNewTextListener;
import rwd.gui.editor.listeners.FormatThicknessListener;
import rwd.gui.editor.listeners.FormatTypeListener;
import rwd.gui.editor.listeners.EditZoomListener;
import rwd.gui.editor.viewers.*;
/**
 * 
 * @author RWD, Richard Bosworth
 * 
 * @version 13-Apr-2005 14:46:01
 */
public class ControlPanel 
	extends JToolBar 
	implements ActionListener 
{
	public final static String SELECT = "Select";
	public final static String BOX = "Box";
	public final static String CONTOUR = "Contour";
	public final static String FOOT = "Foot";
	public final static String LEG = "Leg";
	public final static String TIE = "Tie";
	public final static String STRAND = "Strand";
	public final static String USPIDER = "USpider";
	public final static String SSPIDER = "SSpider";
	public final static String ARC = "Arc";
	public final static String LINE = "Line";
	public final static String TEXT = "TBox";
	
	public final static String FCOLOR = "FColor";
	public final static String BCOLOR = "BColor";
	public final static String DELETE = "Delete";
	public final static String TYPE = "Type";
	public final static String THICK = "Thick";
	public final static String LABEL = "Label";
	public final static String ZOOM_IN = "ZoomIn";
	public final static String ZOOM_OUT = "ZoomOut";
	public final static String BEZIER = "Bezier";
	public final static String ELLIPSE = "Ellipse";
	public final static String PLACE_FEET = "Feet";
	public final static String AUTO = "Auto";
	public final static String SMOOTH = "Smooth";
	public final static String RANDOM = "Random";
	public final static String MOTION = "Motion";
	
	public static ImageIcon makeImageIcon(String gifLocation, String actionName) {
		java.net.URL imgURL = ControlPanel.class.getResource(gifLocation);
			if (imgURL != null) {
				return new ImageIcon(imgURL, actionName);
			} else {
				return null;
			}
//		ImageIcon icon = new ImageIcon(gifLocation, actionName);
//		return icon;		
	}
	
    public ControlPanel(DiagramViewerInterface v) {
        this.viewer = v;

		System.out.println("new ControlPanel for viewer: "+ viewer);
        System.out.println("isRequestFocusEnabled()="+isRequestFocusEnabled());
		this.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				System.out.println("ControlPanel for viewer " + viewer + " gained focus");
			}
		});

		button = makeButton(
			"images/buttons/select.gif", SELECT, 
			"Select an item", new EditSelectListener(viewer));
		add(button);
		defaultButton = button;

		button = makeButton(
			"images/buttons/newbox.gif", BOX, 
			"Create a new box decoration", new DrawNewBoxListener(viewer));
		add(button);

        button = makeButton(
			"images/buttons/newcontour.gif", CONTOUR, 
			"Create a new contour", new DrawNewContourListener(viewer));
        add(button);

		button = makeButton(
			"images/buttons/foot.gif", FOOT, 
			"Create a new normal foot", new DrawNewFootListener(viewer, ConcreteSpiderFoot.NORMAL));
		add(button);

		button = makeButton(
			"images/buttons/univspider.gif", USPIDER, 
			"Create a new universal spider", new DrawNewFootListener(viewer, ConcreteSpiderFoot.UNIVERSAL));
		add(button);

		button = makeButton(
			"images/buttons/schspider.gif", SSPIDER, 
			"Create a new Schrodinger spider", new DrawNewFootListener(viewer, ConcreteSpiderFoot.SCHROEDINGER));
		add(button);

		button = makeButton(
			"images/buttons/leg.gif", LEG, 
			"Create a new normal leg", new DrawNewLegListener(viewer, ConcreteSpiderLeg.NORMAL));
		add(button);

		button = makeButton(
			"images/buttons/tie.gif", TIE, 
			"Create a new tie", new DrawNewLegListener(viewer, ConcreteSpiderLeg.TIE));
		add(button);

		button = makeButton(
			"images/buttons/strand.gif", STRAND, 
			"Create a new strand", new DrawNewLegListener(viewer, ConcreteSpiderLeg.STRAND));
		add(button);

		button = makeButton(
			"images/buttons/newarc.gif", ARC, 
			"Create a new arrow", new DrawNewArcListener(viewer, ConcreteArc.QUAD));
		add(button);

		button = makeButton(
			"images/buttons/newline.gif", LINE,
			"Create a new arrow", new DrawNewArcListener(viewer, ConcreteArc.LINE));
		add(button);

		button = makeButton(
			"images/buttons/text.gif", TEXT,
			"Create a new comment", new DrawNewTextListener(viewer, ConcreteTextBox.NORMAL));
		add(button);

        button = makeButton(
			"images/buttons/fgcolor.gif", FCOLOR, 
			"Change foreground color", new FormatForeColorListener(viewer));
        add(button);
        
        button = makeButton(
			"images/buttons/bgcolor.gif", BCOLOR, 
			"Change background color", new FormatBackColorListener(viewer));
        add(button);

        button = makeButton(
			"images/buttons/Delete24.gif", DELETE,
			"Delete Item", new EditDeleteListener(viewer));
        add(button);

        button = makeButton(
			"images/buttons/type.gif", TYPE,
			"Change type", new FormatTypeListener(viewer));
        add(button);

		button = makeButton(
			"images/buttons/thickness.gif", THICK,
			"Select Thickness", new FormatThicknessListener(viewer));
        add(button);

        button = makeButton(
        	"images/buttons/label.gif", LABEL,
			"Edit label", new FormatSetLabelListener(viewer));
        add(button);

        button = makeButton(
        	"images/buttons/ZoomIn24.gif", ZOOM_IN,
			"Zoom in", new EditZoomListener(viewer, EditZoomListener.ZOOM_IN, -1));
        add(button);

		button = makeButton(
			"images/buttons/ZoomOut24.gif", ZOOM_OUT,
			"Zoom out", new EditZoomListener(viewer, EditZoomListener.ZOOM_OUT, 1));
        add(button);

		button = makeButton(
			"images/buttons/bezier.gif", BEZIER,
			"Smooth contours using Bezier curves", new FormatBezierListener(viewer));
	    add(button);

		button = makeButton(
			"images/buttons/ellipse.gif", ELLIPSE,
			"Draw contours as ellipses", new FormatEllipseListener(viewer));
	    add(button);

//        button = makeButton(
//        	"images/buttons/placefeet.gif", PLACE_FEET,
//			"Perform initial embedding of spider feet", new PlaceFeetListener(viewer));
//        add(button);
//
//        button = makeButton(
//        	"images/buttons/autospider.gif", AUTO,
//			"Auto-embed spider feet", new AutoSpiderListener(viewer));
//        add(button);
//
//        button = makeButton(
//        	"images/buttons/smooth.gif", SMOOTH,
//			"Adjust drawing parameters", new SmoothListener(viewer));
//        add(button);
//
//        button = makeButton(
//			"images/buttons/random.gif", RANDOM,
//			"Add a random spider", new RandomSpiderListener(viewer));
//        add(button);
//
//		button = makeButton(
//			"images/buttons/wobble.gif", MOTION,
//			"Toggle Movement", new LayoutMotionListener(viewer));
//		add(button);

		((JToolBar)this).setOpaque(true);
        enableButtons(null);
        
        setDefaultButton();
    }

	protected JToggleButton makeButton(String gifLocation, String actionName, String tip, ActionListener action) {
		JToggleButton button = new JToggleButton();
		button.setActionCommand(actionName);
		button.setToolTipText(tip);
		button.addActionListener(this);
		ImageIcon icon = makeImageIcon(gifLocation, actionName);
		button.setBackground(Color.LIGHT_GRAY);
		if (icon != null) {
			button.setIcon(icon);
		} else {
			button.setText(actionName);
			System.err.println("Resource not found: "+ gifLocation);
		}

		buttonGroup.add(button);
		cmd2button.put(actionName, button);

		return button;
	}
	
	public void actionPerformed(ActionEvent e) {
		// Change fore color
		cmd = e.getActionCommand();
		if (cmd == ControlPanel.FCOLOR) {
			(new FormatForeColorListener(viewer)).setForeColor();
			// Reset default button
			setDefaultButton();
		// Change back color
		} else if (cmd == ControlPanel.BCOLOR) {
			(new FormatBackColorListener(viewer)).setBackColor();
			// Reset default button
			setDefaultButton();
		// Delete
		} else if (cmd == ControlPanel.DELETE) {
			(new EditDeleteListener(viewer)).deleteItem();
			// Reset default button
			setDefaultButton();
		// Change type
		} else if (cmd == ControlPanel.TYPE) {
			(new FormatTypeListener(viewer)).changeType();
			// Reset default button
			setDefaultButton();
		// Thick
		} else if (cmd == ControlPanel.THICK) {
			(new FormatThicknessListener(viewer)).setThickness();
			// Reset default button
			setDefaultButton();
		// Set label
		} else if (cmd == ControlPanel.LABEL) {
			(new FormatSetLabelListener(viewer)).setLabel();
			// Reset default button
			setDefaultButton();
		// Zoom in
		} else if (cmd == ControlPanel.ZOOM_IN) {
			(new EditZoomListener(viewer,EditZoomListener.ZOOM_IN, -1)).zoom();
			// Reset default button
			setDefaultButton();
		// Zoom out
		} else if (cmd == ControlPanel.ZOOM_OUT) {
			(new EditZoomListener(viewer, EditZoomListener.ZOOM_OUT, 1)).zoom();
			// Reset default button
			setDefaultButton();
		// Bezier
		} else if (cmd == ControlPanel.BEZIER) {
			(new FormatBezierListener(viewer)).smooth();
			// Reset default button
			setDefaultButton();
		// Ellipse
		} else if (cmd == ControlPanel.ELLIPSE) {
			(new FormatEllipseListener(viewer)).smooth();
			// Reset default button
			setDefaultButton();
		// Place feet
		} else if (cmd == ControlPanel.PLACE_FEET) {
			(new PlaceFeetListener(viewer)).placeFeet();
			// Reset default button
			setDefaultButton();
		// Auto spider
		} else if (cmd == ControlPanel.AUTO) {
			(new AutoSpiderListener(viewer)).auto();
			// Reset default button
			setDefaultButton();
		// Smooth
		} else if (cmd == ControlPanel.SMOOTH) {
			(new SmoothListener(viewer)).smooth();
			// Reset default button
			setDefaultButton();
		// Movement
		} else if (cmd == ControlPanel.MOTION) {
			(new LayoutMotionListener(viewer)).toggleMotion();
			// Reset default button
			setDefaultButton();
		// Random spider
		} //else if (cmd == ControlPanel.RANDOM) {
			//(new RandomSpiderListener(viewer)).random();
			// Reset default button
			// setDefaultButton();
		//}
	}
	
    public void enableButtons(Collection diagramItems) {
    	if (diagramItems != null && diagramItems.size() == 1) {
    		DiagramItem item = (DiagramItem)diagramItems.iterator().next();
			if (item != null) {
				setEnabled("FColor", true);
				setEnabled("BColor", true);
				setEnabled("Thick", true);
				setEnabled("Label", true);
				setText("Thick", String.valueOf(item.getThickness()));
			} else {
				setEnabled("FColor", false);
				setEnabled("BColor", false);
				setEnabled("Thick", false);
				setEnabled("Label", false);
			}
    	}
    }

	public void setDefaultButton() {
		defaultButton.setSelected(true);
		cmd = SELECT;
	}
    
    /** Get current command */
	public String getCmd() {
		return cmd;
	}

    protected void setEnabled(String cmd, boolean flag) {
		button = (JToggleButton)cmd2button.get(cmd);
		if (button != null) {
			button.setEnabled(flag);
		}	
    }

	protected void setText(String cmd, String text) {
		button = (JToggleButton)cmd2button.get(cmd);
		if (button != null) {
			button.setText(text);
		}
	}

	// Local properties
	protected String cmd = "";
	protected JToggleButton defaultButton;
	protected ButtonGroup buttonGroup = new ButtonGroup();
	protected JToggleButton button;
	protected Map cmd2button = new HashMap();
	 
    private DiagramViewerInterface viewer;
}
