package rwd.gui.editor.listeners;

import java.awt.Polygon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFileChooser;

import rwd.concretesyntax.eulerdiagram.DecorationBox;
import rwd.gui.editor.DiagramCanvas;
import rwd.gui.editor.EditorFrame;
import rwd.gui.editor.dialogs.DecorationBoxDialog;
import rwd.gui.editor.viewers.DiagramViewerInterface;

/**
 * @author Octavian Patrascoiu
 *
 */
public class DrawNewBoxListener implements ActionListener {
	public DrawNewBoxListener(EditorFrame frame) {
		this.frame = frame;
	}
	
	public DrawNewBoxListener(DiagramViewerInterface viewer) {
		this.viewer = viewer;
	}

	public void actionPerformed(ActionEvent e) {
		if (frame != null) viewer = frame.getCurrentViewer();
		if (viewer == null) return;
		DiagramCanvas dc = viewer.getDiagramCanvas();
		if (dc == null) return;
		int x = (int)dc.getCurrentX();
		int y = (int)dc.getCurrentY();
		addNewBox(x, y);
	}
	
	public void addNewBox(int x1, int y1) {
		addNewBox(x1, y1, x1+3500, y1+2500);
	}

	public void addNewBox(int x1, int y1, int x2, int y2) {
		if (frame != null) viewer = frame.getCurrentViewer();
		if (viewer == null) return;
//		String label = JOptionPane.showInputDialog("Box label");
//		if (label == null || label.length() == 0) {
//			return;
//		}
//		Polygon polygon =
//			new Polygon(
//				new int[] { x1, x2, x2, x1 },
//				new int[] { y1, y1, y2, y2 },
//				4);
//		DecorationBox box = new DecorationBox(label, polygon);
//		viewer.add(box);

		// Create a dialog to input text
		DecorationBoxDialog dialog = new DecorationBoxDialog(frame, "Decoration");
		int returnVal = dialog.showOpenDialog(frame);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			String label = dialog.getLabel();
			String readingTree = dialog.getReadingForest();
			// Create the decoration box
			Polygon polygon = new Polygon(
				new int[] { x1, x2, x2, x1 },
				new int[] { y1, y1, y2, y2 },
				4);
			DecorationBox box = new DecorationBox(label, readingTree, polygon);
			viewer.add(box);
				
			// Repaint image
			viewer.redrawImage();
		}		

		viewer.redrawImage();
	}

	// Local variables
	private EditorFrame frame;
	private DiagramViewerInterface viewer;
}
