package rwd.gui.editor.listeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JOptionPane;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.gui.editor.EditorFrame;
import rwd.views.abstracttoconcrete.converting.Radialiser;

/**
 * @author Stirling Chow
 *
 */

// This code is adapted from the class GeneratorCirculiserListener to
// call the UVic radialiser.  GeneratorCirculiserListener uses
// EditorFrame.showAbstractDiagram which eventually passes control to
// ResultDirector within which the concrete diagram is created.  This class
// is unable to use this method because there is presently no way to
// tell ResultDirector which generator (Circuliser or Radialiser) to use.
// To maintain consistency, perhaps in the future a field should be added
// to the abstract diagram giving a "hint" as to which abstract-to-concrete
// algorithm should be used; in this way, all the error handling and generic
// concrete operations (e.g., placing the decorating box) can be centralised
// in the ResultDirector.
//
// In lieue of the above, this class performs the abstract-to-concrete
// translation internally and presents the result using EditorFrame.addViewer.
// A downside of this is that the code that parses the input string is
// duplicated in both GeneratorCirculiserListener and this class.
//
// NOTE: this class does not do any error checking on the input.  The UVic
// Radialiser *may* fail on certain inputs (i.e., those without the fullset).
// It is assumed that the radialiser algorithm is only called on valid input.

public class GeneratorRadialiserListener implements ActionListener {
  public GeneratorRadialiserListener(EditorFrame frame) {
    this.frame = frame;
  }

  public void actionPerformed(ActionEvent e) {
    /*** begin - duplicated from GeneratorCirculiserListener */

    String inputString =
      JOptionPane.showInputDialog(
        frame,
        "Please enter zone list",
        "a b c ab ac bc abc");

    if (inputString != null) {
      // This code to generate the AEDView is borrowed from another
      // class of Jean's - pjm2.
      Vector allContours = new Vector();
      letterLoop : for (
        int i = 0; i < inputString.length(); i++) {
        String letter = inputString.substring(i, i + 1);
        if (letter.equals(" ")) {
          continue letterLoop;
        } else {
          Contour c = Contour.getContour(letter);
          if (!allContours.contains(c)) {
            allContours.add(c);
          }
        }
      }

      int charPosition = 0;
      Vector zones = new Vector();
      zones.add(Zone.getZone(new Vector(), allContours));

      zoneLoop : while (charPosition < inputString.length()) {
        Vector containingContours = new Vector();
        contourLoop : while (
          charPosition < inputString.length()) {
          // extract character from inputString
          char ch = inputString.charAt(charPosition);
          charPosition++;
          if (ch != ' ') {
            containingContours.add(
              Contour.getContour("" + ch));
          } else {
            break contourLoop;
          }
        } // finished adding contours
        Vector excludingContours =
          (Vector) allContours.clone();
        excludingContours.removeAll(containingContours);
        zones.add(
          Zone.getZone(
            containingContours,
            excludingContours));
      } // finished adding zones

      // Create an abstract diagram.
      UnitaryDiagram ud = new UnitaryDiagram(zones, "ud");
      frame.getLog().reportMessage(ud.toDebugString());

    /*** end - duplicated from GeneratorCirculiserListener */

      // Create a concrete diagram from the abstract diagram
      // using the UVic radialiser algorithm.
      Radialiser radialiser = new Radialiser(ud);

      // Show the concrete diagram in the editor.
      if (frame.addViewer("Radialiser result", radialiser.getCUD()) ==  null) {
        JOptionPane.showMessageDialog(frame, "Can't be drawn");
      }
    }
  }

  // Local variables
  protected EditorFrame frame;
}
