package rwd.gui.editor.listeners;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JFileChooser;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.gui.editor.EditorFrame;
import rwd.gui.editor.dialogs.CheckReadingTreeDialog;
import rwd.gui.editor.viewers.DiagramViewerInterface;
import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class ReasoningCheckReadingTreeListener implements ActionListener {
	public ReasoningCheckReadingTreeListener(EditorFrame frame) {
		this.frame = frame;
	}
	
	public ReasoningCheckReadingTreeListener(DiagramViewerInterface viewer) {
		this.viewer = viewer;
	}

	public void actionPerformed(ActionEvent e) {
		if (frame != null) viewer = frame.getCurrentViewer();
		if (viewer == null) return;
		checkReadingTree();
	}
	
	public void checkReadingTree() {
		if (frame != null) viewer = frame.getCurrentViewer();
		if (viewer == null) return;
		ILog log = frame.getLog();

		// Create a dialog to input text
		CheckReadingTreeDialog dialog = new CheckReadingTreeDialog(frame, "Check Reading Tree");
		int returnVal = dialog.showOpenDialog(frame);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			String label = dialog.getLabel();
			if (label != null && label.length() != 0) {
				ConcreteUnitaryDiagram cud = viewer.getCUDCalled(label);
				boolean flag = cud.isValidTree(log);
				if (log != null) {
					if (flag)
						log.reportMessage("Reading tree is OK");
					else
						log.reportMessage("Wrong reading tree");
				}
			}
		}		

		viewer.redrawImage();
	}

	// Local variables
	private EditorFrame frame;
	private DiagramViewerInterface viewer;
}
