package rwd.gui.editor.listeners.diagramcontrollers;

import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;

import rwd.gui.editor.DiagramCanvas;

/**
 * The interface for listeners which interpret user gestures on diagrams.
 * These listeners are implemented using the state pattern.
 * Classes which implement this interface represent the Context.
 * States are represented by classes which implement the DiagramControllerState
 * interface.
 * 
 * @author Richard Bosworth
 * 
 * @version 18-Apr-2005 20:57
 */
public interface DiagramController 
	extends MouseMotionListener, MouseListener, KeyListener
{
	// attributes
	// ----------
	
	/**
	 * @return DiagramCanvas controlled by this controller
	 */
	public DiagramCanvas getDiagramCanvas();
	
	
	/**
	 * @return current state of this controller
	 */
	public DiagramControllerState getState();
	
	
	// operations
	// ----------
	
	/**
	 * Post: controller has been set to required state.
	 * @param dcs required state of this controller (must not be null)
	 */
	public void setState(DiagramControllerState dcs);
	
	/**
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e);
	
	/**
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e); 

	/**
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e); 

	/**
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e);
	
	/**
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e);
	
	/**
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 */
	public void mouseDragged(MouseEvent e); 
	
	/**
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent e);

	/**
	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
	 */
	public void keyTyped(KeyEvent e);
	
	/**
	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
	 */
	public void keyPressed(KeyEvent e);
	
	/**
	 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
	 */
	public void keyReleased(KeyEvent e);
	
}
