package rwd.gui.editor.viewers;

import java.awt.Color;
import java.awt.Font;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.Shape;
import java.awt.geom.*;

import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import java.util.TreeSet;
import java.util.Vector;


import javax.swing.JComponent;

import rwd.concretesyntax.*;
import rwd.concretesyntax.Observable;
import rwd.concretesyntax.Observer;
import rwd.concretesyntax.eulerdiagram.*;
import rwd.concretesyntax.spiderdiagram.*;
import rwd.concretesyntax.constraintdiagram.*;

import rwd.abstractsyntax.eulerdiagram.*;
import rwd.abstractsyntax.spiderdiagram.*;

import rwd.gui.editor.*;
import rwd.ocl.constraintdiagram.ReadingTreeString;

/**
 * Note this is a default, rather than an abstract, class.
 * It contains default implementations of the methods of DiagramViewerInterface.
 *  
 * @author Octavian Patrascoiu, rnb
 * @version 12.8.05  13:28
 *
 */
class AbstractDiagramViewer
    extends JComponent
    implements DiagramViewerInterface, Observer 
{    	
    //
    // Constructors
    //
	
    public AbstractDiagramViewer() {
    	strId = ""+id++;
    	store();
    }
    
	public AbstractDiagramViewer(ConcreteDiagram cd) {
		_diagram = cd;
		strId = ""+id++;
		if (_diagram != null) {
			_diagram.addObserver(this);
		}
		store();
		
	}
	
	public void store() {
		_history.store(this);    
	}
	
	
	/* 
	 * Get a given ConcreteUnitaryDiagram 
	 */
	public ConcreteUnitaryDiagram getCUDCalled(String s) {
		ConcreteDiagram cd = getConcreteDiagram();
		// Get the unitary diagrams
		Set unitaryDiags = cd.getConcreteUnitaryDiagrams();
		Iterator it = unitaryDiags.iterator();
		while (it.hasNext()) {
			ConcreteUnitaryDiagram cud = (ConcreteUnitaryDiagram) it.next();
			if (cud.getDecorationBox().getLabel().equals(s)) {
				return cud;
			}
		}
		return null;
	}
	
	/**
	 * 
	 * @param x x-coord of point in diagram
	 * @param y y-coord of point in diagram
	 * @return decoration box containing point (x,y) or null if none
	 */
	/*
	 * test: 
	 *   no boxes in diagram
	 *   boxes in diagram, but point outside all
	 *   point is in one box
	 * 
	 */
	public DecorationBox getBoxContaining(int x, int y) {
		
		// get set of boxes (assumed non-overlapping)
		DecorationSet ds = _diagram.getDecorationSet();
		List bl = ds.getBoxes();
		
		// find box which contains (x,y)
		Iterator it = bl.iterator();
		while (it.hasNext()) {
			DecorationBox db = (DecorationBox) it.next();
			Polygon p = db.getPolygon();
			if (p.contains(x,y)) return db;
		}
		
		return null;
	}

    //
    // Drawing methods
    //
    public void draw(ConcreteDiagram cd) {
        _diagram = cd;
        if (_diagram != null) {
			_diagram.addObserver(this);
        }
        _canvas.drawImageNow();
    }

	// Repaint a image
	public void redrawImage() {
		_canvas.drawImageNow();
	}

	public void drawAgain() {
//		repaintImage();
		redrawImage();
	}
	
	// Adds a unique label to the diagram.
	public void drawString(String label, float x, float y) {
		_canvas.drawString(label, x, y);
	}

    //
    // Copy, Cut, Paste, and Delete
    //
    public void copySelectedItems() {
    	_clipboard.addAll(getDiagramCanvas().getSelectedItems());
    }

    public void cutSelectedItems() {
        copySelectedItems();
        deleteSelectedItems();
    }

	public void pasteSelectedItems() {
		// Paste selected items
		Iterator i = _clipboard.iterator();
		while (i.hasNext()) {
			DiagramItem item = (DiagramItem)i.next();
			paste(item);
		}
		_diagram.notifyObservers();
	}
    protected void paste(DiagramItem _clipboard) {
        if (_clipboard instanceof DecorationBox) {
            // Get old properties
            DecorationBox oldBox = (DecorationBox) _clipboard;
            String oldLabel = oldBox.getLabel();
            Polygon oldPoly = oldBox.getPolygon();
            // Compute new properties: label and polygon
            String newLabel = newName(oldBox.getLabel());
            int newX = oldPoly.xpoints[0]+DX;
			int newY = oldPoly.ypoints[0]+DY;
            int dy = oldPoly.getBounds().height;
            int dx = oldPoly.getBounds().width;
            Polygon newPoly =
                new Polygon(
                    new int[] { newX, newX + dx, newX + dx, newX },
                    new int[] { newY, newY, newY + dy, newY + dy },
                    4);
            // Add new box
            DecorationBox newBox = new DecorationBox(newLabel, "", newPoly);
            add(newBox);
        } else if (_clipboard instanceof ConcreteContour) {
            // Get old properties
            ConcreteContour oldContourPoly = (ConcreteContour) _clipboard;
            Polygon oldPoly = oldContourPoly.getPolygon();
            // Compute new properties: label and polygon
            String newLabel = newName(oldContourPoly.getLabel());
            Contour newContour = Contour.getContour(newLabel);
            int npoints = oldPoly.npoints;
            int xpoints[] = new int[npoints];
            int ypoints[] = new int[npoints];
            for (int i = 0; i < npoints; i++) {
                xpoints[i] = oldPoly.xpoints[i] + DX;
                ypoints[i] = oldPoly.ypoints[i] + DX;
            }
            Polygon newPoly = new Polygon(xpoints, ypoints, npoints);
            // Create and add new contour
            ConcreteContour newContourPoly =
                new ConcreteContour(newContour, newPoly);
            add(newContourPoly);
        } else if (_clipboard instanceof ConcreteSpiderLeg) {
            // Get old properties
            ConcreteSpiderLeg oldLeg = (ConcreteSpiderLeg) _clipboard;
            ConcreteSpiderFoot oldSource = oldLeg.getSource();
            ConcreteSpiderFoot oldTarget = oldLeg.getTarget();
            int oldType = oldLeg.getType();
            // Compute new properties: label and polygon
            int newType = oldType;
            ConcreteSpiderFoot newSource =
                new ConcreteSpiderFoot("", oldSource.getType());
            newSource.setLocation(oldSource.getX()+DX, oldSource.getY()+DY);
            ConcreteSpiderFoot newTarget =
                new ConcreteSpiderFoot("", oldTarget.getType());
            newTarget.setLocation(oldTarget.getX()+DX, oldTarget.getY()+DY);
            // Create and add new foot
            ConcreteSpiderLeg newLeg =
                new ConcreteSpiderLeg("", newSource, newTarget, newType);
            add(newLeg);
        } else if (_clipboard instanceof ConcreteSpiderFoot) {
            // Get old properties
            ConcreteSpiderFoot oldFoot = (ConcreteSpiderFoot) _clipboard;
            int oldType = oldFoot.getType();
            // Compute new properties: label and plygon
            int newType = oldType;
            int newX = oldFoot.getX()+DX;
            int newY = oldFoot.getY()+DY;
            // Create and add new foot
            ConcreteSpiderFoot newFoot = new ConcreteSpiderFoot("", newType);
            newFoot.setLocation(newX, newY);
            add(newFoot);
        } else if (_clipboard instanceof ConcreteArc) {
            // Get old properties
            ConcreteArc oldArc = (ConcreteArc) _clipboard;
            int oldType = oldArc.getType();
            ConcreteArcEnd oldStart = oldArc.getStart();
            ConcreteArcEnd oldEnd = oldArc.getEnd();
            // Compute new properties: label and plygon
            int newType = oldType;
            ConcreteArcEnd newStart = new ConcreteArcEnd(oldStart.getX()+DX, oldStart.getY()+DY);
            ConcreteArcEnd newEnd = new ConcreteArcEnd(oldEnd.getX()+DX, oldEnd.getY()+DY);
            // Create and add new arc
            ConcreteArc newArc = new ConcreteArc(newStart, newEnd, newType);
            newArc.attachEnds(this);
            add(newArc);
        } else if (_clipboard instanceof ConcreteTextBox) {
            // Get old properties
            ConcreteTextBox oldTextBox = (ConcreteTextBox) _clipboard;
            int oldType = oldTextBox.getBoxType();
            String oldText = oldTextBox.getText();
            // Create and add new foot
            ConcreteTextBox newTextBox =
                new ConcreteTextBox(oldType, oldTextBox.getX()+DX, oldTextBox.getY()+DY, oldText);
            add(newTextBox);
        }
    }
	protected long _staticNo = 0;
	protected String newName(String name) {
		return name + "_" + _staticNo++;
	}

    public void deleteSelectedItems() {
        // Remove selected items
		Iterator i = getDiagramCanvas().getSelectedItems().iterator();
		while (i.hasNext()) {
			DiagramItem item = (DiagramItem)i.next();
			
			// remove item
			removeWithoutHistory(item);
		}
		// Deselect them
        getDiagramCanvas().deselect();
        // Notify observers
		_diagram.notifyObservers();
		// Repaint it
		this.repaint();
        // Store state in history
        _history.store(this);
    }

    //
    // Undo and Redo methods
    //
    public void undo() {
        _history.undo(this);
    }

    public void redo() {
        _history.redo(this);
    }

    //
    // Format methods
    //
    public void changeType() { 
		// Remove selected items
		Collection selectedItems = getDiagramCanvas().getSelectedItems();
		if (selectedItems == null)
			return;
		Iterator i = selectedItems.iterator();
		while (i.hasNext()) {
			DiagramItem item = (DiagramItem)i.next();
			changeType(item);
		}

		// Store state in history
		_history.store(this);        
    }
    protected void changeType(DiagramItem selectedItem) {
        // Change type
        if (selectedItem == null)
            return;
        if (selectedItem instanceof Handle) {
        	selectedItem = ((Handle)selectedItem).getItem();
        }
        if (selectedItem instanceof ConcreteSpiderFoot) {
            ConcreteSpiderFoot foot = (ConcreteSpiderFoot) selectedItem;
            if (foot.getType() == ConcreteSpiderFoot.SCHROEDINGER) {
                foot.setType(ConcreteSpiderFoot.NORMAL);
            } else {
                foot.setType(foot.getType() + 1);
            }
        } else if (selectedItem instanceof ConcreteSpiderLeg) {
            ConcreteSpiderLeg leg = (ConcreteSpiderLeg) selectedItem;
            if (leg.getType() == ConcreteSpiderLeg.TIE) {
                leg.setType(ConcreteSpiderLeg.NORMAL);
            } else {
                leg.setType(leg.getType() + 1);
            }
        } else if (selectedItem instanceof ConcreteArc) {
            ConcreteArc arc = (ConcreteArc) selectedItem;
            if (arc.getType() == ConcreteArc.QUAD) {
                arc.setType(ConcreteArc.LINE);
            } else {
                arc.setType(arc.getType() + 1);
            }
        } else if (selectedItem instanceof ConcreteTextBox) {
            ConcreteTextBox textBox = (ConcreteTextBox) selectedItem;
            if (textBox.getBoxType() == ConcreteTextBox.BORDER) {
                textBox.setBoxType(ConcreteTextBox.NORMAL);
            } else {
                textBox.setBoxType(textBox.getBoxType() + 1);
            }
        }
//        getDiagramCanvas().selectItem(selectedItem);

        redrawImage();
        repaint();
    }

    //
    // Add/remove a diagram item
    //
    public void add(DiagramItem item) {
        // Add item
        addWithoutHistory(item);

		// Store state in history
		_history.store(this);
    }
	public void addWithoutHistory(DiagramItem item) {
		// Add item
		if (item instanceof DecorationBox) {
			_diagram.addDecorationBox((DecorationBox)item);
		} else if (item instanceof ConcreteContour) {
			addConcreteContour((ConcreteContour) item);
		} else if (item instanceof ConcreteSpiderLeg) {
			_diagram.addConcreteLeg((ConcreteSpiderLeg)item);
		} else if (item instanceof ConcreteSpiderFoot) {
			_diagram.addConcreteFoot((ConcreteSpiderFoot)item);
		} else if (item instanceof ConcreteArc) {
			_diagram.addConcreteArc((ConcreteArc)item);
		} else if (item instanceof ConcreteTextBox) {
			_diagram.addTextBox((ConcreteTextBox)item);
		}
		_diagram.notifyObservers();
	}
    public void addSpider(Vector zoneAreas, Spider spider) {
        // Add spider
        _diagram.addAbstractSpider(spider);

		// Store state in history
		_history.store(this);
    }

    public void remove(DiagramItem item) {
		removeWithoutHistory(item);
		
		// Store state in history
		_history.store(this);
    }
    
    /**
     * Post: Item is not present in diagram.
     *       If item was concrete spider foot:
     *         any connected leg has been removed,
     *         the reading tree has been reconciled,
     *         if the foot's spider has only one label, 
     *           the label has been moved to an unlabelled foot, if any.
     *  
     * @param item item to be removed from diagram
     */
    /*
     * tests:
     *   item is decoration box
     *   item is concrete contour
     *   item is concrete spider leg
     *   item is concrete spider foot
     *     foot inside decoration box with valid reading tree
     *       foot is unlabelled
     *       foot is labelled
     *         spider with four feet (two labelled) and three legs
     *         spider with three feet (one labelled) and two legs
     *     foot is labelled and outside of any decoration box
     *   item is concrete arc
     *   item is concrete text box
     */
    protected void removeWithoutHistory(DiagramItem item) {
		// Delete element
		if (item instanceof DecorationBox) {
			_diagram.removeDecorationBox((DecorationBox) item);
		} 
		else if (item instanceof ConcreteContour) {
			_diagram.removeConcreteContour((ConcreteContour) item);
		} 
		else if (item instanceof ConcreteSpiderLeg) {
			_diagram.removeConcreteLeg((ConcreteSpiderLeg)item);
		} 
		else if (item instanceof ConcreteSpiderFoot) {
			ConcreteSpiderFoot csf = (ConcreteSpiderFoot) item;
			String oldlabel = csf.getLabel();			
			boolean spiderLabelDeleted = false;
			
			// preserve label if necessary and possible
			if (oldlabel.length() != 0) {
				spiderLabelDeleted = true;
				ConcreteSpider cs = _diagram.getConcreteSpider(csf);
				if (cs.getNumberOfLabels() == 1) {
					Iterator fit = cs.getFeet().iterator();
					while (fit.hasNext()) {
						ConcreteSpiderFoot f = (ConcreteSpiderFoot) fit.next();
						if (f.getLabel().length()==0) {
							f.setLabel(oldlabel);
							spiderLabelDeleted = false;
							break;
						}
					}
				}
			}
			
			if (spiderLabelDeleted) {
				// reconcile reading tree
				DecorationBox db = getBoxContaining(csf.getX(),csf.getY());
				if (db != null) {
					ReadingTreeString rts = 
						new ReadingTreeString(db.getReadingForest());
					rts.update(oldlabel,"");
					db.setReadingForest(rts.toString());
				}
			}
			// remove foot
			_diagram.removeConcreteFoot(csf);
			
			// remove any leg connected to it
			Iterator legIt = _diagram.getFeetAndLegsSet().getLegs().iterator();
			while (legIt.hasNext()) {
				ConcreteSpiderLeg leg = (ConcreteSpiderLeg) legIt.next();
				if (leg.getSource() == csf || leg.getTarget() == csf) {
					legIt.remove();
					_diagram.removeConcreteLeg(leg);
				}
			}
		} 
		else if (item instanceof ConcreteArc) {
			_diagram.removeConcreteArc((ConcreteArc)item);
		} 
		else if (item instanceof ConcreteTextBox) {
			_diagram.removeTextBox((ConcreteTextBox)item);
		}
    }

    //
    // Add/remove contour
    //
    public Vector getAllConcreteZones() {
        return new Vector(_diagram.getConcreteZones());
    }
    
    public void addConcreteContour(ConcreteContour cp) {
        _diagram.addConcreteContour(cp);
    }

    public void removeConcreteContour(ConcreteContour cp) {
        _diagram.removeConcreteContour(cp);
    }

    public Vector getVisibleConcreteZones() {
        return new Vector(_diagram.getVisibleConcreteZones());
    } 

    public DiagramItem getNearestItem(float x, float y) {
        // Start by working out whether the user selected an item...
        // make this larger to make contour selection easier
        int rectSize = DiagramCanvas.SELECTION_SIZE;
        Rectangle2D rect =
            new Rectangle2D.Float(
                x - rectSize,
                y - rectSize,
                rectSize * 2,
                rectSize * 2);

		// Look for spider feet
		Vector feet = new Vector(_diagram.getFeetAndLegsSet().getFeet());
		for (int i = 0; i < feet.size(); i++) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot) feet.get(i);
			Shape shape = foot.getShape();
			if (shape.intersects(rect)) {
				return foot;
			}
		}
		// Look for contours - derived first
		// Make a list and sort them according to the area - containment
		java.util.List contours = new Vector(_diagram.getConcreteContours());
		contours = sort(contours);
		for(int i=0; i<contours.size(); ++i) {
			ConcreteContour cp = (ConcreteContour) contours.get(i);
			Polygon p = cp.getPolygon();
			p.invalidate();
			if (p.contains(rect) || p.intersects(rect)) {
				return cp;
			}
		}
        return null;
    }
    protected java.util.List sort(java.util.List contours) {
    	if (contours == null) return new Vector();
    	boolean sorted = false;
    	while (!sorted) {
    		sorted = true;
    		for(int i=0; i<contours.size()-1; ++i) {
    			ConcreteContour c1 = (ConcreteContour)contours.get(i);
				ConcreteContour c2 = (ConcreteContour)contours.get(i+1);
				Rectangle r1 = c1.getPolygon().getBoundingBox();
				Rectangle r2 = c2.getPolygon().getBoundingBox();
				if (r1.contains(r2)) {
					contours.set(i, c2);
					contours.set(i+1, c1);
					sorted = false;
				}
    		}
    	}
    	return contours;
    }

    // Attach all the arcs to source and target
    public void attachArcs(boolean detachFirst) {
        Vector arcs = _diagram.getArcSet().getArcs();
        for (int i = 0; i < arcs.size(); i++) {
            ConcreteArc arc = (ConcreteArc) arcs.get(i);
            arc.attachEnds(this);
        }
    }

	// Update the location of all arcs if a source or target has moved
	public void updateArcsLocation() {
		Vector arcs = _diagram.getArcSet().getArcs();
		for (int i = 0; i < arcs.size(); i++) {
			ConcreteArc arc = (ConcreteArc) arcs.get(i);
			arc.updateLocation();
		}
	}

	// Attach an arc end
	public void attachArcEnd(ConcreteArcEnd arcEnd) {
		arcEnd.detach();
		DiagramItem item = getNearestItem(arcEnd.getX(), arcEnd.getY()); 
		arcEnd.attachItem(item);
	}
	
	public DecorationBox getBoxAround(Set polygonSet, double padding) {
		// work out a bounding box
		int leftMost = Integer.MAX_VALUE;
		int rightMost = Integer.MIN_VALUE;
		int upperMost = Integer.MIN_VALUE;
		int lowerMost = Integer.MAX_VALUE;
		Iterator i = polygonSet.iterator();
		while(i.hasNext()){
			ConcreteContour cp = (ConcreteContour) i.next();
			//System.out.println("assessing contour polygon " + cp);
			Polygon p = cp.getPolygon();
			p.invalidate();
			Rectangle rect = p.getBounds();
			int boxLeft = rect.x;
			int boxBottom = rect.y;
			int boxRight = boxLeft + rect.width;
			int boxTop = boxBottom + rect.height;
			//System.out.println("box left is " + boxLeft);
			//System.out.println("box right is " + boxRight);
			//System.out.println("box top is " + boxTop);
			//System.out.println("box bottom is " + boxBottom);
			if (leftMost > boxLeft) {
				leftMost = boxLeft;
			}
			if (lowerMost > boxBottom) {
				lowerMost = boxBottom;
			}
			if (rightMost < boxRight) {
				rightMost = boxRight;
			}
			if (upperMost < boxTop) {
				upperMost = boxTop;
			}
		}
		DecorationSet decorationSet = _diagram.getDecorationSet();
		i = decorationSet.getBoxes().iterator();
		while(i.hasNext()){
			DecorationBox b = (DecorationBox) i.next();
			//System.out.println("assessing contour polygon " + cp);
			Polygon p = b.getPolygon();
			p.invalidate();
			Rectangle rect = p.getBounds();
			int boxLeft = rect.x;
			int boxBottom = rect.y;
			int boxRight = boxLeft + rect.width;
			int boxTop = boxBottom + rect.height;
			//System.out.println("box left is " + boxLeft);
			//System.out.println("box right is " + boxRight);
			//System.out.println("box top is " + boxTop);
			//System.out.println("box bottom is " + boxBottom);
			if (leftMost > boxLeft) {
				leftMost = boxLeft;
			}
			if (lowerMost > boxBottom) {
				lowerMost = boxBottom;
			}
			if (rightMost < boxRight) {
				rightMost = boxRight;
			}
			if (upperMost < boxTop) {
				upperMost = boxTop;
			}
			//System.out.println(
			//    "put box left " + leftMost + " right " + rightMost);
			//System.out.println(
			//    "put box top " + upperMost + " bottom " + lowerMost);
		}

		// add padding round box
		int strictWidth = rightMost - leftMost;
		rightMost = rightMost + (int) (((double)strictWidth) *padding);
		leftMost = leftMost - (int) (((double)strictWidth) *padding);
		int strictHeight = upperMost - lowerMost;
		upperMost = upperMost + (int) (strictHeight *padding);
		lowerMost = lowerMost - (int) (strictHeight *padding);
		//System.out.println(
		//    "finally, box left " + leftMost + " right " + rightMost);
		//System.out.println(
		//    "finally, box top " + upperMost + " bottom " + lowerMost);
		Polygon p =
			new Polygon(
				new int[] { leftMost, rightMost, rightMost, leftMost },
				new int[] { lowerMost, lowerMost, upperMost, upperMost },
				4);
		if (polygonSet.size() == 0) {
			p =
				new Polygon(
					new int[] {
						-ConcreteDiagram.SCALE,
						ConcreteDiagram.SCALE,
						ConcreteDiagram.SCALE,
						-ConcreteDiagram.SCALE },
					new int[] {
						-ConcreteDiagram.SCALE,
						-ConcreteDiagram.SCALE,
						ConcreteDiagram.SCALE,
						ConcreteDiagram.SCALE },
					4);
		}
		DecorationBox box = new DecorationBox("", "", p);
		return box;
	}

	public Rectangle getBoxAround(double padding) {
		Set polygonSet = _diagram.getConcreteContours();
		DecorationBox box = getBoxAround(polygonSet, padding);
		return box.getPolygon().getBoundingBox();
	}

	//
	// Observer
	//
	public void update(Observable obj, Object arg) {
		if (obj == null) return;
		if (obj instanceof ConcreteDiagram) {
			if (arg == rwd.concretesyntax.Event.ADD_ITEM_NO_ANIMATION) ;
			if (arg == rwd.concretesyntax.Event.REMOVE_ITEM_NO_ANIMATION) ;
			if (arg == rwd.concretesyntax.Event.CHANGE_ITEM_NO_ANIMATION) ;
			else this.drawAgain();
		}
	}
	
	
	/**
	 * dummy version for default class
	 */
    public void fitToWidth(int width) {
    }

    //
    // Getters and setters
    //
    public String getTitle() {
        return _title;
    }
    public void setTitle(String title) {
        _title = title;
    }

	public ConcreteDiagram getConcreteDiagram(){
		return _diagram;
	}
	public void setConcreteDiagram(ConcreteDiagram _diagram){
		this._diagram = _diagram;
	}

	public Set getContradictionZones() {
		return _contZones;
	}
	public void setContradictionZones(Set contZones) {
		_contZones = contZones;		
	}

	public DiagramCanvas getDiagramCanvas() {
		return _canvas;
	}

	public ControlPanel getControlPanel() {
		return _controls;
	}

	public History getHistory() {
		return _history;
	}
	
	public java.util.List getSelectedItems() {
		if (_canvas == null) return new Vector();
		java.util.List res = new Vector(_canvas.getSelectedItems());
		return res;
	}

	
    public boolean isBezier() {
		if (_canvas == null) return false;
        return _canvas.getSmoothed();
    }
    public void setBezier(boolean smoothed) {
    	this._smoothed = smoothed;
		if (_canvas == null) return;
        _canvas.setSmoothed(smoothed);
    }

	public void setMotionFlag(boolean b) {
		_motionFlag = b;
	}
	public boolean getMotionFlag() {
		return _motionFlag;
	}

	public void setBackColor(Color color) {
		// Store state in history
		_history.store(this);

		// Set back color for selected items
		if (_canvas == null) return;
		Collection selectedItems = _canvas.getSelectedItems();
		if (selectedItems == null) return;
		Iterator i = selectedItems.iterator();
		while (i.hasNext()) {
			DiagramItem item = (DiagramItem)i.next();
			if (item != null) {
				// Add in zone colors
				if (item instanceof ConcreteZone) {
					ConcreteZone cz = (ConcreteZone) item;
					cz.setBackColor(color);
				// Add in zone colors
				} else if (item instanceof ConcreteContour) {
//					ConcreteDiagram cd = this.getConcreteDiagram();
//					Set zones = cd.getConcreteZones((ConcreteContour)item);
//					Iterator j = zones.iterator();
//					while (j.hasNext()) {
//						ConcreteZone cz = (ConcreteZone) j.next();
//						System.out.println(
//							"colouring "
//							+ cz.toString()
//							+ ""
//							+ cz.hashCode()
//							+ " color "
//							+ color);
//						 cz.mixBackColor(color);						
//					}
				}
			} 
		}

		// Store state in history
		_history.store(this);
	}
	public void setForeColor(Color color) {
		// Store state in history
		_history.store(this);

		// Set label for selected items
		if (_canvas == null) return;
		Collection selectedItems = _canvas.getSelectedItems();
		if (selectedItems == null) return;
		Iterator i = selectedItems.iterator();
		while (i.hasNext()) {
			DiagramItem item = (DiagramItem)i.next();
			if (item != null) {
				item.setForeColor(color);		
			} 
		}

		// Store state in history
		_history.store(this);
	}
	public void setFont(Font font) {
		// Store state in history
		_history.store(this);

		// Set font for selected items
		if (_canvas == null) return;
		Collection selectedItems = _canvas.getSelectedItems();
		if (selectedItems == null) return;
		Iterator i = selectedItems.iterator();
		while (i.hasNext()) {
			DiagramItem item = (DiagramItem)i.next();
			if (item != null) {
				item.setFont(font);		
			} 
		}

		// Store state in history
		_history.store(this);
	}
	
	public void setLabel(String label) { 
		// Store state in history
		_history.store(this);

		// Set label for selected items
		if (_canvas == null) return;
		Collection selectedItems = _canvas.getSelectedItems();
		if (selectedItems == null) return;
		Iterator i = selectedItems.iterator();
		while (i.hasNext()) {
			DiagramItem item = (DiagramItem)i.next();
			if (item != null && !(item instanceof Handle)) {
				// reconcile reading tree
				if (item instanceof ConcreteSpiderFoot) {
					ConcreteSpiderFoot csf = (ConcreteSpiderFoot) item;
					String oldlabel = csf.getLabel();
					DecorationBox db = getBoxContaining(csf.getX(),csf.getY());
					if (db!=null) {
						ReadingTreeString rts = 
							new ReadingTreeString(db.getReadingForest());
						rts.update(oldlabel,label);
						db.setReadingForest(rts.toString());
					}
				}
					
				// set item's label
				item.setLabel(label);		
			} 
		}

		// Store state in history
		_history.store(this);
	}

	public void setThickness(float thickness) {
		// Store state in history
		_history.store(this);

		// Set thickness
		if (_canvas == null) return;
		Collection selectedItems = _canvas.getSelectedItems();
		if (selectedItems == null) return;
		Iterator i = selectedItems.iterator();
		while (i.hasNext()) {
			DiagramItem item = (DiagramItem)i.next();
			if (item != null) {
				item.setThickness(thickness);		
			} 
		}

		// Store state in history
		_history.store(this);
	}

	public String toString() {
		return "Viewer - "+strId;
	}
	//
	// Local properties
	//
	protected String _title = "Untitled diagram";
	protected boolean _smoothed = false;
	protected boolean _justForViewing = false;
	protected DiagramCanvas _canvas;
	protected ControlPanel _controls;
	protected History _history = new History();

	// This contains all of the things that we need to draw.
	protected ConcreteDiagram _diagram = new ConcreteDiagram();
	protected Set _contZones = new TreeSet();

	// For copy/cut/paste
	protected Collection _clipboard = new LinkedHashSet();
	public static final int DX = 100;
	public static final int DY = 100;
	
	// For graph motion
	protected boolean _motionFlag = false;
	
	// Id
	protected String strId; 
	protected static int id = 0;
	
	/*
	// test program
    // ------------

    private static void pr(String s) {
      System.out.println(s);
    }

    private static void hd(String s) {
      System.out.println("\n" + s + "\n");
    }

    private static void pg() {
      try {
        int i;
        System.out.print("next page? (CR)");
        do i = System.in.read(); while (i != '\n');
        System.out.println();
      }
      catch (Exception e) {System.out.println("raised exception : " + e);}
    }

    public static void main(String args[]) throws Exception {
    	
    	ConcreteDiagram cd = new ConcreteDiagram();
    	AbstractDiagramViewer adv = new AbstractDiagramViewer(cd);
    	pr("adv="+adv);
    	pg();
    	
    	{
	    	hd("Test of getBoxContaining");
	    	
	    	pr("adv.getBoxContaining(2,2)="+adv.getBoxContaining(2,2));
	    	pg();
	    	
	    	Polygon p = new Polygon();
	    	p.addPoint(1,1);
	    	p.addPoint(1,3);
	    	p.addPoint(3,3);
	    	p.addPoint(3,1);
	    	
	    	DecorationBox db = new DecorationBox("d1","(a)",p); 
	    	
	    	cd = new ConcreteDiagram();
	    	cd.addDecorationBox(db);
	    	
	    	adv = new AbstractDiagramViewer(cd);
	    	pr("adv="+adv);
	    	pr("adv.getBoxContaining(2,2)="+adv.getBoxContaining(2,2));
	    	pg();
	    	pr("adv.getBoxContaining(0,0)="+adv.getBoxContaining(0,0));
	    	pg();	   
    	}
    }
    */
}
