package rwd.metrics;

import java.awt.Color;
import java.awt.Font;
import java.awt.Frame;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Polygon;
import java.awt.Rectangle;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.AffineTransform;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.Vector;
import java.util.Iterator;
import java.util.Set;

import javax.swing.JFrame;
import javax.swing.JPanel;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.utilities.AreaRWD;

public class ContourAreaDifferenceComparison
	extends Comparison 
{
	public ContourAreaDifferenceComparison(ConcreteUnitaryDiagram cud, double weighting, ConcreteUnitaryDiagram movecud) {
		super(cud, weighting, movecud);
		usedByPolygonMove = false;
	}


	protected double giveScore() {
		double ret = 0.0;

		Set moveContourPolys = cud.getConcreteContours();
		Set stillContourPolys = stillCud.getConcreteContours();

        Iterator i = stillContourPolys.iterator();
        while(i.hasNext()){

			ConcreteContour stillcp = (ConcreteContour)i.next();
			Polygon stillp = stillcp.getPolygon();

			Point2D.Double stillCentre = ContourPositionComparison.polygonCentre(stillp);

            Iterator j = moveContourPolys.iterator();
            while(j.hasNext()){
				ConcreteContour movecp = (ConcreteContour)j.next();

				if(stillcp.getContour().equals(movecp.getContour())) {

					Polygon movep = movecp.getPolygon();

					Point2D.Double moveCentre = ContourPositionComparison.polygonCentre(movep);

					double diffX = stillCentre.x-moveCentre.x;
					double diffY = stillCentre.y-moveCentre.y;

					// copy one polygon, because it will move to get an accurate overlay
					Polygon newMovep = new Polygon(movep.xpoints, movep.ypoints, movep.npoints);
					shiftPolygon(newMovep,diffX,diffY);

					// do the area difference calculation
					double diff = findAreaDifference(stillp,newMovep);

					// add the square of area difference to ret
					ret += diff*diff;
				}
			}
		}
		
		ret = ret/100000000;
		
System.out.println("quality "+ret);
		return ret;
	}	




/** moves the polygon in the direction specifed by x and y */
	public static void shiftPolygon(Polygon p,double movex, double movey) {
	
		int[] xs = p.xpoints;
		int[] ys = p.ypoints;
		for(int j = 0; j<p.npoints; j++) {
			xs[j] += movex;
			ys[j] += movey;

		}
	}

//debug
static Vector tops = new Vector();
//end debug


	public static double findAreaDifference(Polygon poly1,Polygon poly2) {
		AreaRWD area1 = new AreaRWD(poly1);
		AreaRWD area2 = new AreaRWD(poly2);

		AreaRWD areaDiff = new AreaRWD(poly1);
		areaDiff.exclusiveOr(area2);
		Vector polys = areaToPolygons(areaDiff);

//debug
  class TestPanel extends JPanel  {

	AreaRWD a1;
	AreaRWD a2;
	AreaRWD aDiff;
	public Vector polyList = new Vector();
	public Vector areaValueList = new Vector();

	  public TestPanel(AreaRWD a1, AreaRWD a2, AreaRWD aDiff) {
		  this.a1 = a1;
		  this.a2 = a2;
		  this.aDiff = aDiff;
		  setBackground(Color.white);
	  }

	  public void paintComponent(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		super.paintComponent(g2);

		Rectangle r = null;
		int xOffset = 0;
		int yOffset = 0;
		AffineTransform translate = null;
		
		r = aDiff.getBounds();
		xOffset = 10-r.x;
		yOffset = 10-r.y;
		translate = AffineTransform.getTranslateInstance(xOffset,yOffset);
		g2.setTransform(translate);
		g2.setColor(Color.yellow);
		g2.fill(aDiff);

		AffineTransform diffTransform = translate;

		r = a1.getBounds();
		xOffset = 10-r.x;
		yOffset = 10-r.y;
		translate = AffineTransform.getTranslateInstance(xOffset,yOffset);
		g2.setTransform(translate);
		g2.setColor(Color.black);
		g2.draw(a1);

		r = a2.getBounds();
		xOffset = 10-r.x;
		yOffset = 10-r.y;
		translate = AffineTransform.getTranslateInstance(xOffset,yOffset);
		g2.setTransform(translate);
		g2.setColor(Color.blue);
		g2.draw(a2);

		Iterator it = polyList.iterator();
		Iterator it2 = areaValueList.iterator();
		int count = 1;
		while(it.hasNext()) {

			Polygon poly = (Polygon)it.next();
			Double size = (Double)it2.next();

			double polyArea = AreaPolygonAdapter.getAreaOf(poly);

			String pointString = numberOfPointsString(poly,g2,diffTransform);

			Font font = new Font("Arial",Font.BOLD,12);
			FontRenderContext frc = g2.getFontRenderContext();
			String zoneLabel = "Zone "+Integer.toString(count)+" size "+size.toString()+" "+pointString;
			TextLayout labelLayout = new TextLayout(zoneLabel, font, frc);

			Rectangle bounds = poly.getBounds();
			int xPos = (int)(bounds.getX()+bounds.getWidth()/2);
			int yPos = (int)(bounds.getY()+bounds.getHeight()/2);

			g2.setColor(Color.black);
			g2.setTransform(diffTransform);

			labelLayout.draw(g2,xPos,yPos);

			count++;
		}
	}

	public String numberOfPointsString(Polygon poly, Graphics2D g2,AffineTransform transform) {
		int countLineTo = 0;
		int countMoveTo = 0;
		int squareSize = 4;
		double[] pathPoint = {0.0,0.0,0.0,0.0,0.0,0.0};
		PathIterator pi = poly.getPathIterator(null);
		while(!pi.isDone()) {
			int segmentType = pi.currentSegment(pathPoint);
			if(segmentType == PathIterator.SEG_MOVETO) {
				g2.setColor(Color.green);
				g2.setTransform(transform);
				Rectangle2D.Double r = new Rectangle2D.Double(pathPoint[0]-squareSize/2,pathPoint[1]-squareSize/2,squareSize,squareSize);
				g2.draw(r);
				countMoveTo++;
			}
			if(segmentType == PathIterator.SEG_LINETO) {
				g2.setColor(Color.black);
				g2.setTransform(transform);
				Rectangle2D.Double r = new Rectangle2D.Double(pathPoint[0]-squareSize/2,pathPoint[1]-squareSize/2,squareSize,squareSize);
				g2.draw(r);
				countLineTo++;
			}
			if(segmentType == PathIterator.SEG_CLOSE) {
				g2.setColor(Color.orange);
				g2.setTransform(transform);
				Rectangle2D.Double r = new Rectangle2D.Double(pathPoint[0]-squareSize/2,pathPoint[1]-squareSize/2,squareSize,squareSize);
				g2.draw(r);
			}
			pi.next();
			squareSize += 2;
		}
		return "MoveTo "+Integer.toString(countMoveTo)+" LineTo "+Integer.toString(countLineTo);
	}
  }

while(tops.size() > 0) {
	Frame f = (Frame)tops.get(0);
	tops.remove(0);
	f.dispose();
}

JFrame top = new JFrame("Comparison");
TestPanel panel = new TestPanel(area1,area2,areaDiff);
//end debug

		int count = 0;
		double totalArea = 0.0;

		Iterator it = polys.iterator();
		while(it.hasNext()) {
			Polygon poly = (Polygon)it.next();

			double polyArea = AreaPolygonAdapter.getAreaOf(poly);
			totalArea += polyArea;

//			debug
panel.polyList.add(poly);
panel.areaValueList.add(new Double(polyArea));
//			end debug

			count++;
		}
System.out.println("Number of Polys " + count + " total area "+totalArea);


// debug
top.setSize(600,600);
top.getContentPane().add(panel);
top.setVisible(true);
tops.add(top);
// end debug



		return totalArea;
	}



/** 
 * this converts an area, assumed to be a polygon type, to an array list
 * of Polygons, one polygon for each separate segment
 */
	public static Vector areaToPolygons(AreaRWD a) {

		Vector retPolys = new Vector();
	    double[] pathPoint = {0.0,0.0,0.0,0.0,0.0,0.0};
		Polygon poly = null;
		PathIterator pi = a.getPathIterator(null);
		while(!pi.isDone()) {
			int segmentType = pi.currentSegment(pathPoint);
			if(segmentType == PathIterator.SEG_MOVETO) {
				poly = new Polygon();
			}
			if(segmentType == PathIterator.SEG_LINETO) {
				poly.addPoint((int)pathPoint[0],(int)pathPoint[1]);
			}
			if(segmentType == PathIterator.SEG_CLOSE) {
				retPolys.add(poly);
			}
			pi.next();
		}
		return retPolys;
	}


	public static void outpoly(Polygon p) {
		double[] pathPoint = {0.0,0.0,0.0,0.0,0.0,0.0};
		PathIterator pi = p.getPathIterator(null);
		while(!pi.isDone()) {
			int segmentType = pi.currentSegment(pathPoint);
			System.out.print("("+segmentType+","+pathPoint[0]+","+pathPoint[1]+")" );
			pi.next();
		}
	}

}




