package rwd.metrics;

import java.awt.Polygon;
import java.util.Iterator;
import java.util.Set;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.DecorationBox;

public class ContourCloseness
	extends EulerMetric 
{
	// this looks more complicated than it needs to be
	public ContourCloseness(ConcreteUnitaryDiagram cud, double weighting) {
		super(cud, weighting);
	}

	protected double giveScore() {
		double total = 0.0;
		// Here's the strategy::
		// for each contour c
		// for each edge of c, e
		// work out a unit vector w "along the edge"
		// and remember the point p at the "start of the edge" p
		// keep a minForCE and a maxForCE
		// for each node n on contour c
		// work out the vector v from n to p
		// see whether perpx = (v - (v.w)w)_x+y changes min / max
		// we know the min and the max for c, e
		// for each other contour c2
		// keep a min and a max
		// for each node n on contours c2
		// work out the vector v from n to p
		// see whether (v - (v.w)w)_x changes min / max
		// we know the min and the max for c2, e
		// check to see how the min/max for c2,e compare with those for c,e
		// next contour c2
		// next e and therefore w
		// next c

		// here starts the implementation
		// for each contour c
        
		Set contourPolygons = cud.getConcreteContours();
		if (contourPolygons.size() < 2) {
			return 0.0;
		}
		Iterator it = contourPolygons.iterator();
		while (it.hasNext()) {
			ConcreteContour cp = (ConcreteContour) (it.next());
			Polygon c = cp.getPolygon();
			// for each edge of c, e
			int[] cxs = c.xpoints;
			int[] cys = c.ypoints;

			for (int cindex = 0;
				cindex < c.npoints;
				cindex++) { // work out a unit vector w "along the edge"
				// and remember the point p at the "start of the edge" p
				int px = cxs[cindex % c.npoints];
				int py = cys[cindex % c.npoints];
				double wx = cxs[(cindex + 1) % c.npoints] - px;
				double wy = cys[(cindex + 1) % c.npoints] - py;
				double wlength = Math.sqrt(wx * wx + wy * wy);
				wx = wx / wlength;
				wy = wy / wlength;
				//System.out.println("unit vector w is ("+wx+","+wy+")");
				// keep a minForCE and a maxForCE
				int minForCE = Integer.MAX_VALUE;
				int maxForCE = Integer.MIN_VALUE;
				// for each node n on contour c
				for (int cindex2 = 0; cindex2 < c.npoints; cindex2++) {
					int nx = cxs[cindex2];
					int ny = cys[cindex2];
					// work out the vector v from n to p
					int vx = nx - px;
					int vy = ny - py;
					// see whether (v - (v.w)w)_x+y changes min / max
					int perpxy =
						(int) (vx - (vx * wx + vy * wy) * wx)
							+ (int) (vy - (vx * wx + vy * wy) * wy);
					//System.out.println("perp x+ycoord for ("+vx+","+vy+") is "+perpxy);
					if (minForCE > perpxy) {
						minForCE = perpxy;
					}
					if (maxForCE < perpxy) {
						maxForCE = perpxy;
					}
				} // we know the min and the max for c, e
				//System.out.println("min x perp is "+minForCE);
				//System.out.println("max x perp is "+maxForCE);
				{
					// for each other contour c2
					Iterator it2 = contourPolygons.iterator();
					while (it2.hasNext()) {
						ConcreteContour cp2 = (ConcreteContour) (it2.next());
						if (cp2 != cp) {
							Polygon c2 = cp2.getPolygon();
							int[] c2xs = c2.xpoints;
							int[] c2ys = c2.ypoints;

							int minForC2E = Integer.MAX_VALUE;
							int maxForC2E = Integer.MIN_VALUE;
							// for each node n on contours c2
							for (int cindex2 = 0;
								cindex2 < c2.npoints;
								cindex2++) {
								int nx = c2xs[cindex2];
								int ny = c2ys[cindex2];
								// work out the vector v from n to p
								int vx = nx - px;
								int vy = ny - py;
								// see whether (v - (v.w)w)_x+y changes min / max
								int perpxy =
									(int) (vx - (vx * wx + vy * wy) * wx)
										+ (int) (vy - (vx * wx + vy * wy) * wy);
								//System.out.println("perp x+ycoord for ("+vx+","+vy+") is "+perpxy);
								if (minForC2E > perpxy) {
									minForC2E = perpxy;
								}
								if (maxForC2E < perpxy) {
									maxForC2E = perpxy;
								}
							} // we know the min and the max for c2, e
							//System.out.println("min x perp is "+minForC2E);
							//System.out.println("max x perp is "+maxForC2E);
							// we know the min and the max for c2, e

							// check to see how the min/max for c2,e compare with those for c,e
							// ?????????????????????????
							// we have min1, min2, max1, max2
							// or      min1, max1, min2, max2
							// or      min1, min2, max2, max1
							// need to check some proportional aspect of this pair of bounds
							// basically, are the four numbers, when placed in order,
							// roughly equally spaced?
							// could do mean/square of diffs again.

							double ra, rb, rc, rd;
							// r for ranked - will get ra<=rb<=rc<=rd
							if (minForCE < minForC2E) {
								ra = minForCE;
								if (minForC2E < maxForCE) {
									rb = minForC2E;
									if (maxForCE < maxForC2E) {
										rc = maxForCE;
										rd = maxForC2E;
									} else {
										rc = maxForC2E;
										rd = maxForCE;
									}
								} else {
									rb = maxForCE;
									rc = minForC2E;
									rd = maxForC2E;
								}
							} else {
								ra = minForC2E;
								if (minForCE < maxForC2E) {
									rb = minForCE;
									if (maxForC2E < maxForCE) {
										rc = maxForC2E;
										rd = maxForCE;
									} else {
										rc = maxForCE;
										rd = maxForC2E;
									}
								} else {
									rb = maxForC2E;
									rc = minForCE;
									rd = maxForCE;
								}
							}
							// calculate sum/sqrs/diffs for the gaps amongst these
							//System.out.println("in order: "+ra+","+rb+","+rc+","+rd);
							double gap1 = (rb - ra) / (rd - ra);
							double gap2 = (rc - rb) / (rd - ra);
							double gap3 = (rd - rc) / (rd - ra);
							//System.out.println("ratios: "+gap1+","+gap2+","+gap3);
							/*double diffSumSqs = (gap1-0.3333)*(gap1-0.3333)
							                +(gap2-0.3333)*(gap2-0.3333)
							                +(gap3-0.3333)*(gap3-0.3333);
							//System.out.println("adds "+diffSumSqs+" to overall score");
							total += diffSumSqs;
							*/
							total += 1 / gap1;
							total += 1 / gap2;
							total += 1 / gap3;
						}
					}
				} // next (other) contour c2
			} // next e and therefore w
		} // next c

		return total;
	}

	public static void main(String[] args) { //build PolygonSet with null zones is OK
		Polygon p1 =
			new Polygon(
				new int[] { 0, 20, 20, 0 },
				new int[] { 0, 0, 20, 20 },
				4);
		Polygon p2 =
			new Polygon(
				new int[] { 5, 35, 30, 10 },
				new int[] { 5, 10, 30, 25 },
				4);
		ConcreteContour cp1 = new ConcreteContour(Contour.getContour("a"), p1);
		ConcreteContour cp2 = new ConcreteContour(Contour.getContour("b"), p2);
        ConcreteUnitaryDiagram cud = new ConcreteUnitaryDiagram(
        new DecorationBox("box", "", new Polygon(new int[]{0,50,50,0},new int[]{0,0,50,50},4)));
		cud.addConcreteContour(cp1);
        cud.addConcreteContour(cp2);
		new ContourCloseness(cud, 1.0).giveScore();
	}
}
