package rwd.metrics;

import java.awt.Polygon;
import java.util.Iterator;
import java.util.Set;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;

public class ContourRoundnessAngle
	extends EulerMetric 
{
	public ContourRoundnessAngle(ConcreteUnitaryDiagram cud, double weighting) {
		super(cud, weighting);
	}

	public double giveScore() {
		double score = 0.0;

		Set contourPolys = cud.getConcreteContours();
        Iterator i = contourPolys.iterator();
        while(i.hasNext()){
			ConcreteContour cp = (ConcreteContour)i.next();
			Polygon p = cp.getPolygon();
			score += giveShapeAngleScore(p);
		}
		score = score / 1000.0;

		//System.out.println("ContourRoundednessMetric returning "+score);
		return score;
	}

	// Go through each point of the shape and return some score
	// based on the square of the difference between each angle
	// and a regular polygon of the same number of points.
	public static double giveShapeAngleScore(Polygon p) {
		double score = 0.0;
		int[] xs = p.xpoints;
		;
		int[] ys = p.ypoints;
		;

		int numPts = p.npoints;
		double idealAngle = (Math.PI * (numPts - 2)) / numPts;
		//System.out.println("ideal angle is "+idealAngle);
		double[] angles = new double[numPts];

		double x1 = xs[0]; // don't be tempted to use ints here
		double y1 = ys[0]; // the test comparing sum to ideal angle multiple
		double x2 = xs[1]; // will fail and the smoothing won't work
		double y2 = ys[1];
		double angleTotal = 0.0;
		for (int i = 2; i <= numPts + 1; i++) {
			double x3 = xs[i % numPts];
			double y3 = ys[i % numPts];
			double alpha1;
			//if(x1!=x2)
			{
				alpha1 = Math.atan((y2 - y1) / (x2 - x1));
			}
			//else
			//{ alpha1 = Math.PI/2; }
			if (x1 > x2) {
				alpha1 = alpha1 + Math.PI;
			}

			double alpha2;
			//if(x2!=x3)
			{
				alpha2 = Math.atan((y2 - y3) / (x2 - x3));
			}
			//else
			//{ alpha2 = Math.PI/2; }
			if (x3 > x2) {
				alpha2 = alpha2 + Math.PI;
			}

			if (alpha2 < alpha1) {
				alpha2 = alpha2 + 2 * Math.PI;
			}
			//double angle2 = alpha2 - alpha1;
			angles[i - 2] = alpha2 - alpha1;
			angleTotal = angleTotal+angles[i-2];
			x1 = x2;
			y1 = y2;
			x2 = x3;
			y2 = y3;
			//System.out.println("angle is "+angles[i-2]);
		}
		if (angleTotal != p.npoints * idealAngle) { 
			// do 2pi - angles
			for (int i = 0; i <= numPts - 1; i++) {
				angles[i] = 2 * Math.PI - angles[i];
			}
		}
		//System.out.println("idealAngle is "+idealAngle);
		for (int i = 0; i <= numPts - 1; i++) {
			//System.out.println("found angle is "+angles[i]);
			//System.out.println("increment score by "+((idealAngle - angles[i]) * (idealAngle - angles[i])));
			score = score + (idealAngle - angles[i]) * (idealAngle - angles[i]);

		}
		//System.out.println("unscaled score sum is "+score);

		//System.out.println("returning angle score "+(score*100/numPts));
		return score * 100 / numPts; // scaled by JAF, normalized by pjm2
	}
}
