package rwd.metrics;

import java.awt.Polygon;
import java.util.Iterator;
import java.util.Set;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.DecorationBox;

public class ContourRoundnessSegmentLength
	extends EulerMetric 
{
	public ContourRoundnessSegmentLength(ConcreteUnitaryDiagram cud, double weighting) {
		super(cud, weighting);
	}

	public double giveScore() {
		double score = 0.0;
		double totalLength = 0.0;

		Set contourPolys = cud.getConcreteContours();
        Iterator i = contourPolys.iterator();
        while(i.hasNext()){
			ConcreteContour cp = (ConcreteContour)i.next();
			Polygon p = cp.getPolygon();
			int[] xs = p.xpoints;
			int[] ys = p.ypoints;
			//int numPts = p.npoints;
			double[] edgeLengths = new double[xs.length];
			double total = 0.0;
			int x1 = xs[p.npoints - 1];
			int y1 = ys[p.npoints - 1];
			for (int i2 = 0; i2 < p.npoints; i2++) {
				int x2 = xs[i2];
				int y2 = ys[i2];
				//System.out.print("("+x1+","+y1+"),("+x2+","+y2+")");
				edgeLengths[i2] =
					Math.sqrt((x2 - x1) * (x2 - x1) + (y2 - y1) * (y2 - y1));
				total += edgeLengths[i2];
				totalLength += edgeLengths[i2];
				//System.out.println(" length"+edgeLengths[i]);
				x1 = x2;
				y1 = y2;
			}
			double mean = total / edgeLengths.length;

			for (int i2 = 0; i2 < edgeLengths.length; i2++) {
				double d = edgeLengths[i2] - mean;
				score += d * d;
			}
		}
		//double diagramArea = DiagramAreaMetric.getDiagramArea(polys);
		//score = (score / diagramArea) / 1000.0; // normalized by pjm2
		score = score / (totalLength * totalLength);
		return score;
	}

	public static void main(String[] args) { //build PolygonSet with null zones is OK
		Polygon p1 =
			new Polygon(
				new int[] { 0, 20, 20, 0 },
				new int[] { 0, 0, 20, 20 },
				4);
		Polygon p2 =
			new Polygon(
				new int[] { 5, 35, 30, 10 },
				new int[] { 5, 10, 30, 25 },
				4);
		ConcreteContour cp1 = new ConcreteContour(Contour.getContour("a"), p1);
		ConcreteContour cp2 = new ConcreteContour(Contour.getContour("b"), p2);
        ConcreteUnitaryDiagram cud = new ConcreteUnitaryDiagram(
        new DecorationBox("box", "", new Polygon(new int[]{0,50,50,0},new int[]{0,0,50,50},4)));
        cud.addConcreteContour(cp1);
        cud.addConcreteContour(cp2);

		new ContourRoundnessSegmentLength(cud, 1.0).giveScore();
	}
}
