package rwd.metrics;

import java.awt.Polygon;
import java.util.Iterator;
import java.util.Set;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;

public class DiagramArea
	extends EulerMetric 
{
	public DiagramArea(ConcreteUnitaryDiagram cud, double weighting) {
		super(cud, weighting);
	}

	protected double giveScore() {
		//Vector contourPolys = polys.getContourPolys();
		double ret = getDiagramArea(cud);
		ret = ret / 1000000.0;
		//System.out.println("DiagramAreaMetric returning "+ret);
		return ret;
	}

	public static double getDiagramArea(ConcreteUnitaryDiagram cud) {
		Set contourPolys = cud.getConcreteContours();

		if (contourPolys.size() == 0) {
			return (0.0);
		}

		double minX = Double.MAX_VALUE;
		double minY = Double.MAX_VALUE;
		double maxX = -Double.MAX_VALUE;
		double maxY = -Double.MAX_VALUE;

        Iterator i = contourPolys.iterator();
        while(i.hasNext()){
			ConcreteContour cp = (ConcreteContour)i.next();
			Polygon poly = cp.getPolygon();

			int[] xs = poly.xpoints;
			int[] ys = poly.ypoints;
			for (int j = 0; j < poly.npoints; j++) {

				int x = xs[j];
				int y = ys[j];

				if (x > maxX) {
					maxX = x;
				}
				if (x < minX) {
					minX = x;
				}
				if (y > maxY) {
					maxY = y;
				}
				if (y < minY) {
					minY = y;
				}
			}
		}
		return (maxX - minX) * (maxY - minY);
	}
}
