package rwd.metrics;

import java.awt.Polygon;
import java.util.Iterator;
import java.util.Set;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.ConcreteZone;
import rwd.concretesyntax.eulerdiagram.DecorationBox;

public class ZoneArea
	extends EulerMetric 
{
	public ZoneArea(ConcreteUnitaryDiagram cud, double weighting) {
		super(cud, weighting);
	}

	/**
	 * Score is given by calculating the difference in squares from
	 * the mean area. Ergo, the lowest (best) score is achieved when
	 * all zones have the same area.
	 */
	public double giveScore() {
		double score = 0.0;
		double totalArea = 0.0;
        Set zoneAreas = cud.getConcreteZones();
		double[] area = new double[zoneAreas.size()];
        Iterator i = zoneAreas.iterator();
        int counter = 0;
        while(i.hasNext()){
			ConcreteZone zoneArea = (ConcreteZone) i.next();
			//Polygon poly = zonePoly.getPoly();
			//System.out.println("ZAM assessing poly:"+AreaPolygonAdapter.niceString(poly));
			//area[i] =AreaPolygonAdapter.getArea(poly);
			//System.out.println("considering zone area "+zoneArea.getZoneString());
			area[counter] = AreaPolygonAdapter.getAreaOfConnectedArea(zoneArea.getArea());
			score += 1 / area[counter];
			totalArea = totalArea + area[counter];
            counter++;
		}
		//System.out.println("ZAM returning score:"+score*totalArea);
		return score * totalArea;
	}
	public static void main(String[] args) { 
		//build PolygonSet with null zones is OK
		Polygon p1 =
			new Polygon(
				new int[] { 0, 20, 20, 0 },
				new int[] { 0, 0, 20, 20 },
				4);
		Polygon p2 =
			new Polygon(
				new int[] { 5, 35, 30, 10 },
				new int[] { 5, 10, 30, 25 },
				4);
		ConcreteContour cp1 = new ConcreteContour(Contour.getContour("a"), p1);
		ConcreteContour cp2 = new ConcreteContour(Contour.getContour("b"), p2);
        ConcreteUnitaryDiagram cud = new ConcreteUnitaryDiagram(
        new DecorationBox("box", "", new Polygon(new int[]{0,50,50,0},new int[]{0,0,50,50},4)));
        cud.addConcreteContour(cp1);
        cud.addConcreteContour(cp2);
		System.out.println("" + new ZoneArea(cud, 1.0).giveScore());

		p1.xpoints[0] = 10;
		cud.refreshZones(true);
		cud.notifyObservers();
		System.out.println("" + new ZoneArea(cud, 1.0).giveScore());

		p1 =
			new Polygon(
				new int[] { 0, 200, 200, 0 },
				new int[] { 0, 0, 200, 200 },
				4);
		p2 =
			new Polygon(
				new int[] { 50, 350, 300, 100 },
				new int[] { 50, 100, 300, 250 },
				4);
		cp1 = new ConcreteContour(Contour.getContour("a"), p1);
		cp2 = new ConcreteContour(Contour.getContour("b"), p2);
        cud = new ConcreteUnitaryDiagram(
        new DecorationBox("box", "", new Polygon(new int[]{0,400,400,0},new int[]{0,0,400,400},4)));
        cud.addConcreteContour(cp1);
        cud.addConcreteContour(cp2);
		System.out.println("" + new ZoneArea(cud, 1.0).giveScore());
	}
}
