package rwd.metrics;

import java.awt.Polygon;
import java.util.Vector;
import java.util.Iterator;
import java.util.Set;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteZone;
import rwd.utilities.AreaRWD;

/**
 * To avoid narrow, banana-shaped zones, this Metric gives a score
 * of zero for convex zones. A positive non-zero score is given to
 * shapes that are concave. This score is based on the sum of squares
 * of the turn angles that cause concaveness.
 *
 * this throws an IllegalArgumentException if it's
 * asked to assess the ZoneRoundness where zones are not simply connected
 * (i.e. with diagrams which have underlying nested Euler diagrams)
 */
public class ZoneConvexness 
	extends EulerMetric 
{
	public ZoneConvexness(ConcreteUnitaryDiagram cud, double weighting) {
		super(cud, weighting);
	}

	public double giveScore() {
		double score = 0.0;

        Set zoneAreas = cud.getConcreteZones();
        Iterator i = zoneAreas.iterator();
        while(i.hasNext()){
			ConcreteZone zp = (ConcreteZone)i.next();
			AreaRWD a = zp.getArea();
			score += giveShapeConvexnessScore(a);
		}
		return score;
	}

	public static double giveShapeConvexnessScore(AreaRWD a) {
		double score = 0.0;
		
		Vector polys = AreaPolygonAdapter.getPolygonsFor(a);
		if(polys.size()!=1){
			throw new IllegalArgumentException();
		}
		Polygon poly = (Polygon)polys.get(0);

		//boolean clockwise = ShapeUtil.isDrawnClockwise(shape);
		// this is messed up due to the extra point... :-/
		boolean clockwise = true;
		int[] xs = poly.xpoints;
		int[] ys = poly.ypoints;

		for (int i = 0; i < poly.npoints; i++) {
			int x1 = xs[i];
			int y1 = ys[i];
			int x2 = xs[(i + 1) % poly.npoints];
			int y2 = ys[(i + 1) % poly.npoints];
			int x3 = xs[(i + 2) % poly.npoints];
			int y3 = ys[(i + 2) % poly.npoints];

			//System.out.println(x1 + ", " + y1);
			double theta12 = Math.atan2(y2 - y1, x2 - x1);
			double theta23 = Math.atan2(y3 - y2, x3 - x2);

			double turnAngle = theta12 - theta23;
			if (turnAngle > Math.PI) {
				turnAngle -= 2 * Math.PI;
			} else if (turnAngle < -Math.PI) {
				turnAngle += 2 * Math.PI;
			}

			if ((clockwise && turnAngle < 0)
				|| (!clockwise && turnAngle > 0)) {
				score += Math.pow(turnAngle, 2);
			}

			//System.out.println("turn: " + turnAngle + " score: " + score);
		}

		//System.out.println("ZoneConvexnessMeasure score: " + score);
		return score;
	}

}
