package rwd.rules;

import java.util.Vector;

import rwd.abstractsyntax.eulerdiagram.*;
import rwd.abstractsyntax.spiderdiagram.*;
import rwd.gui.ProofViewer;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToJustADiagram;
import rwd.rules.application.RuleAppliesWithString;
import rwd.rules.choicespec.*;
import rwd.rules.inputspec.RuleAddContour;
import rwd.rules.nospec.*;

/**
 * @author jaf28
 */
public class ProofGetterSmall {
    DiagramGetter dg;
    int maxNumContours;
    RuleWithChoice[] rulesWithChoice;
    RuleNoSpec[] rulesNoChoice;
    Vector maxContourSet;
    int numberOfRules;
    int habLimit;
    int shadingLimit;

    // ask GES about which rules should be here just for completeness
    // complete set either includes F->anything 
    // and P->P or rubbish
    public ProofGetterSmall(
        int numContours,
        int maxNumContours,
        int numSpiders,
        int numComponents,
        int habitatLimit,
        int shadingLimit) {
        this(
            numContours,
            maxNumContours,
            numSpiders,
            numComponents,
            habitatLimit,
            shadingLimit,
            new Rule[] {
                (RuleWithChoice) RuleDeleteContour.getRule(),
                (RuleWithChoice) RuleIntroduceShadedZone.getRule(),
                (RuleWithChoice) RuleDeleteShadedZone.getRule(),
                (RuleWithChoice) RuleUnshadeZone.getRule(),
                (RuleWithChoice) RuleDeleteSpider.getRule(),
                (RuleWithChoice) RuleAddSpiderFoot.getRule(),
                (RuleWithChoice) RuleSplitSpider.getRule(),
                (RuleWithChoice) RuleJoinSpiders.getRule(),
                (RuleWithChoice) RuleExcludedMiddle.getRule(),
                (RuleWithChoice) RuleReverseExcludedMiddle.getRule(),
                (RuleWithChoice) MergeContradictionRule.getRule(),
            // reverse?
             (RuleWithChoice) MergeConjunctionRule.getRule(),
            // reverse?
            // reverse of removeNotRule ????????
            // remove not to false is NotFalseIsTrue below
            // reverse also below
            (RuleWithChoice) RuleReverseAndIdempotent.getRule(),
                (RuleWithChoice) RuleReverseOrIdempotent.getRule(),
                (RuleWithChoice) RuleDMPullNotThroughAnd.getRule(),
                (RuleWithChoice) RuleDMPullNotThroughOr.getRule(),
                (RuleWithChoice) RuleDMPushNotThroughAnd.getRule(),
                (RuleWithChoice) RuleDMPushNotThroughOr.getRule(),
                (RuleWithChoice) FalseWithAndRule.getRule(),
                (RuleWithChoice) FalseWithOrRule.getRule(),
                (RuleWithChoice) RuleDistributeAndOverOr.getRule(),
                (RuleWithChoice) RuleDistributeMergeAnds.getRule(),
                (RuleWithChoice) RuleDistributeMergeOrs.getRule(),
                (RuleWithChoice) RuleDistributeOrOverAnd.getRule(),
                (RuleNoSpec) RemoveNotRule.getRule(),
                (RuleNoSpec) NotFalseIsTrueRule.getRule(),
                (RuleNoSpec) NotTrueIsFalseRule.getRule(),
                (RuleNoSpec) AndIdempotentRule.getRule(),
                (RuleNoSpec) OrIdempotentRule.getRule(),
                (RuleNoSpec) RemoveNotNotRule.getRule(),
                (RuleNoSpec) InsertNotNotRule.getRule(),
                (RuleNoSpec) RulePToPorF.getRule(),
                (RuleNoSpec) SimpleCombiningRule.getRule()});
    }

    public ProofGetterSmall(
        int numContours,
        int maxNumContours,
        int numSpiders,
        int numComponents,
        int habitatLimit,
        int shadingLimit,
        Vector rules) {
        Rule[] ruleAr = new Rule[rules.size()];
        for (int i = 0; i < rules.size(); i++) {
            ruleAr[i] = (Rule) rules.get(i);
        }
        this.maxNumContours = maxNumContours;
        dg =
            new DiagramGetterSmall(
                numContours,
                numSpiders,
                numComponents,
                habLimit,
                shadingLimit);

        // RuleAddContour done specially

        int numWChoice = 0;
        int numNChoice = 0;
        for (int i = 0; i < ruleAr.length; i++) {
            Rule r = ruleAr[i];
            if (r instanceof RuleWithChoice) {
                numWChoice++;
            } else if (r instanceof RuleNoSpec) {
                numNChoice++;
            }
        }
        this.rulesWithChoice = new RuleWithChoice[numWChoice];
        this.rulesNoChoice = new RuleNoSpec[numNChoice];
        numWChoice = 0;
        numNChoice = 0;
        for (int i = 0; i < ruleAr.length; i++) {
            Rule r = ruleAr[i];
            if (r instanceof RuleWithChoice) {
                rulesWithChoice[numWChoice] = (RuleWithChoice) r;
                numWChoice++;
            } else if (r instanceof RuleNoSpec) {
                rulesNoChoice[numNChoice] = (RuleNoSpec) r;
                numNChoice++;
            }
        }

        maxContourSet = new Vector();

        if (maxNumContours > 0) {
            maxContourSet.add(Contour.getContour("a"));
        }
        if (maxNumContours > 1) {
            maxContourSet.add(Contour.getContour("b"));
        }
        if (maxNumContours > 2) {
            maxContourSet.add(Contour.getContour("c"));
        }
        if (maxNumContours > 3) {
            maxContourSet.add(Contour.getContour("d"));
        }
        if (maxNumContours > 4) {
            System.out.println("too many contours to handle");
            System.exit(0);
        }
        numberOfRules = rulesWithChoice.length + rulesNoChoice.length;
        //System.out.println("there are "+numberOfRules+" rules");
    }

    public ProofGetterSmall(
        int numContours,
        int maxNumContours,
        int numSpiders,
        int numComponents,
        int habitatLimit,
        int shadingLimit,
        Rule[] rules) {

        this.maxNumContours = maxNumContours;
        dg = new DiagramGetterRandom(numContours, numSpiders, numComponents);

        // RuleAddContour done specially

        int numWChoice = 0;
        int numNChoice = 0;
        for (int i = 0; i < rules.length; i++) {
            Rule r = rules[i];
            if (r instanceof RuleWithChoice) {
                numWChoice++;
            } else if (r instanceof RuleNoSpec) {
                numNChoice++;
            }
        }
        this.rulesWithChoice = new RuleWithChoice[numWChoice];
        this.rulesNoChoice = new RuleNoSpec[numNChoice];
        numWChoice = 0;
        numNChoice = 0;
        for (int i = 0; i < rules.length; i++) {
            Rule r = rules[i];
            if (r instanceof RuleWithChoice) {
                rulesWithChoice[numWChoice] = (RuleWithChoice) r;
                numWChoice++;
            } else if (r instanceof RuleNoSpec) {
                rulesNoChoice[numNChoice] = (RuleNoSpec) r;
                numNChoice++;
            }
        }

        maxContourSet = new Vector();

        if (maxNumContours > 0) {
            maxContourSet.add(Contour.getContour("a"));
        }
        if (maxNumContours > 1) {
            maxContourSet.add(Contour.getContour("b"));
        }
        if (maxNumContours > 2) {
            maxContourSet.add(Contour.getContour("c"));
        }
        if (maxNumContours > 3) {
            maxContourSet.add(Contour.getContour("d"));
        }
        if (maxNumContours > 4) {
            System.out.println("too many contours to handle");
            System.exit(0);
        }
        numberOfRules = rulesWithChoice.length + rulesNoChoice.length;
        //System.out.println("there are "+numberOfRules+" rules");
    }
    public Proof getProof(int depth) {
        // get a random start diagram
        Diagram d = dg.getDiagram();

        // start off proof
        Proof p = new Proof(d);
        stepLoop : for (int numSteps = 0; numSteps < depth; numSteps++) {
            d = p.getLastDiagram();
            //System.out.print("got diag to work from  ");			
            // choose a random choice rule
            //System.out.println(
            //	"there are " + numberOfRules + " rules");
            if (Math.random() < 0.1 && !(d instanceof FalseDiagram)) {
                // try to add contour
                Vector parts = d.getUnitaryParts();
                UnitaryDiagram part =
                    (UnitaryDiagram) parts.get(
                        (int) (Math.random() * parts.size()));
                if (!(part instanceof FalseDiagram)) {
                    if (part.getCloneOfContourSet().size() < maxNumContours) {
                        // can add contour!
                        Vector conts = (Vector) maxContourSet.clone();
                        conts.removeAll(part.getCloneOfContourSet());
                        Contour c = (Contour) conts.get(0);
                        RuleAppliesWithString rws =
                            new RuleAppliesWithString(part, "" + c);
                        Diagram newEnd = null;
                        newEnd =
                            RuleAddContour.getRule().applyRuleToPart(d, rws);
                        // seek equivalent diagram in proof p
                        boolean foundmatch = false;
                        p.goToStart();
                        if (p.getCurrentDiagram().equals(newEnd)) {
                            foundmatch = true;
                        } else {
                            diagramScan : while (p.goToNextStep()) {
                                if (p.getCurrentDiagram().equals(newEnd)) {
                                    foundmatch = true;
                                    break diagramScan;
                                }
                            }
                        }
                        if (!foundmatch) {
                            //System.out.println("\nadd a contour!!");
                            p.addStepAtEnd(
                                RuleAddContour.getRule(),
                                rws,
                                newEnd);
                            continue stepLoop;
                        }
                    }
                }
            }
            ruleSearch : for (
                int ruleAttempt = 0; ruleAttempt < 10; ruleAttempt++) {
                //System.out.println("there are "+numberOfRules+" rules");

                Vector parts = d.getParts();
                int numberOfParts = parts.size();
                // choose part to apply a rule to
                Diagram part =
                    (Diagram) parts.get((int) (Math.random() * numberOfParts));
                if (!(part instanceof UnitaryDiagram)) {
                    part =
                        (Diagram) parts.get(
                            (int) (Math.random() * numberOfParts));
                }
                //System.out.println("seeking a rule to apply...");
                int ruleNumber = (int) ((Math.random() * (numberOfRules)));
                //System.out.println("rule " + ruleNumber);
                if (ruleNumber < rulesWithChoice.length) {
                    //System.out.println("rwc number "+ruleNumber);
                    RuleWithChoice rwc = rulesWithChoice[ruleNumber];
                    if (!rwc.isInvertible() && d.containsNegationOf(part)) {
                        continue ruleSearch;
                    }
                    Vector specs = rwc.getApplicationPossibilities(part);
                    int numWays = specs.size();
                    if (numWays == 0) {
                        continue ruleSearch;
                    } else {
                        int wayNumber = (int) (Math.random() * numWays);
                        RuleApplicationSpec ras =
                            (RuleApplicationSpec) specs.get(wayNumber);
                        Diagram newEnd = rwc.applyRuleToPart(d, ras);
                        boolean foundmatch = false;
                        p.goToStart();
                        if (p.getCurrentDiagram().equals(newEnd)) {
                            foundmatch = true;
                        } else {
                            diagramScan : while (p.goToNextStep()) {
                                if (p.getCurrentDiagram().equals(newEnd)) {
                                    foundmatch = true;
                                    break diagramScan;
                                }
                            }
                        }
                        if (foundmatch) {
                            continue ruleSearch;
                        } else {
                            p.addStepAtEnd(rwc, ras, newEnd);
                            //System.out.println(
                            //	"" + rwc + " applied");
                            //System.out.print("" + ruleNumber);
                            break ruleSearch;
                        }
                    }
                } else {
                    ruleNumber = ruleNumber - rulesWithChoice.length;
                    //System.out.println("rns number "+ruleNumber);
                    RuleNoSpec rns = rulesNoChoice[ruleNumber];
                    boolean applies = rns.canApplyTo(part);
                    if (!applies) {
                        continue ruleSearch;
                    } else {
                        RuleApplicationSpec ras =
                            new RuleAppliesToJustADiagram(part);
                        Diagram newEnd = rns.applyRuleToPart(d, ras);
                        boolean foundmatch = false;
                        p.goToStart();
                        if (p.getCurrentDiagram().equals(newEnd)) {
                            foundmatch = true;
                        } else {
                            diagramScan : while (p.goToNextStep()) {
                                if (p.getCurrentDiagram().equals(newEnd)) {
                                    foundmatch = true;
                                    break diagramScan;
                                }
                            }
                        }
                        if (foundmatch) {
                            continue ruleSearch;
                        } else {
                            //System.out.println("apply");                        
                            p.addStepAtEnd(rns, ras, newEnd);
                            //System.out.println(
                            //	"" + rwc + " applied");
                            //System.out.print("" + ruleNumber);
                            break ruleSearch;
                        }
                    }
                }
            }
        }
        return p;
    }
    public static void main(String args[]) {
        // build a proof generator
        Proof p = new ProofGetterSmall(3, 3, 3, 4,3,3).getProof(20);
        //System.out.println("proof is "+ProofFileUser.toSaveString(p));
        new ProofViewer(p, false, false).show();
        System.out.println("done OK");
    }

}
