package rwd.rules;

import java.util.Vector;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.choicespec.*;
import rwd.rules.inputspec.*;
import rwd.rules.nospec.*;
import rwd.abstractsyntax.spiderdiagram.CompoundDiagram;

import java.io.Serializable;
/**
 * A rule for a diagram.
 * 
 * @author RWD,rnb
 * 
 * @version 23.5.05  20:01
 *
 */
public abstract class Rule implements Serializable {
    public Rule(String name) {
        this.name = name;
    }
    
    private static Vector allConstraintDiagramRules = new Vector();
    private static Vector allSpiderDiagramRules = new Vector();
    private static Vector allRules = null;
    
    static{
        // Constraint diagrams
    	allConstraintDiagramRules.add(rwd.rules.constraintdiagrams.RuleDeleteExistentialSpider.getRule());
    	allConstraintDiagramRules.add(rwd.rules.constraintdiagrams.RuleDeleteArrow.getRule());
    	allConstraintDiagramRules.add(rwd.rules.constraintdiagrams.RuleMoveBranch.getRule());
    	allConstraintDiagramRules.add(rwd.rules.constraintdiagrams.RuleUnshadeZone.getRule());
    	allConstraintDiagramRules.add(rwd.rules.constraintdiagrams.RuleAddSpiderFoot.getRule());

        // Spider diagrams
        allSpiderDiagramRules.add(FalseWithAndRule.getRule());
        allSpiderDiagramRules.add(FalseWithOrRule.getRule());
        allSpiderDiagramRules.add(InvertRemoveNotRule.getRule());
        allSpiderDiagramRules.add(InvertShatterSpidersRule.getRule());
        allSpiderDiagramRules.add(InvertToDNFRule.getRule());
        allSpiderDiagramRules.add(MergeConjunctionRule.getRule());
        allSpiderDiagramRules.add(MergeContradictionRule.getRule());
        allSpiderDiagramRules.add(RuleAddSpiderFoot.getRule());
        allSpiderDiagramRules.add(RuleAndAbsorbtion.getRule());
        allSpiderDiagramRules.add(RuleAndForgetsInfo.getRule());
        allSpiderDiagramRules.add(RuleDeleteContour.getRule());
        allSpiderDiagramRules.add(RuleDeleteContourSimple.getRule());
        allSpiderDiagramRules.add(RuleDeleteShadedZone.getRule());
        allSpiderDiagramRules.add(RuleDeleteSpider.getRule());
        allSpiderDiagramRules.add(RuleDistributeAndOverOr.getRule());
        allSpiderDiagramRules.add(RuleDistributeMergeAnds.getRule());
        allSpiderDiagramRules.add(RuleDistributeMergeOrs.getRule());
        allSpiderDiagramRules.add(RuleDistributeOrOverAnd.getRule());
        allSpiderDiagramRules.add(RuleDMPullNotThroughAnd.getRule());
        allSpiderDiagramRules.add(RuleDMPullNotThroughOr.getRule());
        allSpiderDiagramRules.add(RuleDMPushNotThroughAnd.getRule());
        allSpiderDiagramRules.add(RuleDMPushNotThroughOr.getRule());
        allSpiderDiagramRules.add(RuleExcludedMiddle.getRule());
        allSpiderDiagramRules.add(RuleIntroduceShadedZone.getRule());
        allSpiderDiagramRules.add(RuleJoinSpiders.getRule());
        allSpiderDiagramRules.add(RuleOrAbsorbtion.getRule());
        allSpiderDiagramRules.add(RuleReverseAndIdempotent.getRule());
        allSpiderDiagramRules.add(RuleReverseExcludedMiddle.getRule());
        allSpiderDiagramRules.add(RuleReverseOrIdempotent.getRule());
        allSpiderDiagramRules.add(RuleSplitSpider.getRule());
        allSpiderDiagramRules.add(RuleUnshadeZone.getRule());

        allSpiderDiagramRules.add(RuleAddContour.getRule());
        allSpiderDiagramRules.add(RuleFalseImpliesAnything.getRule());
        allSpiderDiagramRules.add(RulePToPOrRubbish.getRule());
        allSpiderDiagramRules.add(RuleReverseAndAbsorbtion.getRule());
        allSpiderDiagramRules.add(RuleReverseOrAbsorbtion.getRule());
                
        allSpiderDiagramRules.add(AndIdempotentRule.getRule());
        allSpiderDiagramRules.add(InsertNotNotRule.getRule());
        allSpiderDiagramRules.add(NotFalseIsTrueRule.getRule());
        allSpiderDiagramRules.add(NotTrueIsFalseRule.getRule());
        allSpiderDiagramRules.add(OrIdempotentRule.getRule());
        allSpiderDiagramRules.add(RemoveNotNotRule.getRule());
        allSpiderDiagramRules.add(RemoveNotRule.getRule());
        allSpiderDiagramRules.add(RemoveNotRuleComplex.getRule());
        allSpiderDiagramRules.add(RulePToPorF.getRule());
        allSpiderDiagramRules.add(RulePushNotsToLeaves.getRule());
        allSpiderDiagramRules.add(ShatterSpidersRule.getRule());
        allSpiderDiagramRules.add(SimpleCombiningRule.getRule());
        allSpiderDiagramRules.add(ToDNFRule.getRule());
    }
    /**
     * @return list of all spider diagram rules
     */
    public static Vector getAllSDRules(){
        return allSpiderDiagramRules;
    }
    
    /**
     * @return list of all constraint diagram rules
     */
    public static Vector getAllCDRules(){
        return allConstraintDiagramRules;
    }
    
    /**
     * @return list of all rules
     */
    public static Vector getAllRules(){
    	if (allRules == null) {
    		allRules = new Vector();
    		allRules.addAll(allSpiderDiagramRules);
    		allRules.addAll(allConstraintDiagramRules);
    	}	
        return allRules;
    }

    
    /**
     * Applies a spider diagram rule.
     * @param r application spec for rule
     * @return resultant diagram
     */
    public abstract Diagram applyRule(RuleApplicationSpec r);
    
    /**
     * Check applicability of rule for spider diagrams.
     * @param ad diagram to which rule is to be applied
     * @return true iff this rule can be applied to ad
     */
    public abstract boolean canApplyTo(Diagram ad);

    /**
     * The name of the rule, or null if identical to the name of the super-rule
     */
    public String name;

    /**
     * @param whole diagram, part of which is to have rule applied to it
     * @param ras rule application spec
     * @return modified whole
     */
    public Diagram applyRuleToPart(Diagram whole, RuleApplicationSpec ras) {
        if (!isInvertible() && whole.containsNegationOf(ras.d)) {
            System.out.println(
                "applying a noninvertible rule to a negated part");
            System.out.println("whole is " + whole);
            System.out.println("part is " + ras.d);
            System.out.println("ras is " + whole);
            System.out.println("rule is " + this);
            new IllegalArgumentException().printStackTrace();
            System.exit(0);
        }
        Diagram part = ras.d;
        //System.out.println("whole is "+whole+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
        //System.out.println("whole is "+whole.toSaveString());
        //System.out.println("part is "+part+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
        //System.out.println("part is "+part.toSaveString());
        //Vector parentAndPart = whole.findPartLike(part);
        //CompoundDiagram parent = (CompoundDiagram)parentAndPart.get(0);
        CompoundDiagram parent = whole.getParentOf(part);
        if (parent == null) { //System.out.println("parent of part is null");
            //System.out.println("applying rwd.rules "+this+" to "+ras);
            Diagram result = applyRule(ras);
            //System.out.println("returning "+result);
            return result;
        }
        Diagram result = whole.getClone();
        //System.out.println("start of result is "+result.toSaveString());
        //System.out.println("parent is "+parent+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
        //System.out.println("parent is "+parent.toSaveString());
        //System.out.println("part is "+part+"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
        //System.out.println("part is "+part.toSaveString());
        int[] coords = whole.getCoords(parent);
        CompoundDiagram newParent = (CompoundDiagram) result.getPartAt(coords);
        Diagram newd = applyRule(ras);
        //System.out.println("newd is "+newd);
        //System.out.println("parent in result is "+newParent.toSaveString());
        Vector resultingChildren = newParent.getChildren();
        int[] oldPartCoords = whole.getCoords(part);
        Diagram childToReplace = result.getPartAt(oldPartCoords);
        resultingChildren.remove(childToReplace);
        resultingChildren.add(oldPartCoords[oldPartCoords.length - 1], newd);
        //System.out.println("after substituting children, children are \n"
        //                   +resultingChildren);
        newParent.flatten();
        result.renameOutside(new Vector());
        //System.out.println("about to return result"+result);
        return result;
    }
    
    /**
     * @param whole diagram to be checked for applicable part
     * @return part of diagram to which this rule is applicable
     */
    public Diagram findApplicablePart(Diagram whole) {
        if (!isInvertible()) {
            throw new IllegalArgumentException();
        }
        // this method needs checking!!
        if (canApplyTo(whole)) {
            return whole;
        }
        if (whole instanceof CompoundDiagram) {
            CompoundDiagram cd = (CompoundDiagram) whole;
            Vector children = cd.getChildren();
            for (int i = 0; i < children.size(); i++) {
                Diagram child = (Diagram) children.get(i);
                Diagram part = findApplicablePart(child);
                if (part != null) {
                    return part;
                }
            }
        }
        return null;
    }
    
    /**
     * @param whole diagram for which applicable spec is to be found
     * @return rule application spec for part of diagram, or null if none
     */
    public RuleApplicationSpec findApplicableSpecForPart(Diagram whole) {
        try{
            Diagram part = findApplicablePart(whole);
            if(part==null){
                return null;
            } else {
                return findApplicableSpec(part);                
            }            
        } catch(IllegalArgumentException x){
            return null;
        }
    }
    
    /**
     * @param part diagram to which this rule is applicable
     * @return rule application spec for part, or null if none
     */
    public abstract RuleApplicationSpec findApplicableSpec(Diagram part);
    
    /**
     * @return save string for this rule
     */
    public String toSaveString() {
        return toString();
    }

    /**
     * @return string representation of this rule
     */
    public String toString() {
    	if (name == null && name.length() == 0) {
    		return super.toString();
    	} else {
    		return name;
    	}
    }
    
    /**
     * @return true iff this rule is invertible
     */
    public abstract boolean isInvertible();
    
    protected java.util.List args;
    
    protected void setArgs(java.util.List args) {
    	this.args = args;
    }    
}
