package rwd.rules;

import java.util.Vector;

import rwd.abstractsyntax.spiderdiagram.*;
import rwd.abstractsyntax.eulerdiagram.*;

/**
 * @author jaf28
 *
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates.
 * To enable and disable the creation of type comments go to
 * Window>Preferences>Java>Code Generation.
 */
public class UnitaryDiagramGetterRandom implements UnitaryDiagramGetter {
	private Vector contourSet;
	private Vector feasibleZoneSets;
	private int numberFeasibleZoneSets;
	private int maxNumSpiders;
	boolean absNumSpiders = false;
	private boolean randomShading;

	public UnitaryDiagramGetterRandom(int numContours, int maxNumSpiders, boolean absNumSpiders, boolean randomShading) {
		this.absNumSpiders = absNumSpiders;
		this.maxNumSpiders = maxNumSpiders;
		this.randomShading = randomShading;
		// build contourSet
		contourSet = new Vector();
		if (numContours > 0) {
			contourSet.add(Contour.getContour("a"));
		}
		if (numContours > 1) {
			contourSet.add(Contour.getContour("b"));
		}
		if (numContours > 2) {
			contourSet.add(Contour.getContour("c"));
		}
		if (numContours > 3) {
			contourSet.add(Contour.getContour("d"));
		}
		if (numContours > 4) {
			System.out.println("too many contours to handle");
			System.exit(0);
		}
		// build feasible zone sets (including outside zone)
		Vector allZoneSet = UnitaryDiagram.allPossibleZones(contourSet);
		//set of sets of contours
		Zone emptyZone = Zone.getZone(new Vector(), contourSet);
		allZoneSet.remove(emptyZone);
		//System.out.println(
		//	"There are " + allZoneSet.size() + " non-outside zones");
		Vector possibleZoneSets = Utilities.powerSet(allZoneSet);
		//System.out.println(
		//	"There are " + possibleZoneSets.size() + " possible zone sets");
		feasibleZoneSets = new Vector();
		// scan through each possible zone set and discard ones with insufficiant zones
		for (int i = 0; i < possibleZoneSets.size(); i++) {
			Vector possibleZoneSet = (Vector) possibleZoneSets.get(i);
			Vector contoursNotContainingSomething =
				(Vector) contourSet.clone();
			for (int j = 0; j < possibleZoneSet.size(); j++) {
				contoursNotContainingSomething.removeAll(
					((Zone) possibleZoneSet.get(j)).getContainingContours());
			}
			if (contoursNotContainingSomething.size() == 0) {
				possibleZoneSet.add(emptyZone);
				feasibleZoneSets.add(possibleZoneSet);
			}
		}
		numberFeasibleZoneSets = feasibleZoneSets.size();
	}

	public UnitaryDiagramGetterRandom(int numContours, int maxNumSpiders) {
		this(numContours, maxNumSpiders, false, false);
	}

	public UnitaryDiagram getDiagram() {
		// choose a zone set
		Vector zones =
			(Vector) feasibleZoneSets.get(
				(int) (Math.random() * numberFeasibleZoneSets));
		UnitaryDiagram d = new UnitaryDiagram(zones, "d1");
		//System.out.println("done zones");

		// build feasible regions
		Vector regions = Utilities.powerSet(zones);
		int numberRegions = regions.size();

		// choose a region for shading
		if (randomShading) {
			Vector sZones =
				(Vector) regions.get((int) (Math.random() * numberRegions));
			d.setShadedZones(sZones);
		}
		//System.out.println("done shading");

		int numSpiders;
		if (absNumSpiders) {
			numSpiders = maxNumSpiders;
		} else {
			numSpiders = (int) (Math.random() * maxNumSpiders);
		}
		//System.out.println("making "+numSpiders+" spiders");

		// choose regions for habitats
		for (int si = 0; si < numSpiders; si++) {
			//System.out.println("get habitat for "+si+"th spider");
			Vector habitat = new Vector();
			while (habitat.size() == 0) {
				habitat =
					(Vector) regions.get(
						(int) (Math.random() * numberRegions));
			}
			d.addSpider(habitat);
			//System.out.println("d has "+d.getSpiders().size()+" spiders");
		}
		// return diagram
		//System.out.println("returning "+DiagramFileUser.toSaveString(d));
		return d;
	}

	public static void main(String args[]) {
		// build a diagram generator
		UnitaryDiagramGetterRandom rdg = new UnitaryDiagramGetterRandom(3, 3);
		for (int i = 0; i < 100000; i++) {
			// get random diagrams
			Diagram d = rdg.getDiagram();
			// save them to file
			DiagramFileUser.addDiagramFreshFile(d, "testingtesting.junk");
			// read it back
			Diagram d2 =
				DiagramFileUser.getDiagramFromStart("testingtesting.junk");
			DiagramFileUser.addDiagramAppend(d, "testingtesting2.junk");
			// read it back
			Diagram d3 =
				DiagramFileUser.getDiagramFromWherever("testingtesting2.junk");
			if (!d.equals(d2) || !d.equals(d3)) {
				throw new IllegalArgumentException();
			} else {
				System.out.println("" + i + " match OK");
			}

		}
		System.out.println("done OK");
	}

}
