package rwd.rules.autoproof;
import rwd.abstractsyntax.spiderdiagram.*;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import java.util.Vector;

import rwd.rules.*;
import rwd.rules.Proof;
import rwd.rules.Rule;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.rules.application.RuleAppliesToDiagramAndDiagrams;
import rwd.rules.application.RuleAppliesToZone;
import rwd.rules.choicespec.RuleUnshadeZone;
import rwd.rules.choicespec.RuleDeleteShadedZone;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.gui.ProofManager;
import rwd.rules.application.RuleAppliesToSpider;
import rwd.rules.choicespec.RuleDeleteSpider;
import rwd.rules.application.RuleAppliesToDiagramAndDiagram;
import rwd.rules.inputspec.RuleFalseImpliesAnything;
import rwd.rules.choicespec.RuleExcludedMiddle;
import rwd.rules.choicespec.RuleIntroduceShadedZone;
import rwd.rules.choicespec.RuleReverseOrIdempotent;
import rwd.rules.inputspec.*;

public class AutoProofWriterWorse {
    public static Proof writeProof(Diagram premis, Diagram conclusion)
        throws NoProofException {
            
        Vector leaves = new Vector();
        leaves.addAll(premis.getParts());
        leaves.addAll(conclusion.getParts());
        for(int i=0; i<leaves.size(); i++){
            Diagram leaf = (Diagram)leaves.get(i);
            if(leaf instanceof NotCompoundDiagram){
                throw new IllegalArgumentException();
            }
        }
        //System.out.println("task is from ");
        //System.out.println("" + DiagramFileUser.toSaveString(premis));
        //System.out.println("task is to ");
        //System.out.println("" + DiagramFileUser.toSaveString(conclusion));

        //System.out.print("go through to betarise with premis...");
        Proof start = startToBetarised(premis, conclusion);
        //new ProofViewer(start, false).show();
        //System.out.println("done");
        //System.out.print("go through to betarise and invert conclusion...");
        //System.out.println("start to betarised is "+ProofFileUser.toSaveString(startToBetarised(conclusion, premis)));
        Proof end =
            ProofInverter.invertProof(startToBetarised(conclusion, premis));
        //new ProofViewer(end, false).show();
        //if(1==1){return end; }
        //System.out.println("done");
        Diagram lhs = start.getLastDiagram();
        Diagram rhs = end.getFirstDiagram();
        //System.out.print("building numerical arrays for rhs ...");
        // work out array of ints for zones in rhs
        Vector rhsParts = rhs.getUnitaryParts();
        UnitaryDiagram rhsUPart = (UnitaryDiagram) rhsParts.get(0);
        Vector rhsCs = null;
        Vector zones = null;
        try {
            rhsCs = rhsUPart.getCloneOfContourSet();
        } catch (NullPointerException x) {
            //System.out.println("rhsCs is null!!");
            //System.out.println("rhsUParts is "+rhsParts);
            zones = new Vector();
            //System.exit(0);
        }
        if (zones == null) {
            zones = UnitaryDiagram.allPossibleZones(rhsCs);
        }
        // use these araylists to build numerical arrays representing the diagrams
        //!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
        int[][] rhsPresentZones = new int[zones.size()][rhsParts.size()];
        int[][] rhsShadedZones = new int[zones.size()][rhsParts.size()];
        int[][] rhsSpiderCountZones = new int[zones.size()][rhsParts.size()];
        for (int i = 0; i < zones.size(); i++) {
            Zone z = (Zone) zones.get(i);
            for (int j = 0; j < rhsParts.size(); j++) {
                UnitaryDiagram rhsPart = (UnitaryDiagram) rhsParts.get(j);
                if (!rhsPart.getCloneOfZoneSet().contains(z)) {
                    rhsPresentZones[i][j] = 0; // zone is absent
                    rhsShadedZones[i][j] = 0; // zone is not shaded
                    rhsSpiderCountZones[i][j] = 0; // zone has no spiders
                } else {
                    rhsPresentZones[i][j] = 1; // zone is present
                    if (rhsPart.containsShaded(z)) {
                        rhsShadedZones[i][j] = 1; // zone is shaded
                    } else {
                        rhsShadedZones[i][j] = 0; // zone is not shaded
                    }
                    rhsSpiderCountZones[i][j] = 0;
                    for (int k = 0; k < rhsPart.getSpiders().size(); k++) {
                        Spider s = (Spider) rhsPart.getSpiders().get(k);
                        if (s.getHabitat().contains(z)) {
                            rhsSpiderCountZones[i][j]++;
                        }
                    }
                }
            }
        }
        // for each zone work out
        // a number of the max number of spiders in a shaded zone on rhs
        int[] maxNumSpidersInShadedZone = new int[zones.size()];
        for (int i = 0; i < maxNumSpidersInShadedZone.length; i++) {
            maxNumSpidersInShadedZone[i] = 0;
        }

        for (int i = 0; i < zones.size(); i++) {
            for (int j = 0; j < rhsParts.size(); j++) {
                if (rhsSpiderCountZones[i][j] > maxNumSpidersInShadedZone[i]
                    && rhsShadedZones[i][j] == 1) {
                    maxNumSpidersInShadedZone[i] = rhsSpiderCountZones[i][j];
                }
            }
        }
        //System.out.println("done");

        //System.out.println("list RHS parts");
        //rhsParts = rhs.getUnitaryParts();
        //for (int j = 0; j < rhsParts.size(); j++) {
        //	Diagram rhsPart = (Diagram)rhsParts.get(j);
        //	System.out.println(
        //		""
        //			+ j
        //			+ "th rhs part is "
        //			+ DiagramFileUser.toSaveString(rhsPart));
        //}
        //System.out.println("end list of RHS parts");
        //////////////////////////////////////////////////////////////////////
        // now go through each lhs diagram, each unshaded zone in lhs
        // and, if it has fewer or equal spiders than maxNumSpidersInShadedZone
        // then apply excluded middle and start again
        //System.out.print("apply excluded middle after consultation with rhs ...");
        ExcludedMiddleLoop : while (true) {
            Vector lhsParts = lhs.getUnitaryParts();
            //System.out.println("in excluded middle loop");
            for (int i = 0; i < zones.size(); i++) {
                Zone z = (Zone) zones.get(i);
                int targetToBeat = maxNumSpidersInShadedZone[i];
                for (int j = 0; j < lhsParts.size(); j++) {
                    UnitaryDiagram ud = (UnitaryDiagram) lhsParts.get(j);
                    if (ud.getCloneOfZoneSet().contains(z)
                        && !ud.containsShaded(z)) { // count spiders in ud's z
                        int num = 0;
                        Vector spiders = ud.getSpiders();
                        for (int k = 0; k < spiders.size(); k++) {
                            Spider s = (Spider) spiders.get(k);
                            if (s.getHabitat().contains(z)) {
                                num++;
                            }
                        }
                        if (num <= targetToBeat) { // apply excludedMiddle
                            Rule rem = RuleExcludedMiddle.getRule();
                            RuleAppliesToZone raz =
                                new RuleAppliesToZone(ud, z);
                            lhs = rem.applyRuleToPart(lhs, raz);
                            start.addStepAtEnd(rem, raz, lhs);
                            continue ExcludedMiddleLoop;
                        }
                    }
                }
            }
            break ExcludedMiddleLoop;
        }
        //new ProofViewer(start,false).show();
        //System.out.println("list LHS parts");
        Vector lhsParts = lhs.getUnitaryParts();
        //for (int j = 0; j < lhsParts.size(); j++) {
        //	Diagram lhsPart = (Diagram)lhsParts.get(j);
        //	System.out.println(
        //		""
        //			+ j
        //			+ "th lhs part is "
        //			+ DiagramFileUser.toSaveString(lhsPart));
        //}
        //System.out.println("end list of LHS parts");
        //if(1==1){return start; }
        //System.out.println("done");
        //System.out.println("exited excluded middle loop");
        // now lhs unitary parts should, one by one, map onto rhs unitary parts
        //System.out.print("find match for each lhs part ...");
        boolean[] haveMatchedOnto = new boolean[rhsParts.size()];
        for (int i = 0; i < rhsParts.size(); i++) {
            haveMatchedOnto[i] = false;
        }

        boolean[] willBeDuplicate = new boolean[lhsParts.size()];
        for (int i = 0; i < lhsParts.size(); i++) {
            willBeDuplicate[i] = false;
        }

        lhsLoop : for (
            int k = 0;
                k < lhsParts.size();
                k++) { //System.out.println("in reduce loop");
            lhsParts = lhs.getUnitaryParts();
            UnitaryDiagram lhsPart = (UnitaryDiagram) lhsParts.get(k);
            //System.out.println("got an lhs part:"+DiagramFileUser.toSaveString(lhsPart));
            if (lhsPart instanceof FalseDiagram) {
                //System.out.println("dealing with a false component on lhs");
                //seek an rhs partner which can be obtained by
                // intro/erase shaded zones,
                // reducing shading
                // removing spiders in lhs
                rhsSearch : for (int j = 0; j < rhsParts.size(); j++) {
                    //System.out.println("seeking partner for lhs false");
                    //System.out.println(DiagramFileUser.toSaveString(lhsPart));
                    // seek
                    // rhs is false
                    UnitaryDiagram rhsPartTmp =
                        (UnitaryDiagram) rhsParts.get(j);
                    //System.out.println("rhstmp is "+DiagramFileUser.toSaveString(rhsPartTmp));
                    if (!(rhsPartTmp instanceof FalseDiagram)) {
                        continue rhsSearch;
                    }
                    //System.out.println(
                    //	"found match for "
                    //		+ k
                    //		+ "th lhs (false) diagram into "
                    //		+ j
                    //		+ "th rhs (false) one");
                    //System.out.println("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
                    //System.out.println("lhs part is   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
                    //System.out.println(""+((UnitaryDiagram)lhsParts.get(k)).toSaveString());
                    if (haveMatchedOnto[j]) {
                        willBeDuplicate[k] = true;
                        //System.out.println("FIRST DUPLICATE FOR ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
                        //System.out.println(""+((UnitaryDiagram)(rhsParts.get(j))).toSaveString());
                    } else {
                        haveMatchedOnto[j] = true;
                        //System.out.println("FIRST MATCH FOR ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
                        //System.out.println(""+((UnitaryDiagram)(rhsParts.get(j
                        //  ))).toSaveString());
                    }
                    continue lhsLoop;
                } // end of rhsPart search
                // if you get here, we had false lhs part but no rhs part was false.
                //System.out.println(
                //	"need to change "
                //		+ k
                //		+ "th lhs (false) diagram into some rhs (non-false) one");
                Rule rfia = RuleFalseImpliesAnything.getRule();
                RuleAppliesToDiagramAndDiagram radd =
                    new RuleAppliesToDiagramAndDiagram(
                        lhsPart,
                        (Diagram) rhsParts.get(0));
                int[] coords = lhs.getCoords(lhsPart);
                lhs = (UnitaryDiagram) rfia.applyRuleToPart(lhs, radd);
                lhsPart = (UnitaryDiagram) lhs.getPartAt(coords);
                start.addStepAtEnd(rfia, radd, lhs);
                //new ProofViewer(start,false).show();
                if (haveMatchedOnto[0]) {
                    willBeDuplicate[k] = true;
                    //System.out.println("FIRST DUPLICATE FOR ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
                    //System.out.println(""+((UnitaryDiagram)(rhsParts.get(j))).toSaveString());
                } else {
                    haveMatchedOnto[0] = true;
                    //System.out.println("FIRST MATCH FOR ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
                    //System.out.println(""+((UnitaryDiagram)(rhsParts.get(j
                    //  ))).toSaveString());
                }
                continue lhsLoop;
            } else {

                int[] lhsPresentZones = new int[zones.size()];
                int[] lhsShadedZones = new int[zones.size()];
                int[] lhsSpiderCountZones = new int[zones.size()];
                for (int i = 0; i < zones.size(); i++) {
                    Zone z = (Zone) zones.get(i);
                    if (!lhsPart.getCloneOfZoneSet().contains(z)) {
                        lhsPresentZones[i] = 0; // zone is absent
                        lhsShadedZones[i] = 0; // zone is not shaded
                        lhsSpiderCountZones[i] = 0; // zone has no spiders
                    } else {
                        lhsPresentZones[i] = 1; // zone is present
                        if (lhsPart.containsShaded(z)) {
                            lhsShadedZones[i] = 1;
                        } // zone is shaded
                        else {
                            lhsShadedZones[i] = 0;
                        } // zone is not shaded
                        lhsSpiderCountZones[i] = 0;
                        for (int l = 0; l < lhsPart.getSpiders().size(); l++) {
                            Spider s = (Spider) lhsPart.getSpiders().get(l);
                            if (s.getHabitat().contains(z)) {
                                lhsSpiderCountZones[i]++;
                            }
                        }
                    }
                } // counted all data for lhsPart
                //seek an rhs partner which can be obtained by
                // intro/erase shaded zones,
                // reducing shading
                // removing spiders in lhs
                rhsSearch : for (int j = 0; j < rhsParts.size(); j++) {

                    //System.out.println("seeking partner for lhs part");
                    //System.out.println(DiagramFileUser.toSaveString(lhsPart));
                    // chec
                    // 1. the sets Srhs, Slhs of ((shaded and touched) or unshaded+present)
                    // match
                    // (this means the difference is in shaded untouched / absent)
                    // 2. the shaded zones in rhs are all shaded in lhs
                    // 3. # spiders in lhs >= # spiders in rhs
                    // all these checks should be done at the numerical array level
                    for (int i = 0;
                        i < zones.size();
                        i++) { // shaded+touched or unshaded+present in lhs = rhs
                        if (((rhsShadedZones[i][j] == 1
                            && rhsSpiderCountZones[i][j] > 0)
                            || rhsShadedZones[i][j] == 0
                            && rhsPresentZones[i][j] == 1)
                            != ((lhsShadedZones[i] == 1
                                && lhsSpiderCountZones[i] > 0)
                                || lhsShadedZones[i] == 0
                                && lhsPresentZones[i] == 1)) {
                            //System.out.println("shaded+empty /absent mismatch :");
                            //System.out.println(""+k+"th lhsPart is "+((UnitaryDiagram)lhsParts.get(k)).toSaveString());
                            //System.out.println(""+j+"th rhsPart is "+((UnitaryDiagram)rhsParts.get(j)).toSaveString());
                            continue rhsSearch;
                        }
                        if (rhsShadedZones[i][j] == 1
                            && lhsShadedZones[i] == 0) {
                            //System.out.println("more shading on right than left :");
                            //System.out.println(""+k+"th lhsPart is "+((UnitaryDiagram)lhsParts.get(k)).toSaveString());
                            //System.out.println(""+j+"th rhsPart is "+((UnitaryDiagram)rhsParts.get(j)).toSaveString());
                            continue rhsSearch;
                        }
                        if (rhsSpiderCountZones[i][j]
                            > lhsSpiderCountZones[i]) {
                            //System.out.println("more spiders on right than left :");
                            //System.out.println(""+k+"th lhsPart is "+((UnitaryDiagram)lhsParts.get(k)).toSaveString());
                            //System.out.println(""+j+"th rhsPart is "+((UnitaryDiagram)rhsParts.get(j)).toSaveString());
                            continue rhsSearch;
                        }
                    }
                    // all previous 3 tests pass - can change lhsPart into rhsPart now
                    // 1. introduce shaded zones which are present in rhs but absent in lhs
                    // 2. erase (empty) shaded zones in lhs but not in rhs
                    // 3. unshade zones which are shaded in lhs but unshaded in rhs
                    // 4. remove spiders feet to compensate for reduction lhs->rhs
                    //
                    //System.out.println("found rhs part OK");
                    for (int i = 0; i < zones.size(); i++) {
                        if (rhsShadedZones[i][j] == 1
                            && lhsPresentZones[i] == 0) {
                            Rule risz = RuleIntroduceShadedZone.getRule();
                            RuleAppliesToZone raz =
                                new RuleAppliesToZone(
                                    lhsPart,
                                    (Zone) zones.get(i));
                            int[] coords = lhs.getCoords(lhsPart);
                            lhs =
                                (UnitaryDiagram) risz.applyRuleToPart(lhs, raz);
                            lhsPart = (UnitaryDiagram) lhs.getPartAt(coords);
                            start.addStepAtEnd(risz, raz, lhs);
                        } else if (
                            rhsPresentZones[i][j] == 0
                                && lhsShadedZones[i] == 1) {
                            Rule rdsz = RuleDeleteShadedZone.getRule();
                            RuleAppliesToZone raz =
                                new RuleAppliesToZone(
                                    lhsPart,
                                    (Zone) zones.get(i));
                            int[] coords = lhs.getCoords(lhsPart);
                            Diagram tmp = rdsz.applyRuleToPart(lhs, raz);
                            //System.out.println("tmp is "+tmp);
                            lhs = tmp;
                            lhsPart = (UnitaryDiagram) lhs.getPartAt(coords);
                            start.addStepAtEnd(rdsz, raz, lhs);
                        } else if (
                            lhsShadedZones[i] == 1
                                && rhsShadedZones[i][j] == 0) {
                            Rule ruz = RuleUnshadeZone.getRule();
                            RuleAppliesToZone raz =
                                new RuleAppliesToZone(
                                    lhsPart,
                                    (Zone) zones.get(i));
                            int[] coords = lhs.getCoords(lhsPart);
                            lhs = ruz.applyRuleToPart(lhs, raz);
                            lhsPart = (UnitaryDiagram) lhs.getPartAt(coords);
                            start.addStepAtEnd(ruz, raz, lhs);
                        }
                        for (int c = rhsSpiderCountZones[i][j];
                            c < lhsSpiderCountZones[i];
                            c++) {
                            Rule rds = RuleDeleteSpider.getRule();
                            Spider s = null;
                            for (int l = 0;
                                l < lhsPart.getSpiders().size();
                                l++) {
                                s = (Spider) lhsPart.getSpiders().get(l);
                                if (s.getHabitat().contains(zones.get(i))) {
                                    break;
                                }
                            }
                            RuleAppliesToSpider ras =
                                new RuleAppliesToSpider(lhsPart, s);
                            int[] coords = lhs.getCoords(lhsPart);
                            lhs = rds.applyRuleToPart(lhs, ras);
                            lhsPart = (UnitaryDiagram) lhs.getPartAt(coords);
                            start.addStepAtEnd(rds, ras, lhs);
                        }
                    } // end of zone loop for reduction
                    // the lhsPart is now changed : lhs is changed too
                    //System.out.println(
                    //	"turned the "
                    //		+ k
                    //		+ "th lhs diagram into "
                    //		+ j
                    //		+ "th rhs one");
                    //System.out.println("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
                    //System.out.println("lhs part is   ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
                    //System.out.println(""+((UnitaryDiagram)lhsParts.get(k)).toSaveString());
                    if (haveMatchedOnto[j]) {
                        willBeDuplicate[k] = true;
                        //System.out.println("FIRST DUPLICATE FOR ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
                        //System.out.println(""+((UnitaryDiagram)(rhsParts.get(j))).toSaveString());
                    } else {
                        haveMatchedOnto[j] = true;
                        //System.out.println("FIRST MATCH FOR ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
                        //System.out.println(""+((UnitaryDiagram)(rhsParts.get(j
                        //  ))).toSaveString());
                    }
                    continue lhsLoop;
                } // end of rhsPart search
            }

            // if you get here then no rhsPart matched onto the lhsPart
            //System.out.println("there is no proof - lhsPart is ");
            //System.out.println("build c/ex from " + DiagramFileUser.toSaveString(lhsPart));
            throw new NoProofException(lhsPart);

        } // end of lhsParts scan
        //System.out.println("now remove duplicates from lhs");
        // now all lhsparts have been changed
        // use willBeDuplicate array to remove some: PorP->P
        //System.out.println("done");
        //System.out.print("remove and duplicate matching lhs parts ...");
        lhsParts = lhs.getUnitaryParts();
        //System.out.println("lhs has "+lhsParts.size()+" children");
        //System.out.println("rhs has "+rhsParts.size()+" children");
        Vector thingsToRemove = new Vector();
        for (int i = 0; i < lhsParts.size(); i++) {
            if (willBeDuplicate[i]) {
                thingsToRemove.add(lhsParts.get(i));
            }
        }
        Vector thingsToAdd = new Vector();
        for (int i = 0; i < rhsParts.size(); i++) {
            if (!haveMatchedOnto[i]) {
                thingsToAdd.add(rhsParts.get(i));
            }
        }
        // pair off equal things to remove and things to add
        Vector dontNeedToRemove = new Vector();
        Vector dontNeedToAdd = new Vector();
        removeLoop : for (int i = 0; i < thingsToRemove.size(); i++) {
            UnitaryDiagram toRemove = (UnitaryDiagram) thingsToRemove.get(i);
            for (int j = 0; j < thingsToAdd.size(); j++) {
                UnitaryDiagram toAdd = (UnitaryDiagram) thingsToAdd.get(j);
                if (toRemove.equals(toAdd) && !dontNeedToAdd.contains(toAdd)) {
                    dontNeedToRemove.add(toRemove);
                    dontNeedToAdd.add(toAdd);
                    continue removeLoop;
                }
            }
        }
        thingsToRemove.removeAll(dontNeedToRemove);
        thingsToAdd.removeAll(dontNeedToAdd);
        //System.out.println("must remove "+thingsToRemove.size()+" things");
        for (int i = 0; i < thingsToRemove.size(); i++) {
            UnitaryDiagram toRemove = (UnitaryDiagram) thingsToRemove.get(i);
            Vector children = new Vector(); // will find 2 matches
            boolean hasFoundFirst = false;
            for (int j = 0;
                j < ((OrCompoundDiagram) lhs).getChildren().size();
                j++) {
                UnitaryDiagram child =
                    (UnitaryDiagram) (((OrCompoundDiagram) lhs)
                        .getChildren()
                        .get(j));
                if (child.equals(toRemove)) {
                    children.add(child);
                    if (hasFoundFirst) {
                        break;
                    } // arraylist children number 2 already
                    else {
                        hasFoundFirst = true;
                    }
                }
            }
            //System.out.println("identify "+children.size()+" children");
            Rule rroi = RuleReverseOrIdempotent.getRule();
            //System.out.println("5. provides d,[d2,d3,..]");
            RuleAppliesToDiagramAndDiagrams radd =
                new RuleAppliesToDiagramAndDiagrams(lhs, children);
            lhs = rroi.applyRuleToPart(lhs, radd);
            start.addStepAtEnd(rroi, radd, lhs);
        } // end of thingsToRemove loop
        //System.out.println("now add extras to lhs");
        //System.out.println("must add "+thingsToAdd.size()+" things");
        for (int i = 0; i < thingsToAdd.size(); i++) {
            UnitaryDiagram toAdd = (UnitaryDiagram) thingsToAdd.get(i);
            Rule rptprr = RulePToPOrRubbish.getRule();
            RuleAppliesToDiagramAndDiagram radd =
                new RuleAppliesToDiagramAndDiagram(lhs, toAdd);
            lhs = rptprr.applyRuleToPart(lhs, radd);
            start.addStepAtEnd(rptprr, radd, lhs);
        } // end of thingsToRemove loop
        //System.out.println("after addition, lhs has "+lhsParts.size()+" children");
        //System.out.println("done: about to concatenate start to end");
        start = concatenateProofs(start, end);
        //System.out.println("time elapsed :"+(int)((startTime.getTime()-new Date().getTime())/1000));
        return start;
    }

    private static Proof concatenateProofs(Proof p1, Proof p2) {
        Proof result = p1.cloneProof();
        if (!result.getLastDiagram().equals(p2.getFirstDiagram())) {
            System.out.println(
                "result.getLastDiagram() is "
                    + DiagramFileUser.toSaveString(result.getLastDiagram()));
            System.out.println(
                "p2.getFirstDiagram() is "
                    + DiagramFileUser.toSaveString(p2.getFirstDiagram()));
            //CompoundDiagram cd1 = (CompoundDiagram)result.getLastDiagram();
            //CompoundDiagram cd2 = (CompoundDiagram)p2.getFirstDiagram();
            //System.out.println(""+cd1.getChildren().size());
            //System.out.println(""+cd2.getChildren().size());
            ProofManager.showNewProof(result,false,true);
            ProofManager.showNewProof(p2,false,true);
            return null;
        }
        while (p1.goToNextStep()) {
        }
        p2.goToStart();
        if (result.trashLastStep()) {
            result.addStepAtEnd(
                p1.getPreviousRule(),
                p1.getPreviousRuleApplicationSpec(),
                p2.getFirstDiagram());
        } else {
            return p2;
        }
        while (p2.goToNextStep()) {
            result.addStepAtEnd(
                p2.getPreviousRule(),
                p2.getPreviousRuleApplicationSpec(),
                p2.getCurrentDiagram());
        }
        return result;
    }

    private static Proof startToBetarised(
        Diagram d,
        Diagram other) { //System.out.print("extending contour set ...");
        Proof result = ContourSetEqualiser.makeContourSetsEqual(d, other);
        //System.out.println("done");
        //System.out.print("equalise zones ...");
        result =
            concatenateProofs(
                result,
                ZoneEqualiser.makeZonesEqual(
                    result.getLastDiagram(),
                    ContourSetEqualiser
                        .makeContourSetsEqual(other, d)
                        .getLastDiagram()));
        //System.out.println("done");
        //System.out.print("splitting spiders ...");
        result =
            concatenateProofs(
                result,
                SlowSpiderSplitter.splitAllSpiders(result.getLastDiagram()));
        //System.out.println("done");
        //System.out.print("turning to DNF ...");
        result =
            concatenateProofs(
                result,
                SlowMakerOfDisNF.turnToDisjNF(result.getLastDiagram()));
        //System.out.println("done");
        //System.out.print("applying 'combine' ...");
        result =
            concatenateProofs(
                result,
                Combiner.combineWherePossible(result.getLastDiagram()));
        //System.out.println("done");
        //System.out.print("removing occurences of False ...");
        result =
            concatenateProofs(
                result,
                FalseRemover.removeFalse(result.getLastDiagram()));
        //System.out.println("done");
        //System.out.print("betarising ...");
        result =
            concatenateProofs(
                result,
                BetaMaker.betarise(result.getLastDiagram()));
        //
        //System.out.println("done");
        return result; // concatenates steps
    }
}
