package rwd.rules.autoproof;

import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.CompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.FalseDiagram;

import java.util.Vector;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.rules.Proof;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.choicespec.RuleIntroduceShadedZone;
import rwd.rules.application.RuleAppliesToZone;

public class ZoneEqualiser {
	public static Proof makeZonesEqual(
		Diagram d,
		Diagram other) { //System.out.println("make zones equal for "+d.toSaveString()
		//                                        +" and "+other.toSaveString());
		Proof p = new Proof(d);
		Vector unitaryPieces = d.getUnitaryParts();
		Vector unionOfZones = new Vector();
		for (int i = 0; i < unitaryPieces.size(); i++) {
			UnitaryDiagram ud = (UnitaryDiagram) unitaryPieces.get(i);
			if (!(ud instanceof FalseDiagram)) {
				Vector zones = ud.getCloneOfZoneSet();
				for (int j = 0; j < zones.size(); j++) {
					Zone z = (Zone) zones.get(j);
					if (!unionOfZones.contains(z)) {
						unionOfZones.add(z);
					}
				}
			}
		}
		unitaryPieces = other.getUnitaryParts();
		for (int i = 0; i < unitaryPieces.size(); i++) {
			UnitaryDiagram ud = (UnitaryDiagram) unitaryPieces.get(i);
			if (!(ud instanceof FalseDiagram)) {
				Vector zones = ud.getCloneOfZoneSet();
				for (int j = 0; j < zones.size(); j++) {
					Zone z = (Zone) zones.get(j);
					if (!unionOfZones.contains(z)) {
						unionOfZones.add(z);
					}
				}
			}
		}
		//System.out.println("union of zones is "+unionOfContours);
		zoneLoop : for (int i = 0; i < unionOfZones.size(); i++) {
			while (true) { //System.out.println("try to add zone "+(Zone)unionOfZones.get(i));
				//System.out.println("to "+d);
				//System.out.println("ie "+d.toSaveString());
				Diagram result = tryToAddZone(d, (Zone) unionOfZones.get(i), p);
				//System.out.println("result is "+result);
				if (result != null) {
					d = result;
					//System.out.println("just added a zone to "+d.toSaveString());
				} else {
					continue zoneLoop;
				}
			}
		}
		return p;
	}

	private static Diagram tryToAddZone(Diagram d, Zone z, Proof p) {
		if (d instanceof UnitaryDiagram) {
			if (d instanceof FalseDiagram) {
				return null;
			}
			UnitaryDiagram ud = (UnitaryDiagram) d;
			if (!ud.getCloneOfZoneSet().contains(z)) { // add z to ud...
				// and substitute back into d... but d = ud
				Rule r = RuleIntroduceShadedZone.getRule();
				RuleApplicationSpec ras = new RuleAppliesToZone(d, z);
				d = r.applyRule(ras);
				p.addStepAtEnd(r, ras, d);
				//System.out.println("added "+z
				//                      +" to diagram has become \n"+d.toSaveString());
				return d;
			}
		} else if (
			d instanceof CompoundDiagram) {
			//System.out.println("getting unitary parts of "+d);
			//System.out.println("getting unitary parts of "+d.toSaveString());
			Vector comps = d.getUnitaryParts();
			compLoop : for (int i = 0; i < comps.size(); i++) {
				UnitaryDiagram ud = (UnitaryDiagram) comps.get(i);
				if (!(ud instanceof FalseDiagram)
					&& !ud.getCloneOfZoneSet().contains(z)) { // add c to ud...
					// and substitute back into d...
					Rule r = RuleIntroduceShadedZone.getRule();
					RuleApplicationSpec ras = new RuleAppliesToZone(ud, z);
					//System.out.println("ud is "+ud.toSaveString());
					//System.out.println("z is "+z);
					Diagram newud = r.applyRule(ras);
					//System.out.println("newud is "+newud);
					//Vector parentAndPart = d.findPartLike(ud);
					//CompoundDiagram parent = (CompoundDiagram)parentAndPart.get(0);
					CompoundDiagram parent = d.getParentOf(ud);
					Vector children = parent.getChildren();
					Diagram result = d.getClone();
					//System.out.println("start of result is "+result.toSaveString());
					int[] coords = d.getCoords(parent);
					CompoundDiagram newParent =
						(CompoundDiagram) result.getPartAt(coords);
					//             (CompoundDiagram)result.findPartLike(parent).get(1);
					//System.out.println("parent in result is "+newParent.toSaveString());
					Vector resultingChildren = newParent.getChildren();
					resultingChildren.clear();
					for (int j = 0; j < children.size(); j++) {
						Diagram child = (Diagram) children.get(j);
						if (child != ud) {
							//System.out.println("add the old child");
							resultingChildren.add(child.getClone());
						} else { //System.out.println("add the new child");
							resultingChildren.add(newud);
						}
					}
					//System.out.println("after substituting children, children are \n"
					//                   +resultingChildren);
					p.addStepAtEnd(r, ras, result);
					//System.out.println("added "+z
					//            +", to diagram \n"+result.toSaveString());
					return result;
				}
			}
		}
		//System.out.println("can't add to diagram");
		return null;
	}
}
