package rwd.rules.choicespec;

import java.util.Vector;

import rwd.abstractsyntax.spiderdiagram.CompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.NotCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleAppliesToDandDandD;
import rwd.rules.application.RuleApplicationSpec;

public class RuleDMPullNotThroughAnd extends RuleWithChoice {
    private RuleDMPullNotThroughAnd() {
        super("sd: notQ and notR -> not(QorR)");
    }
    private static Rule example = null;

    public static Rule getRule() {
        if (example == null) {
            example = new RuleDMPullNotThroughAnd();
        }
        return example;
    }
    public boolean isInvertible() {
        return true;
    }

    public boolean canApplyTo(Diagram d) {
        // search for an AndCompoundDiagram 
        // with two children which are NotCompoundDiagrams        
        if (!(d instanceof AndCompoundDiagram)) {
            return false;
        }
        AndCompoundDiagram acd = (AndCompoundDiagram) d;
        int numberOfNCDs = 0;
        Vector children = acd.getChildren();
        for (int j = 0; j < children.size(); j++) {
            Diagram child = (Diagram) children.get(j);
            if (child instanceof NotCompoundDiagram) {
                numberOfNCDs++;
                if (numberOfNCDs == 2) {
                    return true;
                }
            }
        }
        return false;
    }

    public Vector getApplicationPossibilities(Diagram d) {
        // search for AndCompoundDiagrams
        // search for pairs of children which are NotCompoundDiagrams
        Vector result = new Vector();
        if (!(d instanceof AndCompoundDiagram)) {
            return result;
        }
        AndCompoundDiagram acd = (AndCompoundDiagram) d;
        int numberOfNCDs = 0;
        Vector children = acd.getChildren();
        Vector NCDChildren = new Vector();
        for (int j = 0; j < children.size(); j++) {
            Diagram child = (Diagram) children.get(j);
            if (child instanceof NotCompoundDiagram) {
                NCDChildren.add(child);
            }
        }
        // pick out pairs of NCDChildren and build RASpecs
        for (int j = 0; j < NCDChildren.size(); j++) {
            Diagram childA = (Diagram) NCDChildren.get(j);
            for (int k = j + 1; k < NCDChildren.size(); k++) {
                Diagram childB = (Diagram) NCDChildren.get(k);
                result.add(new RuleAppliesToDandDandD(d, childA, childB));
            }
        }
        return result;
    }

    public Diagram applyRule(RuleApplicationSpec r) {
        try {
            // given d, other1 and other2
            // we should find that other1 and other2 share an And-compoundDiagram
            // as a parent.
            // either parent = other1 and other2
            //  find the grandparent
            //  either its null 
            //     - return not(Other1Child or Other2Child)
            //  or it's an OrDiagram or a NotDiagram
            //     - replace "parent" in d with not(Other1Child or Other2Child)

            // or parent has other components Q1 and Q2 .. and other1 and other2
            //    build Q1 and Q2 and not(Other1Child or Other2Child)
            RuleAppliesToDandDandD raddd = (RuleAppliesToDandDandD) r;
            Diagram d = raddd.d;
            NotCompoundDiagram other1 = (NotCompoundDiagram) raddd.other1;
            NotCompoundDiagram other2 = (NotCompoundDiagram) raddd.other2;
            Diagram parent1 = d.getParentOf(other1);
            Diagram parent2 = d.getParentOf(other2);
            if (parent1 != parent2) {
                throw new IllegalArgumentException();
            }
            Vector justTwoThings = new Vector();
            justTwoThings.add(other1.getChildren().get(0));
            justTwoThings.add(other2.getChildren().get(0));
            OrCompoundDiagram other1Child_or_Other2Child =
                new OrCompoundDiagram(justTwoThings, "ocd");
            Vector justOneThing = new Vector();
            justOneThing.add(other1Child_or_Other2Child);
            NotCompoundDiagram ncd =
                new NotCompoundDiagram(justOneThing, "Not(ocd)");

            int[] parentCoords = d.getCoords(parent1);
            int[] other1Coords = d.getCoords(other1);
            int[] other2Coords = d.getCoords(other2);
            Diagram dClone = d.getClone();
            Diagram parentClone = dClone.getPartAt(parentCoords);
            Diagram other1Clone = dClone.getPartAt(other1Coords);
            Diagram other2Clone = dClone.getPartAt(other2Coords);
            //System.out.println("other1Clone is " + other1Clone);
            //System.out.println("other2Clone is " + other2Clone);
            //System.out.println("parent.getChildren are "+((CompoundDiagram) parent1).getChildren());
            if (((CompoundDiagram) parent1).getChildren().size() == 2) {
                Diagram gparent = d.getParentOf(parent1);
                if (gparent == null) {
                    return ncd;
                } else {
                    Diagram parentOfParentClone =
                        dClone.getParentOf(parentClone);
                    // remove a child and substitute a different one
                    Vector children =
                        ((CompoundDiagram) parentOfParentClone).getChildren();
                    children.remove(parentClone);
                    children.add(ncd);
                    return dClone;
                }
            } else {
                Vector children =
                    ((CompoundDiagram) parentClone).getChildren();
                //System.out.println("children are " + children);
                children.remove(other1Clone);
                //System.out.println("children are " + children);
                children.remove(other2Clone);
                //System.out.println("children are " + children);
                children.add(ncd);
                //System.out.println("children are " + children);
                return parentClone;
            }
        } catch (Exception e) {
            System.out.println("exception thrown : " + e);
            System.exit(0);
            return null;
        }
    }
}
