package rwd.rules.choicespec;
import java.util.Vector;
import java.util.Iterator;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.FalseDiagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleAppliesToContour;
import rwd.rules.application.RuleApplicationSpec;
import rwd.abstractsyntax.spiderdiagram.Spider;

public class RuleDeleteContour extends RuleWithChoice {
	private RuleDeleteContour() {
		super("sd: delete contour (clever)");
	}
	private static Rule example = null;
	public static Rule getRule() {
		if (example == null) {
			example = new RuleDeleteContour();
		}
		return example;
	}
    public boolean isInvertible(){
        return false;
    }

	public boolean canApplyTo(Diagram d) {
        if(!(d instanceof UnitaryDiagram)){
            //System.out.println("DC:isn't unitary");
            return false;
        } else if(d instanceof FalseDiagram){
            //System.out.println("DC:is false");
            return false;
        } else{
            int n =((UnitaryDiagram)d).getCloneOfContourSet().size(); 
            //System.out.println("DC:count contours in "+d+" is "+n);
            return n>0;
        }
	}

	public Vector getApplicationPossibilities(Diagram d) {
		Vector result = new Vector();
		if (d instanceof UnitaryDiagram && !(d instanceof FalseDiagram)) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			Vector contours = ud.getCloneOfContourSet();
			for (int i = 0; i < contours.size(); i++) {
				Contour c = (Contour) (contours.get(i));
				result.add(new RuleAppliesToContour(d, c));
			}
		}
		//System.out.println("returning result with "+result.size()+" options");
		return result;
	}
	public Diagram applyRule(RuleApplicationSpec r) {
		RuleAppliesToContour rac = (RuleAppliesToContour) r;
		Diagram d = rac.d;
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) rac.d;
			Contour c = rac.c;
			Vector newZones = new Vector();
			Vector newShadedZones = new Vector();
			Vector newHabitats = new Vector();
			for (int i = 0; i < ud.getSpiders().size(); i++) {
				newHabitats.add(new Vector());
			}

            Iterator zi = ud.getZoneIterator();
            while(zi.hasNext()){
                Zone oldz = (Zone)zi.next();
				//System.out.println("considering zone "+oldz);
				Vector inContours =
					(Vector) (oldz.getContainingContours().clone());
				Vector outContours =
					(Vector) (oldz.getExcludingContours().clone());
				if (inContours.contains(c)) {
					inContours.remove(c);
				} else {
					outContours.remove(c);
				}
				Zone newz = Zone.getZone(inContours, outContours);
				newZones.add(newz);
				if (ud.containsShaded(oldz)) {
					newShadedZones.add(newz);
				}
				for (int j = 0; j < ud.getSpiders().size(); j++) {
					Spider s = (Spider) (ud.getSpiders().get(j));
					if (s.getHabitat().contains(oldz)) {
						Vector newh = (Vector) (newHabitats.get(j));
						if (!newh.contains(newz)) {
							newh.add(newz);
						}
					}
				}
			}
			// now need to tidy up duplicates from zones
			// and delete from shadedZones if it appeared there less often
			Vector finalZones = new Vector();
			Vector finalShadedZones = new Vector();
			while (newZones.size() > 0) {
				Zone newz = (Zone) (newZones.get(0));
				int count = 0;
				while (newZones.contains(newz)) {
					count++;
					newZones.remove(newz);
				}
				finalZones.add(newz);
				while (newShadedZones.contains(newz)) {
					count--;
					newShadedZones.remove(newz);
				}
				// see whether count>0
				// if count>0 then the zone wasn't always shaded
				if (count == 0) {
					finalShadedZones.add(newz);
				}
			}
			UnitaryDiagram result = new UnitaryDiagram(finalZones, "outcome");
			for (int i = 0; i < finalShadedZones.size(); i++) {
				result.shadeZone((Zone) finalShadedZones.get(i));
			}
			for (int i = 0; i < newHabitats.size(); i++) {
				Vector h = (Vector) newHabitats.get(i);
				result.addSpider(h);
			}
			return result;
		}
		return null;
	}
}
