package rwd.rules.choicespec;
import java.util.Vector;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.rules.Rule;
import rwd.rules.application.*;
import rwd.rules.application.RuleApplicationSpec;

public class RuleDistributeMergeAnds
	extends RuleWithChoice { // takes (P and Q) or (P and R) or ...
	// becomes (P and (Q or R)) or  ...
	// specify (P and Q) and (P and R) and P
	private RuleDistributeMergeAnds() {
		super("sd: (PandQ)or(PandR)or... -> (Pand(QorR))or...");
	}
	private static Rule example = null;

	public static Rule getRule() {
		if (example == null) {
			example = new RuleDistributeMergeAnds();
		}
		return example;
	}
    public boolean isInvertible(){
        return true;
    }

	public boolean canApplyTo(Diagram d) {
		if (!(d instanceof OrCompoundDiagram)) {
			return false;
		}
		OrCompoundDiagram ocd = (OrCompoundDiagram) d;
		// search for a child which is an "or"
		Vector children = ocd.getChildren();
		child1Loop : for (int i = 0; i < children.size(); i++) {
			Diagram child1 = (Diagram) (children.get(i));
			if (!(child1 instanceof AndCompoundDiagram)) {
				continue child1Loop;
			}
			AndCompoundDiagram andChild1 = (AndCompoundDiagram) child1;
			Vector grandchildren1 = andChild1.getChildren();

			child2Loop : for (int j = i + 1; j < children.size(); j++) {
				Diagram child2 = (Diagram) (children.get(j));
				if (!(child2 instanceof AndCompoundDiagram)) {
					continue child2Loop;
				}
				AndCompoundDiagram andChild2 = (AndCompoundDiagram) child2;
				Vector grandchildren2 = andChild2.getChildren();
				// see whether grandchildren1 and grandchildren2 have any common children
				Vector childrenInCommon = new Vector();
				boolean hasMatched[] = new boolean[grandchildren2.size()];
				for (int k = 0; k < hasMatched.length; k++) {
					hasMatched[k] = false;
				}
				for (int k = 0; k < grandchildren1.size(); k++) {
					Diagram gc1 = (Diagram) grandchildren1.get(k);
					findMatch : for (
						int l = 0; l < grandchildren2.size(); l++) {
						if (!hasMatched[l]) {
							Diagram gc2 = (Diagram) grandchildren2.get(l);
							if (gc1.equals(gc2)) {
								childrenInCommon.add(gc1);
								hasMatched[l] = true;
								break findMatch;
							}
						}
					}
				}
				// know commonChildren - find subsets of it
				Vector commonParts =
					rwd.abstractsyntax.spiderdiagram.Utilities.powerSet(
						childrenInCommon);
				for (int k = 0; k < commonParts.size(); k++) {
					Vector commonPart = (Vector) (commonParts.get(k));
					if (commonPart.size() == 0
						|| commonPart.size() == grandchildren1.size()
						|| commonPart.size() == grandchildren2.size()) {
						commonParts.remove(commonPart);
					}
				}
				for (int k = 0; k < commonParts.size(); k++) {
					return true;
				}
			}
		}
		return false;
	}

	public Vector getApplicationPossibilities(Diagram d) {
		Vector result = new Vector();
		if (!(d instanceof OrCompoundDiagram)) {
			return result;
		}
		OrCompoundDiagram ocd = (OrCompoundDiagram) d;
		// search for a child which is an "or"
		Vector children = ocd.getChildren();
		child1Loop : for (int i = 0; i < children.size(); i++) {
			Diagram child1 = (Diagram) (children.get(i));
			if (!(child1 instanceof AndCompoundDiagram)) {
				continue child1Loop;
			}
			AndCompoundDiagram andChild1 = (AndCompoundDiagram) child1;
			Vector grandchildren1 = andChild1.getChildren();

			child2Loop : for (int j = i + 1; j < children.size(); j++) {
				Diagram child2 = (Diagram) (children.get(j));
				if (!(child2 instanceof AndCompoundDiagram)) {
					continue child2Loop;
				}
				AndCompoundDiagram andChild2 = (AndCompoundDiagram) child2;
				Vector grandchildren2 = andChild2.getChildren();
				// see whether grandchildren1 and grandchildren2 have any common children
				Vector childrenInCommon = new Vector();
				boolean hasMatched[] = new boolean[grandchildren2.size()];
				for (int k = 0; k < hasMatched.length; k++) {
					hasMatched[k] = false;
				}
				for (int k = 0; k < grandchildren1.size(); k++) {
					Diagram gc1 = (Diagram) grandchildren1.get(k);
					findMatch : for (
						int l = 0; l < grandchildren2.size(); l++) {
						if (!hasMatched[l]) {
							Diagram gc2 = (Diagram) grandchildren2.get(l);
							if (gc1.equals(gc2)) {
								childrenInCommon.add(gc1);
								hasMatched[l] = true;
								break findMatch;
							}
						}
					}
				}
				// know commonChildren - find subsets of it
				Vector commonParts =
					rwd.abstractsyntax.spiderdiagram.Utilities.powerSet(
						childrenInCommon);
				for (int k = 0; k < commonParts.size(); k++) {
					Vector commonPart = (Vector) (commonParts.get(k));
					if (commonPart.size() == 0
						|| commonPart.size() == grandchildren1.size()
						|| commonPart.size() == grandchildren2.size()) {
						commonParts.remove(commonPart);
					}
				}
				for (int k = 0; k < commonParts.size(); k++) {
					Vector commonPart = (Vector) (commonParts.get(k));
					//System.out.println("provides d,[d2,d3,[d4,d5..]]!!");
					result.add(
						new RuleAppliesToDiDiAL(
							d,
							andChild1,
							andChild2,
							commonPart));
				}
			}
		}
		return result;
	}

	// takes (P and Q) or (P and R) or ...
	// becomes (P and (Q or R)) or  ...
	// specify (P and Q) , (P and R) , P(as Vector entries)
	public Diagram applyRule(RuleApplicationSpec r) {
		RuleAppliesToDiDiAL radi = (RuleAppliesToDiDiAL) r;
		OrCompoundDiagram ocd = (OrCompoundDiagram) radi.d;
		Object o = radi.others;
		Vector commonPart = (Vector) o;
		AndCompoundDiagram andChild1 = (AndCompoundDiagram) radi.d1;
		AndCompoundDiagram andChild2 = (AndCompoundDiagram) radi.d2;

		// we need to build or( and( andchild1 less commonPart),and(andChild2 less commonpart))
		// then "and" it with or-ed of commonpart
		// then "or" that with whatever's left of original children

		Vector gChildren1 = (Vector) (andChild1.getChildren().clone());
		//System.out.println("gChildren1 is "+gChildren1);
		Vector childrenToRemove = new Vector();
		for (int j = 0; j < commonPart.size(); j++) {
			Diagram d2 = (Diagram) (commonPart.get(j));
			matchSearch : for (int i = 0; i < gChildren1.size(); i++) {
				Diagram d = (Diagram) gChildren1.get(i);
				if (d.equals(d2)) {
					childrenToRemove.add(d);
					break matchSearch;
				}
			}
		}
		gChildren1.removeAll(childrenToRemove);
		//System.out.println("after removal gChildren1 is "+gChildren1);
		Vector gChildren2 = (Vector) (andChild2.getChildren().clone());
		//System.out.println("gChildren2 is "+gChildren2);
		childrenToRemove = new Vector();
		for (int j = 0; j < commonPart.size(); j++) {
			Diagram d2 = (Diagram) (commonPart.get(j));
			matchSearch : for (int i = 0; i < gChildren2.size(); i++) {
				Diagram d = (Diagram) gChildren2.get(i);
				if (d.equals(d2)) {
					childrenToRemove.add(d);
					break matchSearch;
				}
			}
		}
		gChildren2.removeAll(childrenToRemove);
		//System.out.println("after removal gChildren2 is "+gChildren2);

		// we need to build or( and( andchild1 less commonPart),and(andChild2 less commonpart))
		Diagram acd1;
		if (gChildren1.size() > 1) {
			acd1 = new AndCompoundDiagram(gChildren1, "Q");
		} else {
			acd1 = (Diagram) gChildren1.get(0);
		} // gChildren1 is size 0
		Diagram acd2;
		if (gChildren2.size() > 1) {
			acd2 = new AndCompoundDiagram(gChildren2, "R");
		} else {
			acd2 = (Diagram) (gChildren2.get(0));
		}
		Vector temp = new Vector();
		temp.add(acd1);
		temp.add(acd2);
		OrCompoundDiagram ocd2 = new OrCompoundDiagram(temp, "QorR");
		//System.out.println("QorR is "+ocd2);

		// then "and" it with or-ed of commonpart
		Diagram rest;
		if (commonPart.size() > 1) {
			rest = new AndCompoundDiagram(commonPart, "P");
		} else {
			rest = (Diagram) commonPart.get(0);
		}
		temp = new Vector();
		temp.add(ocd2);
		temp.add(rest);
		AndCompoundDiagram QorR_and_P =
			new AndCompoundDiagram(temp, "QorR and P");
		//System.out.println("andChild1 is "+andChild1.toSaveString());
		//System.out.println("andChild2 is "+andChild2.toSaveString());
		//

		// then "or" that with whatever's left of original children
		Vector originalChildren = (Vector) (ocd.getChildren().clone());
		//System.out.println("originalChildren is "+originalChildren);
		boolean haveRemoved1 = false;
		boolean haveRemoved2 = false;
		Vector toRemove = new Vector();
		matchSearch : for (int i = 0; i < originalChildren.size(); i++) {
			Diagram originalChild = (Diagram) originalChildren.get(i);
			//System.out.println("originalChild is "+originalChild);
			// look for matches for andChild1 and andChild2
			if (!haveRemoved1 && originalChild.equals(andChild1)) {
				toRemove.add(originalChild);
				haveRemoved1 = true;
				continue matchSearch;
			}
			if (!haveRemoved2 && originalChild.equals(andChild2)) {
				toRemove.add(originalChild);
				haveRemoved2 = true;
			}
		}
		originalChildren.removeAll(toRemove);
		originalChildren.add(QorR_and_P);
		//System.out.println("after removing children and putting back new one, ");
		//System.out.println("originalChildren is "+originalChildren);
		//System.out.println("about to enter if-then return part");
		if (originalChildren.size() == 1) {
			//System.out.println("returning from merge ands");
			//System.out.println(""+QorR_and_P);
			return QorR_and_P;
		} else {
			Diagram result = new OrCompoundDiagram(originalChildren, "outcome");
			//System.out.println("returning result from merge ands");
			//System.out.println(""+result);
			return result;
		}

	}
}
