package rwd.rules.choicespec;
import java.util.Vector;

import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleAppliesToZone;
import rwd.rules.application.RuleApplicationSpec;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;

public class RuleExcludedMiddle extends RuleWithChoice {
	private RuleExcludedMiddle() {
		super("sd: excluded middle");
	}
	private static Rule example = null;

	public static Rule getRule() {
		if (example == null) {
			example = new RuleExcludedMiddle();
		}
		return example;
	}
    public boolean isInvertible(){
        return true;
    }

	public boolean canApplyTo(Diagram d) {
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			Vector zones = (Vector) (ud.getCloneOfZoneSet());
			Vector shadedZones = ud.getCloneOfShadedZones();
			zones.removeAll(shadedZones);
			return zones.size()>0;
		}
		//System.out.println("returning result with "+result.size()+" options");
		return false;
	}

	public Vector getApplicationPossibilities(Diagram d) {
		Vector result = new Vector();
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			Vector zones = (Vector) (ud.getCloneOfZoneSet());
			Vector shadedZones = ud.getCloneOfShadedZones();
			zones.removeAll(shadedZones);
			zoneLoop : for (int i = 0; i < zones.size(); i++) {
				Zone z = (Zone) (zones.get(i));
				result.add(new RuleAppliesToZone(d, z));
				//System.out.println("result now has "+result.size()+" options");
			}
		}
		//System.out.println("returning result with "+result.size()+" options");
		return result;
	}

	public Diagram applyRule(RuleApplicationSpec r) {
		RuleAppliesToZone raz = (RuleAppliesToZone) r;
		Diagram d = raz.d;
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) raz.d;
			Zone z = raz.z;
			UnitaryDiagram ud1 = (UnitaryDiagram) (ud.getClone());
			UnitaryDiagram ud2 = (UnitaryDiagram) (ud.getClone());
			ud1.shadeZone(z);
			Vector habitat = new Vector();
			habitat.add(z);
			ud2.addSpider(habitat);
			Vector diagrams = new Vector();
			diagrams.add(ud1);
			diagrams.add(ud2);
			return new OrCompoundDiagram(diagrams, "outcome");
		}
		return null;
	}
}
