package rwd.rules.choicespec;
import java.util.Vector;

import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.FalseDiagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToZone;
public class RuleIntroduceShadedZone extends RuleWithChoice {
	private RuleIntroduceShadedZone() {
		super("sd: add shaded zone");
	}
	private static Rule example = null;

	public static Rule getRule() {
		if (example == null) {
			example = new RuleIntroduceShadedZone();
		}
		return example;
	}
    public boolean isInvertible(){
        return true;
    }

	public boolean canApplyTo(Diagram d) {
		if (d instanceof UnitaryDiagram
        && !(d instanceof FalseDiagram)) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			Vector allZones =
				UnitaryDiagram.allPossibleZones(ud.getCloneOfContourSet());
			allZones.removeAll(ud.getCloneOfZoneSet());
			return allZones.size()>0;
		}
		return false;
	}

	public Vector getApplicationPossibilities(Diagram d) {
		//System.out.println("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
		//System.out.println("get possibilities for "+d.toSaveString());
		Vector result = new Vector();
		if (d instanceof UnitaryDiagram && !(d instanceof FalseDiagram)) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			Vector allZones =
				UnitaryDiagram.allPossibleZones(ud.getCloneOfContourSet());
			//System.out.println("num possible zones: "+allZones.size());
			//System.out.println("remove existing ie: "+ud.getZones().size());
			//for(int i=0; i<ud.getZones().size(); i++)
			//{ Zone z = (Zone)ud.getZones().get(i);
			//	System.out.println("z"+i+"["+z.getContainingContours()+","+z.getExcludingContours()+"}");
			//}
			allZones.removeAll(ud.getCloneOfZoneSet());
			for (int i = 0; i < allZones.size(); i++) {
				Zone z = (Zone) (allZones.get(i));
				result.add(new RuleAppliesToZone(d, z));
			}
		}
		//System.out.println("found this many: "+result.size());
		return result;
	}

	public Diagram applyRule(RuleApplicationSpec r) {
		RuleAppliesToZone raz = (RuleAppliesToZone) r;
		Diagram d = raz.d;
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) raz.d;
			Zone z = raz.z;
			UnitaryDiagram result = (UnitaryDiagram) ud.getClone();
			result.addZone(z);
			result.shadeZone(z);
			return result;
		}
		//System.out.println("tried to add zone "+raz.z+" to "+d.toSaveString());
		//System.out.println("returning null from add zone");
		return null;
	}

	public static void main(String args[]) {
		Zone z = Zone.getZone(new Vector(), new Vector());
		Vector zones = new Vector();
		zones.add(z);
		UnitaryDiagram ud = new UnitaryDiagram(zones, "test");
		System.out.println(
			"RuleIntroduceShadedZone can be applied:"
				+ RuleIntroduceShadedZone.getRule().canApplyTo(ud));
		System.out.println(
			"RuleIntroduceShadedZone can be applied:"
				+ ((RuleWithChoice)RuleIntroduceShadedZone.getRule()).getApplicationPossibilities(
					ud));

	}
}
