package rwd.rules.choicespec;
import java.util.Vector;

import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.rules.application.RuleAppliesToDandDandD;

public class RuleJoinSpiders extends RuleWithChoice {
	private RuleJoinSpiders() {
		super("sd: join spiders");
	}
	private static Rule example = null;

	public static Rule getRule() {
		if (example == null) {
			example = new RuleJoinSpiders();
		}
		return example;
	}
    public boolean isInvertible(){
        return true;
    }

	public boolean canApplyTo(Diagram d) {
		if (!(d instanceof OrCompoundDiagram)) {
			return false;
		}
		OrCompoundDiagram ocd = (OrCompoundDiagram) d;
		// take pairs of children and see
		Vector children = ocd.getChildren();
		d1Loop : for (int i = 0; i < children.size(); i++) {
			Diagram d1 = (Diagram) children.get(i);
			UnitaryDiagram ud1;
			try {
				ud1 = (UnitaryDiagram) d1;
			} catch (ClassCastException e) {
				continue d1Loop;
			}
			d2Loop : for (int j = i + 1; j < children.size(); j++) {
				Diagram d2 = (Diagram) children.get(j);
				UnitaryDiagram ud2;
				try {
					ud2 = (UnitaryDiagram) d2;
				} catch (ClassCastException e) {
					continue d2Loop;
				}
				UnitaryDiagram ud1Copy = (UnitaryDiagram) (ud1.getClone());
				UnitaryDiagram ud2Copy = (UnitaryDiagram) (ud2.getClone());
				for (int n1 = 0; n1 < ud1Copy.getSpiders().size(); n1++) {
					Spider s1 = (Spider) (ud1Copy.getSpiders().get(n1));
					for (int n2 = 0; n2 < ud2Copy.getSpiders().size(); n2++) {
						Spider s2 = (Spider) (ud2Copy.getSpiders().get(n2));
						Vector temp = (Vector) (s1.getHabitat().clone());
						temp.addAll(s2.getHabitat());
						//System.out.println("consider spiders "+s1+" and "+s2);
						//System.out.println("see if habitats are disjoint");
						if (temp.size()
							== s1.getHabitat().size() + s2.getHabitat().size()) {
							//System.out.println("yes they are disjoint");
							ud1Copy.getSpiders().remove(s1);
							ud2Copy.getSpiders().remove(s2);
							//System.out.println("after removing spiders we have");
							//System.out.println("ud1Copy is "+ud1Copy);
							//System.out.println("ud2Copy is "+ud2Copy);
							if (ud1Copy.equals(ud2Copy)) {
								return true;
							}
							//System.out.println("replace the spiders:");
							ud1Copy.addSpider(s1);
							ud2Copy.addSpider(s2);
						}
					}
				}
			}
		}
		return false;
	}

	public Vector getApplicationPossibilities(Diagram d) {
		Vector result = new Vector();
		if (!(d instanceof OrCompoundDiagram)) {
			return result;
		}
		OrCompoundDiagram ocd = (OrCompoundDiagram) d;
		// take pairs of children and see
		Vector children = ocd.getChildren();
		d1Loop : for (int i = 0; i < children.size(); i++) {
			Diagram d1 = (Diagram) children.get(i);
			UnitaryDiagram ud1;
			try {
				ud1 = (UnitaryDiagram) d1;
			} catch (ClassCastException e) {
				continue d1Loop;
			}
			d2Loop : for (int j = i + 1; j < children.size(); j++) {
				Diagram d2 = (Diagram) children.get(j);
				UnitaryDiagram ud2;
				try {
					ud2 = (UnitaryDiagram) d2;
				} catch (ClassCastException e) {
					continue d2Loop;
				}
				UnitaryDiagram ud1Copy = (UnitaryDiagram) (ud1.getClone());
				UnitaryDiagram ud2Copy = (UnitaryDiagram) (ud2.getClone());
				for (int n1 = 0; n1 < ud1Copy.getSpiders().size(); n1++) {
					Spider s1 = (Spider) (ud1Copy.getSpiders().get(n1));
					for (int n2 = 0; n2 < ud2Copy.getSpiders().size(); n2++) {
						Spider s2 = (Spider) (ud2Copy.getSpiders().get(n2));
						Vector temp = (Vector) (s1.getHabitat().clone());
						temp.removeAll(s2.getHabitat());
						//System.out.println("consider spiders "+s1+" and "+s2);
						//System.out.println("see if habitats are disjoint");
						if (temp.size() == s1.getHabitat().size()) {
							//System.out.println("yes they are disjoint");
							ud1Copy.getSpiders().remove(s1);
							ud2Copy.getSpiders().remove(s2);
							//System.out.println("after removing spiders we have");
							//System.out.println("ud1Copy is "+ud1Copy);
							//System.out.println("ud2Copy is "+ud2Copy);
							if (ud1Copy.equals(ud2Copy)) {
								result.add(
									new RuleAppliesToDandDandD(d, ud1, ud2));
							}
							//System.out.println("replace the spiders:");
							ud1Copy.addSpider(s1);
							ud2Copy.addSpider(s2);
						}
					}
				}
			}
		}
		return result;
	}

	public Diagram applyRule(RuleApplicationSpec r) {
		RuleAppliesToDandDandD raddd = (RuleAppliesToDandDandD) r;
		Diagram d = raddd.d;
		Diagram d1 = raddd.other1;
		Diagram d2 = raddd.other2;
		//System.out.println("joining "+d1+" and "+d2);
		OrCompoundDiagram ocd = (OrCompoundDiagram) d;
		UnitaryDiagram ud1;
		UnitaryDiagram ud2;
		try {
			ud1 = (UnitaryDiagram) d1;
			ud2 = (UnitaryDiagram) d2;
		} catch (ClassCastException e) {
			return null;
		}
		UnitaryDiagram ud1Copy = (UnitaryDiagram) (ud1.getClone());
		UnitaryDiagram ud2Copy = (UnitaryDiagram) (ud2.getClone());
		Vector spiders1 = ud1Copy.getSpiders();
		Vector spiders2 = ud2Copy.getSpiders();
		for (int n1 = 0; n1 < spiders1.size(); n1++) {
			Spider s1 = (Spider) (spiders1.get(n1));
			//System.out.println("consider spider s1 "+s1);
			// is spider s1 in ud1?
			for (int n2 = 0; n2 < spiders2.size(); n2++) {
				Spider s2 = (Spider) (spiders2.get(n2));
				Vector temp = (Vector) (s1.getHabitat().clone());
				temp.removeAll(s2.getHabitat());
				//System.out.println("consider spider s2 "+s2);
				//System.out.println("see if habitats are disjoint");
				if (temp.size() == s1.getHabitat().size()) {
					//System.out.println("yes they are disjoint");
					spiders1.remove(s1);
					spiders2.remove(s2);
					if (ud1Copy.equals(ud2Copy)) {
						//System.out.println("after removing spiders, diagrams become equal");
						spiders1.add(s1);
						Vector hab = s2.getHabitat();
						for (int i = 0; i < hab.size(); i++) {
							Zone z = (Zone) (hab.get(i));
							//if(s1.getHabitat().contains(z))
							//{ System.out.println("trying to join spiders "+s1+" and "+s2);
							//  throw new RuntimeException();
							//}
							s1.addFoot(z);
						}
						Vector newChildren =
							(Vector) (ocd.getChildren().clone());
						boolean removedd1 = false;
						boolean removedd2 = false;
						Vector toRemove = new Vector();
						childLoop : for (
							int i = 0; i < newChildren.size(); i++) {
							Diagram newChild = (Diagram) newChildren.get(i);
							if (newChild.equals(d1) && !removedd1) {
								toRemove.add(newChild);
								removedd1 = true;
								continue childLoop;
							}
							if (newChild.equals(d2) && !removedd2) {
								toRemove.add(newChild);
								removedd2 = true;
							}
						}
						newChildren.removeAll(toRemove);
						newChildren.add(ud1Copy);
						if (newChildren.size() == 1) {
							return ud1Copy;
						} else {
							Diagram result =
								new OrCompoundDiagram(newChildren, "outcome");
							return result;
						}
					}
					//System.out.println("replace the spiders:");
					spiders1.add(n1, s1);
					spiders2.add(n2, s2);
				} // grab next s2
				//System.out.println("try next candidate for s2:");
			}
			//System.out.println("try next candidate for s1:");
		}
		return null;
	}
}
