package rwd.rules.choicespec;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import java.util.Vector;

import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToDiagramAndDiagrams;

public class RuleReverseAndIdempotent extends RuleWithChoice {
	private RuleReverseAndIdempotent() {
		super("sd: PandP... -> P...");
	}
	private static Rule example = null;

	public static Rule getRule() {
		if (example == null) {
			example = new RuleReverseAndIdempotent();
		}
		return example;
	}
    public boolean isInvertible(){
        return true;
    }

	public boolean canApplyTo(Diagram d) {
		if (d instanceof AndCompoundDiagram) {
			AndCompoundDiagram acd = (AndCompoundDiagram) d;
			Vector children = acd.getChildren();
			boolean[] hasMatch = new boolean[children.size()];
			for (int i = 0; i < hasMatch.length; i++) {
				hasMatch[i] = false;
			}
			for (int i = 0; i < children.size(); i++) {
				if (!hasMatch[i]) {
					Diagram di = (Diagram) (children.get(i));
					Vector similarDiags = new Vector();
					for (int j = i + 1; j < children.size(); j++) {
						Diagram dj = (Diagram) (children.get(j));
						if (di.equals(dj)) {
							hasMatch[j] = true;
							similarDiags.add(dj);
						}
					}
					if (similarDiags.size() > 0) {
						similarDiags.add(di);
						return true;
					}
				}
			}
		}
		return false;
	}

	public Vector getApplicationPossibilities(Diagram d) {
		Vector result = new Vector();
		if (d instanceof AndCompoundDiagram) {
			AndCompoundDiagram acd = (AndCompoundDiagram) d;
			Vector children = acd.getChildren();
			boolean[] hasMatch = new boolean[children.size()];
			for (int i = 0; i < hasMatch.length; i++) {
				hasMatch[i] = false;
			}
			for (int i = 0; i < children.size(); i++) {
				if (!hasMatch[i]) {
					Diagram di = (Diagram) (children.get(i));
					Vector similarDiags = new Vector();
                    searchForSimilar:
					for (int j = i + 1; j < children.size(); j++) {
						Diagram dj = (Diagram) (children.get(j));
						if (di.equals(dj)) {
							hasMatch[j] = true;
							similarDiags.add(dj);
                            break searchForSimilar;
						}
					}
					if (similarDiags.size() > 0) {
						similarDiags.add(di);
						//System.out.println("9. provides d,[d1,d2,..]");
						result.add(
							new RuleAppliesToDiagramAndDiagrams(
								acd,
								similarDiags));
					}
				}
			}
		}
		return result;
	}

	public Diagram applyRule(RuleApplicationSpec r) {
		RuleAppliesToDiagramAndDiagrams radd =
			(RuleAppliesToDiagramAndDiagrams) r;
		Diagram d = radd.d;
		AndCompoundDiagram acd = (AndCompoundDiagram) d;
		Vector children = (Vector) (acd.getChildren().clone());
		Vector equivalents = radd.others;
		if (equivalents.size() == children.size()) {
			return ((Diagram) equivalents.get(0)).getClone();
		} else {
			children.removeAll(equivalents);
			children.add(equivalents.get(0));
			if (children.size() > 1) {
				return new AndCompoundDiagram(children, "outcome");
			} else {
				return (Diagram) children.get(0);
			}
		}
	}
}
