package rwd.rules.choicespec;

import rwd.abstractsyntax.spiderdiagram.Diagram;
import java.util.Vector;

import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToDiagramAndDiagrams;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;

public class RuleReverseOrIdempotent extends RuleWithChoice {
	private RuleReverseOrIdempotent() {
		super("sd: PorP... -> P...");
	}
	private static Rule example = null;

	public static Rule getRule() {
		if (example == null) {
			example = new RuleReverseOrIdempotent();
		}
		return example;
	}
    public boolean isInvertible(){
        return true;
    }

	public boolean canApplyTo(Diagram d) {
		if (!(d instanceof OrCompoundDiagram)) {
			return false;
		}
		OrCompoundDiagram ocd = (OrCompoundDiagram) d;
		Vector children = ocd.getChildren();
		boolean[] hasMatch = new boolean[children.size()];
		for (int i = 0; i < hasMatch.length; i++) {
			hasMatch[i] = false;
		}
		for (int i = 0; i < children.size(); i++) {
			if (!hasMatch[i]) {
				Diagram di = (Diagram) (children.get(i));
				Vector similarDiags = new Vector();
				for (int j = i + 1; j < children.size(); j++) {
					Diagram dj = (Diagram) (children.get(j));
					if (di.equals(dj)) {
						hasMatch[j] = true;
						similarDiags.add(dj);
					}
				}
				if (similarDiags.size() > 0) {
					similarDiags.add(di);
					return true;
				}
			}
		}
		return false;
	}

	public Vector getApplicationPossibilities(Diagram d) {
		Vector result = new Vector();
		if (!(d instanceof OrCompoundDiagram)) {
			return result;
		}
		OrCompoundDiagram ocd = (OrCompoundDiagram) d;
		Vector children = ocd.getChildren();
		boolean[] hasMatch = new boolean[children.size()];
		for (int i = 0; i < hasMatch.length; i++) {
			hasMatch[i] = false;
		}
		for (int i = 0; i < children.size(); i++) {
			if (!hasMatch[i]) {
				Diagram di = (Diagram) (children.get(i));
				Vector similarDiags = new Vector();
                searchForSimilar:
				for (int j = i + 1; j < children.size(); j++) {
					Diagram dj = (Diagram) (children.get(j));
					if (di.equals(dj)) {
						hasMatch[j] = true;
						similarDiags.add(dj);
                        break searchForSimilar;
					}
				}
				if (similarDiags.size() > 0) {
					similarDiags.add(di);
					//System.out.println("10. provides d,[d1,d2,..]");
					result.add(
						new RuleAppliesToDiagramAndDiagrams(ocd, similarDiags));
				}
			}
		}
		return result;
	}

	public Diagram applyRule(RuleApplicationSpec r) {
		RuleAppliesToDiagramAndDiagrams radd =
			(RuleAppliesToDiagramAndDiagrams) r;
		Diagram d = radd.d;
		OrCompoundDiagram ocd = (OrCompoundDiagram) d;
		Vector children = (Vector) (ocd.getChildren().clone());
		//System.out.println("before removal, we have "+children.size()+" children");
		Vector equivalents = radd.others;
		//System.out.println("identify equivalents[0] "+equivalents.get(0));
		//System.out.println("identify equivalents[1] "+equivalents.get(1));
		//System.out.println("before removal, we have "+equivalents.size()+" equivalents");
		if (equivalents.size() == children.size()) {
			//System.out.println("removal will leave a single unitary diagram");
			return ((Diagram) equivalents.get(0)).getClone();
		} else {
			children.removeAll(equivalents);
			//System.out.println("after removal, we have "+children.size()+" children");
			children.add(equivalents.get(0));
			//System.out.println("removal+replacement leaves "+children.size()+" children");
			return new OrCompoundDiagram(children, "outcome");
		}
	}
}
