package rwd.rules.choicespec;
import java.util.Vector;

import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.abstractsyntax.spiderdiagram.Utilities;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToSpiderAndRegion;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
public class RuleSplitSpider extends RuleWithChoice {
	private RuleSplitSpider() {
		super("sd: split spider");
	}
	private static Rule example = null;

	public static Rule getRule() {
		if (example == null) {
			example = new RuleSplitSpider();
		}
		return example;
	}
    public boolean isInvertible(){
        return true;
    }

	public boolean canApplyTo(Diagram d) {
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			// for each spider
            for (int i = 0; i < ud.getSpiders().size(); i++) {
                Spider s = (Spider) (ud.getSpiders().get(i));
                // get its habitat and split it non-trivially
                //System.out.println("spider is "+s);
                Vector habitat = s.getHabitat();
                //System.out.println("habitat is "+habitat);
                Vector subsets = Utilities.powerSet(habitat);
                Zone faveZone = (Zone)habitat.get(0);
                for(int i2=0; i2<subsets.size(); i2++){
                    Vector subset = (Vector)subsets.get(i2);
                    
                    if(subset.size()<habitat.size() && subset.contains(faveZone)){
                        return true;
                    }
                }
            }
			return false;
		}
		return false;
	}

	public Vector getApplicationPossibilities(Diagram d) {
		Vector result = new Vector();
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			// for each spider
			for (int i = 0; i < ud.getSpiders().size(); i++) {
				Spider s = (Spider) (ud.getSpiders().get(i));
				// get its habitat and split it non-trivially
				//System.out.println("spider is "+s);
				Vector habitat = s.getHabitat();
				//System.out.println("habitat is "+habitat);
                Vector subsets = Utilities.powerSet(habitat);
                Zone faveZone = (Zone)habitat.get(0);
                for(int i2=0; i2<subsets.size(); i2++){
                    Vector subset = (Vector)subsets.get(i2);
                    if(subset.size()<habitat.size() && subset.contains(faveZone)){
                        result.add(
							new RuleAppliesToSpiderAndRegion(
								ud,
								new Spider((Vector) s.getHabitat().clone()),
  								subset));
                    }
                }
			}
			return result;
		}
		return result;
	}

	private static boolean hasFalse(boolean[] flags) {
		for (int i = 1; i < flags.length; i++) {
			if (flags[i] == false) {
				return true;
			}
		}
		return false;
	}

	public Diagram applyRule(RuleApplicationSpec r) {
		try {
			RuleAppliesToSpiderAndRegion rasr =
				(RuleAppliesToSpiderAndRegion) r;
			Spider s = rasr.s;
			Diagram d = rasr.d;
			UnitaryDiagram ud = (UnitaryDiagram) d;
			UnitaryDiagram ud1 = (UnitaryDiagram) (ud.getClone());
			UnitaryDiagram ud2 = (UnitaryDiagram) (ud.getClone());
			Spider s1 = null;
			Spider s2 = null;
			for (int i = 0; i < ud1.getSpiders().size(); i++) {
				s1 = (Spider) (ud1.getSpiders().get(i));
				s2 = (Spider) (ud2.getSpiders().get(i));
				if (s1.equals(s)) {
					break;
				}
			}
			// spider s is in ud and s1 is in ud1
			Vector subHabitat = rasr.region;
			Vector bigHabitat = (Vector) s2.getHabitat().clone();
			//if(subHabitat.size()==0 || subHabitat.size()==bigHabitat.size())
			//{ throw new RuntimeException(); }

			s1.setHabitat(subHabitat);
			bigHabitat.removeAll(subHabitat);
			//if(bigHabitat.size()==0)
			//{ System.out.println("rasr.region is "+rasr.region);
			//  System.out.println("original habitat is "+s2.getHabitat());
			//  throw new RuntimeException();
			//}
			s2.setHabitat(bigHabitat);
			Vector diagrams = new Vector();
			diagrams.add(ud1);
			diagrams.add(ud2);
			return new OrCompoundDiagram(diagrams, "outcome");
		} catch (ClassCastException e) {
		}
		return null;
	}
}
