package rwd.rules.constraintdiagrams;

import java.util.*;

import rwd.rules.CDRule;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;

import rwd.abstractsyntax.constraintdiagram.*;
import rwd.abstractsyntax.eulerdiagram.*;
import rwd.abstractsyntax.spiderdiagram.*;

import rwd.concretesyntax.*;
import rwd.concretesyntax.DiagramItem;
import rwd.concretesyntax.constraintdiagram.*;
import rwd.concretesyntax.eulerdiagram.*;
import rwd.concretesyntax.spiderdiagram.ConcreteSpider;

public class RuleDeleteArrow
	extends CDRule 
{
	private RuleDeleteArrow() {
		super("cd: Delete Arrow");
	}
	private static Rule example = null;

	public static Rule getRule() {
		if (example == null) {
			example = new RuleDeleteArrow();
		}
		return example;
	}

	//
	// Concrete level reasoning
	//
	public boolean canApplyTo(List args, String extraArgs) {
		// Check if there is a diagram
		ConcreteUnitaryDiagram cud = getCUD(args);
		if (cud == null) return false;
		// Check if there is an arrow
		DiagramItem item = (DiagramItem)getArg(args, 2);
		if (item == null || !(item instanceof ConcreteArc)) return false;
		return true;
	}
	// Delete arrow
	// If the arrow targets a derived contour C, delete contour if the conditions
	// 	1. there are no arrows emanating from the target contour
	// 	2. the derived contour is not needed to describe the domain of any universal spider
	// 	3. the reading tree for the premise diagram is valid for the conclusion diagram
	// are true
	public ConcreteUnitaryDiagram applyRule(List args, String extraArgs) {
		// Get cud and arc
		ConcreteUnitaryDiagram cud = getCUD(args);
		// Get arrow
		ConcreteArc arrow = (ConcreteArc)getArg(args, 2);
		// Remove arrow
		cud.removeConcreteArc(arrow);
		// If target is a derived contour
		DiagramItem target = arrow.getTarget();
		if (target instanceof ConcreteContour) {
			ConcreteContour c = (ConcreteContour)target;
			if (c.isDerived()) {
				// Condition 1
				boolean flag1 = true;
				Iterator i = cud.getConcreteArcs().iterator();
				while (i.hasNext()) {
					ConcreteArc arrow1 = (ConcreteArc)i.next();
					DiagramItem source1 = arrow1.getSource();
					DiagramItem target1 = arrow1.getTarget();
					if (arrow != arrow1 && target == target1) {
						flag1 = false;
						break;
					}
					if (arrow != arrow1 && target == source1) {
						flag1 = false;
						break;
					}
				}
				// Condition 2
				boolean flag2 = true;
				i = cud.getConcreteSpiders().iterator();
				while (i.hasNext()) {
					ConcreteSpider sp1 = (ConcreteSpider)i.next();
					if (sp1.isUniversal()) {
						if (sp1.inDomain(cud, c)) {
							flag2 = false;
						}
					}
				}
				// Condition 3
				boolean flag3 = cud.isValidTree();
								
				// Remove contour
				if (flag1 && flag2 && flag3) {
					cud.removeConcreteContour(c);
					cud.notifyObservers();
				}
			}
		}
		return cud;
	}
	
	//
	// Abstract level reasoning
	//
	public boolean canApplyTo(List args) {
		if (args == null || args.size() != 2) return false;
		Object arg1 = args.get(0);
		Object arg2 = args.get(1);
		if (!(arg1 instanceof UnitaryDiagram)) return false;
		if (!(arg2 instanceof Arrow)) return false;
		UnitaryDiagram ud = (UnitaryDiagram)arg1;
		Arrow a = (Arrow)arg2;
		if (!(ud.getArrows().contains(a))) return false;
		return false;
	}

	// Delete arrow
	// If the arrow targets a derived contour C, delete contour if the conditions
	// 	1. there are no arrows emanating from the target contour
	// 	2. the derived contour is not needed to describe the domain of any universal spider
	// 	3. the reading tree for the premise diagram is valid for the conclusion diagram
	// are true
	public Diagram applyRule(List args) {
		Object arg1 = args.get(0);
		Object arg2 = args.get(1);
		UnitaryDiagram ud = (UnitaryDiagram)arg1;
		ud = (UnitaryDiagram)ud.getClone();
		Arrow arrow = (Arrow)arg2;
		ud.removeArrow(arrow);
		// If target is a derived contour
		Object target = arrow.getTarget();
		if (target instanceof Contour) {
			Contour c = (Contour)target;
			if (c.isDerived()) {
				// Condition 1
				boolean flag1 = true;
				Iterator i = ud.getArrows().iterator();
				while (i.hasNext()) {
					Arrow arrow1 = (Arrow)i.next();
					Object source1 = arrow1.getSource();
					Object target1 = arrow1.getTarget();
					if (arrow != arrow1 && target == target1) {
						flag1 = false;
						break;
					}
					if (arrow != arrow1 && target == source1) {
						flag1 = false;
						break;
					}
				}
				// Condition 2
				boolean flag2 = true;
				i = ud.getSpiders().iterator();
				while (i.hasNext()) {
					Spider sp1 = (Spider)i.next();
					if (sp1.isUniversal()) {
						if (sp1.inDomain(ud, c)) {
							flag2 = false;
						}
					}
				}
				// Condition 3
				boolean flag3 = ud.isValidTree();
								
				// Remove contour
				if (flag1 && flag2 && flag3) {
					ud = removeContour(ud, c);
				}
			}
		}
		return ud;
	}

	public Diagram applyRule(RuleApplicationSpec r) {
		return null;
	}

	public boolean canApplyTo(Diagram ad) {
		return false;
	}

	public RuleApplicationSpec findApplicableSpec(Diagram part) {
		return null;
	}

	public boolean isInvertible() {
		return false;
	}
	
	public UnitaryDiagram removeContour(UnitaryDiagram ud, Contour c) {
		Vector newZones = new Vector();
		Vector newShadedZones = new Vector();
		Vector newHabitats = new Vector();
		for (int i = 0; i < ud.getSpiders().size(); i++) {
			newHabitats.add(new Vector());
		}

        Iterator zi = ud.getZoneIterator();
        while(zi.hasNext()){
            Zone oldz = (Zone)zi.next();
			//System.out.println("considering zone "+oldz);
			Vector inContours =
				(Vector) (oldz.getContainingContours().clone());
			Vector outContours =
				(Vector) (oldz.getExcludingContours().clone());
			if (inContours.contains(c)) {
				inContours.remove(c);
			} else {
				outContours.remove(c);
			}
			Zone newz = Zone.getZone(inContours, outContours);
			newZones.add(newz);
			if (ud.containsShaded(oldz)) {
				newShadedZones.add(newz);
			}
			for (int j = 0; j < ud.getSpiders().size(); j++) {
				Spider s = (Spider) (ud.getSpiders().get(j));
				if (s.getHabitat().contains(oldz)) {
					Vector newh = (Vector) (newHabitats.get(j));
					if (!newh.contains(newz)) {
						newh.add(newz);
					}
				}
			}
		}
		// now need to tidy up duplicates from zones
		// and delete from shadedZones if it appeared there less often
		Vector finalZones = new Vector();
		Vector finalShadedZones = new Vector();
		while (newZones.size() > 0) {
			Zone newz = (Zone) (newZones.get(0));
			int count = 0;
			while (newZones.contains(newz)) {
				count++;
				newZones.remove(newz);
			}
			finalZones.add(newz);
			while (newShadedZones.contains(newz)) {
				count--;
				newShadedZones.remove(newz);
			}
			// see whether count>0
			// if count>0 then the zone wasn't always shaded
			if (count == 0) {
				finalShadedZones.add(newz);
			}
		}
		UnitaryDiagram result = new UnitaryDiagram(finalZones, "outcome");
		for (int i = 0; i < finalShadedZones.size(); i++) {
			result.shadeZone((Zone) finalShadedZones.get(i));
		}
		for (int i = 0; i < newHabitats.size(); i++) {
			Vector h = (Vector) newHabitats.get(i);
			result.addSpider(h);
		}
		return result;
	}

}
