package rwd.rules.heuristics;
import rwd.abstractsyntax.eulerdiagram.*;
import rwd.abstractsyntax.spiderdiagram.*;
import rwd.rules.choicespec.*;
import rwd.rules.nospec.*;

import java.util.Vector;

import rwd.rules.*;
import rwd.rules.application.*;
import rwd.rules.autoproof.*;
import rwd.rules.inputspec.*;

import javax.swing.*;
import java.io.BufferedWriter;
import java.io.File;
//import java.io.FileWriter;
import java.io.IOException;

import rwd.gui.*;

public abstract class HeuristicProofWriter {

    public static double proofLengthLimit = Double.MAX_VALUE;

    public static Vector unitaryHRules;
    public static Vector compoundHRules;
    private static Diagram target = null;
    protected static boolean isLogging = false;
    protected static int costFound = 0;
    private static BufferedWriter logWriter = null;
    private static int stepNum = 0;
    protected static int numberStubs = 0;
    private static StringProgressShower requester;
    private static String tempDirectory = null;
    private static Vector proofsFound;
    private static boolean collectAll;
    private static RuleNeedsInput[] rulesNeedInput;
    private static boolean debug = true;
    private static CostCalculator cc = new DefaultCostCalculator();

    static {
        unitaryHRules = new Vector();
        unitaryHRules.add(RuleAddContour.getRule());
        unitaryHRules.add(RuleDeleteContour.getRule());
        unitaryHRules.add(RuleIntroduceShadedZone.getRule());
        unitaryHRules.add(RuleDeleteShadedZone.getRule());
        unitaryHRules.add(RuleAddSpiderFoot.getRule());
        unitaryHRules.add(RuleUnshadeZone.getRule());
        unitaryHRules.add(RuleDeleteSpider.getRule());

        compoundHRules = new Vector();
        compoundHRules.add(RuleAddContour.getRule());
        compoundHRules.add(RuleDeleteContourSimple.getRule());
        compoundHRules.add(RuleIntroduceShadedZone.getRule());
        compoundHRules.add(RuleDeleteShadedZone.getRule());
        compoundHRules.add(RuleUnshadeZone.getRule());
        compoundHRules.add(RuleDeleteSpider.getRule());
        compoundHRules.add(RuleAddSpiderFoot.getRule());

        compoundHRules.add(RuleSplitSpider.getRule());
        compoundHRules.add(RuleJoinSpiders.getRule());

        compoundHRules.add(RuleExcludedMiddle.getRule());
        //compoundHRules.add(RuleReverseExcludedMiddle.getRule());
        // removed because it's derived (GES)

        //compoundHRules.add(MergeContradictionAvoidingFalseRule.getRule());
        //compoundHRules.add(RuleFalseImpliesAContradiction.getRule());
        // needs input
        //compoundHRules.add(MergeConjunctionRule.getRule());
        // removed because its's derived (GES)
        // inverse?- derived - idempotency and erasure 
        //compoundHRules.add(RemoveNotRule.getRule());
        //compoundHRules.add(ReverseNegationEquivalenceRule.getRule());
        // removed because it's derived (GES)
        
        compoundHRules.add(AndIdempotentRule.getRule());
        compoundHRules.add(RuleReverseAndIdempotent.getRule());

        compoundHRules.add(OrIdempotentRule.getRule());
        compoundHRules.add(RuleReverseOrIdempotent.getRule());

        compoundHRules.add(RuleDMPullNotThroughAnd.getRule());
        compoundHRules.add(RuleDMPullNotThroughOr.getRule());
        compoundHRules.add(RuleDMPushNotThroughAnd.getRule());
        compoundHRules.add(RuleDMPushNotThroughOr.getRule());

        compoundHRules.add(RemoveNotNotRule.getRule());
        compoundHRules.add(InsertNotNotRule.getRule());

        compoundHRules.add(RuleAndAbsorbtion.getRule());
        compoundHRules.add(RuleOrAbsorbtion.getRule());
        //compoundHRules.add(RulePToPand_PorQ.getRule());
        // needs input
        //compoundHRules.add(RulePToPor_PandQ.getRule());
        // needs input

        compoundHRules.add(RuleDistributeAndOverOr.getRule());
        compoundHRules.add(RuleDistributeOrOverAnd.getRule());
        compoundHRules.add(RuleDistributeMergeAnds.getRule());
        compoundHRules.add(RuleDistributeMergeOrs.getRule());        
    }

    protected HeuristicProofWriter() {
    } // prevents external construction of objects

    public static synchronized Vector seekProof(
        Diagram start,
        Diagram target,
        int costLimit,
        HeuristicCalculator hc,
        CostCalculator cc,
        Vector ruleSet,
        StringProgressShower requester,
        boolean isUnitary,
        boolean isCompound,
        BufferedWriter toLog,
        boolean collectAllProofs)
        throws NoProofException {
        if (toLog != null) {
            isLogging = true;
            logWriter = toLog;
            Vector a =
                seekProof(
                    start,
                    target,
                    costLimit,
                    hc,
                    cc,
                    ruleSet,
                    requester,
                    isUnitary,
                    isCompound,
                    collectAllProofs);
            isLogging = false;
            logWriter = null;
            return a;
        } else {            
            debug = false;
            // because logWriter will be null
            return seekProof(
                start,
                target,
                costLimit,
                hc,
                cc,
                ruleSet,
                requester,
                isUnitary,
                isCompound,
                collectAllProofs);
        }
    }

    public static synchronized Vector seekProof(
        Diagram start,
        Diagram target,
        int costLimit,
        HeuristicCalculator hc,
        CostCalculator cc_p,
        Vector ruleSet,
        StringProgressShower requester,
        boolean isUnitary,
        boolean isCompound,
        boolean collectAllProofs)
        throws NoProofException {
        if (cc_p != null) {
            cc = cc_p;
        }

        debug("starting a heuristic search from " + start);
        debug("starting a heuristic search to " + target);
        debug("using " + hc.getClass());
        debug("collecting all? " + collectAllProofs);
        for (int i = 0; i < ruleSet.size(); i++) {
            debug("use rule " + ruleSet.get(i));
        }
        if (costLimit >= proofLengthLimit) {
            throw new IllegalArgumentException();
        }
        costFound = costLimit;

        collectAll = collectAllProofs;

        if (tempDirectory == null) {
            setTempDirectory();
        }

        if (start == null || target == null) {
            throw new IllegalArgumentException();
        }

        HeuristicProofWriter.requester = requester;
        HeuristicProofWriter.target = target;

        for (int i = 0; i < costFound + 1; i++) {
            //System.out.println("clearing file " + getFileName(i));
            ProofFileUser.clearFile(getFileName(i));
        }

        Vector RWCAL = new Vector();
        Vector RNSAL = new Vector();
        Vector RNIAL = new Vector();
        Vector allRules = null;
        if (isUnitary) {
            allRules = unitaryHRules;
        } else {
            allRules = compoundHRules;
        }

        for (int i = 0; i < allRules.size(); i++) {
            Rule r = (Rule) allRules.get(i);
            debug("shall we include " + r + " ?");
            if (ruleSet.contains(r)) {
                debug("yes : it's a ");
                if (r instanceof RuleWithChoice) {
                    RWCAL.add(r);
                    debug("rule with choice");
                } else if (r instanceof RuleNoSpec) {
                    RNSAL.add(r);
                    debug("rule no spec");
                } else if (r instanceof RuleNeedsInput) {
                    RNIAL.add(r);
                    debug("rule needs input");
                }
            } else {
                debug("no");
            }
        }

        RuleWithChoice[] rulesWithChoice = new RuleWithChoice[RWCAL.size()];
        for (int i = 0; i < RWCAL.size(); i++) {
            rulesWithChoice[i] = (RuleWithChoice) RWCAL.get(i);
            debug(
                    "" + i + "th rule with choice " + rulesWithChoice[i]);
        }
        RuleNoSpec[] rulesNoSpec = new RuleNoSpec[RNSAL.size()];
        for (int i = 0; i < RNSAL.size(); i++) {
            rulesNoSpec[i] = (RuleNoSpec) RNSAL.get(i);
            debug(
                    "" + i + "th rule no spec " + rulesNoSpec[i]);
        }
        rulesNeedInput = new RuleNeedsInput[RNIAL.size()];
        for (int i = 0; i < RNIAL.size(); i++) {
            rulesNeedInput[i] = (RuleNeedsInput) RNIAL.get(i);
            debug(
                    "" + i + "th rule needs input " + rulesNeedInput[i]);
        }

        if (isLogging) {
            writeln("task: to find a proof from diagram");
            writeln(DiagramFileUser.toSaveString(start));
            writeln("to");
            writeln(DiagramFileUser.toSaveString(target));
        }
        //System.out.println("rules with choice are" + newRWCAL);
        //System.out.println("rules no spec are" + newRNSAL);
        //System.out.println("rules need input are" + newRNIAL);

        Proof first = new Proof(start);
        numberStubs = 1;
        proofsFound = new Vector();
        if (isLogging) {
            writeln("inc numberStubs:" + numberStubs);
        }
        if (target.equals(start)) {
            if (isLogging) {
                writeln("proof is trivial");
                writeln(ProofFileUser.toSaveString(first));
            }
            finishUp();
            Vector result = new Vector();
            result.add(first);
            result.add(new Integer(numberStubs));
            return result;
        }
        debug("about to save first proof");
        addProof(first, 0.0);
        if (debug) {
            System.out.println("saved first proof");
        }

        stepNum = 1;
        loop : while (true) {
            debug("in loop doing step " + stepNum);
            Proof nextContender = getBestProofSoFar();

            ProofWithScore bestProofWS = null;
            if (nextContender != null) {
                if (isLogging) {
                    writeln("proof from file was not null");
                }
                bestProofWS = new ProofWithScore(
                    nextContender,
                    cc.cost(nextContender),
                    hc.calcHeuristic(nextContender.getLastDiagram(), target));
            }

            debug("bestPWS so far " + bestProofWS);
            if (bestProofWS == null) {
                // this means that we have exhausted all proof stubs
                // so the proof we have is the right one
                //if (aProofFound != null) {
                //System.out.println("returning " + aProofFound);
                if (isLogging) {
                    writeln("ran out of proof attempts");
                }
                finishUp();
                Vector result = new Vector();
                if (proofsFound.size() == 0) {
                    proofsFound.add(null);
                    result.addAll(proofsFound);
                } else {
                    result.addAll(proofsFound);
                }
                result.add(new Integer(numberStubs));
                return result;
            }
            debug("bestPWS so far has cost " + bestProofWS.cost);
            debug("bestPWS so far has heuristic " + bestProofWS.heuristic);
            Proof bestProof = bestProofWS.proof;
            if (bestProof.getLastDiagram() == null) {
                throw new IllegalArgumentException();
            }
            double bestCost = bestProofWS.cost;            
            double bestScore = bestProofWS.heuristic + bestCost;
            if (isLogging) {
                writeln("best score is " + bestScore);
                debug("costFound is "+costFound+" and bestScore is "+bestScore);
            }

            if (bestScore > costFound // heuristic predicts higher cost from here 
            || bestCost==costFound // got equal cost and bound to make it worse 
            || bestScore >= costFound && !collectAll) {
                if (isLogging) {
                    writeln("no proofs with lower cost");
                }
                finishUp();
                //System.exit(0); // remove this line!!! it's just for debugging
                Vector result = new Vector();
                if (proofsFound.size() == 0) {
                    proofsFound.add(null);
                    result.addAll(proofsFound);
                } else {
                    result.addAll(proofsFound);
                }
                result.add(new Integer(numberStubs));
                return result;
            }

            announce();

            if (bestScore >= proofLengthLimit) // all proofs have been blocked
                {
                if (isLogging) {
                    writeln("proofs have all been assessed as blocked");
                }
                AutoProofWriterBetter.writeProof(start, target);
                finishUp();
                throw new NoProofException("can't find a proof with this rule-set");
            }

            if (isLogging) {
                writeln("~~ in loop extending this proof ~~:");
                writeln(ProofFileUser.toSaveString(bestProof));
            }
            //System.out.println("apply input-based rules");
            //Proof tmpPf =
            if (isUnitary) {
                doUnitaryInputRules(
                    (UnitaryDiagram) target,
                    bestProof,
                    hc,
                    stepNum,
                    bestScore,
                    requester);
            } else if (isCompound) {
                doCompoundInputRules(
                    target,
                    bestProof,
                    hc,
                    stepNum,
                    bestScore,
                    requester);
            }
            //System.out.println("apply other rules");
            // apply each rule in turn to bestStub
            Diagram d1 = bestProof.getLastDiagram();
            //System.out.println("bestProof is " + ProofFileUser.toSaveString(bestProof));
            Vector partsOfD1 = d1.getParts();
            for (int i = 0; i < partsOfD1.size(); i++) {
                Diagram part = (Diagram) partsOfD1.get(i);
                debug("take " + i + "th part");
                    // = " + DiagramFileUser.toSaveString(part));
                //System.out.println("consider part = " + part.toSaveString());
                // rule with choice loop
                for (int j = 0; j < rulesWithChoice.length; j++) {
                    RuleWithChoice rwc = rulesWithChoice[j];
                    debug(
                            "consider rule = " + rwc.toSaveString());
                    Vector specs = rwc.getApplicationPossibilities(part);
                    debug("num of specs is = " + specs.size());
                    for (int k = 0; k < specs.size(); k++) {
                        RuleApplicationSpec ras =
                            (RuleApplicationSpec) specs.get(k);
                        Proof newp = bestProof.cloneProof();
                        if (!d1.containsNegationOf(part)
                            || rwc.isInvertible()) {
                            ////////////////////////////////////
                            /////////////////////////////////////////
                            Diagram newEnd = rwc.applyRuleToPart(d1, ras);
                            double newH = hc.calcHeuristic(newEnd, target);
                            newp.addStepAtEnd(rwc, ras, newEnd);
                            numberStubs++;
                            if (isLogging) {
                                writeln("inc numberStubs:" + numberStubs);
                            }
                            double newScore = cc.cost(newp) + newH;
                            if (newScore <= costFound) {
                                addProof(newp, newScore);
                                stepNum++;
                            }
                            if (isLogging) {
                                writeln("apply rule " + rwc.name);
                                writeln("with spec " + ras);
                                writeln("heuristic+score " + newH);
                            }
                        }
                    }
                } // rule without choice loop
                for (int j = 0; j < rulesNoSpec.length; j++) {
                    RuleNoSpec rns = rulesNoSpec[j];
                    debug(
                            "consider rule = " + rns.toSaveString());
                    if (rns.canApplyTo(part)
                        && ((!d1.containsNegationOf(part))
                            || rns.isInvertible())) {
                        ////////////////////////////////////////
                        debug("can apply to part");
                        RuleApplicationSpec ras =
                            new RuleAppliesToJustADiagram(part);
                        Proof newp = bestProof.cloneProof();
                        Diagram newEnd = rns.applyRuleToPart(d1, ras);
                        double score = hc.calcHeuristic(newEnd, target);
                        newp.addStepAtEnd(rns, ras, newEnd);
                        numberStubs++;
                        if (isLogging) {
                            writeln("inc numberStubs:" + numberStubs);
                        }

                        double newH = score + cc.cost(newp);
                        if (isLogging) {
                            writeln("apply rule " + rns.name);
                            writeln("heuristic+score " + newH);
                        }
                        debug(
                                "proof is now "
                                    + ProofFileUser.toSaveString(newp));
                        debug(
                                "cost so far " + cc.cost(newp));
                        debug("heuristic " + score);
                        debug("costFound " + costFound);
                        if (newH <= costFound) {
                            debug("add this proof");
                            addProof(newp, newH);
                            stepNum++;
                        } else {
                            debug("don't add this proof");
                        }
                    }
                }
            }
        }
    }

    protected static String announce() {
        if (numberStubs == 0) {
            return "no proofs in progress";
        }
        return numberStubs
            + " proof attempts built so far";
        //return "";
    }

    private static void finishUp() {
        announce();
        if (logWriter != null) {
            try {
                logWriter.flush();
                logWriter.close();
            } catch (IOException e) {
                System.out.println("io exception closing log file " + e);
            }
        }
        
    }

    protected static void addProof(Proof p, double score) {
        debug("shall we really add this proof? " + stepNum);
        Diagram end = p.getLastDiagram();
        int countMatches = 0;
        p.goToStart();
        if (p.getCurrentDiagram().equals(end)) {
            countMatches++;
        }
        while (p.goToNextStep()) {
            if (p.getCurrentDiagram().equals(end)) {
                countMatches++;
            }
        }
        if (countMatches != 1) {
            debug("no - it circles");
            return;
        }

        int intScore = (int) score;
        debug("OK yes: save proof to " + intScore);
        ProofFileUser.addProofAppend(p, getFileName(intScore));
        //System.out.println("saved proof to file "+score);

        if (p.getLastDiagram().equals(target)) {
            p.setLastDiagram(target);
            //System.out.println("proof found!");
            costFound = (int)cc.cost(p);
            if (isLogging) {
                writeln("successful proof found:");
                writeln("" + ProofFileUser.toSaveString(p));
                writeln("costFound is now " + costFound);
            }
            if (proofsFound.size() > 0
                && cc.cost((Proof) proofsFound.get(0)) > cc.cost(p)
                || collectAll == false) {
                proofsFound.clear();
            }
            proofsFound.add(p);
            debug("proof added to proofsFound array");
        }
        if (requester != null) {
            requester.update(announce());
        }
    }

    private static Proof getBestProofSoFar() {
        //System.out.println("getting best proof so far");
        if (isLogging) {
            writeln("seeking best proof so far: costFound = " + costFound);
        }
        costLoop : for (int i = 0; i <= costFound; i++) {
            // try to get an object from each in turn
            debug("try to read proof of length " + i);
            if (i == costFound && !collectAll && proofsFound.size() > 0) {
                debug(
                        "would've carried on looking but don't want to find _all_ proofs");
                return null;
            }
            if (proofsFound.size() > 0) {
                Proof p = (Proof) proofsFound.get(0);
                if (cc.cost(p) < i) { // changed to ensure catch all diags
                    debug(
                            "return null bcs numsteps<i ???? not sure why");
                    return null;
                }
            }
            Proof p = null;
            try {
                p = ProofFileUser.getProofFromWherever(getFileName(i));
                p.getClass();
                if (isLogging) {
                    writeln("got a proof attempt from file : h+c=" + i);
                    //System.out.println("got a proof from file "+i);
                }
            } catch (NullPointerException e) {
                debug(
                        "can't read proof from file " + i + ":" + e);
                continue costLoop;
            }
            debug("got proof is " + ProofFileUser.toSaveString(p));
                //System.out.println("got a proof from file "+i);
            if (requester != null) {
                requester.update(announce());
            }
            return p;
        }
        debug("best proof so far is null");
        return null;
    }

    protected static void writeln(String message) {
        try {
            //debug(message);
            while (message.length() > 0) {
                int n = message.indexOf("\n");
                if (n == -1) {
                    logWriter.write(message);
                    message = "";
                } else {
                    logWriter.write(message.substring(0, n));
                    logWriter.newLine();
                    message = message.substring(n + 1, message.length());
                }
            }
            logWriter.write(message);
            logWriter.newLine();
        } catch (IOException x) {
            System.out.println("IOException in log writing:" + x);
            x.printStackTrace();
            System.exit(0);
        }
    }

    private static void doUnitaryInputRules(
        UnitaryDiagram target_d,
        Proof bestProof,
        HeuristicCalculator hc,
        int stepNum,
        double score,
        StringProgressShower requester) {
        //System.out.println("doing input rules");
        UnitaryDiagram d1 = (UnitaryDiagram) bestProof.getLastDiagram();
        //System.out.println("d1 in unitary proof writer is "+d1);
        ////////////////////////////////////////////////////////////////////
        // apply each rule in turn to bestStub
        // Add Contour
        if (rulesNeedInput.length > 0) {
            Rule rac = RuleAddContour.getRule();
            Vector justD2 = (Vector) target_d.getCloneOfContourSet();
            justD2.removeAll(d1.getCloneOfContourSet());
            for (int i = 0; i < justD2.size(); i++) {
                Contour c = (Contour) justD2.get(i);
                //System.out.println("add contour "+c);
                Proof newp = bestProof.cloneProof();
                RuleAppliesWithString rws =
                    new RuleAppliesWithString(d1, "" + c);
                UnitaryDiagram newEnd = (UnitaryDiagram) rac.applyRule(rws);
                if (newEnd.equals(target)) {
                    newp.addStepAtEnd(rac, rws, target);
                    if (isLogging) {
                        writeln("inc numberStubs");
                    }
                    numberStubs++;
                    double newH =
                        cc.cost(newp) + hc.calcHeuristic(newEnd, target);
                    if (isLogging) {
                        writeln("apply rule " + rac.name);
                        writeln("with spec " + rws);
                        writeln("h+score " + newH);
                    }
                    if (newH <= costFound) {
                        addProof(newp, newH);
                        stepNum++;
                    }
                    announce();
                } else {
                    newp.addStepAtEnd(rac, rws, newEnd);
                    numberStubs++;
                    if (isLogging) {
                        writeln("inc numberStubs");
                    }
                    double newH =
                        cc.cost(newp) + hc.calcHeuristic(newEnd, target);
                    if (isLogging) {
                        writeln("apply rule " + rac.name);
                        writeln("with spec " + rws);
                        writeln("h+score " + newH);
                    }
                    if (newH <= costFound) {
                        addProof(newp, newH);
                        stepNum++;
                    }
                    announce();
                }
            }
        }
    }
    private static void doCompoundInputRules(
        Diagram target,
        Proof bestProof,
        HeuristicCalculator hc,
        int stepNum,
        double score,
        StringProgressShower requester) {
        //System.out.println("doing input rules");
        Diagram d1 = bestProof.getLastDiagram();
        //System.out.println("d1 in unitary proof writer is "+d1);
        ////////////////////////////////////////////////////////////////////
        // apply each rule in turn to bestStub
        // Add Contour
        if (rulesNeedInput.length > 0) {
            Rule rac = RuleAddContour.getRule();
            Vector leaves = target.getUnitaryParts();
            Vector targetContours = new Vector();
            leafLoop : for (int i = 0; i < leaves.size(); i++) {
                UnitaryDiagram ud = (UnitaryDiagram) leaves.get(i);
                if (ud instanceof FalseDiagram) {
                    continue leafLoop;
                }
                Vector leafContours = ud.getCloneOfContourSet();
                for (int j = 0; j < leafContours.size(); j++) {
                    Contour c = (Contour) leafContours.get(j);
                    if (!targetContours.contains(c)) {
                        targetContours.add(c);
                    }
                }
            }
            Vector d1leaves = d1.getUnitaryParts();
            ruleLoop : for (int i = 0; i < d1leaves.size(); i++) {
                UnitaryDiagram d1leaf = (UnitaryDiagram) d1leaves.get(i);
                if (d1leaf instanceof FalseDiagram) {
                    continue ruleLoop;
                }
                Vector contoursToAdd = (Vector) targetContours.clone();
                contoursToAdd.removeAll(d1leaf.getCloneOfContourSet());
                for (int j = 0; j < contoursToAdd.size(); j++) {
                    Contour c = (Contour) contoursToAdd.get(j);
                    //System.out.println("add contour "+c);
                    Proof newp = bestProof.cloneProof();
                    RuleAppliesWithString rws =
                        new RuleAppliesWithString(d1leaf, "" + c);
                    Diagram newEnd = (Diagram) rac.applyRuleToPart(d1, rws);
                    if (newEnd.equals(target)) {
                        newp.addStepAtEnd(rac, rws, target);
                        if (isLogging) {
                            writeln("inc numberStubs");
                        }
                        numberStubs++;
                        double newH =
                            cc.cost(newp) + hc.calcHeuristic(newEnd, target);
                        if (isLogging) {
                            writeln("apply rule " + rac.name);
                            writeln("with spec " + rws);
                            writeln("h+score " + newH);
                        }
                        if (newH <= costFound) {
                            addProof(newp, newH);
                            stepNum++;
                        }
                        announce();
                    } else {
                        newp.addStepAtEnd(rac, rws, newEnd);
                        numberStubs++;
                        if (isLogging) {
                            writeln("inc numberStubs");
                        }
                        double newH =
                            cc.cost(newp) + hc.calcHeuristic(newEnd, target);
                        if (isLogging) {
                            writeln("apply rule " + rac.name);
                            writeln("with spec " + rws);
                            writeln("h+score " + newH);
                        }
                        if (newH <= costFound) {
                            addProof(newp, newH);
                            stepNum++;
                        }
                        announce();
                    }
                }
            }
        }
    }

    private static void setTempDirectory() {
        File f;
        //try {
        //    f = new File("C:\\TEMP\\proofsTest.tmp");
        //    FileWriter fw = new FileWriter(f);
        //    fw.write("test");
        //    fw.close();
        //    tempDirectory = "C:\\TEMP";
        //} catch (Exception x) {
            // show file chooser and filter just for folders		
            JFileChooser jf = new JFileChooser();
            jf.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
            jf.showDialog(null, "choose folder for temporary files");
            File tempFile = jf.getSelectedFile();
            if (tempFile != null)
            	tempDirectory = tempFile.getPath();
        //}
    }
    private static String getFileName(int i) {
        return tempDirectory + "\\" + i + "proofs.tmp";
    }
    private static void debug(String message){
        if(debug){
            if(logWriter!=null)
            {writeln(message);}
            System.out.println(message);
        }
    }
}
class ProofWithScore {
    Proof proof;
    double cost;
    double heuristic;
    ProofWithScore(Proof p, double c, double h) {
        proof = p;
        cost = c;
        heuristic = h;
    }

}
