package rwd.rules.nospec;

import java.util.Vector;
import java.util.Iterator;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.spiderdiagram.CompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.FalseDiagram;
import rwd.abstractsyntax.spiderdiagram.NotCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;
import rwd.rules.application.RuleAppliesToContour;
import rwd.rules.choicespec.RuleDeleteContourSimple;
import rwd.rules.choicespec.RuleWithChoice;

public class NotTrueIsFalseComplexRule extends RuleNoSpec {
    private NotTrueIsFalseComplexRule() {
        super("Not(True) is False (complex)");
    }
    private static Rule example = null;

    public static Rule getRule() {
        if (example == null) {
            example = new NotTrueIsFalseComplexRule();
        }
        return example;
    }
    public boolean isInvertible() {
        return true;
    }

    public boolean canApplyTo(Diagram d) {
        if (!(d instanceof NotCompoundDiagram)) {
            return false;
        } else {
            Diagram child =
                (Diagram) (((NotCompoundDiagram) d).getChildren().get(0));
            if (child instanceof FalseDiagram
                || child instanceof CompoundDiagram) {
                return false;
            } else {
                UnitaryDiagram ud = ((UnitaryDiagram) child);
                //System.out.println("ud is "+DiagramFileUser.toSaveString(ud));
                // apply DeleteContourSimple until you can't any more
                RuleWithChoice rdcs = (RuleWithChoice)RuleDeleteContourSimple.getRule();
                Vector specs = rdcs.getApplicationPossibilities(ud);
                //System.out.println("specs is "+specs);
                Iterator i = specs.iterator();
                while(i.hasNext()){
                    RuleApplicationSpec ras = (RuleApplicationSpec)i.next();
                    Contour c = ((RuleAppliesToContour)(ras)).c;
                    //System.out.println("removing "+c);
                    RuleAppliesToContour rac = new RuleAppliesToContour(ud,c);
                    Diagram newd = rdcs.applyRuleToPart(ud,rac);
                    ud = (UnitaryDiagram)newd;
                    //System.out.println("ud is "+DiagramFileUser.toSaveString(ud));
                }                
                if (ud.getCloneOfContourSet().size() > 0) {
                    return false;
                } else if (ud.getNumShadedZones() > 0) {
                    return false;
                } else if (ud.getSpiders().size() > 0) {
                    return false;
                } else {
                    return true;
                }
            }
        }
    }
    public Diagram applyRule(RuleApplicationSpec r) {
        return new FalseDiagram("f");
    }
}
