package rwd.rules.nospec;
import java.util.Vector;
import java.util.Iterator;

import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.FalseDiagram;
import rwd.abstractsyntax.spiderdiagram.NotCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.abstractsyntax.spiderdiagram.Utilities;
import rwd.rules.Rule;
import rwd.rules.application.RuleApplicationSpec;

public class RemoveNotRuleComplex extends RuleNoSpec {
    private RemoveNotRuleComplex() {
        super("sd: Not(d) is expanded - complex");
    }
    private static Rule example = null;

    public static Rule getRule() {
        if (example == null) {
            example = new RemoveNotRuleComplex();
        }
        return example;
    }

    public boolean isInvertible() {
        return true;
    }

    public boolean canApplyTo(Diagram d) {
        //System.out.println("checking canApply to RNR:"+d);
        if (!(d instanceof NotCompoundDiagram)) {
            //System.out.println("RNR: not a not diagram");
            return false;
        } else {
            Diagram child =
                (Diagram) (((NotCompoundDiagram) d).getChildren().get(0));
            if (!(child instanceof UnitaryDiagram)) {
                //System.out.println("RNR: isn't unitary child");
                return false;
            } else if (child instanceof FalseDiagram) {
                //System.out.println("RNR: is false child");
                return false;
            } else {
                // check there's a spider or some shading
                UnitaryDiagram ud = (UnitaryDiagram) child;
                // check it's an alpha diagram
                Vector spiders = ud.getSpiders();
                for (int si = 0; si < spiders.size(); si++) {
                    Spider s = (Spider) spiders.get(si);
                    if (s.getHabitat().size() > 1) {
                        //System.out.println("RNR: not alpha diagram");
                        return false;
                    }
                }
                // check it's in Venn form
                Vector allZones = Utilities.powerSet(ud.getCloneOfContourSet());
                //System.out.println("all zones is "+allZones);
                //System.out.println("ud.getZones is "+ud.getZones());
                if (ud.getNumZones() != allZones.size()) {
                    //System.out.println("RNR: not vennified");
                    return false;
                }
                //System.out.println("shading "+ud.getCloneOfShadedZones());
                //System.out.println("spiders "+ud.getSpiders());
                return ud.getNumShadedZones()>0 ||
                            ud.getSpiders().size()>0;
            }
        }
    }
    public Diagram applyRule(RuleApplicationSpec r) {
        NotCompoundDiagram ncd = (NotCompoundDiagram) r.d;
        UnitaryDiagram ud = (UnitaryDiagram) (ncd.getChildren().get(0));
        UnitaryDiagram basicUD = (UnitaryDiagram) ud.getClone();
        basicUD.clearShadedZones();
        basicUD.getSpiders().clear();
        Vector newChildren = new Vector();
        //build a disjunction of alpha diagrams built with same zone set 
        //start with one with nothing in it iterate through zones
        Iterator zi = basicUD.getZoneIterator();
        while(zi.hasNext()){
            Zone z = (Zone)zi.next();
            int numSpiders = 0;
            for (int j = 0; j < ud.getSpiders().size(); j++) {
                Spider s = (Spider) ud.getSpiders().get(j);
                if (s.getHabitat().contains(z)) {
                    numSpiders++;
                }
            }
            for (int j = 0; j < numSpiders; j++) {
                UnitaryDiagram child = (UnitaryDiagram) basicUD.getClone();
                for (int k = 0; k < j; k++) {
                    Vector hab = new Vector();
                    hab.add(z);
                    child.addSpider(hab);
                }
                Vector hab = new Vector();
                hab.add(z);
                child.setShadedZones(hab);
                newChildren.add(child);
            }
            if (ud.containsShaded(z)) {
                UnitaryDiagram child = (UnitaryDiagram) basicUD.getClone();
                for (int k = 0; k < numSpiders + 1; k++) {
                    Vector hab = new Vector();
                    hab.add(z);
                    child.addSpider(hab);
                }
                newChildren.add(child);
            }
        }
        if (newChildren.size() > 1) {
            return new OrCompoundDiagram(newChildren, "orcd");
        } else {
            return (Diagram) newChildren.get(0);
        }
    }
}
