package rwd.tableaux.diagram;

import java.util.Vector;
import java.util.Collection;
import java.util.HashMap;
import java.util.TreeSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import rwd.abstractsyntax.eulerdiagram.Contour;
import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.AndCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.CompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Diagram;
import rwd.abstractsyntax.spiderdiagram.FalseDiagram;
import rwd.abstractsyntax.spiderdiagram.NotCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.OrCompoundDiagram;
import rwd.abstractsyntax.spiderdiagram.Spider;
import rwd.abstractsyntax.spiderdiagram.TrueDiagram;
import rwd.abstractsyntax.spiderdiagram.UnitaryDiagram;
import rwd.tableaux.semantic.Options;
import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class EqualiserImpl implements Equaliser {
	/** Constructors */
	public EqualiserImpl(Options opt, ILog log) {
		this.opt = opt;
		this.log = log;
	}
	
	/** Add both missing contours and zones */
	public List equaliseDiagrams(List diags) {
		// Make an AND
		AndCompoundDiagram and = new AndCompoundDiagram(new Vector(), "");
		and.setChildren(new Vector(diags));
		// Equalise
		AndCompoundDiagram newAnd = (AndCompoundDiagram)equaliseDiagram(and);
		return newAnd.getChildren();
	}
	public Diagram equaliseDiagram(Diagram d) {
		d = equaliseContours(d);
		d = equaliseZones(d);
		return d;
	}
		
	/** Add missing contours */
	public List equaliseContours(List diags) {
		// Make an AND
		AndCompoundDiagram and = new AndCompoundDiagram(new Vector(), "");
		and.setChildren(new Vector(diags));
		// Equalise
		AndCompoundDiagram newAnd = (AndCompoundDiagram)equaliseContours(and);
		return newAnd.getChildren();
	}
	public Diagram equaliseContours(Diagram d) {
		// Compute all the contours
		if (opt.isDebug()) {
			log.reportMessage("Compute contours ...");
		}
		Set contours = new TreeSet();
		computeContours(d, contours); 
		if (opt.isDebug()) {
			log.reportMessage(contours.toString());
		}
		// Add missing contours to all the unitary diagrams
		if (opt.isDebug()) {
			log.reportMessage("Add missing contours ...");
		}
		d = equaliseContours(d, contours);

		return d;
	}
	public Diagram equaliseContours(Diagram d, Set contours) {
		// Unitary diagram
		Diagram newDiagram = d; 
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			// true or false
			if (ud instanceof FalseDiagram || ud instanceof TrueDiagram) {
			// User diagram
			} else {
				// Add contours to ud
				for(Iterator i = contours.iterator(); i.hasNext();) {
					Contour c = (Contour)i.next();
					newDiagram = addContour((UnitaryDiagram)newDiagram, c);					
				}
			}
		} else if (d instanceof CompoundDiagram) {
			CompoundDiagram cd = (CompoundDiagram)d;
			// For each child
			Collection children = cd.getChildren();
			Vector newChildren = new Vector();
			for(Iterator i = children.iterator(); i.hasNext();) {
				Diagram child = (Diagram)i.next();
				// Make a new child
				Diagram newChild = equaliseContours(child, contours);
				newChildren.add(newChild);
			}
			// Make a new compound diagram
			CompoundDiagram newParent = null;
			if (cd instanceof NotCompoundDiagram) {
				newParent = new NotCompoundDiagram(newChildren, "");
			} else if (cd instanceof AndCompoundDiagram) {
				newParent = new AndCompoundDiagram(newChildren, "");
			} else if (cd instanceof OrCompoundDiagram) {
				newParent = new OrCompoundDiagram(newChildren, "");
			}
			newDiagram = newParent;
		}
		return newDiagram;
	}

	//
	// Compute contours and zones section
	//
	/** Compute the set of contours used in expression */
	public void computeContours(Diagram d, Set contours) {
		// Null expression
		if (d == null) return;
		// atomic
		if (d instanceof UnitaryDiagram) {
			if (d instanceof TrueDiagram) {
			} else if (d instanceof FalseDiagram) {
			} else {
				// Get the unitary diagram
				UnitaryDiagram ud = (UnitaryDiagram)d;
				// Add contours
				List cs = ud.getCloneOfContourSet();
				for(Iterator i = cs.iterator(); i.hasNext();) {
					Contour c = (Contour)i.next();
					addContour(contours, c); 
				}
			}
		// Not, And, Or, Inmplies, Equivalent
		} else if (d instanceof CompoundDiagram){
			Collection children = ((CompoundDiagram)d).getChildren();
			for(Iterator i = children.iterator(); i.hasNext();) {
				Diagram child = (Diagram)i.next();
				computeContours(child, contours);
			}
		}				
	}
	/** Add contour to a set **/
	protected void addContour(Set set, Contour c) {
		Iterator i = set.iterator();
		boolean isIn = false;
		while (i.hasNext()) {
			Contour c1 = (Contour)i.next();
			if (c1.equals(c)) {
				isIn = true;
				break;
			}
		}
		if (!isIn) 
			set.add(c);
	}
	
	/** Add contours to a unitary diagram */
	public UnitaryDiagram addContours(UnitaryDiagram ud, List cs) {
		for(int i=0; i<cs.size(); i++) {
			ud = addContour(ud, (Contour)cs.get(i));
		}
		return ud;
	}
	
	/** Add a contour to a unitary diagram */
	public UnitaryDiagram addContour(UnitaryDiagram ud, Contour c) {
		// Check if it's already there
		if (ud instanceof TrueDiagram || ud instanceof FalseDiagram) {
			return ud;
		}
		if (ud.getCloneOfContourSet().contains(c)) {
			return ud;
		}
		// Initializes the components of the new diagram: zones, shaded zones, and habitats
		Vector newZones = new Vector();
		Vector newShadedZones = new Vector();
		Map newHabitats = new HashMap();
		Collection oldSpiders = ud.getSpiders();
		for (Iterator i = oldSpiders.iterator(); i.hasNext();) {
			Spider oldSpider = (Spider) i.next();
			newHabitats.put(oldSpider, new Vector());
		}
		// For each old zone
		Collection oldZones = ud.getCloneOfZoneSet();
		for(Iterator i = oldZones.iterator(); i.hasNext();) { 
			Zone oldZone = (Zone) i.next();
			
			// Make a zone contained in c  
			Vector newInContours1 =	(Vector) (oldZone.getContainingContours().clone());
			newInContours1.add(c);
			Vector newOutContours1 = (Vector) (oldZone.getExcludingContours().clone());
			Zone newZone1 = Zone.getZone(newInContours1, newOutContours1);

			// Make a zone not included in c
			Vector newInContours2 =	(Vector) (oldZone.getContainingContours().clone());
			Vector newOutContours2 = (Vector) (oldZone.getExcludingContours().clone());
			newOutContours2.add(c);
			Zone newZone2 = Zone.getZone(newInContours2, newOutContours2);
			
			// Add zones to newZones and newShadedZones
			newZones.add(newZone1);
			newZones.add(newZone2);
			if (ud.containsShaded(oldZone)) {
				newShadedZones.add(newZone1);
				newShadedZones.add(newZone2);
			}
			
			// For each old spider
			for(Iterator j = oldSpiders.iterator(); j.hasNext();) {  
				Spider oldSpider = (Spider) j.next();
				// Add new zones if old zone belongs to the habitat
				if (oldSpider.getHabitat().contains(oldZone)) {
					Vector newHabitat = (Vector) newHabitats.get(oldSpider);
					newHabitat.add(newZone1);
					newHabitat.add(newZone2);
				}
			}
		}
		
		// Make the new diagram
		UnitaryDiagram newDiagram = new UnitaryDiagram(newZones, ud.getName());
		// Set habitats
		for (Iterator i = oldSpiders.iterator(); i.hasNext();) {
			Spider oldSpider = (Spider) i.next();
			newDiagram.addSpider((Vector) newHabitats.get(oldSpider));
		}
		newDiagram.setShadedZones(newShadedZones);
		return newDiagram;		
	}

	/** Add missing (shaded) zones */
	public List equaliseZones(List diags) {
		// Make an AND
		AndCompoundDiagram and = new AndCompoundDiagram(new Vector(), "");
		and.setChildren(new Vector(diags));
		// Equalise
		AndCompoundDiagram newAnd = (AndCompoundDiagram)equaliseZones(and);
		return newAnd.getChildren();
	}
	public Diagram equaliseZones(Diagram d) {
		// Equalise contours first
		d = equaliseContours(d);
		// Compute all the zones
		if (opt.isDebug()) {
			log.reportMessage("Compute zones ...");
		}
		Set zones = new TreeSet();
		computeZones(d, zones); 
		if (opt.isDebug()) {
			log.reportMessage(zones.toString());
		}
				
		// Add missing zones to all the unitary diagrams
		if (opt.isDebug()) {
			log.reportMessage("Add missing zones ...");
		}
		d = equaliseZones(d, zones);
		
		return d;
	}
	public Diagram equaliseZones(Diagram d, Set zones) {
		// Unitary diagram
		Diagram newDiagram = d; 
		if (d instanceof UnitaryDiagram) {
			UnitaryDiagram ud = (UnitaryDiagram) d;
			// true or false
			if (ud instanceof FalseDiagram || ud instanceof TrueDiagram) {
			// User diagram
			} else {
				// Add contours to ud
				for(Iterator i = zones.iterator(); i.hasNext();) {
					Zone z = (Zone)i.next();
					newDiagram = addZone((UnitaryDiagram)newDiagram, z);					
				}
			}
		} else if (d instanceof CompoundDiagram) {
			CompoundDiagram cd = (CompoundDiagram)d;
			// For each child
			Collection children = cd.getChildren();
			Vector newChildren = new Vector();
			for(Iterator i = children.iterator(); i.hasNext();) {
				Diagram child = (Diagram)i.next();
				// Make a new child
				Diagram newChild = equaliseZones(child, zones);
				newChildren.add(newChild);
			}
			// Make a new compound diagram
			CompoundDiagram newParent = null;
			if (cd instanceof NotCompoundDiagram) {
				newParent = new NotCompoundDiagram(newChildren, "");
			} else if (cd instanceof AndCompoundDiagram) {
				newParent = new AndCompoundDiagram(newChildren, "");
			} else if (cd instanceof OrCompoundDiagram) {
				newParent = new OrCompoundDiagram(newChildren, "");
			}
			newDiagram = newParent;
		}
		return newDiagram;
	}

	/** Compute the set of contours used in expression */
	public void computeZones(Diagram d, Set zones) {
		// Null expression
		if (d == null) return;
		// atomic
		if (d instanceof UnitaryDiagram) {
			if (d instanceof TrueDiagram) {
			} else if (d instanceof FalseDiagram) {
			} else {
				// Get the unitary diagram
				UnitaryDiagram ud = (UnitaryDiagram)d;
				// Add zones
				List zs = ud.getCloneOfZoneSet();
				for(Iterator i = zs.iterator(); i.hasNext();) {
					Zone z = (Zone)i.next();
					zones.add(z); 
				}
			}
		// Not, And, Or, Inmplies, Equivalent
		} else if (d instanceof CompoundDiagram){
			Collection children = ((CompoundDiagram)d).getChildren();
			for(Iterator i = children.iterator(); i.hasNext();) {
				Diagram child = (Diagram)i.next();
				computeZones(child, zones);
			}
		}				
	}

	/** Add a zone to a unitary diagram */
	public UnitaryDiagram addZone(UnitaryDiagram ud, Zone z) {
		// Check if it's already there
		if (ud instanceof TrueDiagram || ud instanceof FalseDiagram) {
			return ud;
		}
		if (ud.getCloneOfZoneSet().contains(z)) {
			return ud;
		}
		// Check zone
		List zAll = z.getAllContours();
		List udAll = ud.getCloneOfContourSet();
		if (zAll.containsAll(udAll) && udAll.containsAll(zAll)) {
			// Make a clone
			UnitaryDiagram newUD = (UnitaryDiagram)ud.getClone();
			newUD.getCloneOfZoneSet().add(z);
			newUD.shadeZone(z);
			return newUD;		
		} else {
			log.reportMessage("Cannot add zone '"+z+"'");
			return ud;
		}
	}

	//
	// Local attributes
	//
	protected Options opt;
	protected ILog log;
}
