package rwd.tableaux.semantic;

import java.util.List;

import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.abstractsyntax.spiderdiagram.Diagram;

/**
 * @author Octavian Patrascoiu
 *
 */
public interface ENode {
	/** Search for a compound diagram and create a list of nodes */
	public List expand();

	/** 
	 * Check if the node can be checked for contradictions:
	 * - all diagrams are literals
	 * - there is no not
	 * - all the diagrams are equalised
	 * - all the spiders have one foot
	 */
	public boolean isLeaf();
	
	/** Check if all the diagrams are literals: X and not X */
	public boolean allDiagramsAreLiterals();

	/** Check if there is at least Not diagram (skip not true, not false) */
	public boolean containsNot();

	/** Check if all the unitary diagrams are equalised */
	public boolean allDiagramsAreEqualised();

	/** Check if all the contours are equalised */
	public boolean allContoursAreEqualised();

	/** Check if all the zones are equalised */
	public boolean allZonesAreEqualised();

	/** Check if all the spiders have one foot */
	public boolean allSpidersHaveOneFoot();

	/** Check if all the unitary diagrams contain the contours */
	public boolean allDiagramsContainContours(List contours);
	
	/** Check if all the unitary diagrams contain the zone */
	public boolean allDiagramsContainZone(Zone z);
	
	/** Check if there are some obvious cases */
	public boolean isObviousCase();
	
	/** Check is there are some obvious true cases */
	public boolean isObviousTrueCase();

	/** Check is there are some obvious false cases */
	public boolean isObviousFalseCase();
	
//	/** Check if the node contains only trues */
//	public boolean onlyTrues();
//	
//	/** Check if the node contains only falses */
//	public boolean onlyFalses();
//	
//	/** Check if the node contains only one unitary diagram */
//	public boolean isUnitary();
//	
//	/** Check if the node contains only one AND diagram */
//	public boolean isAnd();
//	
//	/** Check if the node contains only one OR diagram */
//	public boolean isOr();
//	
//	/** Check if the node contains only one NOT diagram */
//	public boolean isNot();
//
	/** Compute contradiction if it's a leaf */
	public Contradiction computeContradiction();
	
	/** Operation kind */
	public OperationKind getOperationKind();
	public void setOperationKind(OperationKind kind);
	
	/** Contained diagrams */
	public List getDiagrams();
	public void setDiagrams(List diags);
	public void addDiagram(Diagram exp);

	/** Override toString */
	public String toString();
}
