package rwd.tableaux.semantic;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.EmptyBorder;

import uk.ac.kent.cs.kmf.util.ILog;

/**
 * @author Octavian Patrascoiu
 *
 */
public class TableauDialogImpl
	extends JDialog
{
	public TableauDialogImpl(JFrame parent, String title, ILog log) {
		super(parent, title);	
		getContentPane().setLayout(new GridBagLayout());
		this.log = log;
		JDialog dialog = new JDialog();

		//
        //--- Create and fill PROJECT panel
        //
		generalPanel = new MatrixPanel();
	    //--- Give the panel a border gap of 5 pixels ---
	    generalPanel.setBorder(new EmptyBorder(new Insets(10, 10, 10, 10)));
	    //--- Get the constraints ---
	    GridBagConstraints c = new GridBagConstraints();
	    //--- Add space around all components to avoid clutter ---
	    c.insets = new Insets(2, 2, 2, 2);
	    c.ipadx = 5;
	    c.ipady = 5;
	    c.weightx = 0.0;
	    c.weighty = 0.0;
		//--- Initialize common variables ---
		int line = 0;
		int column = 0;
    	EmptyBorder border = new EmptyBorder(new Insets(0, 0, 0, 10));
    	Dimension buttonSize = new Dimension(80, 25);
    	int fieldSize = 30;
		JLabel label;
		JButton button;

    	//--- Logical expression label ---
		label = new JLabel("Logical expression");
	    label.setBorder(border);
	    c.gridx = column;
	    c.gridy = line;
	    c.anchor = GridBagConstraints.WEST;
	    generalPanel.add(label, c);
        //--- Logical expression area ---
        logicField = new JTextField(fieldSize);
        logicField.setText("d");
	    c.gridx = column+1;
	    c.gridy = line;
	    generalPanel.add(logicField, c);
	    line++;

		//--- Options area ---
		//--- Release
		label = new JLabel("Debug version");
		label.setBorder(border);
		c.gridx = column;
		c.gridy = line;
		c.anchor = GridBagConstraints.WEST;
		generalPanel.add(label, c);
		debugCheckBox = new JCheckBox();
		debugCheckBox.setSelected(false);
		c.gridx = column+1;
		c.gridy = line;
		generalPanel.add(debugCheckBox, c);
		line++;
		//--- Smooth
		label = new JLabel("Smooth");
		label.setBorder(border);
		c.gridx = column;
		c.gridy = line;
		c.anchor = GridBagConstraints.WEST;
		generalPanel.add(label, c);
		smoothCheckBox = new JCheckBox();
		smoothCheckBox.setSelected(false);
		c.gridx = column+1;
		c.gridy = line;
		generalPanel.add(smoothCheckBox, c);
		line++;
		//--- Diagram size
		label = new JLabel("Diagram size");
		label.setBorder(border);
		c.gridx = column;
		c.gridy = line;
		c.anchor = GridBagConstraints.WEST;
		generalPanel.add(label, c);
		sizeField = new JTextField(fieldSize);
		sizeField.setText("200");
		c.gridx = column+1;
		c.gridy = line;
		generalPanel.add(sizeField, c);
		line++;
		//--- Diagrams per line
		label = new JLabel("Diagrams per line");
		label.setBorder(border);
		c.gridx = column;
		c.gridy = line;
		c.anchor = GridBagConstraints.WEST;
		generalPanel.add(label, c);
		noField = new JTextField(fieldSize);
		noField.setText("5");
		c.gridx = column+1;
		c.gridy = line;
		generalPanel.add(noField, c);
		line++;
		// Display direction
		label = new JLabel("Horizontal display");
		label.setBorder(border);
		c.gridx = column;
		c.gridy = line;
		c.anchor = GridBagConstraints.WEST;
		generalPanel.add(label, c);
		horCheckBox = new JCheckBox();
		horCheckBox.setSelected(true);
		c.gridx = column+1;
		c.gridy = line;
		generalPanel.add(horCheckBox, c);
		line++;
    
		//
        //--- Create and fill CONFIRM panel ---
        //
        JPanel confirmPanel = new JPanel(new GridBagLayout());
        //--- OK button ---
		JButton okButton = new JButton("OK");
		okButton.setPreferredSize(buttonSize);
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                okAction(evt);
            }
        });
	    c.gridx = 0;
	    c.gridy = 0;
		confirmPanel.add(okButton);
        //--- Cancel button ---
		JButton cancelButton = new JButton("Cancel");
		cancelButton.setPreferredSize(buttonSize);
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancelAction(evt);
            }
        });
	    c.gridx = 0;
	    c.gridy = 1;
		confirmPanel.add(cancelButton);

	    //--- Add TAB panel to MAIN panel ---
	    c.gridx = 0;
	    c.gridy = 0;
		getContentPane().add(generalPanel, c);

		//--- Add CONFIRM to MAIN panel ---		
	    c.gridx = 0;
	    c.gridy = 1;
	    c.anchor = GridBagConstraints.CENTER;
	    getContentPane().add(confirmPanel, c);
    		
        // Put frame in the middle of the screen
        pack();
        Dimension screen = getToolkit().getScreenSize();
        int width  = this.getSize().width+20;
        int height = this.getSize().height+20;
        int left = (screen.width - width)/2;
        int top = (screen.height - height)/2;
        setBounds(left, top, width, height);
	}

	//--- OK action ---
	protected int returnVal = JFileChooser.CANCEL_OPTION;
    protected void okAction(java.awt.event.ActionEvent evt) {
    	returnVal = JFileChooser.APPROVE_OPTION;
    	String input = getLogicExp();
		if (input != null && input.length() != 0)  	
			dispose();
    }

	//--- Cancel action ---
    protected void cancelAction(java.awt.event.ActionEvent evt) {
		returnVal = JFileChooser.CANCEL_OPTION;   	
    	dispose();
    }

	//--- Get XMI file name ----
	public int showOpenDialog(JFrame parent) {
		this.setModal(true);
		show();
		return returnVal;
	}
	
	//--- Get the logical expression ----
	public String getLogicExp() {
		return logicField.getText();
	}
	
	//--- Get the options ---
	public Options getOptions() {
		Options opt = new OptionsImpl();
		opt.setDebug(debugCheckBox.isSelected());
		opt.setBinaryOperators(smoothCheckBox.isSelected());
		opt.setDiagramSize(Integer.parseInt(sizeField.getText()));
		opt.setDiagramsPerLine(Integer.parseInt(noField.getText()));
		opt.setHorizontal(horCheckBox.isSelected());
		return opt;
	}

	protected MatrixPanel generalPanel;
	protected JTextField logicField;
	protected JCheckBox debugCheckBox;	
	protected JCheckBox smoothCheckBox;	
	protected JTextField sizeField;	
	protected JTextField noField;
	protected JCheckBox horCheckBox;

	protected ILog log;
}
