package rwd.views.abstracttoconcrete.forcemodel;

import java.util.*;
import java.io.Serializable;

/**
 * @author  $Author: op $
 * @version $Id: Graph.java,v 1.7 2004/10/20 10:08:39 op Exp $
 */
public class Graph implements Serializable, Constants {
    
    public Graph() {
        
    }
    
    // eg "a b ab"
    public Graph(String zones) {
        // Add nodes to the graph.
        StringTokenizer tokenizer = new StringTokenizer(zones, " \t\n\r\f,.;:-");
        while (tokenizer.hasMoreTokens()) {
            String label = sortString(tokenizer.nextToken());
            _nodes.add(new Node(label));
            
            char[] chars = label.toCharArray();
            for (int i = 0; i < chars.length; i++) {
                Character character = new Character(chars[i]);
                _contours.put(character, null);
            }
        }
        
        // Work out where to add edges.
        Node[] nodes = (Node[]) _nodes.toArray(new Node[0]);
        for (int i = 0; i < nodes.length; i++) {
            for (int j = i + 1; j < nodes.length; j++) {
                Node source = nodes[i];
                Node target = nodes[j];
                if (areConnected(source, target)) {
                    _edges.add(new Edge(source, target));
                }
            }
        }
    }
    
    public Node[] getNodeArray() {
        return (Node[]) _nodes.toArray(new Node[0]);
    }
    
    public Edge[] getEdgeArray() {
        return (Edge[]) _edges.toArray(new Edge[0]);
    }

    /**
     * Sorts all chars in a string in ascending order.
     */
    private static String sortString(String input) {
        char[] chars = input.toCharArray();
        Arrays.sort(chars);
        return new String(chars);
    }
    
    /**
     * Returns true is two zones should be connected.
     */
    private boolean areConnected(Node source, Node target) {
        char[] sourceChars = source.getLabel().toCharArray();
        char[] targetChars = target.getLabel().toCharArray();
        // Make sure that targetChars is always the largest of the two arrays.
        if (targetChars.length < sourceChars.length) {
            char[] temp = sourceChars;
            sourceChars = targetChars;
            targetChars = temp;
        }
        if (targetChars.length - sourceChars.length != 1) {
            return false;
        }
        int j = 0;
        for (int i = 0; i < sourceChars.length; i++) {
            while (j < targetChars.length && sourceChars[i] != targetChars[j]) {
                j++;
            }
            if (j == targetChars.length) {
                return false;
            }
        }
        return true;
    }
    
    public void center() {
        
        // Work out the drawing boundaries...
        double minX = Double.POSITIVE_INFINITY;
        double maxX = Double.NEGATIVE_INFINITY;
        double minY = Double.POSITIVE_INFINITY;
        double maxY = Double.NEGATIVE_INFINITY;

        Iterator nodeIt = _nodes.iterator();
        while (nodeIt.hasNext()) {
            Node node = (Node) nodeIt.next();
            
            if (node.getX() > maxX) {
                maxX = node.getX();
            }
            if (node.getX() < minX) {
                minX = node.getX();
            }
            if (node.getY() > maxY) {
                maxY = node.getY();
            }
            if (node.getY() < minY) {
                minY = node.getY();
            }
        }
        
        double xOffset = minX + (maxX - minX) / 2;
        double yOffset = minY + (maxY - minY) / 2;
        
        nodeIt = _nodes.iterator();
        while (nodeIt.hasNext()) {
            Node node = (Node) nodeIt.next();
            node.setX(node.getX() - xOffset);
            node.setY(node.getY() - yOffset);
        }
    }
    
    // Draw this graph on the supplied Graphics2D context.
    // THIS METHOD IS NO LONGER USED.
    /*public void draw(Graphics2D g, int width, int height) {
        AffineTransform transform = new AffineTransform();
        g.setTransform(transform);
        g.setColor(Color.white);
        g.fillRect(0, 0, width, height);
        g.setColor(Color.lightGray);
        g.setStroke(new BasicStroke(EDGE_WIDTH));
        
        center();
        
        double zoomFactor = ZOOM_FACTOR;
        transform.scale(zoomFactor, zoomFactor);
        transform.translate(width / 2 / zoomFactor, height / 2 / zoomFactor);
        g.setTransform(transform);
        
        
        // Draw edges.
        Iterator edgeIt = _edges.iterator();
        while (edgeIt.hasNext()) {
            Edge edge = (Edge) edgeIt.next();
            Node source = edge.getSource();
            Node target = edge.getTarget();
            Line2D line = new Line2D.Double(source.getX(), source.getY(), target.getX(), target.getY());
            g.draw(line);
        }
        
        // Draw nodes.
        Iterator nodeIt = _nodes.iterator();
        while (nodeIt.hasNext()) {
            Node node = (Node) nodeIt.next();
            double radius = NODE_RADIUS;
            Ellipse2D ellipse = new Ellipse2D.Double(node.getX() - radius, node.getY() - radius, 2 * radius, 2 * radius);
            g.setColor(Color.lightGray);
            g.fill(ellipse);
            
            // Draw label, centered.
            g.setColor(Color.white);
            String label = node.getLabel();
            FontMetrics metrics = g.getFontMetrics(g.getFont());
            Rectangle2D rect = metrics.getStringBounds(label, g);
            g.drawString(label, (int) (node.getX() - rect.getWidth() / 2), (int) (node.getY() + rect.getHeight() / 2));
        }
    }*/
    
    public HashMap getContours() {
        return _contours;
    }
    
    public LinkedHashSet getNodes() {
        return _nodes;
    }
    
    public String toString() {
        StringBuffer buffer = new StringBuffer();
        buffer.append("GRAPH:\n");
        buffer.append("NODES:\n");
        Iterator nodeIt = _nodes.iterator();
        while (nodeIt.hasNext()) {
            Node node = (Node) nodeIt.next();
            buffer.append(node.toString() + '\n');
        }
        buffer.append("EDGES:\n");
        Iterator edgeIt = _edges.iterator();
        while (edgeIt.hasNext()) {
            Edge edge = (Edge) edgeIt.next();
            buffer.append(edge.toString() + '\n');
        }
        return buffer.toString();
    }
    
    private LinkedHashSet _nodes = new LinkedHashSet();
    private LinkedHashSet _edges = new LinkedHashSet();
    private HashMap _contours = new HashMap();
    
}
