package rwd.views.abstracttoconcrete.forcemodel;

import rwd.utilities.*;
import rwd.concretesyntax.*;
import rwd.concretesyntax.eulerdiagram.*;
import rwd.concretesyntax.spiderdiagram.*;
import rwd.gui.editor.viewers.*;

import java.awt.geom.GeneralPath;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;
import java.awt.geom.Rectangle2D;
import java.util.*;

public class SpiderPlacer {
    
    public SpiderPlacer(DiagramViewerInterface viewer) {
        this(viewer, null, null);
    }
    
    /**
     * @param viewer
     * @param footMapping maps the moveDiagram feet to the stillDiagram feet
     * @param stillDiagram the still (or previous) diagram
     */
    public SpiderPlacer(DiagramViewerInterface viewer, HashMap footMapping, ConcreteUnitaryDiagram stillDiagram) {
        _viewer = viewer;
        _footMapping = footMapping;
        _stillDiagram = stillDiagram;

        _nodes = new Vector();
    }
    
    public void embed() {
        // group the spider feet into zone groups.
        groups = new HashMap();
        feetToZones = new HashMap();
        zoneAreas = new Vector(_viewer.getConcreteDiagram().getVisibleConcreteZones());
        spiderFeet = _viewer.getConcreteDiagram().getFeetAndLegsSet().getFeet();
        //for (int j = 0; j < zoneAreas.size(); j++) {
        //        ConcreteZone za = (ConcreteZone) zoneAreas.get(j);
        //        za.recalcArea();
        //}        
        for (int i = 0; i < spiderFeet.size(); i++) {
            ConcreteSpiderFoot foot = (ConcreteSpiderFoot) spiderFeet.get(i);
            for (int j = 0; j < zoneAreas.size(); j++) {
                ConcreteZone za = (ConcreteZone) zoneAreas.get(j);
                AreaRWD a = za.getArea();
                if (a.contains(foot.getX(), foot.getY())) {

                    feetToZones.put(foot, za);
                    //System.out.println("putting foot "+foot+" into zone "+za.getZoneString());
                    
                    foot.setZone(za.toString());
                    
                    if (!groups.containsKey(za)) {
                        groups.put(za, new Vector());
                    }
                    Vector nodes = (Vector) groups.get(za);
                    Node node = new Node();
                    node.setObject(foot);
                    node.setX(foot.getX());
                    node.setY(foot.getY());
                    nodes.add(node);
                    _nodes.add(node);
                    
                    //System.out.println("Added foot to " + za.getZoneString());
                    
                    continue;
                }
            }
            
        }

        for (int i = 0; i < _nodes.size(); i++) {
            Node node = (Node) _nodes.get(i);
            ConcreteSpiderFoot foot = (ConcreteSpiderFoot) node.getObject();
            //foot.setLocation((int) node.getX(), (int) node.getY());
            node.setX(foot.getX());
            node.setY(foot.getY());            
        }
        
        //ContourPolygonSet polygonSet = _viewer.getContourPolygonSet();
        ConcreteSpiderFeetAndLegsSet spiderSet = _viewer.getConcreteDiagram().getFeetAndLegsSet();
        //polygonSet.calcZoneAreasForStructureChecking();

        // Get the set of line segments for each zone area.
        //System.out.println("- Getting set of line segments for each zone area");
        HashMap segmentGroup = new HashMap();
        for (int i = 0; i < zoneAreas.size(); i++) {
            ConcreteZone za = (ConcreteZone) zoneAreas.get(i);
            AreaRWD a = za.getArea();
            a = makeMorePoints(a, 100);
            Vector segments = getLineSegments(a);
            segmentGroup.put(za, segments);
        }
       


        if (_animated) {
//            _viewer.draw(polys);
//            _viewer.getDiagramCanvas().repaintImage();
            _viewer.getDiagramCanvas().drawImageNow();
        }
        // For each group of nodes...
        //System.out.println("- Embedding spider feet with force model");
        Iterator it = groups.keySet().iterator();
        while (it.hasNext()) {
            ConcreteZone za = (ConcreteZone) it.next();
            Vector nodes = (Vector) groups.get(za);
            Vector segments = (Vector) segmentGroup.get(za);
            
            //System.out.println("Embedding group of size " + nodes.size() + " for zone " + za.getZoneString());
            
            for (int iteration = 0; iteration < 70; iteration++) {
                
                // Calculate node-node repulsions...
                for (int a = 0; a < nodes.size(); a++) {
                    for (int b = a + 1; b < nodes.size(); b++) {
                        Node nodeA = (Node) nodes.get(a);
                        Node nodeB = (Node) nodes.get(b);
                        
                        double deltaX = nodeB.getX() - nodeA.getX();
                        double deltaY = nodeB.getY() - nodeA.getY();
                        
                        double distanceSquared = deltaX * deltaX + deltaY * deltaY;
                        
                        if (distanceSquared < 0.01) {
                            deltaX = Math.random() / 10 + 0.1;
                            deltaY = Math.random() / 10 + 0.1;
                            distanceSquared = deltaX * deltaX + deltaY * deltaY;
                        }
                        
                        double distance = Math.sqrt(distanceSquared);
                        
                        double repulsiveForce = nodes.size() * (_k * _k) / (distance);
                        
                        nodeB.setFX(nodeB.getFX() + (repulsiveForce * deltaX / distance));
                        nodeB.setFY(nodeB.getFY() + (repulsiveForce * deltaY / distance));
                        nodeA.setFX(nodeA.getFX() - (repulsiveForce * deltaX / distance));
                        nodeA.setFY(nodeA.getFY() - (repulsiveForce * deltaY / distance));
                        
                    }
                }
                
                // calculate segment-node repulsions
                for (int a = 0; a < nodes.size(); a++) {
                    for (int b = 0; b < segments.size(); b++) {
                        for (int c = 0; c < 2; c++) {
                            Node nodeA = (Node) nodes.get(a);
                            Line2D segment = (Line2D) segments.get(b);
                            
                            double x1 = segment.getX1();
                            double y1 = segment.getY1();
                            double x2 = segment.getX2();
                            double y2 = segment.getY2();
                            
                            double length = Math.sqrt((x1 - x2) * (x1 - x2) + (y1 - y2) * (y1 - y2));
                            
                            double deltaX = x1 - nodeA.getX();
                            double deltaY = y1 - nodeA.getY();
                            if (c == 1) {
                                deltaX = x2 - nodeA.getX();
                                deltaY = y2 - nodeA.getY();
                            }
                            
                            double distance = segment.ptSegDist(nodeA.getX(), nodeA.getY());
                            //double distance = Math.sqrt(deltaX * deltaX + deltaY * deltaY);
                            
                            double repulsiveForce = nodes.size() * nodes.size() * length * (_k * _k) / (distance * distance);
                            nodeA.setFX(nodeA.getFX() - (repulsiveForce * deltaX / distance));
                            nodeA.setFY(nodeA.getFY() - (repulsiveForce * deltaY / distance));
                        }
                    }
                }
                
                // For each node, add an extra force towards it previous
                // location in the stillDiagram.
                if (_stillDiagram != null) {

                    for (int a = 0; a < nodes.size(); a++) {
                        Node node = (Node) nodes.get(a);
                        ConcreteSpiderFoot moveFoot = (ConcreteSpiderFoot) node.getObject();
                        ConcreteSpiderFoot stillFoot = (ConcreteSpiderFoot) _footMapping.get(moveFoot);

                        //System.out.println(_footMapping);
                        //System.out.println(moveFoot);
                        
                        if (stillFoot != null) {
                        
                            //double x1 = moveFoot.getX();
                            //double y1 = moveFoot.getY();
                            double x1 = node.getX();
                            double y1 = node.getY();

                            double x2 = stillFoot.getX();
                            double y2 = stillFoot.getY();
                            
                            double dx = x2 - x1;
                            double dy = y2 - y1;
                            
                            double distSq = dx * dx + dy * dy;
                            double dist = Math.sqrt(distSq);
                            
                            double attractiveForce = 1.2 * 1e-5 * nodes.size() * distSq / _k;
                            node.setFX(node.getFX() + (attractiveForce * dx / dist));
                            node.setFY(node.getFY() + (attractiveForce * dy / dist));
                        }
                    }
                }
                
                // move the nodes
                for (int a = 0; a < nodes.size(); a++) {
                    Node node = (Node) nodes.get(a);
                    
                    double xMovement = _c * node.getFX();
                    double yMovement = _c * node.getFY();
                    
                    // System.out.println("x movement " + xMovement + ", y movement" + yMovement);

                    if(xMovement > _maxMovement) {
                        xMovement = _maxMovement;
                    } else if(xMovement < -_maxMovement) {
                        xMovement = -_maxMovement;
                    } 
                                        
                    if(yMovement > _maxMovement) {
                        yMovement = _maxMovement;
                    } else if(yMovement < -_maxMovement) {
                        yMovement = -_maxMovement;
                    }
                                        

                    node.setX(node.getX() + xMovement);
                    node.setY(node.getY() + yMovement);
                    
                    // Reset the forces
                    node.setFX(0);
                    node.setFY(0);
                }

				// Update the feet positions
				for (int i = 0; i < nodes.size(); i++) {
					Node node = (Node) nodes.get(i);
					ConcreteSpiderFoot foot = (ConcreteSpiderFoot) node.getObject();
					System.out.println("Move spider foot "+foot);
					foot.setLocation((int) node.getX(), (int) node.getY());
					System.out.println("to "+foot);
				}

				if (_animated) {
					_viewer.redrawImage();
				}

            }
            
        }
        
        // Work out the bounding box.
        DecorationBox box = _viewer.getBoxAround(_viewer.getConcreteDiagram().getConcreteContours(), 0);
        Rectangle2D rect = box.getShape().getBounds2D();
        double area = rect.getWidth() * rect.getHeight();
        double sqrtArea = Math.sqrt(area);
        
        boolean swapFeet = true;
        if (swapFeet) {
            // Now we attempt to swap feet in order to reduce edge crossings.
            //System.out.println("- Trying to minimize number of edge crossings");
            // Only bother looking at zones with more than one unmapped node in!
            it = groups.keySet().iterator();
            Vector nodeGroups = new Vector();
            while (it.hasNext()) {
                ConcreteZone za = (ConcreteZone) it.next();
                Vector nodes = (Vector) groups.get(za);
                    
                if (_footMapping != null) {
                    // remove nodes which are mapped in footmapper                    
                    Iterator it2 = nodes.iterator();
                    Vector toRemove = new Vector();
                    while(it2.hasNext()){
                        Node n = (Node)it2.next();
                        ConcreteSpiderFoot foot = (ConcreteSpiderFoot)n.getObject();
                        if(_footMapping.get(foot)!=null){
                            toRemove.add(n);
                        }
                    }
                    nodes.removeAll(toRemove);
                }
                
                if (nodes.size() > 1) {
                    nodeGroups.add(nodes);
                }
            }
            System.out.println("Found " + nodeGroups.size() + " zones with more than one unmapped node.");
            if (nodeGroups.size() > 0) {
                for (int iteration = 0; iteration < 100; iteration++) {
                    
                    // select two random feet from the zone
                    Vector group = (Vector) nodeGroups.get(RAND.nextInt(nodeGroups.size()));
                    int selection1 = RAND.nextInt(group.size());
                    int selection2 = selection1;
                    while (selection2 == selection1) {
                        selection2 = RAND.nextInt(group.size());
                    }
                    Node node1 = (Node) group.get(selection1);
                    Node node2 = (Node) group.get(selection2);
                    
                    double fitness = spiderSet.getEdgeCrossings() + 0.0001 * (spiderSet.totalSquareEdgeLengths() / sqrtArea);
                    swapNodes(node1, node2);
                    //System.out.println("Trial edge crossings: " + spiderSet.getEdgeCrossings());
                    double newFitness = spiderSet.getEdgeCrossings() + 0.0001 * (spiderSet.totalSquareEdgeLengths() / sqrtArea);
                    // Fittest value is 0. Worse is >0
                    if (newFitness > fitness) {
                        // put the nodes back
                        swapNodes(node1, node2);
                    }
                    //System.out.println("Fitness: " + fitness);
                    
                }
            }
        }
        
        //System.out.println("- Done... updating display");
        _viewer.redrawImage();
    }
    
    // swap the location of two nodes
    private void swapNodes(Node node1, Node node2) {
        // remember each node wraps a concrete spider foot object
        ConcreteSpiderFoot foot1 = (ConcreteSpiderFoot) node1.getObject();
        ConcreteSpiderFoot foot2 = (ConcreteSpiderFoot) node2.getObject();
        int tempX = foot1.getX();
        int tempY = foot1.getY();
        foot1.setLocation(foot2.getX(), foot2.getY());
        foot2.setLocation(tempX, tempY);
    }

    private Vector getLineSegments(AreaRWD area) {
        Vector segments = new Vector();
        int type = 0;
        float oldX = 0;
        float oldY = 0;
        float[] coords = new float[6];
        PathIterator it = area.getPathIterator(null);
        while (!it.isDone()) {
            type = it.currentSegment(coords);
            float x1 = coords[0];
            float y1 = coords[1];
            
            if (type == PathIterator.SEG_MOVETO) {
                oldX = x1;
                oldY = y1;
            }
            else if (type == PathIterator.SEG_LINETO) {
                segments.add(new Line2D.Double(oldX, oldY, x1, y1));
                oldX = x1;
                oldY = y1;
            }
            it.next();
        }
        return segments;
    }

    // returns a copy of the area that has more than n points in it.
    private AreaRWD makeMorePoints(AreaRWD area, int n) {
        AreaRWD newArea = new AreaRWD(area);
        int npoints = 0;
        while (npoints < n) {
            GeneralPath path = new GeneralPath();
            npoints = 0;
            int type = 0;
            float oldX = 0;
            float oldY = 0;
            float[] coords = new float[6];
            PathIterator it = newArea.getPathIterator(null);
            while (!it.isDone()) {
                type = it.currentSegment(coords);
                float x1 = coords[0];
                float y1 = coords[1];
                
                if (type == PathIterator.SEG_MOVETO) {
                    path.moveTo(x1, y1);
                    oldX = x1;
                    oldY = y1;
                }
                else if (type == PathIterator.SEG_LINETO || type == PathIterator.SEG_CUBICTO || type == PathIterator.SEG_QUADTO) {
                    path.lineTo(oldX + (x1 - oldX) / 2, oldY + (y1 - oldY) / 2);
                    path.lineTo(x1, y1);
                    npoints += 2;
                    oldX = x1;
                    oldY = y1;
                }
                else if (type == PathIterator.SEG_CLOSE) {
                    path.closePath();
                }
                it.next();
            }
            newArea = new AreaRWD(path);
        }
        
        return newArea;
    }

    // In case the underlying Euler diagram has been altered, this method
    // places the spider feet back in the correct zones. It makes sure that
    // the label of each foot corresponds to the zone it belongs to. 
    public void reinitializeSpiderFeet() {
        Vector newZoneAreas = new Vector(_viewer.getConcreteDiagram().getVisibleConcreteZones());
        
        for (int i = 0; i < _nodes.size(); i++) {
            Node node = (Node) _nodes.get(i);
            ConcreteSpiderFoot foot = (ConcreteSpiderFoot) node.getObject();
            foot.setLocation((int) foot.getX(), (int) foot.getY());
        }
    }
    
   	public boolean getAnimated() {
		return _animated;
	}

	public void setAnimated(boolean b) {
		_animated = b;
	}

    
    private DiagramViewerInterface _viewer;
    private HashMap _footMapping;    
    private ConcreteUnitaryDiagram _stillDiagram;
    
    private List zoneAreas;
    private List spiderFeet;
    private List _nodes = new Vector();
    private HashMap groups;
    private HashMap feetToZones;
    
    private double _k = 2.0;
    private double _c = 3.0;
    private static final Random RAND = new Random();

	private boolean _animated = false;
	private double _maxMovement = 20.0;

}
