package rwd.views.graph;
import java.util.Vector;
import java.util.Iterator;
import java.util.TreeSet;

public class BlockGraph {
	private BlockNode parentNode; // will be the node with the smallest label
	private Vector childNodes = new Vector();
	// will be cutNodes of this rwd.views.graph
	private Graph g;

	public BlockGraph(
		BlockNode parent,
		Graph g) { //System.out.println(""+this.hashCode()+":In BlockGraph constructor with "+g);
		this.parentNode = parent;
		//System.out.println("set parent to be BlockNode "+parent);
		this.g = g;
		this.split();
		//System.out.println(""+this.hashCode()+":Finished BlockGraph constructor");
		//System.out.println("having built "+this);
	}

	public Graph graph() {
		return g;
	}

	public BlockNode parent() {
		return parentNode;
	}

	public Vector children() {
		return childNodes;
	}

	void split() { // iterate through the nodes of g
		// find a cut vertex - then
		// split the rwd.views.graph into pieces and build new BlockNodes
		boolean haveSplit = false;
		//System.out.println("in split() for "+this.hashCode());
		//System.out.println(": iterate through nodes "+g.nodes());
		Iterator nodeIt = g.nodes().iterator();
		loop : while (nodeIt.hasNext()) {
			Node n = (Node) (nodeIt.next());
			//System.out.println("consider node "+n);
			TreeSet nodesCopy = (TreeSet) (g.nodes().clone());
			TreeSet edgesCopy = (TreeSet) (g.edges().clone());
			nodesCopy.remove(n);
			//System.out.println("removing node "+n);
			// remove edges adjacent to n, see if remainder is connected
			Iterator j = g.edges().iterator();
			while (j.hasNext()) {
				Edge e = (Edge) (j.next());
				TreeSet ends = e.nodes;
				if (ends.contains(n)) {
					edgesCopy.remove(e);
				}
			}
			Graph temp = new Graph(nodesCopy, edgesCopy);
			//System.out.println("rwd.views.graph with node removed is "+temp);
			// see if this rwd.views.graph is connected
			Vector components = temp.components();
			if (components.size() == 1) // n was a cutvx
				{ //System.out.println("was not a cut vertex");
				//System.out.println("node was not a cut vertex");
				continue loop;
			}
			//System.out.println("found a cutvertex "+n);
			haveSplit = true;
			// replace the cutvertex n in all components
			Iterator compIt = components.iterator();
			while (compIt.hasNext()) {
				Graph component = (Graph) (compIt.next());
				component.nodes().add(n);
				Iterator edgeIt = g.edges().iterator();
				while (edgeIt.hasNext()) {
					Edge e = (Edge) (edgeIt.next());
					component.checkAndAdd(e);
				}
			}
			//System.out.println("after replacing "+n+" in each component");
			//System.out.println("components are "+components);
			// go through each component
			// if they contain the cutNode of this block
			//System.out.println("set childNode "+childNode);
			BlockNode child = new BlockNode(n, this);
			childNodes.add(child);
			//System.out.println("set childNode "+childNode);
			// then just replace the rwd.views.graph of this with this component
			// otherwise build a new BlockGraph for each
			compIt = components.iterator();
			boolean haveFoundAParent = false;
			while (compIt.hasNext()) {
				Graph component = (Graph) (compIt.next());
				if (component.nodes().contains(parentNode.node())
					&& !haveFoundAParent) {
					this.g = component;
					haveFoundAParent = true;
					//System.out.println("adding component attached to parent:"+component);
				} else // attach a new BlockGraph to the right ChildNode
					{
					BlockGraph bg = new BlockGraph(child, component);
					child.addChildGraph(bg);
					bg.split();
					//System.out.println("adding child for:"+child+" and component "+component);
				}
			}
			break loop;
		}
		if (haveSplit) {
			split();
		}
	}

	public String toString() {
		return "bg: " + g;
	}
/*
	public static void main(String[] args) {
		TreeSet nodes = new TreeSet();
		Node n = Node.getNode(new TreeSet());
		nodes.add(n);
		TreeSet labels = new TreeSet();
		labels.add("a");
		Node na = Node.getNode(labels);
		nodes.add(na);
		labels = new TreeSet();
		labels.add("a");
		labels.add("b");
		Node nab = Node.getNode(labels);
		nodes.add(nab);
		labels = new TreeSet();
		labels.add("a");
		labels.add("c");
		Node nac = Node.getNode(labels);
		nodes.add(nac);
		TreeSet edges = new TreeSet();
		edges.add(Edge.getEdge(n, na));
		edges.add(Edge.getEdge(na, nab));
		edges.add(Edge.getEdge(na, nac));
		Graph g = new Graph(nodes, edges);

		BlockNode bn = new BlockNode(Node.getNode(new TreeSet()), null);
		BlockGraph bg = new BlockGraph(bn, g);
		System.out.println("parentNode is " + bg.parentNode);
		System.out.println("top of tree is " + bg);
		System.out.println("childNodes are " + bg.childNodes);
		System.out.println("all blocks are " + bn.getBlocks());
	}
	*/

}
