package rwd.views.smoother;

import java.awt.*;
import java.awt.event.ActionEvent;
import java.io.*;
import java.net.*;
import java.util.*;

import javax.swing.*;
import javax.swing.border.Border;

import rwd.abstractsyntax.eulerdiagram.*;
import rwd.concretesyntax.*;
import rwd.concretesyntax.eulerdiagram.*;
import rwd.concretesyntax.spiderdiagram.*;
import rwd.concretesyntax.constraintdiagram.*;

import rwd.gui.*;
import rwd.gui.editor.*;
import rwd.gui.editor.viewers.*;
import rwd.metrics.*;

/**
 * @author Octavian Patrascoiu
 *
 */
public abstract class AbstractParameterWindow 
	extends JFrame 
{		
	public AbstractParameterWindow(String title) {
		super(title);
	}

	public AbstractParameterWindow(
		EditorFrame frame,
		ConcreteUnitaryDiagram cud,
		DiagramViewerComponent viewer,
		String title,
		int offset,
		boolean noWindow) 
	{	
		super(title);
		if (!noWindow) {
			startup(frame, cud, viewer, false);
		}
	}

	public AbstractParameterWindow(
		EditorFrame frame,
		ConcreteUnitaryDiagram cud,
		DiagramViewerInterface viewer) {
		super("Drawing Parameters");
		startup(frame, cud, viewer, false);
	}

	/** 
	 * The integrateViewer parameter means the viewer appears inside the parameter window
	 * if it is false, the viewer should be provided with an alternative top level window.
	 */
	public AbstractParameterWindow(
		EditorFrame frame,
		ConcreteUnitaryDiagram cud,
		DiagramViewerInterface viewer,
		boolean integrateViewer) {
		super("Drawing Parameters");
		startup(frame, cud, viewer, integrateViewer);
	}

	protected void startup(
		EditorFrame frame,
		ConcreteUnitaryDiagram cud,
		DiagramViewerInterface viewer,
		boolean integrateViewer) 
	{
		System.out.println("C " + frame);
		this.frame = frame;
		this.cud = cud;
		this.viewer = viewer;
		this.integrateViewer = integrateViewer;

		setupMetrics();
		setupManipulators();
		setupPanels();
		// Notify observers
		Iterator it = cud.getDiagramItems().iterator();
		while (it.hasNext()) {
			DiagramItem item = (DiagramItem)it.next();
			item.setChanged();
			item.notifyObservers();
		}

		Dimension d = Toolkit.getDefaultToolkit().getScreenSize();
		double maxX = d.getWidth() - this.getWidth();
		this.setLocation((int) maxX, 0);

		setVisible(true);

		System.out.println("invoke draw in viewer");
		viewer.draw(cud); ///////////////////////////////!!

		loadDefaultParameters();
	}

	protected void abstractButton(ActionEvent e) {
	}
	public void actionPerformed(ActionEvent event) {
	}

	protected void setupPanels() {
		variousPanel = new JPanel();
		metricPanel = new JPanel();
		manipulatorPanel = new JPanel();
		buttonPanel = new JPanel();

		progressPanel = new JPanel();

		setupVarious(variousPanel);
		setupMetricsPanel(metricPanel);
		setupManipulatorsPanel(manipulatorPanel);
		setupButtons(buttonPanel);
		setupProgress(progressPanel);

		Border etchedBorder = BorderFactory.createEtchedBorder();
		Border spaceBorder1 = BorderFactory.createEmptyBorder(1, 1, 1, 1);
		Border spaceBorder2 = BorderFactory.createEmptyBorder(1, 1, 1, 1);
		Border compoundBorder =
			BorderFactory.createCompoundBorder(etchedBorder, spaceBorder1);
		Border panelBorder =
			BorderFactory.createCompoundBorder(spaceBorder2, compoundBorder);

		outputDataArea = new JTextArea();
		outputDataArea.setMinimumSize(new Dimension(200,42));
		FontScrollPane scroller = new FontScrollPane(outputDataArea);
		scroller.setPreferredSize(new Dimension(400, 200));
		scroller.setMinimumSize(new Dimension(250, 100));

		variousPanel.setBorder(panelBorder);
		metricPanel.setBorder(panelBorder);
		//scalePanel.setBorder(panelBorder);
		manipulatorPanel.setBorder(panelBorder);

		GridBagLayout gridbag = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();

		getContentPane().setLayout(gridbag);

		c.ipadx = 2;
		c.ipady = 2;

		if (integrateViewer) {
			c.gridx = 0;
			c.gridy = 0;
			c.gridwidth = 1;
			c.gridheight = 4;
			c.anchor = GridBagConstraints.NORTH;
			gridbag.setConstraints((Component) viewer, c);
			getContentPane().add((Component) viewer);
		}

		c.gridx = 1;
		c.gridy = 0;
		c.gridwidth = 2;
		c.gridheight = 1;
		gridbag.setConstraints(metricPanel, c);
		getContentPane().add(metricPanel);

		c.gridx = 1;
		c.gridy = 1;
		c.gridwidth = 1;
		gridbag.setConstraints(variousPanel, c);
		getContentPane().add(variousPanel);

		c.gridx = 2;
		c.gridy = 1;
		c.gridwidth = 1;
		gridbag.setConstraints(manipulatorPanel, c);
		getContentPane().add(manipulatorPanel);

		c.gridx = 1;
		c.gridy = 2;
		c.gridwidth = 2;
		gridbag.setConstraints(buttonPanel, c);
		getContentPane().add(buttonPanel);

		c.gridx = 1;
		c.gridy = 3;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(progressPanel, c);
		getContentPane().add(progressPanel);

		c.gridx = 1;
		c.gridy = 4;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.CENTER;
		gridbag.setConstraints(scroller, c);
		getContentPane().add(scroller);
		pack();
	}

	protected abstract void setupMetrics();
	protected abstract void setupManipulators();
	protected abstract void setupButtons(JPanel panel);

	protected void setupMetricsPanel(JPanel panel) {
		// display the fields
		GridBagLayout gridbag = new GridBagLayout();
		panel.setLayout(gridbag);

		GridBagConstraints c = new GridBagConstraints();

		c.ipadx = 2;
		c.ipady = 2;
		c.anchor = GridBagConstraints.WEST;

		int yLevel = 0;

		JCheckBox button;
		JTextField field;

		for (int index = 0; index < metrics.size(); index++) {
			Metric m = (Metric) (metrics.get(index));

			button = new JCheckBox((m.getClass().getName().substring(12)));
			metricButtons.add(button);
			field = new JTextField(5);
			field.setMinimumSize(MINIMUM_TEXT_SIZE);
			field.setText(Double.toString(m.getWeighting()));
			metricFields.add(field);

			Boolean use = (Boolean) metricBooleans.get(index);
			if (use.equals(Boolean.TRUE)) {
				button.setSelected(true);
				metricValues.add(m);
			}

			if (index < numRows) {
				c.gridx = 0;
				c.gridy = yLevel;
				gridbag.setConstraints(button, c);
				panel.add(button);

				c.gridx = 1;
				gridbag.setConstraints(field, c);
				panel.add(field);

				yLevel = (yLevel + 1) % (numRows + 1);

			} else {
				c.gridx = 2;
				c.gridy = yLevel - numRows;
				gridbag.setConstraints(button, c);
				panel.add(button);

				c.gridx = 3;
				gridbag.setConstraints(field, c);
				panel.add(field);

				yLevel++;
			}
		}

		/*		updateButton = new JButton("Update Weights");
				updateButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent event) {
						updateButton(event);
					}
				});
		
				c.gridx = 0;
				c.gridy = yLevel;
				c.gridwidth = 2;
				c.anchor = GridBagConstraints.CENTER;
				gridbag.setConstraints(updateButton, c);
				panel.add(updateButton);
		*/
	}

	protected void setupManipulatorsPanel(JPanel panel) {
		// display the buttons
		GridBagLayout gridbag = new GridBagLayout();
		panel.setLayout(gridbag);

		GridBagConstraints c = new GridBagConstraints();

		c.ipadx = 2;
		c.ipady = 2;
		c.anchor = GridBagConstraints.CENTER;

		int yLevel = 0;

		JRadioButton button;

		manipulatorGroup = new ButtonGroup();

		Iterator mi = manipulators.iterator();
		while (mi.hasNext()) {
			Manipulator m = (Manipulator) mi.next();
			button = new JRadioButton((m.getClass().getName()).substring(19));
			manipulatorGroup.add(button);
			manipulatorButtons.add(button);
			if (manipulatorButtons.size() == 1) {
				button.setSelected(true);
				manipulator = m;
			}
			c.gridx = 0;
			c.gridy = yLevel;
			gridbag.setConstraints(button, c);
			panel.add(button);
			yLevel++;
		}
	}

	protected void setupProgress(JPanel panel) {
		GridBagLayout gridbag = new GridBagLayout();
		panel.setLayout(gridbag);

		GridBagConstraints c = new GridBagConstraints();

		c.ipadx = 2;
		c.ipady = 2;
		c.anchor = GridBagConstraints.CENTER;

		statusLabel = new JLabel("");
		progressLabel = new JLabel("0");
		iterationsLabel = new JLabel("-1");
		JLabel firstLabel =
			new JLabel("Iterations started: ", SwingConstants.LEFT);
		JLabel middleLabel = new JLabel(" of ", SwingConstants.LEFT);

		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.EAST;
		gridbag.setConstraints(statusLabel, c);
		panel.add(statusLabel);

		c.gridx = 1;
		c.gridy = 0;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(firstLabel, c);
		panel.add(firstLabel);

		c.gridx = 2;
		c.gridy = 0;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(progressLabel, c);
		panel.add(progressLabel);

		c.gridx = 3;
		c.gridy = 0;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(middleLabel, c);
		panel.add(middleLabel);

		c.gridx = 4;
		c.gridy = 0;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(iterationsLabel, c);
		panel.add(iterationsLabel);
	}

	protected void setupVarious(JPanel panel) {
		GridBagLayout gridbag = new GridBagLayout();
		panel.setLayout(gridbag);

		GridBagConstraints c = new GridBagConstraints();

		c.ipadx = 2;
		c.ipady = 2;

		int yLevel = 0;

		JLabel label;

		// iterations field
		iterationsField = new JTextField(5);
		iterationsField.setMinimumSize(MINIMUM_TEXT_SIZE);
		iterationsField.setText(Integer.toString(ITERATIONS));
		label = new JLabel("Iterations: ", SwingConstants.LEFT);

		c.gridx = 0;
		c.gridy = yLevel;
		c.anchor = GridBagConstraints.EAST;
		gridbag.setConstraints(label, c);
		panel.add(label);

		c.gridx = 1;
		c.gridy = yLevel;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(iterationsField, c);
		panel.add(iterationsField);
		iterationsField.requestFocus();

		yLevel++;

		// distance to move contour field
		moveContourDistanceField = new JTextField(7);
		moveContourDistanceField.setMinimumSize(MINIMUM_TEXT_SIZE);
		moveContourDistanceField.setText(Double.toString(MOVE_CONTOUR_DISTANCE));
		label = new JLabel("Movement Contour Distance: ", SwingConstants.LEFT);

		c.gridx = 0;
		c.gridy = yLevel;
		c.anchor = GridBagConstraints.EAST;
		gridbag.setConstraints(label, c);
		panel.add(label);

		c.gridx = 1;
		c.gridy = yLevel;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(moveContourDistanceField, c);
		panel.add(moveContourDistanceField);

		yLevel++;

		// distance to move feet field
		moveFeetDistanceField = new JTextField(7);
		moveFeetDistanceField.setMinimumSize(MINIMUM_TEXT_SIZE);
		moveFeetDistanceField.setText(Double.toString(MOVE_FEET_DISTANCE));
		label = new JLabel("Movement Feet Distance: ", SwingConstants.LEFT);

		c.gridx = 0;
		c.gridy = yLevel;
		c.anchor = GridBagConstraints.EAST;
		gridbag.setConstraints(label, c);
		panel.add(label);

		c.gridx = 1;
		c.gridy = yLevel;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(moveFeetDistanceField, c);
		panel.add(moveFeetDistanceField);

		yLevel++;

		// normalize field
		normalizeBox = new JCheckBox("Normalize", MOVE_FEET);

		c.gridx = 0;
		c.gridy = yLevel;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(normalizeBox, c);
		panel.add(normalizeBox);

		yLevel++;

		// move contours box
		moveContoursBox = new JCheckBox("Move Contours", MOVE_CONTOURS);

		c.gridx = 0;
		c.gridy = yLevel;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(moveContoursBox, c);
		panel.add(moveContoursBox);

		yLevel++;

		// move spider feet box
		moveFeetBox = new JCheckBox("Move Feet", MOVE_FEET);

		c.gridx = 0;
		c.gridy = yLevel;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(moveFeetBox, c);
		panel.add(moveFeetBox);

		yLevel++;

		// cooling field
		coolBox = new JCheckBox("Cool", COOL);

		c.gridx = 0;
		c.gridy = yLevel;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(coolBox, c);
		panel.add(coolBox);

		yLevel++;

		// show steps field
		animateBox = new JCheckBox("Display Steps", SHOW_STEPS);

		c.gridx = 0;
		c.gridy = yLevel;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(animateBox, c);
		panel.add(animateBox);

		yLevel++;

		// output data
		outputDataBox = new JCheckBox("Output Data", OUTPUTDATA);

		c.gridx = 0;
		c.gridy = yLevel;
		c.gridwidth = 2;
		c.anchor = GridBagConstraints.WEST;
		gridbag.setConstraints(outputDataBox, c);
		panel.add(outputDataBox);
	}

	public void say(String message) {
		outputBuffer += "\n" + message;
		try {
			// End of report
			if (message.substring(0, 5).equals("SCORE")) {
				// Store the values into a file to compute the average value
				try {
					BufferedWriter out = new BufferedWriter(new FileWriter("metrics.txt", true)); 
					out.write("\n"+outputBuffer); 
					out.close();					 
				} catch (Exception e1) {
					e1.printStackTrace();
				}

				// Print metric values
				// Store the values into a file to compute the average value
				try {
					File fOut = new File("metrics.txt");
					PrintStream sOut = new PrintStream(new FileOutputStream(fOut, true));
					sOut.println("\n"+outputBuffer);
					sOut.close();					 
				} catch (Exception e1) {
					e1.printStackTrace();
				}

				// Print metric values
				outputDataArea.append("\n" + outputBuffer);
				outputDataArea.setCaretPosition(
					outputDataArea.getText().length());
				outputBuffer = "";
			}
		} catch (StringIndexOutOfBoundsException x) {
		}
	}

	//
	// Actions on events
	//
	protected void stopButton(ActionEvent event) {
		worker = new SwingWorker() {
			public Object construct() {
				manipulator.setStopFlag(true);
				return manipulator;
			}
		};
		worker.start();
	}

	protected void bezierButton(ActionEvent event) {
		if (viewer.isBezier() == true) {
			viewer.setBezier(false);
		} else {
			viewer.setBezier(true);
		}

		//if (!running) {
		viewer.redrawImage();
		//}
	}
	protected void loadButton(ActionEvent event) {
		if (running) {
			return;
		}

		JFileChooser chooser = null;
		if (currentFile == null) {
			chooser = new JFileChooser(System.getProperty("user.dir"));
		} else {
			chooser = new JFileChooser(currentFile);
		}

		int returnVal = chooser.showOpenDialog(this);
		if (returnVal != JFileChooser.APPROVE_OPTION) {
			return;
		}

		currentFile = chooser.getSelectedFile();

		//loadSettings(currentFile);
		loadSettings(currentFile);
	}
	protected void scaleDownButton(ActionEvent event) {
		scaleDiagram(cud, 0.5);
	}

	protected void scaleUpButton(ActionEvent event) {
		scaleDiagram(cud, 2.0);
	}

	protected void qualityButton(ActionEvent event) {
		if (!running) {
			assignValues();
		}

		boolean oldOutput = manipulator.outputData;
		manipulator.outputData = true;
		Quality quality = manipulator.currentQuality();
		String qualityString = quality.toString();
		try {
			statusLabel.setText(
				"QUALITY: " + qualityString.substring(0, 5) + " ");
		} catch (StringIndexOutOfBoundsException e) {
			statusLabel.setText(
				"QUALITY: " + qualityString.substring(0, 1) + " ");
		}

		manipulator.outputData = oldOutput;

	}

	protected void saveButton(ActionEvent event) {
		if (running) {
			return;
		}

		JFileChooser chooser = null;
		if (currentFile == null) {
			chooser = new JFileChooser(System.getProperty("user.dir"));
		} else {
			chooser = new JFileChooser(currentFile);
		}

		int returnVal = chooser.showSaveDialog(this);

		if (returnVal != JFileChooser.APPROVE_OPTION) {
			return;
		}

		currentFile = chooser.getSelectedFile();

		saveSettings(currentFile);
	}

	//
	// Auxiliary functions
	//
	protected void addMetric(Metric m, boolean active) {
		metrics.add(m);
		metricBooleans.add(new Boolean(active));
	}

	protected void loadDefaultParameters() {
		//String fileName = System.getProperty("user.dir")+File.separator+DEFAULT_PROPERTY_FILE_NAME;
		//File file = new File(fileName);
		//loadSettings(file);
		loadSettings(new File(DEFAULT_PROPERTY_FILE_NAME));
	}

	protected void loadSettings(File file) {
		InputStream inStream = null;
		//by default, look for defaults in filespace
		try {
			inStream = new FileInputStream(file);
		} catch (FileNotFoundException x) {
			try {
				System.out.println("file " + file + " not found in filespace");
				URL url =
					new ContourPolygonSet(new Vector())
						.getClass()
						.getClassLoader()
						.getResource(
						file.toString());
				URLConnection con = url.openConnection();
				con.setDoInput(true);
				con.setDoOutput(false);
				con.setUseCaches(false);
				inStream = con.getInputStream();
			} catch (IOException ex) {
				ex.printStackTrace();
			}
		}
		//System.out.println("inStream is " + inStream);

		try {
			//System.out.println("interpreting from stream");
			int separatorInd;
			//BufferedReader b = new BufferedReader(new FileReader(file));
			InputStreamReader isReader = new InputStreamReader(inStream);
			BufferedReader b = new BufferedReader(isReader);
			String line = b.readLine();
			//System.out.println("line is "+line);
			while (line != null) {
				if (line.equals("")) {
					line = b.readLine();
					continue;
				}

				StringBuffer parseLine = new StringBuffer(line);

				separatorInd = parseLine.indexOf(" ");
				String part = parseLine.substring(0, separatorInd);
				parseLine.delete(0, separatorInd + 1);
				//System.out.println("line is "+line);

				if (part.equals("iterations")) {
					iterationsField.setText(parseLine.toString());
				}
				if (part.equals("move_contour_distance")) {
					moveContourDistanceField.setText(parseLine.toString());
				}
				if (part.equals("move_foot_distance")) {
					moveFeetDistanceField.setText(parseLine.toString());
				}
				if (part.equals("normalize")) {
					if (parseLine.toString().equals("true")) {
						normalizeBox.setSelected(true);
					} else {
						normalizeBox.setSelected(false);
					}
				}
				if (part.equals("move_contour")) {
					if (parseLine.toString().equals("true")) {
						moveContoursBox.setSelected(true);
					} else {
						moveContoursBox.setSelected(false);
					}
				}
				if (part.equals("move_foot")) {
					if (parseLine.toString().equals("true")) {
						moveFeetBox.setSelected(true);
					} else {
						moveFeetBox.setSelected(false);
					}
				}
				if (part.equals("cool")) {
					if (parseLine.toString().equals("true")) {
						coolBox.setSelected(true);
					} else {
						coolBox.setSelected(false);
					}
				}
				if (part.equals("animate")) {
					if (parseLine.toString().equals("true")) {
						animateBox.setSelected(true);
					} else {
						animateBox.setSelected(false);
					}
				}
				if (part.equals("outputData")) {
					if (parseLine.toString().equals("true")) {
						outputDataBox.setSelected(true);
					} else {
						outputDataBox.setSelected(false);
					}
				}

				if (part.equals("measurer")) {
					separatorInd = parseLine.indexOf(" ");
					parseLine.delete(0, separatorInd + 1);

					separatorInd = parseLine.indexOf(" ");
					String metricName = parseLine.substring(0, separatorInd);
					parseLine.delete(0, separatorInd + 1);
					//System.out.println("metricName is "+metricName);

					separatorInd = parseLine.indexOf(" ");
					String metricValue = parseLine.substring(0, separatorInd);
					parseLine.delete(0, separatorInd + 1);
					//System.out.println("metricValue is "+metricValue);

					String metricSelected = parseLine.toString();
					//System.out.println("metricSelected is "+metricSelected);

					for (int index = 0; index < metrics.size(); index++) {
						Metric metric = (Metric) (metrics.get(index));
						String derivedName = metric.getClass().toString();

						int derivedPlaceOfS = derivedName.indexOf("rwd.");
						String shortDerivedName = derivedName.substring(derivedPlaceOfS, derivedName.length());

						if (shortDerivedName.equals(metricName)) {
							JTextField mf =
								(JTextField) metricFields.get(index);
							mf.setText(metricValue);

							JCheckBox mb =
								(JCheckBox) (metricButtons.get(index));
							if (metricSelected.equals("true")) {
								mb.setSelected(true);
							} else {
								mb.setSelected(false);
							}
						}
					}
				}

				if (part.equals("manipulator")) {
					separatorInd = parseLine.indexOf(" ");
					parseLine.delete(0, separatorInd + 1);

					separatorInd = parseLine.indexOf(" ");
					String manipulatorName =
						parseLine.substring(0, separatorInd);
					parseLine.delete(0, separatorInd + 1);

					String manipulatorSelected = parseLine.toString();

					for (int index = 0; index < manipulators.size(); index++) {
						Manipulator manipulator =
							(Manipulator) (manipulators.get(index));

						if (manipulator
							.getClass()
							.toString()
							.equals("class " + manipulatorName)) {

							JRadioButton mb =
								(JRadioButton) (manipulatorButtons.get(index));
							if (manipulatorSelected.equals("true")) {
								mb.setSelected(true);
							} else {
								mb.setSelected(false);
							}
						}
					}
				}
				line = b.readLine();
			}
			b.close();
			//System.out.println("done interpreting from stream");
		} catch (IOException e) {
			e.printStackTrace();
		}

	}

	protected void saveSettings(File file) {

		try {
			BufferedWriter b = new BufferedWriter(new FileWriter(file));

			// save the various stuff
			b.write("iterations " + iterationsField.getText());
			b.newLine();
			b.write("move_contour_distance " + moveContourDistanceField.getText());
			b.newLine();
			b.write("move_foot_distance " + moveFeetDistanceField.getText());
			b.newLine();
			b.write("normalize " + Boolean.toString(normalizeBox.isSelected()));
			b.newLine();
			b.write("move_contour " + Boolean.toString(moveContoursBox.isSelected()));
			b.newLine();
			b.write("move_foot " + Boolean.toString(moveFeetBox.isSelected()));
			b.newLine();
			b.write("cool " + Boolean.toString(coolBox.isSelected()));
			b.newLine();
			b.write("animate " + Boolean.toString(animateBox.isSelected()));
			b.newLine();
			b.write("outputData " + Boolean.toString(outputDataBox.isSelected()));
			b.newLine();

			// save the Metrics
			for (int index = 0; index < metrics.size(); index++) {
				Metric metric = (Metric) (metrics.get(index));

				JTextField mf = (JTextField) metricFields.get(index);
				b.write("measurer " + metric.getClass() + " " + mf.getText());

				JCheckBox mb = (JCheckBox) (metricButtons.get(index));
				b.write(" " + Boolean.toString(mb.isSelected()));
				b.newLine();
			}

			// save the manipulators
			for (int index = 0; index < manipulators.size(); index++) {
				Manipulator manipulator =
					(Manipulator) (manipulators.get(index));
				JRadioButton mb =
					(JRadioButton) (manipulatorButtons.get(index));
				b.write(
					"manipulator "
						+ manipulator.getClass()
						+ " "
						+ Boolean.toString(mb.isSelected()));
				b.newLine();
			}

			b.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	/**
	 * this scale moves the points of all the polys, by a factor given
	 * in the multiplier.
	 */
	protected void scaleDiagram(
		ConcreteUnitaryDiagram cud,
		double multiplier) {

		if (cud == null)
			return;

		// scale contours
		Set contourPolys = cud.getConcreteContours();
		Iterator polyIt = contourPolys.iterator();
		while (polyIt.hasNext()) {
			ConcreteContour cp = (ConcreteContour) polyIt.next();
			Polygon poly = cp.getPolygon();

			int[] xs = poly.xpoints;
			int[] ys = poly.ypoints;
			for (int j = 0; j < poly.npoints; j++) {

				xs[j] = (int) (xs[j] * multiplier);
				ys[j] = (int) (ys[j] * multiplier);
			}
			cp.autoPlaceLabel();
		}

		// scale spiders, only need to change feet location
		java.util.List spiderFeet = cud.getFeetAndLegsSet().getFeet();
		Iterator feetIt = spiderFeet.iterator();
		while (feetIt.hasNext()) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot) feetIt.next();
			foot.setLocation(
				(int) (foot.getX() * multiplier),
				(int) (foot.getY() * multiplier));
		}

		// scale arcs
		Vector arcs = cud.getArcSet().getArcs();
		Iterator arcIt = arcs.iterator();
		while (arcIt.hasNext()) {
			ConcreteArc arc = (ConcreteArc) arcIt.next();

			ConcreteArcEnd start = arc.getStart();
			ConcreteArcEnd middle = arc.getMiddle();
			ConcreteArcEnd end = arc.getEnd();
			start.setLocation(
				(int) (start.getX() * multiplier),
				(int) (start.getY() * multiplier));
			middle.setLocation(
				(int) (middle.getX() * multiplier),
				(int) (middle.getY() * multiplier));
			end.setLocation(
				(int) (end.getX() * multiplier),
				(int) (end.getY() * multiplier));
		}

		// scale box
		Polygon boxPoly = cud.getDecorationBox().getPolygon();

		int[] boxxs = boxPoly.xpoints;
		int[] boxys = boxPoly.ypoints;
		for (int j = 0; j < boxPoly.npoints; j++) {

			boxxs[j] = (int) (boxxs[j] * multiplier);
			boxys[j] = (int) (boxys[j] * multiplier);
		}

		cud.refreshZones(true);
		cud.notifyObservers();

		viewer.redrawImage();
	}

	protected void assignValues() {
		// distance to move
		manipulator.maxMoveContourDistance = Double.parseDouble(moveContourDistanceField.getText());
		manipulator.maxMoveFeetDistance = Double.parseDouble(moveFeetDistanceField.getText());
		// animate the movement
		manipulator.animated = animateBox.isSelected();
		// cool the movement
		manipulator.cooling = coolBox.isSelected();
		// output data
		manipulator.outputData = outputDataBox.isSelected();
		// move contours
		manipulator.moveContours = moveContoursBox.isSelected();
		// move feet
		manipulator.moveFeet = moveFeetBox.isSelected();
		manipulator.dataDisplay = this;

		// get the Metrics
		metricValues = new Vector();
		for (int index = 0; index < metrics.size(); index++) {
			Metric m = (Metric) (metrics.get(index));

			JTextField mf = (JTextField) metricFields.get(index);
			double weight = Double.parseDouble(mf.getText());
			m.setWeighting(weight);

			JCheckBox mb = (JCheckBox) (metricButtons.get(index));
			if (mb.isSelected()) {
				metricValues.add(m);
				if (m instanceof ZoneAreaProportional) {
					zonesToPoints.clear();
					Set cZones = cud.getConcreteZones();
					Iterator i = cZones.iterator();
					while (i.hasNext()) {
						ConcreteZone cz = (ConcreteZone) i.next();
						Zone z = cz.getAbstractZone();
						boolean gotInput = false;
						while (!gotInput) {
							String pointText =
								JOptionPane.showInputDialog(
									this,
									"points for " + z);
							try {
								if (pointText == null) {
									zonesToPoints.put(z, new Double(-1.0));
									gotInput = true;
								} else {
									double d = Double.parseDouble(pointText);
									zonesToPoints.put(z, new Double(d));
									System.out.println(
										"mapping " + z + " to " + d);
									gotInput = true;
								}
							} catch (NumberFormatException x) {
								JOptionPane.showMessageDialog(
									this,
									"can't interpret "
										+ pointText
										+ " as a number");
							}
						}
					}
				}
			}

			/*
						int mIndex = -1;
						for (int i = 0; i < metricValues.size(); i++) {
							Metric mi = (Metric) (metricValues.get(i));
							if (mi == m) {
								mIndex = i;
							}
						}
			
						if (mb.isSelected()) {
							if (mIndex == -1) {
								metricValues.add(m);
							}
						} else {
							if (mIndex != -1) {
								metricValues.remove(mIndex);
							}
						}
			*/
		}

		// get the manipulator
		for (int index = 0; index < manipulators.size(); index++) {
			Manipulator m = (Manipulator) (manipulators.get(index));
			JRadioButton mb = (JRadioButton) (manipulatorButtons.get(index));
			if (mb.isSelected()) {
				manipulator = m;
			}
		}

		// add the Metrics to the manipulator
		manipulator.metrics = metricValues;
	}

	//
	// Local properties
	//
	protected EditorFrame frame;
	protected DiagramViewerInterface viewer;
	protected ConcreteUnitaryDiagram cud;

	protected Vector metricValues = new Vector();
	protected Vector manipulators = new Vector();
	protected Vector metrics = new Vector();
	protected int numRows;
	protected Vector metricBooleans = new Vector();
	protected HashMap zonesToPoints = new LinkedHashMap();

	protected Manipulator manipulator = null;

	protected ButtonGroup manipulatorGroup;

	protected Vector manipulatorButtons = new Vector();
	protected Vector metricFields = new Vector();
	protected Vector metricButtons = new Vector();
	protected JButton bezierButton;
	protected JButton loadButton;
	protected JButton saveButton;
	protected JButton qualityButton;
	protected JButton scaleUpButton;
	protected JButton scaleDownButton;
	protected JButton startButton;
	protected JButton stopButton;

	protected boolean integrateViewer = false;

	protected JPanel variousPanel;
	protected JPanel manipulatorPanel;
	protected JPanel metricPanel;
	protected JPanel buttonPanel;
	protected JPanel progressPanel;

	protected JTextField iterationsField;
	protected JTextField moveContourDistanceField;
	protected JTextField moveFeetDistanceField;

	protected JCheckBox normalizeBox;
	protected JCheckBox moveContoursBox;
	protected JCheckBox moveFeetBox;
	protected JCheckBox coolBox;
	protected JCheckBox animateBox;
	protected JCheckBox outputDataBox;

	protected JLabel statusLabel;
	protected JLabel progressLabel;
	protected JLabel iterationsLabel;
	protected JTextArea outputDataArea;

	protected boolean running = false;
	protected File currentFile = null;
	private String outputBuffer = "";

	protected SwingWorker worker;
	protected MetricSmoother smoother = null;

	public static int ITERATIONS = 20;
	protected static Dimension MINIMUM_TEXT_SIZE = new Dimension(80,21);
	protected static double MOVE_CONTOUR_DISTANCE = 200.0;
	protected static double MOVE_FEET_DISTANCE = MOVE_CONTOUR_DISTANCE;
	protected static boolean COOL = true;
	protected static boolean MOVE_CONTOURS = true;
	protected static boolean MOVE_FEET = true;
	protected static boolean OUTPUTDATA = false;
	protected static boolean SHOW_STEPS = true;
	protected static String DEFAULT_PROPERTY_FILE_NAME =
		"default.parametersettings";
}
