package rwd.views.smoother;

import java.awt.Polygon;
import java.util.Vector;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import javax.swing.JLabel;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.spiderdiagram.ConcreteSpiderFoot;
import rwd.gui.editor.viewers.DiagramViewerInterface;
import rwd.metrics.AreaPolygonAdapter;
import rwd.metrics.DiagramArea;

public class ComparisonFastHillClimber
	extends AbstractHillClimber 
{
	public ComparisonFastHillClimber(
		List checkers,
		ConcreteUnitaryDiagram cud,
		Vector metrics,
		DiagramViewerInterface viewer,
		boolean moveContours,
		boolean moveFeet,
		boolean isAnimated) {
		super(checkers, cud, metrics, viewer, moveContours, moveFeet, isAnimated);
	}
	int stepNumber = 0;

	protected void makeChange(int numberOfSteps, JLabel changeLabel) {
		setStopFlag(false);
		Quality quality = currentQuality();
		Quality pointsQuality = currentQualityPoints();
		Quality polygonsQuality = currentQualityPolygons();
		double move = maxMoveContourDistance;

        double diagramArea = DiagramArea.getDiagramArea(cud);
        
		for (int j = 1; j <= numberOfSteps; j++) {
			if (stopFlag) {
				viewer.draw(cud);
				return;
			}
			if (changeLabel != null) {
				changeLabel.setText(Integer.toString(j));
			}
			if (outputData) {
				/**/
				//System.out.println("");
				/**/
				//System.out.println("ITERATION in " + getClass() + ": " + j);
			}
            
            // Move contours
            if (moveContours)
				moveContours(cud.getConcreteContours(), diagramArea, move, pointsQuality, polygonsQuality);

			// Move feet
			if (moveFeet)
				moveSpiderFeet(cud.getConcreteFeet(), diagramArea, move, quality);
			
			if (cooling) {
				move = move - (maxMoveContourDistance / numberOfSteps);
			}
			if (Math.random() > 0.5) {
				stepNumber++;
			}
		}
	}
	
	protected void moveContours(Set contours, double diagramArea, double move, Quality pointsQuality, Quality polygonsQuality) {
		Iterator i=contours.iterator();
		while(i.hasNext()){
			ConcreteContour cp = (ConcreteContour)i.next();
			Polygon p = cp.getPolygon();

			// this allows the scaling of movement according to the size of contours
			double pArea = AreaPolygonAdapter.getAreaOf(p);
			double sizeFactor = Math.sqrt(pArea/diagramArea);
			double adjustedMove = move*sizeFactor;

			if(hasPointMetrics()) {
				int[] xs = p.xpoints;
				int[] ys = p.ypoints;

				for (int posNumber = 0; posNumber < p.npoints; posNumber++) {
					int oldX = xs[posNumber];
					int oldY = ys[posNumber];

					for (int direction = 0; direction <= 3; direction++) {
						double moveX = 0.0;
						double moveY = 0.0;
						if ((direction + stepNumber) % 4 == 0) {
							moveX = adjustedMove;
						}
						if ((direction + stepNumber) % 4 == 1) {
							moveX = -adjustedMove;
						}
						if ((direction + stepNumber) % 4 == 2) {
							moveY = adjustedMove;
						}
						if ((direction + stepNumber) % 4 == 3) {
							moveY = -adjustedMove;
						}

						xs[posNumber] += moveX;
						ys[posNumber] += moveY;

						// Notify observers (metrics and viewers)
						notifyObservers(cp, false, animated);
							
						Quality newPointsQuality = currentQualityPoints();
						if (structureStillOK() && newPointsQuality.compare(pointsQuality) <= 0) {
							pointsQuality = newPointsQuality;
							stepNumber++;
							if (animated) {
								viewer.draw(cud);
							}
							break;
						} else {
							xs[posNumber] = oldX;
							ys[posNumber] = oldY;
						}
					}
					p.invalidate();
				}
			}

			if (contourMove && hasPolygonMetrics()) {
				Vector oldPoints = getPolygonPoints(p);

				for (int direction = 0; direction <= 3; direction++) {
					double moveX = 0.0;
					double moveY = 0.0;
					if ((direction + stepNumber) % 4 == 0) {
						moveX = adjustedMove;
					}
					if ((direction + stepNumber) % 4 == 1) {
						moveX = -adjustedMove;
					}
					if ((direction + stepNumber) % 4 == 2) {
						moveY = adjustedMove;
					}
					if ((direction + stepNumber) % 4 == 3) {
						moveY = -adjustedMove;
					}

					movePolygonPoints(p, (int) moveX, (int) moveY);

					// Notify observers (metrics and viewers)
					notifyObservers(cp, false, animated);
						
					Quality newPolygonsQuality = currentQualityPolygons();

					if (structureStillOK() && newPolygonsQuality.compare(polygonsQuality) <= 0) {
						polygonsQuality = newPolygonsQuality;
						stepNumber++;
						if (animated) {
							viewer.draw(cud);
						}
						break;
					} else {
						setPolygonPoints(oldPoints, p);
					}
				}
			}

			cp.autoPlaceLabel();
		}		
	}
	
	protected void moveSpiderFeet(Set feet, double diagramArea, double move, Quality quality) {
		// For each feet
		Iterator i = feet.iterator();
		double adjustedMove = move;
		while (i.hasNext()) {
			ConcreteSpiderFoot foot = (ConcreteSpiderFoot) i.next();
			int oldX = foot.getX();
			int oldY = foot.getY();

			// For each possible directions
			for (int direction = 0; direction <= 3; direction++) {
				double moveX = 0.0;
				double moveY = 0.0;
				if ((direction + stepNumber) % 4 == 0) {
					moveX = adjustedMove;
				}
				if ((direction + stepNumber) % 4 == 1) {
					moveX = -adjustedMove;
				}
				if ((direction + stepNumber) % 4 == 2) {
					moveY = adjustedMove;
				}
				if ((direction + stepNumber) % 4 == 3) {
					moveY = -adjustedMove;
				}

				foot.setLocation((int)(oldX + moveX), (int)(oldY + moveY));

				// Notify observers
				notifyObservers(foot, false, animated);

				// Check structure
				boolean ok = structureStillOK();

				// If ok compute quality 
				if (ok) {
					// Compute quality
					Quality newQuality = this.currentQuality();

					// If better quality accept position 
					if (newQuality.compare(quality) <= 1) {
						// ComputeQuality
						quality = newQuality;
						stepNumber++;
						// Notify observers
						notifyObservers(foot, false, animated);
						break;
					// Move back
					} else {
						foot.setLocation(oldX, oldY);
					}
				// Move back
				} else {
					foot.setLocation(oldX, oldY);
				}
			}
		}		
	}

}
