package rwd.views.smoother;

import java.util.Vector;
import java.util.List;

import javax.swing.JLabel;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.gui.editor.viewers.DiagramViewerInterface;
import rwd.metrics.*;

/**
 * This runs the smoothing. Construct a new one with the first constructor
 * using the default metrics or use the second constructor to pass a
 * list of metrics, registered with the given manipulator, use the run
 * method to start the smoothin process. The second constructor
 * also allows a parameter window to be associated with the smoothing, for output
 * purposes. Set values such as animation, movement and cooling directly in the
 * manipulator.
 */
public class MetricSmoother 
	implements Runnable 
{
    public int iterations = 20;
    public JLabel progressLabel = null;
    public Manipulator manipulator = null;

    protected StructureChecker sc;
    protected ConcreteUnitaryDiagram cud;
    protected DiagramViewerInterface viewer;

    public MetricSmoother(
        ConcreteUnitaryDiagram cud,
        DiagramViewerInterface viewer,
        int iterations) 
    {
    	System.out.println("build a MetricSmoother");
        Vector metrics = new Vector();
        metrics.add(new ContourArea(cud, 100));
        metrics.add(new ContourRoundnessAngle(cud, 2000));
        metrics.add(new ContourRoundnessSegmentLength(cud, 8000));
        metrics.add(new ContourClosenessSegmentPoint(cud, 6));
        metrics.add(new ContourClosenessPointPoint(cud, .02));
        metrics.add(new ZoneArea(cud, 1));

		List checkers = new Vector();
		checkers.add(new ZonePresenceChecker(cud));
		checkers.add(new SpiderChecker(cud));
        manipulator =
            new FastHillClimber(
                checkers,
                cud,
                metrics,
                viewer,
                false,
                false,
                false);

        this.cud = cud;
        this.viewer = viewer;
        this.iterations = iterations;
    }

    /**
     * This allows a manipulator to be passed to the smoother, if a dataDisplay 
     * is required, it should be set directly in the manipulator
     */
    public MetricSmoother(
        ConcreteUnitaryDiagram cud,
        DiagramViewerInterface viewer,
        Manipulator manipulator,
        JLabel progressLabel,
        int iterations) {
        this.cud = cud;
        this.viewer = viewer;
        this.manipulator = manipulator;
        this.progressLabel = progressLabel;
        this.iterations = iterations;
    }

    /** Use this to execute the smoother */
    public void run() {
        manipulator.makeChange(iterations, progressLabel);
        System.out.println("invoke draw on " + manipulator.viewer.hashCode());
        manipulator.viewer.draw(manipulator.cud);
    }
}
