package rwd.views.smoother;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Date;
import java.util.List;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;

import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.gui.editor.EditorFrame;
import rwd.gui.editor.viewers.DiagramViewerComponent;
import rwd.gui.editor.viewers.DiagramViewerInterface;
import rwd.metrics.*;

/**
 */
public class MetricSmootherParameterWindow
	extends AbstractParameterWindow
	implements ActionListener 
{
	protected StructureChecker sc;

	protected JButton updateButton;
	protected JButton closeButton;

	public MetricSmootherParameterWindow(
		EditorFrame frame,
		ConcreteUnitaryDiagram cud,
		DiagramViewerComponent viewer,
		String title,
		int offset,
		boolean noWindow) {
		super(frame, cud, viewer, title, offset, noWindow);
		if (!noWindow) {
			startup(frame, cud, viewer, false);
		}
	}

	public MetricSmootherParameterWindow(
		EditorFrame frame,
		ConcreteUnitaryDiagram cud,
		DiagramViewerInterface viewer) {
		this(frame, cud, viewer, false);
	}

	/** 
	 * The integrateViewer parameter means the viewer appears inside the parameter window
	 * if it is false, the viewer should be provided with an alternative top level window.
	 */
	public MetricSmootherParameterWindow(
		EditorFrame frame,
		ConcreteUnitaryDiagram cud,
		DiagramViewerInterface viewer,
		boolean integrateViewer) {
		super(frame, cud, viewer, integrateViewer);
	}

	int indexOfProportionalMetric;

	protected void setupMetrics() {
		// register the Metrics
//		addMetric(new ContourRoundnessMetric(cud, 2.0), true);
//		addMetric(new ContourEdgeLengthMetric(cud, 2.0), true);
//		addMetric(new ContourAreaMetric(cud, 1.0), false);
//		addMetric(new ContourClosenessMetricPts(cud, 3.0), true);
//		addMetric(new ContourClosenessMetricEdgePt(cud, 1.0), true);
//		addMetric(new DiagramAreaMetric(cud, 1.0), false);
//		addMetric(new ZoneAreaMetric(cud, 1.0), true);
//		addMetric(new ZoneAreaMetricProportional(cud, 1.0, zonesToPoints), false);

//		addMetric(new ContourRoundnessAngle(cud, 1E2), true);
//		addMetric(new ContourRoundnessSegmentLength(cud, 1E4), true);
//		addMetric(new ContourArea(cud, 1.0), false);
//		addMetric(new ContourClosenessPointPoint(cud, 2E-4), true);
//		addMetric(new ContourClosenessSegmentPoint(cud, 4E-6), true);
//		addMetric(new DiagramArea(cud, 1.0), false);
//		addMetric(new ZoneArea(cud, 2E-1), true);
//		addMetric(new ZoneAreaProportional(cud, 1.0, zonesToPoints), false);
//		numRows = 8;

//		addMetric(new ContourRoundnessAngle(cud, 1E0), true);
//		addMetric(new ContourRoundnessSegmentLength(cud, 1E0), true);
//		addMetric(new ContourArea(cud, 1E0), false);
//		addMetric(new ContourClosenessPointPoint(cud, 1E0), true);
//		addMetric(new ContourClosenessSegmentPoint(cud, 1E0), true);
//		addMetric(new DiagramArea(cud, 1.0), false);
//		addMetric(new ZoneArea(cud, 1E0), true);
//		addMetric(new ZoneAreaProportional(cud, 1.0, zonesToPoints), false);
//		numRows = 8;
//		
//		addMetric(new NodeNodeSeparation(cud, 1.0), true);
//		addMetric(new NodeContourSeparation(cud, 1.0), true);
//		addMetric(new EdgeLength(cud, 1.0), true);
//		addMetric(new AngularResolution(cud, 1.0), true);
//		addMetric(new EdgeEdgeCrossing(cud, 1.0), true);
//		addMetric(new EdgeContourCrossing(cud, 1.0), true);
//		indexOfProportionalMetric = 4;

		addMetric(new ContourRoundnessAngle(cud, 4E2), true);
		addMetric(new ContourRoundnessSegmentLength(cud, 2E2), true);
		addMetric(new ContourArea(cud, 1E0), false);
		addMetric(new ContourClosenessPointPoint(cud, 3.25E-1), true);
		addMetric(new ContourClosenessSegmentPoint(cud, 25E3), true);
		addMetric(new DiagramArea(cud, 1.0), false);
		addMetric(new ZoneArea(cud, 57E-3), true);
		addMetric(new ZoneAreaProportional(cud, 1.0, zonesToPoints), false);
		numRows = 8;
		
		addMetric(new NodeNodeSeparation(cud, 2E-3), true);
		addMetric(new NodeContourPointSeparation(cud, 2E-3), true);
		addMetric(new NodeContourSegmentSeparation(cud, 2E-3), false);
		addMetric(new NodeEdgeSeparation(cud, 25E3), true);
		addMetric(new EdgeLength(cud, 7E1), true);
		addMetric(new AngularResolution(cud, 16E-2), true);
		addMetric(new EdgeEdgeCrossing(cud, 37E-1), true);
		addMetric(new EdgeContourCrossing(cud, 83e-2), true);
		indexOfProportionalMetric = 4;
		//System.out.println("metrics as built in ParameterWindow are" + metrics);
	}
	
	protected void setupManipulators() {
		// register the manipulators
		List checkers = new Vector();
		checkers.add(new ZonePresenceChecker(cud));
		checkers.add(new SpiderChecker(cud));
		manipulators.add(
			new FastHillClimber(
				checkers,
				cud,
				metrics,
				viewer,
				MOVE_CONTOURS,
				MOVE_FEET,
				SHOW_STEPS));
		manipulators.add(
			new RandomHillClimber(
				new Vector(checkers),
				cud,
				metrics,
				viewer,
				MOVE_CONTOURS,
				MOVE_FEET,
				SHOW_STEPS));
	}

	protected void setupButtons(JPanel panel) {
		GridBagLayout gridbag = new GridBagLayout();
		panel.setLayout(gridbag);

		GridBagConstraints c = new GridBagConstraints();

		c.ipadx = 2;
		c.ipady = 2;
		c.anchor = GridBagConstraints.CENTER;

		scaleDownButton = new JButton("Scale Down");
		getRootPane().setDefaultButton(startButton);
		scaleDownButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				scaleDownButton(event);
			}
		});

		c.gridx = 0;
		c.gridy = 0;
		gridbag.setConstraints(scaleDownButton, c);
		panel.add(scaleDownButton);

		scaleUpButton = new JButton("Scale Up");
		scaleUpButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				scaleUpButton(event);
			}
		});

		c.gridx = 1;
		c.gridy = 0;
		gridbag.setConstraints(scaleUpButton, c);
		panel.add(scaleUpButton);

		startButton = new JButton("Start");
		getRootPane().setDefaultButton(startButton);
		startButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				startButton(event);
			}
		});

		c.gridx = 2;
		c.gridy = 0;
		gridbag.setConstraints(startButton, c);
		panel.add(startButton);

		stopButton = new JButton("Stop");
		stopButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				stopButton(event);
			}
		});

		c.gridx = 3;
		c.gridy = 0;
		gridbag.setConstraints(stopButton, c);
		panel.add(stopButton);

		iterationsField.requestFocus();

		qualityButton = new JButton("Get Quality");
		qualityButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				qualityButton(event);
			}
		});

		c.gridx = 0;
		c.gridy = 1;
		gridbag.setConstraints(qualityButton, c);
		panel.add(qualityButton);

		bezierButton = new JButton("Bezier");
		bezierButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				bezierButton(event);
			}
		});

		c.gridx = 1;
		c.gridy = 1;
		gridbag.setConstraints(bezierButton, c);
		panel.add(bezierButton);

		loadButton = new JButton("Load Settings");
		loadButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				loadButton(event);
			}
		});

		c.gridx = 2;
		c.gridy = 1;
		gridbag.setConstraints(loadButton, c);
		panel.add(loadButton);

		saveButton = new JButton("Save Settings");
		saveButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				saveButton(event);
			}
		});

		c.gridx = 3;
		c.gridy = 1;
		gridbag.setConstraints(saveButton, c);
		panel.add(saveButton);
		/*
				closeButton = new JButton("Close");
				closeButton.addActionListener(new ActionListener() {
					public void actionPerformed(ActionEvent event) {
						closeButton(event);
					}
				});
		
				xLevel++;
				c.gridx = xLevel;
				c.gridy = 1;
				gridbag.setConstraints(closeButton, c);
				panel.add(closeButton);
		*/
	}

	/** 
	 * For changing the Metric constants of selected Metrics
	 * whilst a manipulator thread is running.
	 */
	/*	protected void updateButton(ActionEvent event) {
			for (int index = 0; index < metrics.size(); index++) {
	
				Metric m = (Metric) (metrics.get(index));
				if (metricValues.contains(m)) {
					JTextField mf = (JTextField) metricFields.get(index);
					double weight = Double.parseDouble(mf.getText());
					m.setWeighting(weight);
				}
			}
	
		}
	*/

	public void smooth() {
		startButton(null);
	}

	public void stop() {
		if (manipulator != null) {
			manipulator.setStopFlag(true);
		}
	}

	public void startButton(ActionEvent event) {
		if (running) {
			return;
		}
	
		assignValues();
	
		worker = new SwingWorker() {
			public Object construct() {
				running = true;
				statusLabel.setText("RUNNING  ");
				iterationsLabel.setText(iterationsField.getText());
				Metric.NORMALIZE = normalizeBox.isSelected();
	
				Date start = new Date();
				manipulator.viewer.draw(cud);
				int iterations =
					(int) Double.parseDouble(iterationsField.getText());
	
				smoother =
					generateSmoother(
						cud,
						viewer,
						manipulator,
						progressLabel,
						iterations);
				smoother.run();
	
				Date stop = new Date();
				outputDataArea.append(
					"\nDone in " + (stop.getTime() - start.getTime()) / 1000.0);
				running = false;
				statusLabel.setText(
					"Completed in "
						+ Double.toString(
							((int) (stop.getTime() - start.getTime())
								/ 1000.0))
						+ "s. ");
				return manipulator;
			}
		};
	
		worker.start();
	}

	protected MetricSmoother generateSmoother(
		ConcreteUnitaryDiagram cud,
		DiagramViewerInterface viewer,
		Manipulator manipulator,
		JLabel progressLabel,
		int iterations) {
		return new MetricSmoother(
			cud,
			viewer,
			manipulator,
			progressLabel,
			iterations);
	}
}
