package rwd.views.smoother;

import java.util.*;

import rwd.abstractsyntax.eulerdiagram.*;
import rwd.abstractsyntax.spiderdiagram.*;

import rwd.concretesyntax.*;
import rwd.concretesyntax.eulerdiagram.*;
import rwd.concretesyntax.spiderdiagram.*;

public class SpiderChecker 
	implements StructureChecker 
{
	public SpiderChecker(ConcreteUnitaryDiagram cud) {
		startAt(cud);
	}
	
    public void startAt(ConcreteUnitaryDiagram cud) {
    	if (cud == null) return;
    	cud.refreshZones(false);
    	cud.notifyObservers();
		initialAbstractSpiders = getAbstractSpiders(cud);
		System.out.println("starting with "+initialAbstractSpiders); 
    }

    public boolean checkStillOK(ConcreteUnitaryDiagram cud) {
    	// Get current set of abstract spiders
    	Set abstractSpiders = getAbstractSpiders(cud);
    	// Check if the initial and current set of spiders is equal
		if(abstractSpiders == null) {
			System.out.println("Duplicate abstract spider in SpiderChecker");
			return false;
		}
		if (abstractSpiders.size() != initialAbstractSpiders.size()) {
			System.out.println("Could not match up zones in ZonePresentChecker, by counting");            
			return false;
		}
		Iterator i = abstractSpiders.iterator();
		while(i.hasNext()){
			Spider newas = (Spider) i.next();
			boolean foundMatch = false;
			Iterator i2 = initialAbstractSpiders.iterator();
			while(i2.hasNext()){
				Spider oldas = (Spider)i2.next();
				if (oldas.equals(newas)) {
					foundMatch = true;
					break;
				}
			}
			if (!foundMatch) { 
				System.out.println("Could not match up spiders in SpiderChecker, by pairing");
				return false;
			}
		}
		return true;
    }
    
	protected Set getAbstractSpiders(Set concreteSpiders, ConcreteUnitaryDiagram cud) {
		Set res = new LinkedHashSet();
		Iterator i = concreteSpiders.iterator();
		while (i.hasNext()) {
			ConcreteSpider cs = (ConcreteSpider)i.next();
			Spider as = cs.getAbstractSpider(cud); 
			if(res.contains(as)){
				return null;
			} else {
				res.add(as);
			}
		}
		return res;
	}
   
    protected Set getAbstractZones(Set concreteZones) {
    	Set res = new LinkedHashSet();
    	Iterator i = concreteZones.iterator();
    	while (i.hasNext()) {
    		ConcreteZone cz = (ConcreteZone)i.next();
            Zone z = cz.getAbstractZone(); 
            if(res.contains(z)){
                return null;
            } else {
                res.add(z);
            }
    	}
    	return res;
    }

	protected Set getAbstractSpiders(ConcreteUnitaryDiagram cud) {
		Set res = new LinkedHashSet();
		// Get concrete zones, spiders
		cud.refreshZones(false);
		Set cZones = cud.getVisibleConcreteZones();
		Set cSpiders = cud.getConcreteSpiders();
		// For each spider
		Iterator i = cSpiders.iterator();
		while (i.hasNext()) {
			ConcreteSpider cs = (ConcreteSpider)i.next();
			// Compute abstract spider
			Spider as = cs.getAbstractSpider(cud);
			if (as != null) {
				res.add(as);
			}
		}
		return res;
	}

	//
	// Local properties
	//
	// Remember initial initial abstract spiders 
	protected Set initialAbstractSpiders;
}
