package rwd.views.smoother;
import java.awt.Polygon;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;
import java.util.Vector;
import java.util.Iterator;
import java.util.LinkedHashSet;
import java.util.Set;

import rwd.abstractsyntax.eulerdiagram.Zone;
import rwd.concretesyntax.ConcreteUnitaryDiagram;
import rwd.concretesyntax.eulerdiagram.ConcreteContour;
import rwd.concretesyntax.eulerdiagram.ConcreteZone;

public class ZonePresenceChecker
	implements StructureChecker 
{
	public ZonePresenceChecker(ConcreteUnitaryDiagram cud) {
		startAt(cud);
	}
	
    public void startAt(ConcreteUnitaryDiagram cud) {
    	if (cud == null) return;
		cud.refreshZones(false);
		cud.notifyObservers();
        presentZones = getAbstractZones(cud.getVisibleConcreteZones());
		System.out.println("starting with "+presentZones); 
    }

    public boolean checkStillOK(ConcreteUnitaryDiagram cud) {
		//System.out.println("checking "+cud);
		// check polygons don't self-intersect (convex)     
        Iterator cpolyIt = cud.getConcreteContours().iterator();
        Vector lines = new Vector();
        while (cpolyIt.hasNext()) {
            lines = new Vector();
            ConcreteContour cp = (ConcreteContour) (cpolyIt.next());
            Polygon p = cp.getPolygon();
            PathIterator pathIt = cp.getPolygon().getPathIterator(null);
            double[] pointInfo = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
            pathIt.currentSegment(pointInfo);
            double firstX = pointInfo[0];
            double firstY = pointInfo[1];
            double startX = firstX;
            double startY = firstY;
			//System.out.println("got point "+firstX+","+firstY);
            double nextX, nextY;
            for (int dummy = 0; dummy < p.npoints - 1; dummy++) {
                pathIt.next();
                pathIt.currentSegment(pointInfo);
                nextX = pointInfo[0];
                nextY = pointInfo[1];
                //System.out.println("got point "+nextX+","+nextY);
                // build a line segment
                lines.add(new Line2D.Double(firstX, firstY, nextX, nextY));
                firstX = nextX;
                firstY = nextY;
            }
            lines.add(new Line2D.Double(firstX, firstY, startX, startY));

            // we have an array of lines.
            // now use intersectsLine to check well-formedness of Polygon
            //System.out.println("we have "+lines.size()+" lines");
            for (int index1 = 0; index1 < lines.size() - 1; index1++) {
                Line2D line1 = (Line2D) (lines.get(index1));
                for (int index2 = index1 + 2;
                    index2 < lines.size();
                    index2++) {
                    if (!(index1 == 0 && index2 == lines.size() - 1)) {
                        Line2D line2 = (Line2D) (lines.get(index2));
                        //System.out.println("does line "+index1+" cross line "+index2+"?");
   	                    //System.out.println("line  "+line1.getP1()+" to "+line1.getP2());
       	                //System.out.println("line  "+line2.getP1()+" to "+line2.getP2());
                        if (line1.intersectsLine(line2)) {
                            System.out.println("polygon self-intersects!");
                            return false;
                        }
                    }
                }
            }
        }

		// Check if the abstract zones are the same
		cud.refreshZones(true);
		Set zoneAreas = cud.getVisibleConcreteZones();
		Set abstractZoneAreas = getAbstractZones(zoneAreas);
		if(abstractZoneAreas == null) {
			System.out.println("Duplicate abstract zone in ZonePresenceChecker");
			return false;
		}
		if (abstractZoneAreas.size() != presentZones.size()) {
			System.out.println("Could not match up zones in ZonePresentChecker, by counting");            
			return false;
		}
		Iterator i = abstractZoneAreas.iterator();
		while(i.hasNext()){
			Zone z = (Zone) i.next();
			boolean foundMatch = false;
			Iterator i2 = presentZones.iterator();
			while(i2.hasNext()){
				Zone presentOne = (Zone)i2.next();
				if (z.equals(presentOne)) {
					foundMatch = true;
					break;
				}
			}
			if (!foundMatch) { 
				System.out.println("Could not match up zones in ZonePresentChecker, by pairing");
				return false;
			}
		}

		// Check if all zones are singulars		
        i = zoneAreas.iterator();
        while(i.hasNext()){
            ConcreteZone cz = (ConcreteZone)i.next();
            Zone az = cz.getAbstractZone();
            Vector cCont = az.getContainingContours();
			Vector eCont = az.getExcludingContours();
            if(cCont.size()>0) {
            	boolean isSingular = cz.getArea().isSingular(); 
            	if (!isSingular){
					System.out.println("area is not singular");
                	return false;
            	}
            }
        }

        //System.out.println("structure still OK");
        return true;
    }
    
    protected Set getAbstractZones(Set concreteZones) {
    	Set res = new LinkedHashSet();
    	Iterator i = concreteZones.iterator();
    	while (i.hasNext()) {
    		ConcreteZone cz = (ConcreteZone)i.next();
            Zone z = cz.getAbstractZone(); 
            if(res.contains(z)){
                return null;
            } else {
                res.add(z);
            }
    	}
    	return res;
    }

	// remember which zones are present at start
	protected Set presentZones;
}
